package com.zillion.database.agent;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import java.util.UUID;

import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.client.Delete;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.util.Bytes;
import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.data.Stat;
import org.zillion.util.common.HexToByte;
import org.zillion.util.common.MemoryQueryUtil;
import org.zillion.util.common.MyException;
import org.zillion.util.json.Convertor_JSON;
import org.zillion.util.json.FastJsonReaderUtil;
import org.zillion.util.json.FastJsonUtil;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.zillion.database.async.AsyncJob;
import com.zillion.database.consistency.ZillionConsistencyThread;
import com.zillion.database.consistency.ZillionRedissonLockThread;
import com.zillion.database.util.Configuration_ElasticSearch;
import com.zillion.database.util.Configuration_MySQL;
import com.zillion.database.util.Configuration_Redis;
import com.zillion.database.util.DataSourceConfig_HikariCP;
import com.zillion.database.util.DatabaseImpl_HBase;
import com.zillion.database.util.DatabaseImpl_Mix;
import com.zillion.database.util.DatabaseImpl_MySQL;
import com.zillion.database.util.ITableWrapper;
import com.zillion.database.util.KerberosUtil;
import com.zillion.database.util.ObserverCheck;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.common.ElasticSearchUtil;
import com.zillion.util.common.HashObject;
import com.zillion.util.common.HashUtil;
import com.zillion.util.common.SplitUtil;
import com.zillion.util.common.SplitWrapper;
import com.zillion.util.common.ZKRepositoryUtil;
import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.SchemaParser;
import com.zillion.util.table.SchemaRepository;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.table.SchemaRepositoryHelper;
import com.zillion.util.table.ZillionTableColumn;
import com.zillion.util.table.ZillionTableColumnEnum;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;
import com.zillion.util.zk.ZKHelper;
import com.zillion.util.zk.ZKLockCEDU;
import com.zillion.util.zk.ZKLockRW;
import com.zillion.util.zk.ZKWrapper;

import redis.clients.jedis.ShardedJedis;

public class ZillionAgent implements IZillionAgent {
	public String database_type;
	public Configuration configuration;
	public Configuration configuration_bak;
	public Configuration_MySQL configuration_MySQL;
	public Configuration_ElasticSearch configuration_ElasticSearch;
	public Configuration_Redis configuration_Redis;
	public String full_name = "undefined";

	public SchemaRepositoryHelper helper;

	private static String generateName() {
		String result = "null";
		try {
			Random rand = new Random();
			result = InetAddress.getLocalHost().getHostAddress() + ":" + rand.nextInt(65536);
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
		return result;
	}

	private static String generateName(String name) {
		String result = "null";
		try {
			result = InetAddress.getLocalHost().getHostAddress() + ":" + name;
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
		return result;
	}

	public ZillionAgent(Configuration configuration) {
		this.database_type = "HBase";
		this.configuration = configuration;
		this.configuration_bak = null;
		this.full_name = generateName();
		this.CheckConfig(configuration);
	}

	public ZillionAgent(Configuration configuration, String name) {
		this.database_type = "HBase";
		this.configuration = configuration;
		this.configuration_bak = null;
		this.full_name = generateName(name);
		this.CheckConfig(configuration);
	}

	public ZillionAgent(Configuration configuration, Configuration configuration_bak) {
		this.database_type = "HBase";
		this.configuration = configuration;
		this.configuration_bak = configuration_bak;
		this.full_name = generateName();
		this.CheckConfig(configuration);
	}

	public ZillionAgent(Configuration configuration, Configuration configuration_bak, String name) {
		this.database_type = "HBase";
		this.configuration = configuration;
		this.configuration_bak = configuration_bak;
		this.full_name = generateName(name);
		this.CheckConfig(configuration);
	}

	private void CheckConfig(Configuration configuration) {
		String database_type = configuration.get("zillion.database_type");
		if (database_type != null) {
			this.database_type = database_type;
		}
		if (this.database_type.equals("MySQL") || this.database_type.equals("Mix")) {
			DataSourceConfig_HikariCP DataSourceConfig_c3p0 = new DataSourceConfig_HikariCP();
			DataSourceConfig_c3p0.jdbcUrl = configuration.get("MySQL.c3p0.jdbcUrl");
			DataSourceConfig_c3p0.user = configuration.get("MySQL.c3p0.user");
			DataSourceConfig_c3p0.password = configuration.get("MySQL.c3p0.password");
			DataSourceConfig_c3p0.acquireIncrement = configuration.getInt("MySQL.c3p0.acquireIncrement", 3);
			DataSourceConfig_c3p0.idleConnectionTestPeriod = configuration.getInt("MySQL.c3p0.idleConnectionTestPeriod", 60);
			DataSourceConfig_c3p0.initialPoolSize = configuration.getInt("MySQL.c3p0.initialPoolSize", 10);
			DataSourceConfig_c3p0.maxIdleTime = configuration.getInt("MySQL.c3p0.maxIdleTime", 60);
			DataSourceConfig_c3p0.maxPoolSize = configuration.getInt("MySQL.c3p0.maxPoolSize", 50);
			DataSourceConfig_c3p0.maxStatements = configuration.getInt("MySQL.c3p0.maxStatements", 100);
			DataSourceConfig_c3p0.minPoolSize = configuration.getInt("MySQL.c3p0.minPoolSize", 10);
			String zookeeper_connectionAddress = configuration.get("MySQL.zookeeper.quorum");
			int zookeeper_sessionTimeout = configuration.getInt("MySQL.zookeeper.session.timeout", 30000);
			Configuration_MySQL configuration_MySQL = new Configuration_MySQL(DataSourceConfig_c3p0, zookeeper_connectionAddress,
					zookeeper_sessionTimeout);
			this.configuration_MySQL = configuration_MySQL;
		}
		{
			String ElasticSearch_enable = configuration.get("ElasticSearch.enable");
			if (ElasticSearch_enable != null && ElasticSearch_enable.equalsIgnoreCase("true")) {
				String ElasticSearch_hostname = configuration.get("ElasticSearch.hostname");
				int ElasticSearch_port = configuration.getInt("ElasticSearch.port", 9200);
				String ElasticSearch_scheme = configuration.get("ElasticSearch.scheme");
				Configuration_ElasticSearch configuration_ElasticSearch = new Configuration_ElasticSearch(ElasticSearch_hostname, ElasticSearch_port,
						ElasticSearch_scheme);
				this.configuration_ElasticSearch = configuration_ElasticSearch;
			}
		}
		{
			String Redis_enable = configuration.get("Redis.enable");
			if (Redis_enable != null && Redis_enable.equalsIgnoreCase("true")) {
				String address = configuration.get("Redis.address");
				int index_ = address.indexOf(':');
				String Redis_hostname = address.substring(0, index_);
				int Redis_port = Integer.parseInt(address.substring(index_ + 1));
				String Redis_password = configuration.get("Redis.password");
				int Redis_lock_thread_count = Integer.parseInt(configuration.get("Redis.lock_thread_count"));
				Configuration_Redis configuration_Redis = new Configuration_Redis(Redis_hostname, Redis_port, Redis_password,
						Redis_lock_thread_count);
				this.configuration_Redis = configuration_Redis;
			}

		}
	}

	public String getFull_name() {
		return full_name;
	}

	public void setFull_name(String full_name) {
		this.full_name = full_name;
	}

	public SchemaRepositoryContainer container = new SchemaRepositoryContainer();

	public boolean print = true;
	public boolean observer = true;

	public void Start() throws Exception {
		boolean without_lock = false;
		this.Start(without_lock);
	}

	public void Start(boolean without_lock) throws Exception {
		if (!this.container.initialized) {
			this.container.agent = this;
			this.container.without_lock = without_lock;
			this.helper = new SchemaRepositoryHelper(this.full_name);
			this.helper.Initialize(container, database_type, configuration, configuration_bak, configuration_MySQL, configuration_ElasticSearch,
					configuration_Redis, print, observer);
			this.container.consitencyThread = new ZillionConsistencyThread(this.container);
			this.container.consitencyThread.start();
			if (configuration_Redis != null) {
				for (int i = 0; i < configuration_Redis.lock_thread_count; i++) {
					ZillionRedissonLockThread redissonThread = new ZillionRedissonLockThread(this.container);
					redissonThread.start();
					this.container.redissonLockThreadList.add(redissonThread);
				}
			}
		}
	}

	public void Stop() {
		if (this.container.initialized) {
			try {
				this.container.consitencyThread.RequestStop();
				this.container.consitencyThread.join();
				if (configuration_Redis != null) {
					for (int i = 0; i < this.container.redissonLockThreadList.size(); i++) {
						ZillionRedissonLockThread redissonThread = this.container.redissonLockThreadList.get(i);
						redissonThread.RequestStop();
					}
					for (int i = 0; i < this.container.redissonLockThreadList.size(); i++) {
						ZillionRedissonLockThread redissonThread = this.container.redissonLockThreadList.get(i);
						redissonThread.join();
					}
				}
				this.container.thread.RequestStop();
				this.container.thread.join();
				this.container.DatabaseImpl.Stop();
				if (this.container.esClient != null) {
					this.container.esClient.close();
				}
				this.container.ZK.close();
				this.container.initialized = false;
				if (this.container.redisPool != null) {
					this.container.redisPool.close();
				}
				if (this.container.redisson != null) {
					this.container.redisson.shutdown();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public com.sagacloud.json.JSONObject Query(com.sagacloud.json.JSONObject queryJSON) throws Exception {
		com.sagacloud.json.JSONObject result = Query(queryJSON, false);
		return result;
	}

	public com.sagacloud.json.JSONObject Query(com.sagacloud.json.JSONObject queryJSON_Saga, boolean unuse) throws Exception {
		JSONObject queryJSON = (JSONObject) Convertor_JSON.SagaJson2FastJson(queryJSON_Saga);
		JSONObject result = this.Query(queryJSON, unuse);
		com.sagacloud.json.JSONObject result_Saga = (com.sagacloud.json.JSONObject) Convertor_JSON.FastJson2SagaJson(result);
		return result_Saga;
	}

	public JSONObject Query(JSONObject queryJSON) throws Exception {
		return this.Query(queryJSON, false);
	}

	public JSONObject Query(JSONObject queryJSON, boolean unuse) throws Exception {
		String uuid = UUID.randomUUID().toString();
		AsyncJob job = new AsyncJob(uuid, queryJSON);
		JSONObject result = this.Query(queryJSON, unuse, job);
		return result;
	}

	public JSONObject Query(JSONObject queryJSON, boolean unuse, AsyncJob job) throws Exception {
		if (!this.container.initialized) {
			throw new MyException("ZillionAgent: " + "container not initialized");
		}

		KerberosUtil.reloginFromKeytab(container.DatabaseImpl, false, true);

		JSONObject result = new JSONObject();
		result.put("Result", "failure");

		String QueryType = (String) queryJSON.get("QueryType");
		if (QueryType.startsWith("monitor_")) {
			ZillionMonitorUtil.process(this, queryJSON, result);
		} else if (QueryType.equals("admin_password_clear")) {
			Stat stat = container.ZK.exists(ZillionConstant.zk_root + "/password", false);
			if (stat != null) {
				container.ZK.delete(ZillionConstant.zk_root + "/password", -1);
			}
			result.remove("Result");
			result.put("Result", "success");
		} else if (QueryType.equals("admin_password_set")) {
			String Password = (String) queryJSON.get("Password");
			Stat stat = container.ZK.exists(ZillionConstant.zk_root + "/password", false);
			if (stat == null) {
				container.ZK.create(ZillionConstant.zk_root + "/password", Password.getBytes(), CreateMode.PERSISTENT);
			} else {
				container.ZK.setData(ZillionConstant.zk_root + "/password", Password.getBytes(), -1);
			}
			result.remove("Result");
			result.put("Result", "success");
		} else if (QueryType.equals("admin_password_get")) {
			Stat stat = container.ZK.exists(ZillionConstant.zk_root + "/password", false);
			String Content;
			if (stat == null) {
				Content = null;
			} else {
				byte[] bytes = container.ZK.getData(ZillionConstant.zk_root + "/password", false, stat);
				Content = new String(bytes);
			}
			result.remove("Result");
			result.put("Result", "success");
			result.put("Content", Content);
		} else if (QueryType.equals("database_list")) {
			List<String> itemList = new ArrayList<String>();
			itemList.addAll(this.container.namespaceList);
			Collections.sort(itemList);
			JSONArray content = new JSONArray();
			for (String item : itemList) {
				content.add(item);
			}
			result.remove("Result");
			result.put("Result", "success");
			result.put("Count", (long) itemList.size());
			result.put("Content", content);
		} else if (QueryType.equals("database_create")) {
			String Database = (String) queryJSON.get("Database");
			try {
				boolean success = this.NDL_AddDatabase(Database);
				if (success) {
					result.remove("Result");
					result.put("Result", "success");
				}
			} catch (Exception e) {
				throw e;
			}
		} else if (QueryType.equals("database_drop")) {
			String Database = (String) queryJSON.get("Database");
			try {
				boolean success = this.NDL_DropDatabase(Database);
				if (success) {
					result.remove("Result");
					result.put("Result", "success");
				}
			} catch (Exception e) {
				throw e;
			}
		} else if (QueryType.equals("table_list_strict")) {
			String Database = (String) queryJSON.get("Database");
			List<String> itemList = this.container.repositoryMap.get(Database).TableList();
			Collections.sort(itemList);
			JSONArray content = new JSONArray();
			for (String item : itemList) {
				content.add(item);
			}
			result.remove("Result");
			result.put("Result", "success");
			result.put("Count", (long) content.size());
			result.put("Content", content);
		} else if (QueryType.equals("table_list")) {
			String Database = (String) queryJSON.get("Database");
			List<String> itemList = this.container.repositoryMap.get(Database).TableList();
			Collections.sort(itemList);
			JSONArray content = new JSONArray();
			for (String item : itemList) {
				content.add(item);
				ZillionTableSchema table_schema = this.container.repositoryMap.get(Database).GetTable(item);
				String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + table_schema.Namespace;
				List<String> child_nameList = container.ZK
						.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + table_schema.table_name, false);
				Collections.sort(child_nameList);
				for (String child_name : child_nameList) {
					content.add(item + "_" + child_name);
				}
			}
			result.remove("Result");
			result.put("Result", "success");
			result.put("Count", (long) content.size());
			result.put("Content", content);
		} else if (QueryType.equals("split_list")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			ZillionTableSchema table_schema = this.container.repositoryMap.get(Database).GetTable(Datatable);
			JSONArray content = new JSONArray();
			{
				String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + table_schema.Namespace;
				List<String> child_nameList = container.ZK
						.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + table_schema.table_name, false);
				Collections.sort(child_nameList);
				for (String child_name : child_nameList) {
					content.add(child_name);
				}
			}
			result.remove("Result");
			result.put("Result", "success");
			result.put("Count", (long) content.size());
			result.put("Content", content);
		} else if (QueryType.equals("table_detail")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			String tableName = Datatable;
			ZillionTableSchema table_schema = this.container.repositoryMap.get(Database).GetTable(tableName);
			if (table_schema == null) {
				SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
				int index_ = tableName.lastIndexOf("_");
				if (index_ != -1) {
					String prefix = tableName.substring(0, index_);
					String suffix = tableName.substring(index_ + 1);
					try {
						boolean is_split_table = false;
						String query_month = null;
						SplitWrapper sw = new SplitWrapper(tableName, is_split_table, query_month, table_schema);
						SplitUtil.SplitParse(sdf, this.container.repositoryMap.get(Database), prefix, suffix, sw);
						tableName = sw.tableName;
						is_split_table = sw.is_split_table;
						query_month = sw.query_month;
						table_schema = sw.schema;
					} catch (ParseException e) {
						// e.printStackTrace();
					}
				}
			}
			List<ZillionTableIndex> index_schemaList = this.container.repositoryMap.get(Database).GetIndex(tableName);
			JSONArray Indexes = new JSONArray();
			for (ZillionTableIndex index_schema : index_schemaList) {
				JSONObject indexJSON = new JSONObject();
				indexJSON.put("Name", index_schema.index_name);
				JSONArray indexColumns = new JSONArray();
				for (String column : index_schema.Key) {
					indexColumns.add(column);
				}
				indexJSON.put("Columns", indexColumns);
				Indexes.add(indexJSON);
			}
			JSONObject content = (JSONObject) FastJsonReaderUtil.Instance()
					.parse(JSON.toJSONString(table_schema.JSON, SerializerFeature.WriteMapNullValue));
			content.put("Indexes", Indexes);
			result.remove("Result");
			result.put("Result", "success");
			result.put("Content", content);
		} else if (QueryType.equals("table_create")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			JSONObject Definition = (JSONObject) queryJSON.get("Definition");
			JSONObject table_schema = (JSONObject) FastJsonReaderUtil.Instance()
					.parse(JSON.toJSONString(Definition, SerializerFeature.WriteMapNullValue));
			JSONArray Indexes = (JSONArray) table_schema.get("Indexes");
			table_schema.remove("Indexes");
			try {
				boolean success = this.DDL_AddTable(Database, Datatable, table_schema, Indexes);
				if (success) {
					result.remove("Result");
					result.put("Result", "success");
				}
			} catch (Exception e) {
				throw e;
			}
		} else if (QueryType.equals("table_clear")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			try {
				String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + Database;

				ZillionTableSchema schema = this.container.repositoryMap.get(Database).GetTable(Datatable);
				boolean is_split_table = false;
				String selected_query_month = null;
				if (schema == null) {
					SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
					int index_ = Datatable.lastIndexOf("_");
					if (index_ != -1) {
						String prefix = Datatable.substring(0, index_);
						String suffix = Datatable.substring(index_ + 1);
						try {
							String tableName = null;
							SplitWrapper sw = new SplitWrapper(tableName, is_split_table, selected_query_month, schema);
							SplitUtil.SplitParse(sdf, this.container.repositoryMap.get(Database), prefix, suffix, sw);
							tableName = sw.tableName;
							is_split_table = sw.is_split_table;
							selected_query_month = sw.query_month;
							schema = sw.schema;
						} catch (ParseException e) {
							throw e;
						}
					}
				}

				String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();
				String parent_path_meta = zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + schema.table_name;
				String lock_type_meta = "w";
				// ��ȡmeta��
				String node_path_meta = ZKLockRW.Create(container.ZK, parent_path_meta, lock_type_meta, this.full_name);
				try {
					LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + node_path_meta);
					boolean wait_success = ZKLockRW.WaitFor(container.ZK, parent_path_meta, node_path_meta, ZillionConstant.WaitFor_seconds);
					LogUtil.info(
							" ********************************** " + method_name + " metalock:" + " WaitFor " + wait_success + " " + node_path_meta);
					if (!wait_success) {
						throw new MyException("ZillionAgent: " + "WaitFor fail " + parent_path_meta);
					}
					int count;
					if (schema.table_type.startsWith("split_")) {
						List<String> query_monthList = new ArrayList<String>();
						if (is_split_table) {
							query_monthList.add(selected_query_month);
						} else {
							List<String> children = container.ZK
									.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name, false);
							query_monthList.addAll(children);
						}
						count = container.DatabaseImpl.clear_table_split(Database, schema, query_monthList, container.ZK);
					} else {
						count = container.DatabaseImpl.clear_table_not_split(Database, schema, container.ZK);
					}

					if (schema.elastic_search) {
						ElasticSearchUtil.clearIndex(container.esClient, schema.Namespace + "_" + schema.table_name);
					}

					result.remove("Result");
					result.put("Result", "success");
					result.put("Count", (long) count);
				} catch (Exception e) {
					throw e;
				} finally {
					// �ͷ�meta��
					ZKLockRW.Finish(container.ZK, node_path_meta);
					LogUtil.info(" ********************************** " + method_name + " metalock:" + " finish " + node_path_meta);
				}
			} catch (Exception e) {
				throw e;
			}
		} else if (QueryType.equals("table_drop")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			try {
				String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + Database;

				ZillionTableSchema schema = this.container.repositoryMap.get(Database).GetTable(Datatable);
				boolean is_split_table = false;
				String query_month = null;
				if (schema == null) {
					SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
					int index_ = Datatable.lastIndexOf("_");
					if (index_ != -1) {
						String prefix = Datatable.substring(0, index_);
						String suffix = Datatable.substring(index_ + 1);
						try {
							String tableName = null;
							SplitWrapper sw = new SplitWrapper(tableName, is_split_table, query_month, schema);
							SplitUtil.SplitParse(sdf, this.container.repositoryMap.get(Database), prefix, suffix, sw);
							tableName = sw.tableName;
							is_split_table = sw.is_split_table;
							query_month = sw.query_month;
							schema = sw.schema;
						} catch (ParseException e) {
							throw e;
						}
					}
				}

				Stat stat = new Stat();
				if (is_split_table) {
					String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();
					String parent_path_meta = zkpath_namespace + ZillionConstant.zk_splitlocks + "/" + schema.table_name;
					String lock_type_meta = "w";
					// ��ȡmeta��
					String node_path_meta = ZKLockRW.Create(container.ZK, parent_path_meta, lock_type_meta, this.full_name);
					try {
						LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + node_path_meta);
						boolean wait_success = ZKLockRW.WaitFor(container.ZK, parent_path_meta, node_path_meta, ZillionConstant.WaitFor_seconds);
						LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + wait_success + " "
								+ node_path_meta);
						if (!wait_success) {
							throw new MyException("ZillionAgent: " + "WaitFor fail " + parent_path_meta);
						}
						byte[] hash = container.ZK
								.getData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name + "/" + query_month, false, stat);
						HashObject hashObject = HashUtil.Parse(hash);
						hashObject.delete_count = hashObject.insert_count - hashObject.delete_count;
						hashObject.insert_count = 0;
						container.ZK.create(
								zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + schema.table_name + "/" + query_month + "/" + "tmp-",
								hashObject.toBytes(), CreateMode.PERSISTENT_SEQUENTIAL);
						for (ZillionTableIndex schema_index : schema.Indexes) {
							String indexName = schema_index.index_name;
							container.ZK.create(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (schema.table_name + "." + indexName) + "/"
									+ query_month + "/" + "tmp-", hashObject.toBytes(), CreateMode.PERSISTENT_SEQUENTIAL);
						}
						for (int index_wait = 0; index_wait < 15; index_wait++) {
							try {
								Thread.sleep(1000L);
							} catch (Exception e) {
								e.printStackTrace();
							}
							boolean hash_tmp_finish = true;
							List<String> children = container.ZK
									.getChildren(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + schema.table_name + "/" + query_month, false);
							if (children.size() > 0) {
								hash_tmp_finish = false;
							}
							for (ZillionTableIndex schema_index : schema.Indexes) {
								String indexName = schema_index.index_name;
								children = container.ZK.getChildren(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/"
										+ (schema.table_name + "." + indexName) + "/" + query_month, false);
								if (children.size() > 0) {
									hash_tmp_finish = false;
									break;
								}
							}
							if (hash_tmp_finish) {
								try {
									Thread.sleep(1000L);
								} catch (Exception e) {
									e.printStackTrace();
								}
								break;
							}
						}
						container.ZK.delete(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name + "/" + query_month, -1);
						ZKHelper.recursive(container.ZK,
								zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + schema.table_name + "/" + query_month);
						container.DatabaseImpl.meta_query_delete_split(Database, schema.table_name + "_" + query_month);
						container.DatabaseImpl.ori_drop_table(Database, ZillionConstant.prefix_data + schema.table_name + "_" + query_month);
						for (ZillionTableIndex schema_index : schema.Indexes) {
							String indexName = schema_index.index_name;
							container.ZK.delete(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (schema.table_name + "." + indexName) + "/"
									+ query_month, -1);
							ZKHelper.recursive(container.ZK,
									zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (schema.table_name + "." + indexName) + "/" + query_month);
							container.DatabaseImpl.meta_query_delete_split(Database, (schema.table_name + "." + indexName) + "_" + query_month);
							container.DatabaseImpl.ori_drop_table(Database,
									ZillionConstant.prefix_index + (schema.table_name + "." + indexName) + "_" + query_month);
						}

						hashObject = HashUtil.Parse(hash);
						long count = hashObject.insert_count - hashObject.delete_count;
						result.remove("Result");
						result.put("Result", "success");
						result.put("Count", (long) count);
					} catch (Exception e) {
						throw e;
					} finally {
						// �ͷ�meta��
						ZKLockRW.Finish(container.ZK, node_path_meta);
						LogUtil.info(" ********************************** " + method_name + " metalock:" + " finish " + node_path_meta);
					}
				} else {
					boolean success = this.DDL_DropTable(Database, Datatable);
					if (success) {
						result.remove("Result");
						result.put("Result", "success");
					}
				}
			} catch (Exception e) {
				throw e;
			}
		} else if (QueryType.equals("index_create")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			String Name = (String) queryJSON.get("Name");
			JSONArray Columns = (JSONArray) queryJSON.get("Columns");

			JSONArray indexArray = new JSONArray();
			JSONObject index_schema = new JSONObject();
			index_schema.put("Name", Name);
			index_schema.put("Columns", Columns);
			indexArray.add(index_schema);
			try {
				boolean success;
				if (queryJSON.containsKey("thread_count")) {
					long thread_count = (Long) queryJSON.get("thread_count");
					success = this.DDL_AddIndex(Database, Datatable, indexArray, (int) thread_count, job);
				} else {
					success = this.DDL_AddIndex(Database, Datatable, indexArray, job);
				}
				if (success) {
					result.remove("Result");
					result.put("Result", "success");
				}
			} catch (Exception e) {
				throw e;
			}
		} else if (QueryType.equals("index_drop")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			try {
				JSONArray indexArray = new JSONArray();
				String Name = (String) queryJSON.get("Name");
				indexArray.add(Name);
				boolean success = this.DDL_DropIndex(Database, Datatable, indexArray);
				if (success) {
					result.remove("Result");
					result.put("Result", "success");
				}
			} catch (Exception e) {
				throw e;
			}
		} else if (QueryType.equals("indexs_create")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			JSONArray indexArray = (JSONArray) queryJSON.get("Indexs");
			try {
				boolean success;
				if (queryJSON.containsKey("thread_count")) {
					long thread_count = (Long) queryJSON.get("thread_count");
					success = this.DDL_AddIndex(Database, Datatable, indexArray, (int) thread_count, job);
				} else {
					success = this.DDL_AddIndex(Database, Datatable, indexArray, job);
				}
				if (success) {
					result.remove("Result");
					result.put("Result", "success");
				}
			} catch (Exception e) {
				throw e;
			}
		} else if (QueryType.equals("indexs_drop")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			JSONArray Names = (JSONArray) queryJSON.get("Names");
			try {
				boolean success = this.DDL_DropIndex(Database, Datatable, Names);
				if (success) {
					result.remove("Result");
					result.put("Result", "success");
				}
			} catch (Exception e) {
				throw e;
			}
		} else if (QueryType.equals("split_table_create")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			int index_ = Datatable.lastIndexOf('_');
			String tableName = Datatable.substring(0, index_);
			String splitName = Datatable.substring(index_ + 1);
			ZillionTableSchema table_schema = this.container.repositoryMap.get(Database).GetTable(tableName);
			boolean exist_split = false;
			{
				String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + table_schema.Namespace;
				List<String> child_nameList = container.ZK
						.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + table_schema.table_name, false);
				Collections.sort(child_nameList);
				for (String child_name : child_nameList) {
					if (child_name.equals(splitName)) {
						exist_split = true;
						break;
					}
				}
			}
			if (!exist_split) {
				ITableWrapper dataWrapper = table_schema.connect(container, splitName, true);
				dataWrapper.unuse();
			}
			result.remove("Result");
			result.put("Result", "success");
		} else if (QueryType.equals("table_alter")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			String AlterType = (String) queryJSON.get("AlterType");
			JSONObject AlterContent = (JSONObject) queryJSON.get("AlterContent");
			try {
				boolean success = this.DDL_AlterTable(Database, Datatable, AlterType, AlterContent);
				if (success) {
					result.remove("Result");
					result.put("Result", "success");
				}
			} catch (Exception e) {
				throw e;
			}
		} else if (QueryType.equals("table_count")) {
			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			try {
				String namespace = Database;
				String tableName = Datatable;
				String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;

				ZillionTableSchema schema = this.container.repositoryMap.get(Database).GetTable(Datatable);
				boolean is_split_table = false;
				String query_month = null;
				if (schema == null) {
					SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
					int index_ = Datatable.lastIndexOf("_");
					if (index_ != -1) {
						String prefix = Datatable.substring(0, index_);
						String suffix = Datatable.substring(index_ + 1);
						try {
							SplitWrapper sw = new SplitWrapper(tableName, is_split_table, query_month, schema);
							SplitUtil.SplitParse(sdf, this.container.repositoryMap.get(Database), prefix, suffix, sw);
							tableName = sw.tableName;
							is_split_table = sw.is_split_table;
							query_month = sw.query_month;
							schema = sw.schema;
						} catch (ParseException e) {
							// e.printStackTrace();
						}
					}
				}

				if (!(this.database_type.equals("HBase") || (this.database_type.equals("Mix") && !schema.store_in_minor_db))) {
					DatabaseImpl_MySQL mysql;
					if (this.container.DatabaseImpl instanceof DatabaseImpl_MySQL) {
						mysql = (DatabaseImpl_MySQL) this.container.DatabaseImpl;
					} else {
						mysql = ((DatabaseImpl_Mix) this.container.DatabaseImpl).mysql;
					}
					if (is_split_table) {
						String ori_tableName = ZillionConstant.prefix_data + tableName + "_" + query_month;
						int count = mysql.ori_table_count(namespace, ori_tableName);
						result.remove("Result");
						result.put("Result", "success");
						result.put("Count", (long) count);
					} else {
						if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
							List<String> child_nameList = container.ZK
									.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name, false);
							Collections.sort(child_nameList);
							int count = 0;
							for (String child_name : child_nameList) {
								String ori_tableName = ZillionConstant.prefix_data + tableName + "_" + child_name;
								count += mysql.ori_table_count(namespace, ori_tableName);
							}
							result.remove("Result");
							result.put("Result", "success");
							result.put("Count", (long) count);
						} else {
							String ori_tableName = ZillionConstant.prefix_data + tableName;
							int count = mysql.ori_table_count(namespace, ori_tableName);
							result.remove("Result");
							result.put("Result", "success");
							result.put("Count", (long) count);
						}
					}
				} else {
					Stat stat = new Stat();
					if (is_split_table) {
						String split_hash_exist = zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName + "/" + query_month;
						stat = this.container.ZK.exists(split_hash_exist, false);
						if (stat != null) {
							stat = new Stat();
							byte[] hash = this.container.ZK.getData(split_hash_exist, false, stat);
							HashObject hashObject = HashUtil.Parse(hash);
							long count = hashObject.insert_count - hashObject.delete_count;
							result.remove("Result");
							result.put("Result", "success");
							result.put("Count", (long) count);
						} else {
							result.remove("Result");
							result.put("Result", "failure");
							result.put("Reason", "split_table not exist");
						}
					} else {
						stat = new Stat();
						byte[] hash = this.container.ZK.getData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName, false, stat);
						HashObject hashObject = HashUtil.Parse(hash);
						long count = hashObject.insert_count - hashObject.delete_count;
						result.remove("Result");
						result.put("Result", "success");
						result.put("Count", (long) count);

						if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
							List<String> child_nameList = container.ZK
									.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name, false);
							Collections.sort(child_nameList);
							JSONArray Content = new JSONArray();
							for (String child_name : child_nameList) {
								stat = new Stat();
								byte[] hashInner = this.container.ZK
										.getData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName + "/" + child_name, false, stat);
								HashObject hashObjectInner = HashUtil.Parse(hashInner);
								long countInner = hashObjectInner.insert_count - hashObjectInner.delete_count;
								JSONObject month = new JSONObject();
								month.put("Child", child_name);
								month.put("Count", countInner);
								Content.add(month);
							}
							result.put("Content", Content);
						}
					}
				}
			} catch (Exception e) {
				throw e;
			}
		} else {
			// ����OrderBy
			if (queryJSON.containsKey("OrderBy")) {
				JSONArray OrderBy = (JSONArray) queryJSON.get("OrderBy");
				if (OrderBy == null || OrderBy.size() == 0) {
					queryJSON.remove("OrderBy");
				}
			}
			// ��ָ����ѯ��������ж��β�ѯ
			if ((QueryType.equals("select") || QueryType.equals("combine_select")) && queryJSON.containsKey("QueryTarget")) {
				JSONObject QueryTarget = (JSONObject) queryJSON.get("QueryTarget");
				JSONObject QueryResultInner = this.Query(QueryTarget);
				JSONArray resultContentInner = (JSONArray) QueryResultInner.get("Content");
				List<JSONObject> objectList = new ArrayList<JSONObject>();
				for (int i = 0; i < resultContentInner.size(); i++) {
					objectList.add((JSONObject) resultContentInner.get(i));
				}
				JSONArray OrderBy = null;
				JSONObject Limit = null;
				if (queryJSON.containsKey("OrderBy")) {
					OrderBy = (JSONArray) queryJSON.get("OrderBy");
				}
				if (queryJSON.containsKey("Limit")) {
					Limit = (JSONObject) queryJSON.get("Limit");
				}
				JSONArray Content = new JSONArray();
				if (QueryType.equals("select")) {
					JSONObject Criteria = (JSONObject) queryJSON.get("Criteria");
					Content = MemoryQueryUtil.Instance().Select(objectList, Criteria, OrderBy);
				} else if (QueryType.equals("combine_select")) {
					JSONArray Criterias = (JSONArray) queryJSON.get("Criterias");
					Content = MemoryQueryUtil.Instance().Select(objectList, Criterias, OrderBy);
				}

				long Final_Limit_Skip = 0;
				long Final_Limit_Count = Integer.MAX_VALUE;
				if (Limit != null) {
					Long Skip_tmp = FastJsonUtil.getLong(Limit, "Skip");
					Long Count_tmp = FastJsonUtil.getLong(Limit, "Count");
					if (Skip_tmp != null) {
						Final_Limit_Skip = Skip_tmp;
					}
					if (Count_tmp != null) {
						Final_Limit_Count = Count_tmp;
					}
				}
				if (Final_Limit_Skip != 0 || Final_Limit_Count != Integer.MAX_VALUE) {
					JSONArray LimitContent = new JSONArray();
					for (int i = (int) Final_Limit_Skip; i < Final_Limit_Skip + Final_Limit_Count && i < Content.size(); i++) {
						LimitContent.add(Content.get(i));
					}
					Content = LimitContent;
				}

				String UniqueReturnColumn = null;
				if (queryJSON.containsKey("UniqueReturnColumn")) {
					UniqueReturnColumn = (String) queryJSON.get("UniqueReturnColumn");
				}
				JSONArray ReturnColumns = (JSONArray) queryJSON.get("ReturnColumns");
				Content = ZillionQueryUtil.cut(Content, UniqueReturnColumn, ReturnColumns);
				result.put("Result", "success");
				result.put("Content", Content);
				result.put("Count", (long) Content.size());
				return result;
			}

			String Database = (String) queryJSON.get("Database");
			String Datatable = (String) queryJSON.get("Datatable");
			ZillionTableSchema schema = this.container.repositoryMap.get(Database).GetTable(Datatable);

			String namespace = Database;
			String tableName = Datatable;
			String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
			String query_month = null;
			if (schema == null) {
				SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
				int index_ = Datatable.lastIndexOf("_");
				if (index_ != -1) {
					String prefix = Datatable.substring(0, index_);
					String suffix = Datatable.substring(index_ + 1);
					try {
						boolean is_split_table = false;
						SplitWrapper sw = new SplitWrapper(tableName, is_split_table, query_month, schema);
						SplitUtil.SplitParse(sdf, this.container.repositoryMap.get(Database), prefix, suffix, sw);
						tableName = sw.tableName;
						is_split_table = sw.is_split_table;
						query_month = sw.query_month;
						schema = sw.schema;
					} catch (ParseException e) {
						throw e;
					}
				}
			}

			Stat stat = new Stat();
			if (query_month != null) {
				String split_hash_exist = zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName + "/" + query_month;
				stat = this.container.ZK.exists(split_hash_exist, false);
				if (stat == null) {
					throw new MyException("ZillionAgent: " + namespace + " " + tableName + " is split_table, and not exist");
				}
				if (QueryType.equals("put") || QueryType.equals("batch_put") || QueryType.equals("insert") || QueryType.equals("batch_insert")
						|| QueryType.equals("get") || QueryType.equals("batch_get") || QueryType.equals("remove")
						|| QueryType.equals("batch_remove")) {
					throw new MyException("ZillionAgent: " + namespace + " " + tableName + " is split_table, so can't put or insert");
				}
			}

			if (this.container.without_lock) {
				if (QueryType.contains("insert") || QueryType.contains("update") || QueryType.contains("delete") || QueryType.contains("import")
						|| QueryType.contains("put") || QueryType.contains("remove")) {
					throw new MyException("ZillionAgent: " + namespace + " " + tableName + " without mode can't " + QueryType);
				}
			}

			boolean use;
			if (this.container.without_lock) {
				use = true;
			} else {
				use = schema.useTable(this.container.ZK, this.full_name);
			}
			if (use) {
				// ����ʼ
				schema.usingAdd();
				try {
					JSONArray content = new JSONArray();
					JSONArray SecondCriteria = (JSONArray) queryJSON.get("SecondCriteria");
					JSONArray ConflictArray = new JSONArray();
					if (QueryType.equals("select_count")) {
						JSONObject Criteria = (JSONObject) queryJSON.get("Criteria");
						String Hint = (String) queryJSON.get("Hint");
						ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "select_count", null, Criteria,
								null, SecondCriteria, null, null, null, null, Hint, job, ConflictArray);
						result.put("Count", query_result.count);
					} else if (QueryType.equals("batch_select_count")) {
						JSONArray Criterias = (JSONArray) queryJSON.get("Criterias");
						String Hint = (String) queryJSON.get("Hint");
						JSONArray batch_results = new JSONArray();
						for (int i = 0; i < Criterias.size(); i++) {
							JSONObject Criteria = (JSONObject) Criterias.get(i);
							ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "select_count", null,
									Criteria, null, SecondCriteria, null, null, null, null, Hint, job, ConflictArray);
							JSONObject batch_result = new JSONObject();
							batch_result.put("Count", query_result.count);
							batch_results.add(batch_result);
						}
						result.put("Count", (long) batch_results.size());
						result.put("Content", batch_results);
					} else if (QueryType.equals("combine_select_count")) {
						JSONArray Criterias = (JSONArray) queryJSON.get("Criterias");
						String Hint = (String) queryJSON.get("Hint");
						ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "select_count", null, null,
								Criterias, SecondCriteria, null, null, null, null, Hint, job, ConflictArray);
						result.put("Count", query_result.count);
					} else if (QueryType.equals("select")) {
						String UniqueReturnColumn = null;
						if (queryJSON.containsKey("UniqueReturnColumn")) {
							UniqueReturnColumn = (String) queryJSON.get("UniqueReturnColumn");
						}
						JSONArray ReturnColumns = (JSONArray) queryJSON.get("ReturnColumns");
						JSONObject Criteria = (JSONObject) queryJSON.get("Criteria");
						String Hint = (String) queryJSON.get("Hint");
						JSONArray OrderBy = null;
						JSONObject Limit = null;
						if (queryJSON.containsKey("OrderBy")) {
							OrderBy = (JSONArray) queryJSON.get("OrderBy");
						}
						if (queryJSON.containsKey("Limit")) {
							Limit = (JSONObject) queryJSON.get("Limit");
						}
						ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "select", UniqueReturnColumn,
								ReturnColumns, Criteria, null, SecondCriteria, OrderBy, Limit, null, null, Hint, job, ConflictArray);
						result.put("Count", query_result.count);
						result.put("Content", query_result.content);
					} else if (QueryType.equals("combine_select") || QueryType.equals("batch_select_merge")) {
						String UniqueReturnColumn = null;
						if (queryJSON.containsKey("UniqueReturnColumn")) {
							UniqueReturnColumn = (String) queryJSON.get("UniqueReturnColumn");
						}
						JSONArray ReturnColumns = (JSONArray) queryJSON.get("ReturnColumns");
						JSONArray Criterias = (JSONArray) queryJSON.get("Criterias");
						String Hint = (String) queryJSON.get("Hint");
						JSONArray OrderBy = null;
						JSONObject Limit = null;
						if (queryJSON.containsKey("OrderBy")) {
							OrderBy = (JSONArray) queryJSON.get("OrderBy");
						}
						if (queryJSON.containsKey("Limit")) {
							Limit = (JSONObject) queryJSON.get("Limit");
						}
						ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "select", UniqueReturnColumn,
								ReturnColumns, null, Criterias, SecondCriteria, OrderBy, Limit, null, null, Hint, job, ConflictArray);
						result.put("Count", query_result.count);
						result.put("Content", query_result.content);
					} else if (QueryType.equals("batch_select")) {
						String UniqueReturnColumn = null;
						if (queryJSON.containsKey("UniqueReturnColumn")) {
							UniqueReturnColumn = (String) queryJSON.get("UniqueReturnColumn");
						}
						JSONArray ReturnColumns = (JSONArray) queryJSON.get("ReturnColumns");
						JSONArray Criterias = (JSONArray) queryJSON.get("Criterias");
						String Hint = (String) queryJSON.get("Hint");
						JSONArray batch_results = new JSONArray();
						for (int i = 0; i < Criterias.size(); i++) {
							JSONObject Criteria = (JSONObject) Criterias.get(i);
							ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "select",
									UniqueReturnColumn, ReturnColumns, Criteria, null, SecondCriteria, null, null, null, null, Hint, job,
									ConflictArray);
							JSONObject batch_result = new JSONObject();
							batch_result.put("Count", query_result.count);
							batch_result.put("Content", query_result.content);
							batch_results.add(batch_result);
						}
						result.put("Count", (long) batch_results.size());
						result.put("Content", batch_results);
					} else if (QueryType.equals("delete")) {
						JSONObject Criteria = (JSONObject) queryJSON.get("Criteria");
						String Hint = (String) queryJSON.get("Hint");
						ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "delete", null, Criteria, null,
								SecondCriteria, null, null, null, null, Hint, job, ConflictArray);
						result.put("Count", query_result.count);
					} else if (QueryType.equals("combine_delete")) {
						JSONArray Criterias = (JSONArray) queryJSON.get("Criterias");
						String Hint = (String) queryJSON.get("Hint");
						ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "delete", null, null, Criterias,
								SecondCriteria, null, null, null, null, Hint, job, ConflictArray);
						result.put("Count", query_result.count);
					} else if (QueryType.equals("batch_delete")) {
						JSONArray Criterias = (JSONArray) queryJSON.get("Criterias");
						String Hint = (String) queryJSON.get("Hint");
						JSONArray batch_results = new JSONArray();
						for (int i = 0; i < Criterias.size(); i++) {
							JSONObject Criteria = (JSONObject) Criterias.get(i);
							ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "delete", null, Criteria,
									null, SecondCriteria, null, null, null, null, Hint, job, ConflictArray);
							JSONObject batch_result = new JSONObject();
							batch_result.put("Count", query_result.count);
							batch_results.add(batch_result);
						}
						result.put("Count", (long) batch_results.size());
						result.put("Content", batch_results);
					} else if (QueryType.equals("update")) {
						JSONObject Criteria = (JSONObject) queryJSON.get("Criteria");
						JSONObject Set = (JSONObject) queryJSON.get("Set");
						String Hint = (String) queryJSON.get("Hint");
						ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "update", null, Criteria, null,
								SecondCriteria, null, null, Set, null, Hint, job, ConflictArray);
						result.put("Count", query_result.count);
						if (query_result.reason != null) {
							result.put("Reason", query_result.reason);
						}
					} else if (QueryType.equals("combine_update")) {
						JSONArray Criterias = (JSONArray) queryJSON.get("Criterias");
						JSONObject Set = (JSONObject) queryJSON.get("Set");
						String Hint = (String) queryJSON.get("Hint");
						ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "update", null, null, Criterias,
								SecondCriteria, null, null, Set, null, Hint, job, ConflictArray);
						result.put("Count", query_result.count);
						if (query_result.reason != null) {
							result.put("Reason", query_result.reason);
						}
					} else if (QueryType.equals("batch_update")) {
						JSONArray Criterias = (JSONArray) queryJSON.get("Criterias");
						JSONObject Set = null;
						JSONArray Sets = null;
						if (queryJSON.containsKey("Set")) {
							Set = (JSONObject) queryJSON.get("Set");
						}
						if (queryJSON.containsKey("Sets")) {
							Sets = (JSONArray) queryJSON.get("Sets");
						}
						String Hint = (String) queryJSON.get("Hint");
						JSONArray batch_results = new JSONArray();
						for (int i = 0; i < Criterias.size(); i++) {
							JSONObject Criteria = (JSONObject) Criterias.get(i);
							JSONObject SetInner;
							if (Set != null) {
								SetInner = Set;
							} else {
								SetInner = (JSONObject) Sets.get(i);
							}
							ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "update", null, Criteria,
									null, SecondCriteria, null, null, SetInner, null, Hint, job, ConflictArray);
							JSONObject batch_result = new JSONObject();
							batch_result.put("Count", query_result.count);
							if (query_result.reason != null) {
								batch_result.put("Reason", query_result.reason);
							}
							batch_results.add(batch_result);
						}
						result.put("Count", (long) batch_results.size());
						result.put("Content", batch_results);
					} else if (QueryType.equals("export")) {
						if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
							if (query_month == null) {
								throw new MyException("ZillionAgent: " + namespace + " " + tableName + " export must use child_name");
							}
						}
						JSONObject Criteria = (JSONObject) queryJSON.get("Criteria");
						String ExportZip = (String) queryJSON.get("ExportZip");
						String Hint = (String) queryJSON.get("Hint");
						ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "export", null, Criteria, null,
								SecondCriteria, null, null, null, ExportZip, Hint, job, ConflictArray);
						result.put("Count", query_result.count);
					} else if (QueryType.equals("combine_export")) {
						if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
							if (query_month == null) {
								throw new MyException("ZillionAgent: " + namespace + " " + tableName + " export must use child_name");
							}
						}
						JSONArray Criterias = (JSONArray) queryJSON.get("Criterias");
						String ExportZip = (String) queryJSON.get("ExportZip");
						String Hint = (String) queryJSON.get("Hint");
						ZillionQueryResult query_result = ZillionQueryUtil.query(this.container, schema, query_month, "export", null, null, Criterias,
								SecondCriteria, null, null, null, ExportZip, Hint, job, ConflictArray);
						result.put("Count", query_result.count);
					} else if (QueryType.equals("import")) {
						if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
							if (query_month != null) {
								throw new MyException("ZillionAgent: " + namespace + " " + tableName + " import can't use child_name");
							}
						}
						String ImportZip = (String) queryJSON.get("ImportZip");

						int scanned_count = 0;
						int exist_count = 0;
						ZipArchiveInputStream zis = new ZipArchiveInputStream(new FileInputStream(ImportZip));
						@SuppressWarnings("unused")
						ArchiveEntry entry;
						while ((entry = zis.getNextEntry()) != null) {
							ByteArrayOutputStream out = new ByteArrayOutputStream();
							BufferedOutputStream Bout = new BufferedOutputStream(out);
							int b;
							while ((b = zis.read()) != -1) {
								Bout.write(b);
							}
							Bout.close();
							out.close();
							String JSONString = new String(out.toByteArray(), "UTF-8");
							JSONArray Content = (JSONArray) FastJsonReaderUtil.Instance().parse(JSONString);
							for (int i = 0; i < (Content.size() - 1) / ZillionConstant.import_batch_size + 1; i++) {
								JSONArray InsertObjects = new JSONArray();
								for (int ii = i * ZillionConstant.import_batch_size; ii < (i + 1) * ZillionConstant.import_batch_size
										&& ii < Content.size(); ii++) {
									InsertObjects.add(Content.get(ii));
								}
								JSONObject wrapperObject = new JSONObject();
								wrapperObject.put("QueryType", "batch_insert");
								wrapperObject.put("InsertObjects", InsertObjects);
								wrapperObject.put("Database", Database);
								wrapperObject.put("Datatable", Datatable);
								JSONObject resultJSON = this.Query(wrapperObject);
								exist_count += (Long) resultJSON.get("Count");
								scanned_count += InsertObjects.size();
							}
						}
						zis.close();

						JSONObject Content = new JSONObject();
						Content.put("Count_total", scanned_count);
						Content.put("Count_insert", scanned_count - exist_count);
						Content.put("Count_exist", exist_count);
						result.put("Content", Content);
						result.put("Count", scanned_count);
					} else if (QueryType.equals("put")) {
						JSONObject InsertObject = (JSONObject) queryJSON.get("InsertObject");
						JSONArray InsertObjects = new JSONArray();
						InsertObjects.add(InsertObject);
						ZillionQueryUtil.batch_put_insert(this.container, schema, InsertObjects, true, ConflictArray);
					} else if (QueryType.equals("batch_put")) {
						JSONArray InsertObjects = (JSONArray) queryJSON.get("InsertObjects");
						ZillionQueryUtil.batch_put_insert(this.container, schema, InsertObjects, true, ConflictArray);
					} else if (QueryType.equals("insert")) {
						JSONObject InsertObject = (JSONObject) queryJSON.get("InsertObject");
						JSONObject InsertObject_real = ZillionQueryUtil.getData(this.container, schema, InsertObject, content, null);
						if (InsertObject_real != null) {
							JSONArray InsertObjects_real = new JSONArray();
							InsertObjects_real.add(InsertObject_real);
							ZillionQueryUtil.batch_put_insert(this.container, schema, InsertObjects_real, false, ConflictArray);
						}
					} else if (QueryType.equals("batch_insert")) {
						JSONArray InsertObjects = (JSONArray) queryJSON.get("InsertObjects");
						if (ZillionConstant.check_query) {
							ZillionQueryCheckUtil.Check_batch(schema, InsertObjects);
						}
						JSONArray InsertObjects_real = ZillionQueryUtil.getDatas(this.container, schema, InsertObjects, content, null);
						if (InsertObjects_real.size() > 0) {
							ZillionQueryUtil.batch_put_insert(this.container, schema, InsertObjects_real, false, ConflictArray);
						}
					} else if (QueryType.equals("get")) {
						JSONArray ReturnColumns = (JSONArray) queryJSON.get("ReturnColumns");
						JSONObject InsertObject = (JSONObject) queryJSON.get("Key");
						JSONArray InsertObjects = new JSONArray();
						InsertObjects.add(InsertObject);
						{
							ZillionQueryUtil.getDatas(this.container, schema, InsertObjects, content, ReturnColumns);
						}
					} else if (QueryType.equals("batch_get")) {
						JSONArray ReturnColumns = (JSONArray) queryJSON.get("ReturnColumns");
						JSONArray InsertObjects = (JSONArray) queryJSON.get("Keys");
						if (ZillionConstant.check_query) {
							ZillionQueryCheckUtil.Check_batch(schema, InsertObjects);
						}
						{
							ZillionQueryUtil.getDatas(this.container, schema, InsertObjects, content, ReturnColumns);
						}
					} else if (QueryType.equals("remove")) {
						JSONObject InsertObject = (JSONObject) queryJSON.get("Key");
						ZillionQueryUtil.getData(this.container, schema, InsertObject, content, null);
						ZillionQueryUtil.batch_remove(this.container, schema, content, ConflictArray);
					} else if (QueryType.equals("batch_remove")) {
						JSONArray InsertObjects = (JSONArray) queryJSON.get("Keys");
						if (ZillionConstant.check_query) {
							ZillionQueryCheckUtil.Check_batch(schema, InsertObjects);
						}
						ZillionQueryUtil.getDatas(this.container, schema, InsertObjects, content, null);
						ZillionQueryUtil.batch_remove(this.container, schema, content, ConflictArray);
					}

					if (ConflictArray.size() > 0) {
						result.put("ConflictArray", ConflictArray);
					}
					result.remove("Result");
					result.put("Result", "success");
					if (QueryType.equals("insert") || QueryType.equals("batch_insert") || QueryType.equals("get") || QueryType.equals("batch_get")
							|| QueryType.equals("remove") || QueryType.equals("batch_remove")) {
						result.put("Count", (long) content.size());
						result.put("Content", content);

						if (QueryType.equals("insert") || QueryType.equals("batch_insert")) {
							if (content.size() > 0) {
								result.remove("Result");
								result.put("Result", "failure");
							}
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
				} finally {
					schema.usingDel();

					if (unuse) {
						if (this.container.without_lock) {
						} else {
							schema.unuseTable(this.container.ZK);
						}
					}
				}
				// �������
				return result;
			} else {
				throw new MyException("ZillionAgent: " + namespace + " " + tableName + " useTable fail");
			}
		}

		return result;
	}

	public synchronized boolean NDL_AddDatabase(String namespace) throws Exception {
		String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();

		boolean result = false;
		if (!this.container.initialized) {
			throw new MyException("ZillionAgent: " + "container not initialized");
		}
		if (ZillionConstant.ReservedNamespaceMap.containsKey(namespace)) {
			throw new MyException("ZillionAgent: " + "database can't be " + namespace);
		}
		boolean exist = false;
		for (String namespaceInner : this.container.namespaceList) {
			if (namespaceInner.equals(namespace)) {
				exist = true;
			}
		}
		if (exist) {
			throw new MyException("ZillionAgent: " + "database already exist " + namespace);
		}
		ZKWrapper zookeeper = this.container.ZK;
		String parent_path_ns = ZillionConstant.zk_root + ZillionConstant._nslock;
		String lock_type_ns = "w";
		// ��ȡmeta��
		String node_path_me = ZKLockRW.Create(zookeeper, parent_path_ns, lock_type_ns, this.full_name);
		try {
			LogUtil.info(" ********************************** " + method_name + " nslock:" + " WaitFor " + node_path_me);
			boolean wait_success = ZKLockRW.WaitFor(zookeeper, parent_path_ns, node_path_me, ZillionConstant.WaitFor_seconds);
			LogUtil.info(" ********************************** " + method_name + " nslock:" + " WaitFor " + wait_success + " " + node_path_me);
			if (!wait_success) {
				throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_me);
			}

			boolean exist_namespace = false;
			List<String> namespaceList = this.container.DatabaseImpl.ori_namespaceList();
			for (String namespace_name : namespaceList) {
				if (namespace_name.equals(namespace)) {
					List<String> tableList = this.container.DatabaseImpl.ori_tableList(namespace);
					if (tableList.size() > 0) {
						throw new MyException("ZillionAgent: " + "namespace already exist:" + namespace);
					}
					exist_namespace = true;
					break;
				}
			}
			if (!exist_namespace) {
				this.container.DatabaseImpl.namespace_create(namespace);
			}
			this.container.DatabaseImpl.meta_manage_create_data(namespace);
			this.container.DatabaseImpl.meta_manage_create_index(namespace);
			this.helper.InitializeNamespace(this.container.ZK, this.container.DatabaseImpl, namespace);

			result = true;
		} catch (Exception e) {
			throw e;
		} finally {
			// �ͷ�meta��
			ZKLockRW.Finish(zookeeper, node_path_me);
			LogUtil.info(" ********************************** " + method_name + " nslock:" + " finish " + node_path_me);
		}

		Thread.sleep(1000L * 3);

		return result;
	}

	public synchronized boolean NDL_DropDatabase(String namespace) throws Exception {
		String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();

		boolean result = false;
		if (!this.container.initialized) {
			throw new MyException("ZillionAgent: " + "container not initialized");
		}
		if (ZillionConstant.ReservedNamespaceMap.containsKey(namespace)) {
			throw new MyException("ZillionAgent: " + "database can't be " + namespace);
		}
		boolean exist = false;
		for (String namespaceInner : this.container.namespaceList) {
			if (namespaceInner.equals(namespace)) {
				exist = true;
			}
		}
		if (!exist) {
			throw new MyException("ZillionAgent: " + "database not exist " + namespace);
		}
		SchemaRepository repository = this.container.repositoryMap.get(namespace);
		if (repository.TableList().size() > 0) {
			throw new MyException("ZillionAgent: " + "database not empty " + namespace);
		}
		ZKWrapper zookeeper = this.container.ZK;
		String parent_path_ns = ZillionConstant.zk_root + ZillionConstant._nslock;
		String lock_type_ns = "w";
		// ��ȡmeta��
		String node_path_me = ZKLockRW.Create(zookeeper, parent_path_ns, lock_type_ns, this.full_name);
		try {
			LogUtil.info(" ********************************** " + method_name + " nslock:" + " WaitFor " + node_path_me);
			boolean wait_success = ZKLockRW.WaitFor(zookeeper, parent_path_ns, node_path_me, ZillionConstant.WaitFor_seconds);
			LogUtil.info(" ********************************** " + method_name + " nslock:" + " WaitFor " + wait_success + " " + node_path_me);
			if (!wait_success) {
				throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_me);
			}

			String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
			String parent_path_meta = zkpath_namespace + ZillionConstant.zk_metalock;
			String lock_type_meta = "w";
			// ��ȡmeta��
			String node_path_meta = ZKLockRW.Create(zookeeper, parent_path_meta, lock_type_meta, this.full_name);
			try {
				LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + node_path_meta);
				wait_success = ZKLockRW.WaitFor(zookeeper, parent_path_meta, node_path_meta, ZillionConstant.WaitFor_seconds);
				LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + wait_success + " " + node_path_meta);
				if (!wait_success) {
					throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_meta);
				}

				if (repository.TableList().size() == 0) {
					ZKHelper.recursive(zookeeper, zkpath_namespace);

					if (this.container.DatabaseImpl.ori_tableExists(namespace, "zillion_meta_split")) {
						this.container.DatabaseImpl.meta_manage_drop_split(namespace);
					}
					this.container.DatabaseImpl.meta_manage_drop_index(namespace);
					this.container.DatabaseImpl.meta_manage_drop_data(namespace);
					try {
						this.container.DatabaseImpl.namespace_drop(namespace);
					} catch (Exception e) {
						// e.printStackTrace();
					}

					result = true;
				}
			} catch (Exception e) {
				throw e;
			} finally {
				// �ͷ�meta��
				ZKLockRW.Finish(zookeeper, node_path_meta);
				LogUtil.info(" ********************************** " + method_name + " metalock:" + " finish " + node_path_meta);
			}
		} catch (Exception e) {
			throw e;
		} finally {
			// �ͷ�meta��
			ZKLockRW.Finish(zookeeper, node_path_me);
			LogUtil.info(" ********************************** " + method_name + " nslock:" + " finish " + node_path_me);
		}

		Thread.sleep(1000L * 3);

		return result;
	}

	public synchronized boolean DDL_AddTable(String namespace, String tableName, com.sagacloud.json.JSONObject content_Saga,
			com.sagacloud.json.JSONArray indexes_Saga) throws Exception {
		JSONObject content = (JSONObject) Convertor_JSON.SagaJson2FastJson(content_Saga);
		JSONArray indexes = (JSONArray) Convertor_JSON.SagaJson2FastJson(indexes_Saga);
		return this.DDL_AddTable(namespace, tableName, content, indexes);
	}

	public synchronized boolean DDL_AddTable(String namespace, String tableName, JSONObject content, JSONArray indexes) throws Exception {
		String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();

		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		boolean result = false;
		int repo_version_new = -1;
		if (!this.container.initialized) {
			throw new MyException("ZillionAgent: " + "container not initialized");
		}
		SchemaRepository repository = this.container.repositoryMap.get(namespace);
		ZillionTableSchema schema_exist = repository.GetTable(tableName);
		if (schema_exist != null) {
			throw new MyException("ZillionAgent: " + "schema exist: " + namespace + " " + tableName);
		}
		if (content.containsKey("resident_memory")) {
			boolean resident_memory = (Boolean) content.get("resident_memory");
			if (resident_memory) {
				if (content.containsKey("table_type")) {
					String table_type = (String) content.get("table_type");
					if (table_type.startsWith("split_")) {
						throw new MyException("ZillionAgent: " + "split table can't resident_memory: " + namespace + " " + tableName);
					}
				}
			}
		}
		// ���
		{
			ZillionTableSchema schema = new ZillionTableSchema(namespace, tableName);
			schema.JSON = content;
			SchemaParser.ParseTableSchema(schema, content);
			for (String Column : schema.Key) {
				if (!schema.ColumnMap.containsKey(Column)) {
					throw new MyException("ZillionAgent: " + namespace + " " + tableName + " Column " + Column + " not exist");
				}
				ZillionTableColumn columnDef = schema.ColumnMap.get(Column);
				if (columnDef.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
					throw new MyException("ZillionAgent: " + namespace + " " + tableName + " Column " + Column + " can't use as key or index");
				}
			}
			for (ZillionTableIndex indexDef : schema.Indexes) {
				for (String Column : indexDef.Key) {
					if (!schema.ColumnMap.containsKey(Column)) {
						throw new MyException("ZillionAgent: " + namespace + " " + tableName + " Column " + Column + " not exist");
					}
					ZillionTableColumn columnDef = schema.ColumnMap.get(Column);
					if (columnDef.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
						throw new MyException("ZillionAgent: " + namespace + " " + tableName + " Column " + Column + " can't use as key or index");
					}
				}
			}
		}

		ZKWrapper zookeeper = this.container.ZK;
		String parent_path_meta = zkpath_namespace + ZillionConstant.zk_metalock;
		String lock_type_meta = "w";
		String parent_path_table = zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + tableName;
		String lock_type_table = "c";
		String check_type_table = "c";
		// ��ȡmeta��
		String node_path_meta = ZKLockRW.Create(zookeeper, parent_path_meta, lock_type_meta, this.full_name);
		try {
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + node_path_meta);
			boolean wait_success = ZKLockRW.WaitFor(zookeeper, parent_path_meta, node_path_meta, ZillionConstant.WaitFor_seconds);
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + wait_success + " " + node_path_meta);
			if (!wait_success) {
				throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_meta);
			}

			boolean success = ZKLockCEDU.PreCheck(zookeeper, parent_path_table, check_type_table);
			if (success) {
				// ��ȡtable��
				String node_path_table = ZKLockCEDU.Create(zookeeper, parent_path_table, lock_type_table, this.full_name);
				try {
					int sequence_id = Integer.parseInt(node_path_table.substring(parent_path_table.length() + 3));
					success = ZKLockCEDU.Check(zookeeper, parent_path_table, sequence_id, check_type_table);
					if (success) {
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + node_path_table);
						wait_success = ZKLockCEDU.WaitFor(zookeeper, parent_path_table, node_path_table, ZillionConstant.WaitFor_seconds);
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + wait_success + " "
								+ node_path_table);
						if (!wait_success) {
							throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_table);
						}

						// ����ʼ
						String table_type;
						if (!content.containsKey("table_type")) {
							table_type = "index_data";
							content.put("table_type", table_type);
						} else {
							String table_typeInner = (String) content.get("table_type");
							if (table_typeInner == null) {
							} else if (table_typeInner.equals("index_data")) {
							} else if (table_typeInner.equals("all_column")) {
								if (!content.containsKey("delete_UsePutV")) {
									content.put("delete_UsePutV", false);
								}
							} else if (table_typeInner.startsWith("split_")) {
								if (!content.containsKey("delete_UsePutV")) {
									content.put("delete_UsePutV", false);
								}
								if (!content.containsKey("split_column")) {
									content.put("split_column", "data_time");
								}
							}
							table_type = table_typeInner;
						}
						zookeeper.create(zkpath_namespace + ZillionConstant.zk_meta_data + "/" + tableName,
								ByteToObject.toBytes(JSON.toJSONString(content, SerializerFeature.WriteMapNullValue)), CreateMode.PERSISTENT);
						HashObject hashObject = new HashObject();
						byte[] hash = hashObject.toBytes();
						zookeeper.create(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName, hash, CreateMode.PERSISTENT);
						zookeeper.create(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + tableName, "true".getBytes("UTF-8"),
								CreateMode.PERSISTENT);
						zookeeper.create(zkpath_namespace + ZillionConstant.zk_hashlocks + "/" + tableName, "true".getBytes("UTF-8"),
								CreateMode.PERSISTENT);
						this.container.DatabaseImpl.meta_query_insert_data(namespace, tableName, content, hashObject);

						ZillionTableSchema schema = new ZillionTableSchema(namespace, tableName);
						schema.JSON = content;
						SchemaParser.ParseTableSchema(schema, content);
						if (schema.strong_consistency) {
							this.container.DatabaseImpl.meta_manage_make_sure_cache(namespace);
							zookeeper.create(zkpath_namespace + ZillionConstant.zk_cachelocks + "/" + tableName, "true".getBytes("UTF-8"),
									CreateMode.PERSISTENT);
						}
						if (table_type != null && (table_type.startsWith("split_"))) {
							this.container.DatabaseImpl.meta_manage_make_sure_split(namespace);
						} else {
							byte[][] splitKeys = null;
							if (content.containsKey("region_keys")) {
								JSONArray region_keys = (JSONArray) content.get("region_keys");
								splitKeys = new byte[region_keys.size()][];
								for (int index_key = 0; index_key < region_keys.size(); index_key++) {
									String region_key = (String) region_keys.get(index_key);
									splitKeys[index_key] = HexToByte.hexToByte(region_key);
								}
							}
							this.container.DatabaseImpl.table_manage_create_data(namespace, schema);
						}
						if (indexes != null) {
							for (int i = 0; i < indexes.size(); i++) {
								JSONObject indexJSON = (JSONObject) indexes.get(i);
								String indexName = (String) indexJSON.get("Name");
								JSONArray indexColumns = (JSONArray) indexJSON.get("Columns");

								zookeeper.create(zkpath_namespace + ZillionConstant.zk_meta_index + "/" + (tableName + "." + indexName),
										JSON.toJSONString(indexColumns, SerializerFeature.WriteMapNullValue).getBytes("UTF-8"),
										CreateMode.PERSISTENT);
								zookeeper.create(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName), hash,
										CreateMode.PERSISTENT);
								zookeeper.create(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (tableName + "." + indexName),
										"true".getBytes("UTF-8"), CreateMode.PERSISTENT);
								this.container.DatabaseImpl.meta_query_insert_index(namespace, tableName, indexName, indexColumns, hashObject);
								ZillionTableIndex schema_index = SchemaParser.ParseIndexSchema(schema, indexJSON);
								schema.AddIndex(schema_index);

								if (table_type != null && (table_type.startsWith("split_"))) {
								} else {
									this.container.DatabaseImpl.table_manage_create_index(namespace, tableName, schema_index);
								}
							}
						}

						if (schema.elastic_search) {
							ElasticSearchUtil.create(schema, this.container.esClient, namespace, tableName);
						}

						repo_version_new = ZKRepositoryUtil.ResetRepoVersion(zookeeper, namespace, parent_path_table, lock_type_table);
						LogUtil.info(" ********************************** " + method_name + " work success ");
						// �������

						result = true;
					} else {
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " Check fail " + node_path_table);
					}
				} catch (Exception e) {
					throw e;
				} finally {
					// �ͷ�table��
					ZKLockCEDU.Finish(zookeeper, node_path_table);
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " finish " + node_path_table);
				}
			} else {
				LogUtil.info(" ********************************** " + method_name + " tablelock:" + " PreCheck fail ");
			}
		} catch (Exception e) {
			throw e;
		} finally {
			// �ͷ�meta��
			ZKLockRW.Finish(zookeeper, node_path_meta);
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " finish " + node_path_meta);
		}

		if (result) {
			ObserverCheck.Wait(zookeeper, namespace, repo_version_new);
		}
		return result;
	}

	public synchronized boolean DDL_AlterTable(String namespace, String tableName, String alter_type,
			com.sagacloud.json.JSONObject alter_content_Saga) throws Exception {
		JSONObject alter_content = (JSONObject) Convertor_JSON.SagaJson2FastJson(alter_content_Saga);
		return this.DDL_AlterTable(namespace, tableName, alter_type, alter_content);
	}

	public synchronized boolean DDL_AlterTable(String namespace, String tableName, String alter_type, JSONObject alter_content) throws Exception {
		String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();

		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		boolean result = false;
		int repo_version_new = -1;
		if (!this.container.initialized) {
			throw new MyException("ZillionAgent: " + "container not initialized");
		}
		SchemaRepository repository = this.container.repositoryMap.get(namespace);
		ZillionTableSchema schema_exist = repository.GetTable(tableName);
		if (schema_exist == null) {
			throw new MyException("ZillionAgent: " + "schema not exist: " + namespace + " " + tableName);
		}
		if (alter_type.equals("region_keys")) {
			if (!(schema_exist.table_type != null && (schema_exist.table_type.startsWith("split_")))) {
				throw new MyException("ZillionAgent: " + namespace + " " + tableName + " only child_name can update region_keys");
			}
		} else if (alter_type.equals("delete_UsePutV")) {
			boolean delete_UsePutV = (Boolean) alter_content.get("delete_UsePutV");
			if (delete_UsePutV == schema_exist.delete_UsePutV) {
				return true;
			}
		} else if (alter_type.equals("compress")) {
			boolean compression = (Boolean) alter_content.get("compress");
			if (compression == schema_exist.compress) {
				return true;
			}
		} else if (alter_type.equals("resident_memory")) {
			if (schema_exist.table_type != null && (schema_exist.table_type.startsWith("split_"))) {
				throw new MyException("ZillionAgent: " + "split table can't resident_memory: " + namespace + " " + tableName);
			}
			boolean resident_memory = (Boolean) alter_content.get("resident_memory");
			if (resident_memory == schema_exist.resident_memory) {
				return true;
			}
		} else if (alter_type.equals("Columns_add")) {
			List<String> bad_Columns = new ArrayList<String>();
			JSONArray Columns = (JSONArray) alter_content.get("Columns_add");
			for (int i = 0; i < Columns.size(); i++) {
				JSONObject Column = (JSONObject) Columns.get(i);
				String Name = (String) Column.get("Name");
				if (schema_exist.ColumnMap.containsKey(Name)) {
					bad_Columns.add(Name);
				}
			}
			if (bad_Columns.size() > 0) {
				StringBuffer sb = new StringBuffer();
				for (String Column : bad_Columns) {
					if (sb.length() > 0) {
						sb.append(",");
					}
					sb.append(Column);
				}
				throw new MyException("ZillionAgent: " + namespace + " " + tableName + " Columns exist:" + sb.toString());
			}
			for (int index_col = 0; index_col < Columns.size(); index_col++) {
				JSONObject column = (JSONObject) Columns.get(index_col);
				ZillionTableColumn ztc = SchemaParser.ParseColumn(column);
				if (ztc.NotNull) {
					throw new MyException("ZillionAgent: " + namespace + " " + tableName + " Columns can't be NotNull:" + ztc.Name);
				}
			}
		} else if (alter_type.equals("Columns_drop")) {
			List<String> bad_Columns = new ArrayList<String>();
			JSONArray Columns = (JSONArray) alter_content.get("Columns_drop");
			for (int i = 0; i < Columns.size(); i++) {
				String Name = (String) Columns.get(i);
				if (!schema_exist.ColumnMap.containsKey(Name)) {
					bad_Columns.add(Name);
				} else {
					boolean bad = false;
					for (String key : schema_exist.Key) {
						if (key.equals(Name)) {
							bad = true;
							break;
						}
					}
					for (ZillionTableIndex ZillionTableIndex : schema_exist.Indexes) {
						for (String key : ZillionTableIndex.Key) {
							if (key.equals(Name)) {
								bad = true;
								break;
							}
						}
					}
					if (schema_exist.table_type != null && (schema_exist.table_type.startsWith("split_"))) {
						if (schema_exist.split_column.equals(Name)) {
							bad = true;
						}
					}

					if (bad) {
						bad_Columns.add(Name);
					}
				}
			}
			if (bad_Columns.size() > 0) {
				StringBuffer sb = new StringBuffer();
				for (String Column : bad_Columns) {
					if (sb.length() > 0) {
						sb.append(",");
					}
					sb.append(Column);
				}
				throw new MyException("ZillionAgent: " + namespace + " " + tableName + " Columns not exist or can't drop:" + sb.toString());
			}
		}
		ZKWrapper zookeeper = this.container.ZK;
		String parent_path_meta = zkpath_namespace + ZillionConstant.zk_metalock;
		String lock_type_meta = "w";
		String parent_path_table = zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + tableName;
		String lock_type_table = "e";
		String check_type_table = "d";

		// �����޸ı�ṹ
		if (this.container.without_lock) {
			this.DDL_AlterTable_inner(schema_exist, zookeeper, namespace, tableName, alter_type, alter_content);
			LogUtil.info(" ********************************** " + method_name + " work success ");

			result = true;

			if (result) {
				ObserverCheck.Wait(zookeeper, namespace, repo_version_new);
			}

			return result;
		}

		// ��ȡmeta��
		String node_path_meta = ZKLockRW.Create(zookeeper, parent_path_meta, lock_type_meta, this.full_name);
		try {
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + node_path_meta);
			boolean wait_success = ZKLockRW.WaitFor(zookeeper, parent_path_meta, node_path_meta, ZillionConstant.WaitFor_seconds);
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + wait_success + " " + node_path_meta);
			if (!wait_success) {
				throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_meta);
			}

			boolean success = ZKLockCEDU.PreCheck(zookeeper, parent_path_table, check_type_table);
			if (success) {
				// ��ȡtable��
				String node_path_table = ZKLockCEDU.Create(zookeeper, parent_path_table, lock_type_table, this.full_name);
				try {
					int sequence_id = Integer.parseInt(node_path_table.substring(parent_path_table.length() + 3));
					success = ZKLockCEDU.Check(zookeeper, parent_path_table, sequence_id, check_type_table);
					if (success) {
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + node_path_table);
						wait_success = ZKLockCEDU.WaitFor(zookeeper, parent_path_table, node_path_table, ZillionConstant.WaitFor_seconds);
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + wait_success + " "
								+ node_path_table);
						if (!wait_success) {
							throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_table);
						}

						this.DDL_AlterTable_inner(schema_exist, zookeeper, namespace, tableName, alter_type, alter_content);
						LogUtil.info(" ********************************** " + method_name + " work success ");

						result = true;
					} else {
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " Check fail " + node_path_table);
					}
				} catch (Exception e) {
					throw e;
				} finally {
					// �ͷ�table��
					ZKLockCEDU.Finish(zookeeper, node_path_table);
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " finish " + node_path_table);
				}
			} else {
				LogUtil.info(" ********************************** " + method_name + " tablelock:" + " PreCheck fail ");
			}
		} catch (Exception e) {
			throw e;
		} finally {
			// �ͷ�meta��
			ZKLockRW.Finish(zookeeper, node_path_meta);
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " finish " + node_path_meta);
		}

		if (result) {
			ObserverCheck.Wait(zookeeper, namespace, repo_version_new);
		}
		return result;
	}

	private int DDL_AlterTable_inner(ZillionTableSchema schema_exist, ZKWrapper zookeeper, String namespace, String tableName, String alter_type,
			JSONObject alter_content) throws Exception {
		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		String parent_path_table = zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + tableName;
		String lock_type_table = "e";

		// ����ʼ
		Stat stat = new Stat();
		byte[] data_bytes = zookeeper.getData(zkpath_namespace + ZillionConstant.zk_meta_data + "/" + tableName, false, stat);
		JSONObject content = (JSONObject) FastJsonReaderUtil.Instance().parse(new String(data_bytes, "UTF-8"));

		if (alter_type.equals("region_keys")) {
			if (content.containsKey("region_keys")) {
				content.remove("region_keys");
			}
			JSONArray region_keys = (JSONArray) alter_content.get("region_keys");
			if (region_keys != null) {
				content.put("region_keys", region_keys);
			}
		} else if (alter_type.equals("delete_UsePutV")) {
			boolean delete_UsePutV = (Boolean) alter_content.get("delete_UsePutV");
			if (content.containsKey("delete_UsePutV")) {
				content.remove("delete_UsePutV");
			}
			content.put("delete_UsePutV", delete_UsePutV);
		} else if (alter_type.equals("compress")) {
			boolean compress = (Boolean) alter_content.get("compress");
			if (content.containsKey("compress")) {
				content.remove("compress");
			}
			content.put("compress", compress);
			// ����
			for (ZillionTableIndex schema_index : schema_exist.Indexes) {
				String indexName = schema_index.index_name;
				if (schema_exist.table_type != null && (schema_exist.table_type.startsWith("split_"))) {
					List<String> child_nameList = container.ZK
							.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName), false);
					Collections.sort(child_nameList);
					for (String child_name : child_nameList) {
						String tableNameString = ZillionConstant.prefix_index + (tableName + "." + indexName) + "_" + child_name;
						this.container.DatabaseImpl.table_manage_alter_compression(namespace, tableNameString, compress);
					}
				} else {
					String tableNameString = ZillionConstant.prefix_index + (tableName + "." + indexName);
					this.container.DatabaseImpl.table_manage_alter_compression(namespace, tableNameString, compress);
				}
			}

			if (schema_exist.table_type != null && (schema_exist.table_type.startsWith("split_"))) {
				List<String> child_nameList = container.ZK
						.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema_exist.table_name, false);
				Collections.sort(child_nameList);
				for (String child_name : child_nameList) {
					String tableNameString = ZillionConstant.prefix_data + tableName + "_" + child_name;
					this.container.DatabaseImpl.table_manage_alter_compression(namespace, tableNameString, compress);
				}
			} else {
				String tableNameString = ZillionConstant.prefix_data + tableName;
				this.container.DatabaseImpl.table_manage_alter_compression(namespace, tableNameString, compress);
			}
		} else if (alter_type.equals("resident_memory")) {
			boolean resident_memory = (Boolean) alter_content.get("resident_memory");
			content.put("resident_memory", resident_memory);

			// ����
			for (ZillionTableIndex schema_index : schema_exist.Indexes) {
				String indexName = schema_index.index_name;
				String tableNameString = ZillionConstant.prefix_index + (tableName + "." + indexName);
				this.container.DatabaseImpl.table_manage_alter_in_memory(namespace, tableNameString, resident_memory);
			}
			String tableNameString = ZillionConstant.prefix_data + tableName;
			this.container.DatabaseImpl.table_manage_alter_in_memory(namespace, tableNameString, resident_memory);
		} else if (alter_type.equals("Columns_add")) {
			JSONArray Columns = (JSONArray) alter_content.get("Columns_add");
			JSONArray Columns_current = (JSONArray) content.get("Columns");
			Columns_current.addAll(Columns);
			List<ZillionTableColumn> ztcList = new ArrayList<ZillionTableColumn>();
			for (int index_col = 0; index_col < Columns.size(); index_col++) {
				JSONObject column = (JSONObject) Columns.get(index_col);
				ZillionTableColumn ztc = SchemaParser.ParseColumn(column);
				ztcList.add(ztc);
			}
			if (schema_exist.table_type != null && (schema_exist.table_type.startsWith("split_"))) {
				List<String> child_nameList = container.ZK
						.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema_exist.table_name, false);
				Collections.sort(child_nameList);
				for (String child_name : child_nameList) {
					String tableNameString = ZillionConstant.prefix_data + tableName + "_" + child_name;
					this.container.DatabaseImpl.table_manage_alter_add_column(namespace, tableNameString, ztcList);
				}
			} else {
				String tableNameString = ZillionConstant.prefix_data + tableName;
				this.container.DatabaseImpl.table_manage_alter_add_column(namespace, tableNameString, ztcList);
			}
		} else if (alter_type.equals("Columns_drop")) {
			JSONArray Columns = (JSONArray) alter_content.get("Columns_drop");
			JSONArray Columns_current = (JSONArray) content.get("Columns");
			List<String> columnList = new ArrayList<String>();
			for (int i = 0; i < Columns.size(); i++) {
				String Name = (String) Columns.get(i);
				columnList.add(Name);
				int index = -1;
				for (int ii = 0; ii < Columns_current.size(); ii++) {
					JSONObject ColumnJSON = (JSONObject) Columns_current.get(ii);
					String Name_current = (String) ColumnJSON.get("Name");
					if (Name_current.equals(Name)) {
						index = ii;
						break;
					}
				}
				if (index != -1) {
					Columns_current.remove(index);
				}
			}
			if (schema_exist.table_type != null && (schema_exist.table_type.startsWith("split_"))) {
				List<String> child_nameList = container.ZK
						.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema_exist.table_name, false);
				Collections.sort(child_nameList);
				for (String child_name : child_nameList) {
					String tableNameString = ZillionConstant.prefix_data + tableName + "_" + child_name;
					this.container.DatabaseImpl.table_manage_alter_drop_column(namespace, tableNameString, columnList);
				}
			} else {
				String tableNameString = ZillionConstant.prefix_data + tableName;
				this.container.DatabaseImpl.table_manage_alter_drop_column(namespace, tableNameString, columnList);
			}
		}
		zookeeper.setData(zkpath_namespace + ZillionConstant.zk_meta_data + "/" + tableName,
				ByteToObject.toBytes(JSON.toJSONString(content, SerializerFeature.WriteMapNullValue)), -1);
		this.container.DatabaseImpl.meta_query_update_data(namespace, tableName, content);

		int repo_version_new = ZKRepositoryUtil.ResetRepoVersion(zookeeper, namespace, parent_path_table, lock_type_table);
		// �������

		return repo_version_new;
	}

	public synchronized boolean DDL_DropTable(String namespace, String tableName) throws Exception {
		String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();

		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		boolean result = false;
		int repo_version_new = -1;
		if (!this.container.initialized) {
			throw new MyException("ZillionAgent: " + "container not initialized");
		}
		SchemaRepository repository = this.container.repositoryMap.get(namespace);
		ZillionTableSchema schema_exist = repository.GetTable(tableName);
		if (schema_exist == null) {
			throw new MyException("ZillionAgent: " + "schema not exist: " + namespace + " " + tableName);
		}
		ZKWrapper zookeeper = this.container.ZK;
		String parent_path_meta = zkpath_namespace + ZillionConstant.zk_metalock;
		String lock_type_meta = "w";
		String parent_path_table = zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + tableName;
		String lock_type_table = "d";
		String check_type_table = "d";
		// ��ȡmeta��
		String node_path_meta = ZKLockRW.Create(zookeeper, parent_path_meta, lock_type_meta, this.full_name);
		try {
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + node_path_meta);
			boolean wait_success = ZKLockRW.WaitFor(zookeeper, parent_path_meta, node_path_meta, ZillionConstant.WaitFor_seconds);
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + wait_success + " " + node_path_meta);
			if (!wait_success) {
				throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_meta);
			}

			boolean success = ZKLockCEDU.PreCheck(zookeeper, parent_path_table, check_type_table);
			if (success) {
				// ��ȡtable��
				String node_path_table = ZKLockCEDU.Create(zookeeper, parent_path_table, lock_type_table, this.full_name);
				try {
					int sequence_id = Integer.parseInt(node_path_table.substring(parent_path_table.length() + 3));
					success = ZKLockCEDU.Check(zookeeper, parent_path_table, sequence_id, check_type_table);
					if (success) {
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + node_path_table);
						wait_success = ZKLockCEDU.WaitFor(zookeeper, parent_path_table, node_path_table, ZillionConstant.WaitFor_seconds);
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + wait_success + " "
								+ node_path_table);
						if (!wait_success) {
							throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_table);
						}

						// ����ʼ
						for (ZillionTableIndex schema_index : schema_exist.Indexes) {
							String indexName = schema_index.index_name;
							if (schema_exist.table_type != null && (schema_exist.table_type.startsWith("split_"))) {
								List<String> child_nameList = container.ZK
										.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName), false);
								Collections.sort(child_nameList);
								for (String child_name : child_nameList) {
									zookeeper.delete(
											zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName) + "/" + child_name,
											-1);
									ZKHelper.recursive(zookeeper,
											zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (tableName + "." + indexName) + "/" + child_name);
									this.container.DatabaseImpl.table_manage_drop_split_index(namespace, tableName, indexName, child_name);
									this.container.DatabaseImpl.meta_query_delete_split(namespace, (tableName + "." + indexName) + "_" + child_name);
								}
							} else {
								this.container.DatabaseImpl.table_manage_drop_index(namespace, tableName, indexName);
							}
							zookeeper.delete(zkpath_namespace + ZillionConstant.zk_meta_index + "/" + (tableName + "." + indexName), -1);
							zookeeper.delete(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName), -1);
							ZKHelper.recursive(zookeeper, zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (tableName + "." + indexName));
							this.container.DatabaseImpl.meta_query_delete_index(namespace, tableName, indexName);
							schema_index.close();
						}

						if (schema_exist.table_type != null && (schema_exist.table_type.startsWith("split_"))) {
							List<String> child_nameList = container.ZK
									.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema_exist.table_name, false);
							Collections.sort(child_nameList);
							for (String child_name : child_nameList) {
								zookeeper.delete(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName + "/" + child_name, -1);
								ZKHelper.recursive(zookeeper, zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + tableName + "/" + child_name);
								this.container.DatabaseImpl.table_manage_drop_split_data(namespace, tableName, child_name);
								this.container.DatabaseImpl.meta_query_delete_split(namespace, tableName + "_" + child_name);
							}
						} else {
							this.container.DatabaseImpl.table_manage_drop_data(namespace, tableName);
						}

						zookeeper.delete(zkpath_namespace + ZillionConstant.zk_meta_data + "/" + tableName, -1);
						ZKHelper.recursive(zookeeper, zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName);
						ZKHelper.recursive(zookeeper, zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + tableName);
						ZKHelper.recursive(zookeeper, zkpath_namespace + ZillionConstant.zk_hashlocks + "/" + tableName);
						ZKHelper.recursive(zookeeper, zkpath_namespace + ZillionConstant.zk_cachelocks + "/" + tableName);
						this.container.DatabaseImpl.meta_query_delete_data(namespace, tableName);
						schema_exist.close();

						if (schema_exist.elastic_search) {
							ElasticSearchUtil.drop(this.container.esClient, namespace, tableName);
						}
						if (schema_exist.strong_consistency) {
							if (ZillionConstant.strong_consistency_cache_redis && container.redisConfig != null) {
								ShardedJedis jedis = container.redisPool.getResource();
								jedis.del(namespace + ":" + tableName);
							} else {
								DatabaseImpl_HBase impl = null;
								if (container.DatabaseImpl instanceof DatabaseImpl_HBase) {
									impl = (DatabaseImpl_HBase) container.DatabaseImpl;
								} else if (container.DatabaseImpl instanceof DatabaseImpl_Mix) {
									impl = ((DatabaseImpl_Mix) container.DatabaseImpl).hbase;
								}
								if (impl != null) {
									byte[] table_bytes = ByteToObject.toBytes(schema_exist.table_name);
									byte[] real_bytes_from = new byte[table_bytes.length + 1];
									byte[] real_bytes_to = new byte[table_bytes.length + 1];
									System.arraycopy(table_bytes, 0, real_bytes_from, 0, table_bytes.length);
									System.arraycopy(table_bytes, 0, real_bytes_to, 0, table_bytes.length);
									real_bytes_from[table_bytes.length] = 0x00;
									real_bytes_to[table_bytes.length] = 0x02;

									HTable cache_table = impl.Get_table_meta_cache(namespace);
									List<Delete> batch = new ArrayList<Delete>();
									Scan scan = new Scan();
									scan.setStartRow(real_bytes_from);
									scan.setStopRow(real_bytes_to);
									scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("QueryType"));
									ResultScanner scanner = cache_table.getScanner(scan);
									while (true) {
										Result res = scanner.next();
										if (res == null) {
											break;
										}
										byte[] real_bytes = res.getRow();
										Delete put = new Delete(real_bytes);
										put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("QueryType"));
										put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("InsertObject"));
										put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("Set"));
										batch.add(put);
									}
									cache_table.delete(batch);
									if (ZillionConstant.disable_AutoFlush) {
										cache_table.flushCommits();
									}
								}
							}
						}

						repo_version_new = ZKRepositoryUtil.ResetRepoVersion(zookeeper, namespace, parent_path_table, lock_type_table);
						LogUtil.info(" ********************************** " + method_name + " work success ");
						// �������

						result = true;
					} else {
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " Check fail " + node_path_table);
					}
				} catch (Exception e) {
					throw e;
				} finally {
					// �ͷ�table��
					ZKLockCEDU.Finish(zookeeper, node_path_table);
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " finish " + node_path_table);
				}
			} else {
				LogUtil.info(" ********************************** " + method_name + " tablelock:" + " PreCheck fail ");
			}
		} catch (Exception e) {
			throw e;
		} finally {
			// �ͷ�meta��
			ZKLockRW.Finish(zookeeper, node_path_meta);
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " finish " + node_path_meta);
		}

		if (result) {
			ObserverCheck.Wait(zookeeper, namespace, repo_version_new);
		}
		return result;
	}

	public synchronized boolean DDL_AddIndex(String namespace, String tableName, com.sagacloud.json.JSONArray index_schemaArray_Saga)
			throws Exception {
		return this.DDL_AddIndex(namespace, tableName, index_schemaArray_Saga, ZillionHashUtil.thread_count_default);
	}

	public synchronized boolean DDL_AddIndex(String namespace, String tableName, com.sagacloud.json.JSONArray index_schemaArray_Saga,
			int thread_count) throws Exception {
		JSONArray index_schemaArray = (JSONArray) Convertor_JSON.SagaJson2FastJson(index_schemaArray_Saga);
		return this.DDL_AddIndex(namespace, tableName, index_schemaArray, thread_count);
	}

	public synchronized boolean DDL_AddIndex(String namespace, String tableName, JSONArray index_schemaArray) throws Exception {
		return this.DDL_AddIndex(namespace, tableName, index_schemaArray, ZillionHashUtil.thread_count_default);
	}

	public synchronized boolean DDL_AddIndex(String namespace, String tableName, JSONArray index_schemaArray, AsyncJob job) throws Exception {
		return this.DDL_AddIndex(namespace, tableName, index_schemaArray, ZillionHashUtil.thread_count_default, job);
	}

	public synchronized boolean DDL_AddIndex(String namespace, String tableName, JSONArray index_schemaArray, int thread_count) throws Exception {
		String uuid = UUID.randomUUID().toString();
		AsyncJob job = new AsyncJob(uuid, null);
		return this.DDL_AddIndex(namespace, tableName, index_schemaArray, thread_count, job);
	}

	public synchronized boolean DDL_AddIndex(String namespace, String tableName, JSONArray index_schemaArray, int thread_count, AsyncJob job)
			throws Exception {
		String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();

		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		boolean result = false;
		int repo_version_new = -1;
		if (!this.container.initialized) {
			throw new MyException("ZillionAgent: " + "container not initialized");
		}
		SchemaRepository repository = this.container.repositoryMap.get(namespace);
		ZillionTableSchema schema_exist = repository.GetTable(tableName);
		if (schema_exist == null) {
			throw new MyException("ZillionAgent: " + "schema not exist: " + namespace + " " + tableName);
		}
		// ���
		for (int index_index = 0; index_index < index_schemaArray.size(); index_index++) {
			JSONObject index_schema = (JSONObject) index_schemaArray.get(index_index);
			String indexName = (String) index_schema.get("Name");
			ZillionTableIndex schema_index_exist = repository.GetIndex(tableName, indexName);
			if (schema_index_exist != null) {
				throw new MyException("ZillionAgent: " + "index exist: " + namespace + " " + tableName + " " + indexName);
			}
			JSONArray Columns = (JSONArray) index_schema.get("Columns");
			for (int index_col = 0; index_col < Columns.size(); index_col++) {
				String Column = (String) Columns.get(index_col);
				if (!schema_exist.ColumnMap.containsKey(Column)) {
					throw new MyException("ZillionAgent: " + namespace + " " + tableName + " Column " + Column + " not exist");
				}
				ZillionTableColumn columnDef = schema_exist.ColumnMap.get(Column);
				if (columnDef.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
					throw new MyException("ZillionAgent: " + namespace + " " + tableName + " Column " + Column + " can't use as key or index");
				}
			}

		}
		ZKWrapper zookeeper = this.container.ZK;
		String parent_path_meta = zkpath_namespace + ZillionConstant.zk_metalock;
		String lock_type_meta = "w";
		String parent_path_table = zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + tableName;
		String lock_type_table = "e";
		String check_type_table = "d";
		// ��ȡmeta��
		String node_path_meta = ZKLockRW.Create(zookeeper, parent_path_meta, lock_type_meta, this.full_name);
		try {
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + node_path_meta);
			boolean wait_success = ZKLockRW.WaitFor(zookeeper, parent_path_meta, node_path_meta, ZillionConstant.WaitFor_seconds);
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + wait_success + " " + node_path_meta);
			if (!wait_success) {
				throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_meta);
			}

			boolean success = ZKLockCEDU.PreCheck(zookeeper, parent_path_table, check_type_table);
			if (success) {
				// ��ȡtable��
				String node_path_table = ZKLockCEDU.Create(zookeeper, parent_path_table, lock_type_table, this.full_name);
				try {
					int sequence_id = Integer.parseInt(node_path_table.substring(parent_path_table.length() + 3));
					success = ZKLockCEDU.Check(zookeeper, parent_path_table, sequence_id, check_type_table);
					if (success) {
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + node_path_table);
						wait_success = ZKLockCEDU.WaitFor(zookeeper, parent_path_table, node_path_table, ZillionConstant.WaitFor_seconds);
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + wait_success + " "
								+ node_path_table);
						if (!wait_success) {
							throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_table);
						}

						// ����ʼ
						String table_type = schema_exist.table_type;
						List<ZillionTableIndex> schema_indexList = new ArrayList<ZillionTableIndex>();
						for (int index_index = 0; index_index < index_schemaArray.size(); index_index++) {
							JSONObject index_schema = (JSONObject) index_schemaArray.get(index_index);
							String indexName = (String) index_schema.get("Name");
							JSONArray indexColumns = (JSONArray) index_schema.get("Columns");
							zookeeper.create(zkpath_namespace + ZillionConstant.zk_meta_index + "/" + (tableName + "." + indexName),
									JSON.toJSONString(indexColumns, SerializerFeature.WriteMapNullValue).getBytes("UTF-8"), CreateMode.PERSISTENT);
							ZillionTableIndex schema_index = SchemaParser.ParseIndexSchema(schema_exist, index_schema);
							schema_indexList.add(schema_index);
							schema_exist.AddIndex(schema_index);

							if (table_type != null && (table_type.startsWith("split_"))) {
							} else {
								this.container.DatabaseImpl.table_manage_create_index(namespace, tableName, schema_index);
							}
						}

						for (int index_index = 0; index_index < index_schemaArray.size(); index_index++) {
							JSONObject index_schema = (JSONObject) index_schemaArray.get(index_index);
							String indexName = (String) index_schema.get("Name");
							HashObject hashObject = new HashObject();
							byte[] hash = hashObject.toBytes();
							zookeeper.create(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName), hash,
									CreateMode.PERSISTENT);
							zookeeper.create(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (tableName + "." + indexName),
									"true".getBytes("UTF-8"), CreateMode.PERSISTENT);

							JSONArray indexColumns = (JSONArray) index_schema.get("Columns");
							this.container.DatabaseImpl.meta_query_insert_index(namespace, tableName, indexName, indexColumns, hashObject);
						}

						if (this.database_type.equals("HBase") || (this.database_type.equals("Mix") && !schema_exist.store_in_minor_db)) {
							ZillionHashUtil.AddOrRepairIndex(this.container, schema_exist, schema_indexList, true, thread_count, job);
						}

						repo_version_new = ZKRepositoryUtil.ResetRepoVersion(zookeeper, namespace, parent_path_table, lock_type_table);
						LogUtil.info(" ********************************** " + method_name + " work success ");
						// �������

						result = true;
					} else {
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " Check fail " + node_path_table);
					}
				} catch (Exception e) {
					throw e;
				} finally {
					// �ͷ�table��
					ZKLockCEDU.Finish(zookeeper, node_path_table);
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " finish " + node_path_table);
				}
			} else {
				LogUtil.info(" ********************************** " + method_name + " tablelock:" + " PreCheck fail ");
			}
		} catch (Exception e) {
			throw e;
		} finally {
			// �ͷ�meta��
			ZKLockRW.Finish(zookeeper, node_path_meta);
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " finish " + node_path_meta);
		}

		if (result) {
			ObserverCheck.Wait(zookeeper, namespace, repo_version_new);
		}
		return result;
	}

	public synchronized boolean DDL_DropIndex(String namespace, String tableName, com.sagacloud.json.JSONArray indexNameArray_Saga) throws Exception {
		JSONArray indexNameArray = (JSONArray) Convertor_JSON.SagaJson2FastJson(indexNameArray_Saga);
		return this.DDL_DropIndex(namespace, tableName, indexNameArray);
	}

	public synchronized boolean DDL_DropIndex(String namespace, String tableName, JSONArray indexNameArray) throws Exception {
		String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();

		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		boolean result = false;
		int repo_version_new = -1;
		if (!this.container.initialized) {
			throw new MyException("ZillionAgent: " + "container not initialized");
		}
		SchemaRepository repository = this.container.repositoryMap.get(namespace);
		ZillionTableSchema schema_exist = repository.GetTable(tableName);
		if (schema_exist == null) {
			throw new MyException("ZillionAgent: " + "schema not exist: " + namespace + " " + tableName);
		}
		for (int index_index = 0; index_index < indexNameArray.size(); index_index++) {
			String indexName = (String) indexNameArray.get(index_index);
			ZillionTableIndex schema_index_exist = repository.GetIndex(tableName, indexName);
			if (schema_index_exist == null) {
				throw new MyException("ZillionAgent: " + "index not exist: " + namespace + " " + tableName + " " + indexName);
			}
		}
		ZKWrapper zookeeper = this.container.ZK;
		String parent_path_meta = zkpath_namespace + ZillionConstant.zk_metalock;
		String lock_type_meta = "w";
		String parent_path_table = zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + tableName;
		String lock_type_table = "e";
		String check_type_table = "d";
		// ��ȡmeta��
		String node_path_meta = ZKLockRW.Create(zookeeper, parent_path_meta, lock_type_meta, this.full_name);
		try {
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + node_path_meta);
			boolean wait_success = ZKLockRW.WaitFor(zookeeper, parent_path_meta, node_path_meta, ZillionConstant.WaitFor_seconds);
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " WaitFor " + wait_success + " " + node_path_meta);
			if (!wait_success) {
				throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_meta);
			}

			boolean success = ZKLockCEDU.PreCheck(zookeeper, parent_path_table, check_type_table);
			if (success) {
				// ��ȡtable��
				String node_path_table = ZKLockCEDU.Create(zookeeper, parent_path_table, lock_type_table, this.full_name);
				try {
					int sequence_id = Integer.parseInt(node_path_table.substring(parent_path_table.length() + 3));
					success = ZKLockCEDU.Check(zookeeper, parent_path_table, sequence_id, check_type_table);
					if (success) {
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + node_path_table);
						wait_success = ZKLockCEDU.WaitFor(zookeeper, parent_path_table, node_path_table, ZillionConstant.WaitFor_seconds);
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + wait_success + " "
								+ node_path_table);
						if (!wait_success) {
							throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_table);
						}

						// ����ʼ
						for (int index_index = 0; index_index < indexNameArray.size(); index_index++) {
							String indexName = (String) indexNameArray.get(index_index);
							if (schema_exist.table_type != null && (schema_exist.table_type.startsWith("split_"))) {
								List<String> child_nameList = container.ZK
										.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName), false);
								Collections.sort(child_nameList);
								for (String child_name : child_nameList) {
									zookeeper.delete(
											zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName) + "/" + child_name,
											-1);
									ZKHelper.recursive(zookeeper,
											zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (tableName + "." + indexName) + "/" + child_name);
									this.container.DatabaseImpl.table_manage_drop_split_index(namespace, tableName, indexName, child_name);
									this.container.DatabaseImpl.meta_query_delete_split(namespace, (tableName + "." + indexName) + "_" + child_name);
								}
							} else {
								this.container.DatabaseImpl.table_manage_drop_index(namespace, tableName, indexName);
							}

							ZillionTableIndex schema_index_exist = repository.GetIndex(tableName, indexName);
							zookeeper.delete(zkpath_namespace + ZillionConstant.zk_meta_index + "/" + (tableName + "." + indexName), -1);
							zookeeper.delete(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName), -1);
							ZKHelper.recursive(zookeeper, zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (tableName + "." + indexName));
							this.container.DatabaseImpl.meta_query_delete_index(namespace, tableName, indexName);
							schema_index_exist.close();
						}

						repo_version_new = ZKRepositoryUtil.ResetRepoVersion(zookeeper, namespace, parent_path_table, lock_type_table);
						LogUtil.info(" ********************************** " + method_name + " work success ");
						// �������

						result = true;
					} else {
						LogUtil.info(" ********************************** " + method_name + " tablelock:" + " Check fail " + node_path_table);
					}
				} catch (Exception e) {
					throw e;
				} finally {
					// �ͷ�table��
					ZKLockCEDU.Finish(zookeeper, node_path_table);
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " finish " + node_path_table);
				}
			} else {
				LogUtil.info(" ********************************** " + method_name + " tablelock:" + " PreCheck fail ");
			}
		} catch (Exception e) {
			throw e;
		} finally {
			// �ͷ�meta��
			ZKLockRW.Finish(zookeeper, node_path_meta);
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " finish " + node_path_meta);
		}

		if (result) {
			ObserverCheck.Wait(zookeeper, namespace, repo_version_new);
		}
		return result;
	}

	public synchronized boolean DDL_RepairIndex(String namespace, String tableName, List<String> indexNameList) throws Exception {
		return this.DDL_RepairIndex(namespace, tableName, indexNameList, ZillionHashUtil.thread_count_default);
	}

	public synchronized boolean DDL_RepairIndex(String namespace, String tableName, List<String> indexNameList, int thread_count) throws Exception {
		String uuid = UUID.randomUUID().toString();
		AsyncJob job = new AsyncJob(uuid, null);
		return this.DDL_RepairIndex(namespace, tableName, indexNameList, thread_count, job);
	}

	public synchronized boolean DDL_RepairIndex(String namespace, String tableName, List<String> indexNameList, int thread_count, AsyncJob job)
			throws Exception {
		String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();

		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		boolean result = false;
		if (!this.container.initialized) {
			throw new MyException("ZillionAgent: " + "container not initialized");
		}
		SchemaRepository repository = this.container.repositoryMap.get(namespace);
		ZillionTableSchema schema_exist = repository.GetTable(tableName);
		if (schema_exist == null) {
			throw new MyException("ZillionAgent: " + "schema not exist: " + namespace + " " + tableName);
		}
		if (this.database_type.equals("HBase") || (this.database_type.equals("Mix") && !schema_exist.store_in_minor_db)) {
		} else {
			return true;
		}

		ZKWrapper zookeeper = this.container.ZK;
		String parent_path_table = zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + tableName;
		String lock_type_table = "e";
		String check_type_table = "d";
		boolean success = ZKLockCEDU.PreCheck(zookeeper, parent_path_table, check_type_table);
		if (success) {
			// ��ȡtable��
			String node_path_table = ZKLockCEDU.Create(zookeeper, parent_path_table, lock_type_table, this.full_name);
			try {
				int sequence_id = Integer.parseInt(node_path_table.substring(parent_path_table.length() + 3));
				success = ZKLockCEDU.Check(zookeeper, parent_path_table, sequence_id, check_type_table);
				if (success) {
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + node_path_table);
					boolean wait_success = ZKLockCEDU.WaitFor(zookeeper, parent_path_table, node_path_table, ZillionConstant.WaitFor_seconds);
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + wait_success + " "
							+ node_path_table);
					if (!wait_success) {
						throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_table);
					}

					// ����ʼ
					List<ZillionTableIndex> schema_indexList = new ArrayList<ZillionTableIndex>();
					for (int index = 0; index < indexNameList.size(); index++) {
						String indexName = indexNameList.get(index);
						ZillionTableIndex index_exist = null;
						for (ZillionTableIndex schema_index : schema_exist.Indexes) {
							if (schema_index.index_name.equals(indexName)) {
								index_exist = schema_index;
								break;
							}
						}
						schema_indexList.add(index_exist);
					}
					ZillionHashUtil.AddOrRepairIndex(this.container, schema_exist, schema_indexList, false, thread_count, job);
					// �������

					result = true;
				} else {
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " Check fail " + node_path_table);
				}
			} catch (Exception e) {
				throw e;
			} finally {
				// �ͷ�table��
				ZKLockCEDU.Finish(zookeeper, node_path_table);
				LogUtil.info(" ********************************** " + method_name + " tablelock:" + " finish " + node_path_table);
			}
		} else {
			LogUtil.info(" ********************************** " + method_name + " tablelock:" + " PreCheck fail ");
		}

		return result;
	}

	public synchronized boolean DDL_RefreshHash(String namespace, String tableName) throws Exception {
		return this.DDL_RefreshHash(namespace, tableName, ZillionHashUtil.thread_count_default);
	}

	public synchronized boolean DDL_RefreshHash(String namespace, String tableName, int thread_count) throws Exception {
		String uuid = UUID.randomUUID().toString();
		AsyncJob job = new AsyncJob(uuid, null);
		return this.DDL_RefreshHash(namespace, tableName, thread_count, job);
	}

	public synchronized boolean DDL_RefreshHash(String namespace, String tableName, int thread_count, AsyncJob job) throws Exception {
		String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();

		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		boolean result = false;
		if (!this.container.initialized) {
			throw new MyException("ZillionAgent: " + "container not initialized");
		}
		SchemaRepository repository = this.container.repositoryMap.get(namespace);
		ZillionTableSchema schema_exist = repository.GetTable(tableName);
		if (schema_exist == null) {
			throw new MyException("ZillionAgent: " + "schema not exist: " + namespace + " " + tableName);
		}
		if (this.database_type.equals("HBase") || (this.database_type.equals("Mix") && !schema_exist.store_in_minor_db)) {
		} else {
			return true;
		}

		ZKWrapper zookeeper = this.container.ZK;
		String parent_path_table = zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + tableName;
		String lock_type_table = "e";
		String check_type_table = "d";
		boolean success = ZKLockCEDU.PreCheck(zookeeper, parent_path_table, check_type_table);
		if (success) {
			// ��ȡtable��
			String node_path_table = ZKLockCEDU.Create(zookeeper, parent_path_table, lock_type_table, this.full_name);
			try {
				int sequence_id = Integer.parseInt(node_path_table.substring(parent_path_table.length() + 3));
				success = ZKLockCEDU.Check(zookeeper, parent_path_table, sequence_id, check_type_table);
				if (success) {
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + node_path_table);
					boolean wait_success = ZKLockCEDU.WaitFor(zookeeper, parent_path_table, node_path_table, ZillionConstant.WaitFor_seconds);
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + wait_success + " "
							+ node_path_table);
					if (!wait_success) {
						throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_table);
					}

					// ����ʼ
					ZillionHashUtil.RefreshHash(this.container, schema_exist, thread_count, job);
					// �������

					result = true;
				} else {
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " Check fail " + node_path_table);
				}
			} catch (Exception e) {
				throw e;
			} finally {
				// �ͷ�table��
				ZKLockCEDU.Finish(zookeeper, node_path_table);
				LogUtil.info(" ********************************** " + method_name + " tablelock:" + " finish " + node_path_table);
			}
		} else {
			LogUtil.info(" ********************************** " + method_name + " tablelock:" + " PreCheck fail ");
		}

		return result;
	}

	public synchronized boolean DDL_IndexCheck(String namespace, String tableName, com.sagacloud.json.JSONObject returnJSON_Saga) throws Exception {
		JSONObject returnJSON = (JSONObject) Convertor_JSON.SagaJson2FastJson(returnJSON_Saga);
		return this.DDL_IndexCheck(namespace, tableName, returnJSON);
	}

	public synchronized boolean DDL_IndexCheck(String namespace, String tableName, JSONObject returnJSON) throws Exception {
		String method_name = Thread.currentThread().getStackTrace()[1].getMethodName();

		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		boolean result = false;
		if (!this.container.initialized) {
			throw new MyException("ZillionAgent: " + "container not initialized");
		}
		SchemaRepository repository = this.container.repositoryMap.get(namespace);
		ZillionTableSchema schema_exist = repository.GetTable(tableName);
		if (schema_exist == null) {
			throw new MyException("ZillionAgent: " + "schema not exist: " + namespace + " " + tableName);
		}

		ZKWrapper zookeeper = this.container.ZK;
		String parent_path_table = zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + tableName;
		String lock_type_table = "e";
		String check_type_table = "d";
		boolean success = ZKLockCEDU.PreCheck(zookeeper, parent_path_table, check_type_table);
		if (success) {
			// ��ȡtable��
			String node_path_table = ZKLockCEDU.Create(zookeeper, parent_path_table, lock_type_table, this.full_name);
			try {
				int sequence_id = Integer.parseInt(node_path_table.substring(parent_path_table.length() + 3));
				success = ZKLockCEDU.Check(zookeeper, parent_path_table, sequence_id, check_type_table);
				if (success) {
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + node_path_table);
					boolean wait_success = ZKLockCEDU.WaitFor(zookeeper, parent_path_table, node_path_table, ZillionConstant.WaitFor_seconds);
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " WaitFor " + wait_success + " "
							+ node_path_table);
					if (!wait_success) {
						throw new MyException("ZillionAgent: " + "WaitFor fail " + node_path_table);
					}

					// ����ʼ
					if (schema_exist.Indexes.size() > 0) {
						Stat stat = new Stat();
						byte[] hash_table = this.container.ZK.getData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName, false,
								stat);
						HashObject hashObject_table = HashUtil.Parse(hash_table);
						long count_table = hashObject_table.insert_count - hashObject_table.delete_count;
						returnJSON.put("DataCount", (long) count_table);

						boolean all_equal = true;
						JSONArray content = new JSONArray();
						for (ZillionTableIndex schema_index : schema_exist.Indexes) {
							String indexName = schema_index.index_name;
							byte[] hash_index = zookeeper
									.getData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName), false, stat);
							HashObject hashObject_index = HashUtil.Parse(hash_index);
							long count_index = hashObject_index.insert_count - hashObject_index.delete_count;
							JSONObject compareJSON = new JSONObject();
							compareJSON.put("Index", indexName);
							compareJSON.put("DataCount", (long) count_index);
							if (count_index != count_table) {
								all_equal = false;
							}
							content.add(compareJSON);
						}
						returnJSON.put("Content", content);
						if (all_equal) {
							result = true;
							returnJSON.put("Result", "success");
						} else {
							returnJSON.put("Result", "failure");
						}
					} else {
						returnJSON.put("Result", "success");
					}
					// �������

					result = true;
				} else {
					LogUtil.info(" ********************************** " + method_name + " tablelock:" + " Check fail " + node_path_table);
				}
			} catch (Exception e) {
				throw e;
			} finally {
				// �ͷ�table��
				ZKLockCEDU.Finish(zookeeper, node_path_table);
				LogUtil.info(" ********************************** " + method_name + " tablelock:" + " finish " + node_path_table);
			}
		} else {
			LogUtil.info(" ********************************** " + method_name + " tablelock:" + " PreCheck fail ");
		}

		return result;
	}
}
