package com.zillion.database.agent;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.apache.hadoop.hbase.filter.CompareFilter.CompareOp;
import org.apache.hadoop.hbase.filter.Filter;
import org.apache.hadoop.hbase.filter.FilterList;
import org.apache.hadoop.hbase.filter.FilterList.Operator;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter.RowRange;
import org.apache.hadoop.hbase.filter.RegexStringComparator;
import org.apache.hadoop.hbase.filter.SingleColumnValueFilter;
import org.zillion.util.common.MyException;
import org.zillion.util.json.FastJsonCompareUtil;
import org.zillion.util.json.FastJsonUtil;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.second.SecondCriteria;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.ZillionTableColumn;
import com.zillion.util.table.ZillionTableColumnEnum;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;

public class ZillionCriteriaMatchUtil {
	public static List<ZillionCriteriaGroup> groupList(ZillionTableSchema schema, List<JSONObject> CriteriaList, SecondCriteria scObject, String Hint)
			throws Exception {
		if (Hint == null) {
			return ZillionCriteriaMatchUtil.groupList_without_Hint(schema, CriteriaList, scObject);
		} else {
			String[] keys = schema.Key;
			int index_index = -1;

			for (int ii = 0; ii < schema.Indexes.size(); ii++) {
				ZillionTableIndex index_schema = schema.Indexes.get(ii);
				if (Hint.equals(index_schema.index_name)) {
					keys = index_schema.Key;
					index_index = ii;
					break;
				}
			}
			ZillionCriteriaGroup group = null;
			for (int i = 0; i < CriteriaList.size(); i++) {
				JSONObject CriteriaInner = CriteriaList.get(i);
				ZillionCriteriaMatchItem matchItem = ZillionCriteriaMatchUtil.CriteriaMatch(schema, keys, CriteriaInner);
				if (matchItem != null) {
					matchItem.index_index = index_index;
					if (group == null) {
						group = new ZillionCriteriaGroup();
						group.index_index = index_index;
						group.scObject = scObject;
						group.otherCriteria = matchItem.otherCriteria;
						group.valueFilterList = matchItem.valueFilterList;
					}
					group.CriteriaList.add(CriteriaInner);
					group.itemListOfGroup.add(matchItem);
					group.rangeList_old
							.add(new RowRange(matchItem.bytes_gt, matchItem.bytes_gt_include, matchItem.bytes_lt, matchItem.bytes_lt_include));
				} else {
					throw new MyException("ZillionAgent: Criteria error \n" + FastJsonUtil.toFormatString(CriteriaInner));
				}
			}
			for (RowRange rr_old : group.rangeList_old) {
				int index_exist = -1;
				RowRange rr_exist = null;
				for (int index_tmp = 0; index_tmp < group.rangeList_new.size(); index_tmp++) {
					RowRange rr_new = group.rangeList_new.get(index_tmp);
					if (Arrays.equals(rr_new.getStartRow(), rr_old.getStartRow()) && Arrays.equals(rr_new.getStopRow(), rr_old.getStopRow())) {
						index_exist = index_tmp;
						rr_exist = rr_new;
						break;
					}
				}
				if (rr_exist != null) {
					boolean startRowInclusive = rr_exist.isStartRowInclusive() || rr_old.isStartRowInclusive();
					boolean stopRowInclusive = rr_exist.isStopRowInclusive() || rr_old.isStopRowInclusive();
					if (startRowInclusive == rr_exist.isStartRowInclusive() && stopRowInclusive == rr_exist.isStopRowInclusive()) {
					} else {
						group.rangeList_new.remove(index_exist);
						group.rangeList_new.add(new RowRange(rr_exist.getStartRow(), startRowInclusive, rr_exist.getStopRow(), stopRowInclusive));
					}
				} else {
					group.rangeList_new.add(rr_old);
				}
			}

			List<ZillionCriteriaGroup> groupList = new ArrayList<ZillionCriteriaGroup>();
			groupList.add(group);
			return groupList;
		}
	}

	private static List<ZillionCriteriaGroup> groupList_without_Hint(ZillionTableSchema schema, List<JSONObject> CriteriaList,
			SecondCriteria scObject) throws Exception {
		// ÿƥȷ
		List<List<ZillionCriteriaMatchItem>> matchItemListList = new ArrayList<List<ZillionCriteriaMatchItem>>();
		for (int i = 0; i < CriteriaList.size(); i++) {
			JSONObject CriteriaInner = CriteriaList.get(i);
			List<ZillionCriteriaMatchItem> matchItemList = new ArrayList<ZillionCriteriaMatchItem>();
			{
				ZillionCriteriaMatchItem matchItem = ZillionCriteriaMatchUtil.CriteriaMatch(schema, schema.Key, CriteriaInner);
				if (matchItem != null) {
					matchItem.index_index = -1;
					matchItemList.add(matchItem);
				}
			}
			for (int ii = 0; ii < schema.Indexes.size(); ii++) {
				ZillionTableIndex index_schema = schema.Indexes.get(ii);
				ZillionCriteriaMatchItem matchItem = ZillionCriteriaMatchUtil.CriteriaMatch(schema, index_schema.Key, CriteriaInner);
				if (matchItem != null) {
					matchItem.index_index = ii;
					matchItemList.add(matchItem);
				}
			}
			if (matchItemList.size() == 0) {
				throw new MyException("ZillionAgent: Criteria error \n" + FastJsonUtil.toFormatString(CriteriaInner));
			}

			matchItemListList.add(matchItemList);
		}

		// 
		List<ZillionCriteriaGroup> groupList = new ArrayList<ZillionCriteriaGroup>();
		boolean[] matchItemUsed = new boolean[matchItemListList.size()];
		for (int i = 0; i < CriteriaList.size(); i++) {
			matchItemUsed[i] = false;
		}
		while (true) {
			ZillionCriteriaMatchItem best_matchItem = null;
			int best_i = -1;
			for (int i = 0; i < CriteriaList.size(); i++) {
				if (matchItemUsed[i]) {
					continue;
				}
				List<ZillionCriteriaMatchItem> matchItemList = matchItemListList.get(i);
				for (int ii = 0; ii < matchItemList.size(); ii++) {
					ZillionCriteriaMatchItem matchItem = matchItemList.get(ii);
					if (best_matchItem == null || best_matchItem.matched < matchItem.matched) {
						best_matchItem = matchItem;
						best_i = i;
					}
				}
			}
			if (best_matchItem == null) {
				break;
			}

			ZillionCriteriaGroup group = new ZillionCriteriaGroup();
			group.index_index = best_matchItem.index_index;
			group.CriteriaList.add(CriteriaList.get(best_i));
			group.itemListOfGroup.add(best_matchItem);
			group.rangeList_old.add(
					new RowRange(best_matchItem.bytes_gt, best_matchItem.bytes_gt_include, best_matchItem.bytes_lt, best_matchItem.bytes_lt_include));
			group.otherCriteria = best_matchItem.otherCriteria;
			group.valueFilterList = best_matchItem.valueFilterList;
			group.scObject = scObject;
			matchItemUsed[best_i] = true;
			for (int i = 0; i < CriteriaList.size(); i++) {
				if (matchItemUsed[i]) {
					continue;
				}
				List<ZillionCriteriaMatchItem> matchItemList = matchItemListList.get(i);
				for (int ii = 0; ii < matchItemList.size(); ii++) {
					ZillionCriteriaMatchItem matchItem = matchItemList.get(ii);
					if (matchItem.index_index == best_matchItem.index_index
							&& FastJsonCompareUtil.Instance().Compare(matchItem.otherCriteria, best_matchItem.otherCriteria)) {
						group.itemListOfGroup.add(matchItem);
						matchItemUsed[i] = true;
						group.CriteriaList.add(CriteriaList.get(i));
						group.rangeList_old
								.add(new RowRange(matchItem.bytes_gt, matchItem.bytes_gt_include, matchItem.bytes_lt, matchItem.bytes_lt_include));
						break;
					}
				}
			}
			for (RowRange rr_old : group.rangeList_old) {
				int index_exist = -1;
				RowRange rr_exist = null;
				for (int index_tmp = 0; index_tmp < group.rangeList_new.size(); index_tmp++) {
					RowRange rr_new = group.rangeList_new.get(index_tmp);
					if (Arrays.equals(rr_new.getStartRow(), rr_old.getStartRow()) && Arrays.equals(rr_new.getStopRow(), rr_old.getStopRow())) {
						index_exist = index_tmp;
						rr_exist = rr_new;
						break;
					}
				}
				if (rr_exist != null) {
					boolean startRowInclusive = rr_exist.isStartRowInclusive() || rr_old.isStartRowInclusive();
					boolean stopRowInclusive = rr_exist.isStopRowInclusive() || rr_old.isStopRowInclusive();
					if (startRowInclusive == rr_exist.isStartRowInclusive() && stopRowInclusive == rr_exist.isStopRowInclusive()) {
					} else {
						group.rangeList_new.remove(index_exist);
						group.rangeList_new.add(new RowRange(rr_exist.getStartRow(), startRowInclusive, rr_exist.getStopRow(), stopRowInclusive));
					}
				} else {
					group.rangeList_new.add(rr_old);
				}
			}
			groupList.add(group);
		}

		return groupList;
	}

	public static ZillionCriteriaMatchItem CriteriaMatch(ZillionTableSchema schema, String[] ColumnNames, JSONObject Criteria) throws IOException {
		int equalCount = 0;
		boolean lastRange = false;
		ZillionTableColumn lastRangeColumn = null;
		JSONObject lastRangeJSON = null;
		for (int i = 0; i < ColumnNames.length; i++) {
			String ColumnName = ColumnNames[i];
			ZillionTableColumn column = schema.ColumnMap.get(ColumnName);

			if (!Criteria.containsKey(ColumnName)) {
				break;
			}

			Object columnObject = Criteria.get(ColumnName);
			if (columnObject == null) {
				equalCount++;
			} else {
				if (columnObject instanceof JSONObject) {
					JSONObject columnJSON = (JSONObject) columnObject;
					if (columnJSON.containsKey("$regex")) {

					} else {
						lastRange = true;
						lastRangeColumn = column;
						lastRangeJSON = columnJSON;
					}
					break;
				} else {
					equalCount++;
				}
			}
		}

		int keyMatchCount = equalCount;
		if (lastRange) {
			keyMatchCount++;
		}
		JSONObject otherCriteria = new JSONObject();
		{
			Iterator<String> keyIter = Criteria.keySet().iterator();
			while (keyIter.hasNext()) {
				String key = keyIter.next();
				Object value = Criteria.get(key);
				boolean key_match = false;
				for (int i = 0; i < keyMatchCount; i++) {
					String ColumnName = ColumnNames[i];
					if (ColumnName.equals(key)) {
						key_match = true;
						break;
					}
				}
				if (!key_match) {
					otherCriteria.put(key, value);
				}
			}
		}

		boolean match = false;
		if (schema.table_type.equals("all_column")) {
			match = true;
		} else {
			boolean other_bad = false;
			for (int i = 0; i < ColumnNames.length; i++) {
				String ColumnName = ColumnNames[i];
				if (otherCriteria.containsKey(ColumnName)) {
					other_bad = true;
					break;
				}
			}
			if (!other_bad) {
				match = true;
			}
		}

		if (!match) {
			return null;
		}

		ZillionCriteriaMatchItem result = new ZillionCriteriaMatchItem();
		List<byte[]> bytess = new ArrayList<byte[]>();
		for (int i = 0; i < equalCount; i++) {
			String ColumnName = ColumnNames[i];
			ZillionTableColumn column = schema.ColumnMap.get(ColumnName);
			Object columnObject = Criteria.get(ColumnName);
			byte[] bytes = HTableUtil.GenerateKeyBytes(column, columnObject);
			bytess.add(bytes);
		}
		if (lastRange) {
			int bytes_count = 0;
			for (int i = 0; i < equalCount; i++) {
				bytes_count += bytess.get(i).length;
			}

			byte[] bytes_gte = new byte[bytes_count + 1];
			byte[] bytes_lt = new byte[bytes_count + 1];
			{
				int bytes_index = 0;
				for (int i = 0; i < equalCount; i++) {
					byte[] bytes = bytess.get(i);
					System.arraycopy(bytes, 0, bytes_gte, bytes_index, bytes.length);
					System.arraycopy(bytes, 0, bytes_lt, bytes_index, bytes.length);
					bytes_index += bytes.length;
				}
			}
			bytes_gte[bytes_gte.length - 1] = 0x00;
			bytes_lt[bytes_lt.length - 1] = 0x02;
			result.matched = keyMatchCount;
			// result.bytes_gt = bytes_gte;
			// result.bytes_gt_include = true;
			result.bytes_gt = ByteToObject.lastBytes(bytes_gte);
			result.bytes_gt_include = false;
			result.bytes_lt = bytes_lt;
			result.bytes_lt_include = false;

			Iterator<String> keyIterInner = lastRangeJSON.keySet().iterator();
			while (keyIterInner.hasNext()) {
				String fuhao = keyIterInner.next();
				Object columnObject = lastRangeJSON.get(fuhao);
				byte[] bytes_object = HTableUtil.GenerateKeyBytes(lastRangeColumn, columnObject);
				byte[] bytes_inner;
				if (keyMatchCount == ColumnNames.length) {
					bytes_inner = new byte[bytes_count + bytes_object.length];
				} else {
					bytes_inner = new byte[bytes_count + bytes_object.length + 1];
				}
				int bytes_index = 0;
				for (int i = 0; i < equalCount; i++) {
					byte[] bytes = bytess.get(i);
					System.arraycopy(bytes, 0, bytes_inner, bytes_index, bytes.length);
					bytes_index += bytes.length;
				}
				System.arraycopy(bytes_object, 0, bytes_inner, bytes_index, bytes_object.length);
				if (keyMatchCount == ColumnNames.length) {
					if (fuhao.equals("$lt")) {
						// result.bytes_lt = bytes_inner;
						// result.bytes_lt_include = false;
						result.bytes_lt = ByteToObject.lastBytes(bytes_inner);
						result.bytes_lt_include = true;
					} else if (fuhao.equals("$lte")) {
						result.bytes_lt = bytes_inner;
						result.bytes_lt_include = true;
					} else if (fuhao.equals("$gt")) {
						result.bytes_gt = bytes_inner;
						result.bytes_gt_include = false;
					} else if (fuhao.equals("$gte")) {
						// result.bytes_gt = bytes_inner;
						// result.bytes_gt_include = true;
						result.bytes_gt = ByteToObject.lastBytes(bytes_inner);
						result.bytes_gt_include = false;
					}
				} else {
					if (fuhao.equals("$lt")) {
						byte[] bbbbb = new byte[bytes_inner.length];
						System.arraycopy(bytes_inner, 0, bbbbb, 0, bytes_inner.length);
						bytes_inner[bytes_inner.length - 1] = 0x00;
						// result.bytes_lt = bytes_inner;
						// result.bytes_lt_include = false;
						result.bytes_lt = ByteToObject.lastBytes(bbbbb);
						result.bytes_lt_include = true;
					} else if (fuhao.equals("$lte")) {
						bytes_inner[bytes_inner.length - 1] = 0x02;
						result.bytes_lt = bytes_inner;
						result.bytes_lt_include = false;
					} else if (fuhao.equals("$gt")) {
						bytes_inner[bytes_inner.length - 1] = 0x02;
						result.bytes_gt = bytes_inner;
						result.bytes_gt_include = false;
					} else if (fuhao.equals("$gte")) {
						byte[] bbbbb = new byte[bytes_inner.length];
						System.arraycopy(bytes_inner, 0, bbbbb, 0, bytes_inner.length);
						bytes_inner[bytes_inner.length - 1] = 0x00;
						// result.bytes_gt = bytes_inner;
						// result.bytes_gt_include = true;
						result.bytes_gt = ByteToObject.lastBytes(bbbbb);
						result.bytes_gt_include = false;
					}
				}
			}
		} else {
			int bytes_count = 0;
			for (int i = 0; i < equalCount; i++) {
				bytes_count += bytess.get(i).length;
			}

			result.matched = equalCount;
			if (equalCount == ColumnNames.length) {
				byte[] get_key = new byte[bytes_count];
				int bytes_index = 0;
				for (int i = 0; i < equalCount; i++) {
					byte[] bytes = bytess.get(i);
					System.arraycopy(bytes, 0, get_key, bytes_index, bytes.length);
					bytes_index += bytes.length;
				}
				// result.bytes_gt = get_key;
				// result.bytes_gt_include = true;
				result.bytes_gt = ByteToObject.lastBytes(get_key);
				result.bytes_gt_include = false;
				result.bytes_lt = get_key;
				result.bytes_lt_include = true;
			} else {
				byte[] bytes_gte = new byte[bytes_count + 1];
				byte[] bytes_lt = new byte[bytes_count + 1];
				int bytes_index = 0;
				for (int i = 0; i < equalCount; i++) {
					byte[] bytes = bytess.get(i);
					System.arraycopy(bytes, 0, bytes_gte, bytes_index, bytes.length);
					System.arraycopy(bytes, 0, bytes_lt, bytes_index, bytes.length);
					bytes_index += bytes.length;
				}
				bytes_gte[bytes_gte.length - 1] = 0x00;
				bytes_lt[bytes_lt.length - 1] = 0x02;
				// result.bytes_gt = bytes_gte;
				result.bytes_gt = ByteToObject.lastBytes(bytes_gte);
				result.bytes_gt_include = true;
				result.bytes_lt = bytes_lt;
				result.bytes_lt_include = false;
			}
		}
		// if (result.bytes_gt_include && result.bytes_lt_include && Bytes.compareTo(result.bytes_gt, result.bytes_lt) == 0) {
		// byte[] bytes_lt_new = new byte[result.bytes_lt.length + 1];
		// System.arraycopy(result.bytes_lt, 0, bytes_lt_new, 0, result.bytes_lt.length);
		// bytes_lt_new[bytes_lt_new.length - 1] = 0x00;
		// result.bytes_lt = bytes_lt_new;
		// result.bytes_lt_include = false;
		// }

		result.otherCriteria = otherCriteria;
		{
			Iterator<String> keyIter = otherCriteria.keySet().iterator();
			while (keyIter.hasNext()) {
				String key = keyIter.next();
				Object columnObject = otherCriteria.get(key);
				ZillionTableColumn column = schema.ColumnMap.get(key);
				if (columnObject == null) {
					SingleColumnValueFilter filter = new SingleColumnValueFilter(ByteToObject.toBytes("f"), ByteToObject.toBytes(key),
							CompareOp.EQUAL, new byte[0]);
					filter.setFilterIfMissing(true);
					result.valueFilterList.add(filter);
				} else if (columnObject instanceof JSONObject) {
					JSONObject columnJSON = (JSONObject) columnObject;
					if (columnJSON.containsKey("in")) {
						JSONArray inArray = (JSONArray) columnJSON.get("in");
						Operator operator = Operator.MUST_PASS_ONE;
						List<Filter> rowFilters = new ArrayList<Filter>();
						for (int i = 0; i < inArray.size(); i++) {
							Object inValue = inArray.get(i);
							byte[] bytes_object = HTableUtil.GenerateBytes(column, inValue);
							if (column.Type == ZillionTableColumnEnum.STRING || column.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
								if (bytes_object != null && bytes_object.length == 0) {
									byte[] column_bytesInner = new byte[1];
									column_bytesInner[0] = 0x00;
									bytes_object = column_bytesInner;
								}
							}
							rowFilters.add(
									new SingleColumnValueFilter(ByteToObject.toBytes("f"), ByteToObject.toBytes(key), CompareOp.EQUAL, bytes_object));
						}
						FilterList filter = new FilterList(operator, rowFilters);
						result.valueFilterList.add(filter);
					} else if (columnJSON.containsKey("$regex")) {
						String regex = (String) columnJSON.get("$regex");
						RegexStringComparator comp = new RegexStringComparator(regex);
						SingleColumnValueFilter filter = new SingleColumnValueFilter(ByteToObject.toBytes("f"), ByteToObject.toBytes(key),
								CompareOp.EQUAL, comp);
						filter.setFilterIfMissing(true);
						result.valueFilterList.add(filter);
					} else {
						Iterator<String> keyIterInner = columnJSON.keySet().iterator();
						while (keyIterInner.hasNext()) {
							String fuhao = keyIterInner.next();
							byte[] bytes_object = HTableUtil.GenerateBytes(column, columnJSON.get(fuhao));
							if (column.Type == ZillionTableColumnEnum.STRING || column.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
								if (bytes_object != null && bytes_object.length == 0) {
									byte[] column_bytesInner = new byte[1];
									column_bytesInner[0] = 0x00;
									bytes_object = column_bytesInner;
								}
							}
							if (fuhao.equals("$lt")) {
								SingleColumnValueFilter filter = new SingleColumnValueFilter(ByteToObject.toBytes("f"), ByteToObject.toBytes(key),
										CompareOp.LESS, bytes_object);
								filter.setFilterIfMissing(true);
								result.valueFilterList.add(filter);
							} else if (fuhao.equals("$lte")) {
								SingleColumnValueFilter filter = new SingleColumnValueFilter(ByteToObject.toBytes("f"), ByteToObject.toBytes(key),
										CompareOp.LESS_OR_EQUAL, bytes_object);
								filter.setFilterIfMissing(true);
								result.valueFilterList.add(filter);
							} else if (fuhao.equals("$gt")) {
								SingleColumnValueFilter filter = new SingleColumnValueFilter(ByteToObject.toBytes("f"), ByteToObject.toBytes(key),
										CompareOp.GREATER, bytes_object);
								filter.setFilterIfMissing(true);
								result.valueFilterList.add(filter);
							} else if (fuhao.equals("$gte")) {
								SingleColumnValueFilter filter = new SingleColumnValueFilter(ByteToObject.toBytes("f"), ByteToObject.toBytes(key),
										CompareOp.GREATER_OR_EQUAL, bytes_object);
								filter.setFilterIfMissing(true);
								result.valueFilterList.add(filter);
							}
						}
					}
				} else {
					byte[] bytes_object = HTableUtil.GenerateBytes(column, columnObject);
					if (column.Type == ZillionTableColumnEnum.STRING || column.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
						if (bytes_object != null && bytes_object.length == 0) {
							byte[] column_bytesInner = new byte[1];
							column_bytesInner[0] = 0x00;
							bytes_object = column_bytesInner;
						}
					}
					SingleColumnValueFilter filter = new SingleColumnValueFilter(ByteToObject.toBytes("f"), ByteToObject.toBytes(key),
							CompareOp.EQUAL, bytes_object);
					filter.setFilterIfMissing(true);
					result.valueFilterList.add(filter);
				}
			}
		}

		return result;
	}

}
