package com.zillion.database.agent;

import java.util.ArrayList;
import java.util.List;

import org.zillion.util.common.MyException;
import org.zillion.util.log.LogUtil;

import com.zillion.database.async.AsyncJob;
import com.zillion.util.common.HashObject;
import com.zillion.util.common.HashUtil;
import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.table.SchemaWrapper;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;

public class ZillionHashWorker {
	public String work_type;
	public SchemaRepositoryContainer container;
	public ZillionTableSchema schema;
	public List<ZillionTableIndex> schema_indexList;
	public AsyncJob asyncJob;

	int thread_count = 8;
	List<ZillionHashWorkerItem> jobList;
	int jobIndex = 0;

	public synchronized ZillionHashWorkerItem Pop() {
		ZillionHashWorkerItem result = null;
		if (jobIndex < jobList.size()) {
			result = jobList.get(jobIndex++);
			if (this.work_type.equals("RefreshHash")) {
				String fullName = schema.table_name;
				if (result.schema_index != null) {
					fullName = schema.table_name + "." + result.schema_index.index_name;
				}
				if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
					LogUtil.warn(this.work_type + "\t" + fullName + "_" + result.schemaWrapper.child_name);
				} else {
					LogUtil.warn(this.work_type + "\t" + fullName);
				}
			} else {
				String tableName = schema.table_name;
				StringBuffer sb = new StringBuffer();
				sb.append("[");
				for (ZillionTableIndex index : schema_indexList) {
					if (sb.length() > 1) {
						sb.append(",");
					}
					sb.append(index.index_name);
				}
				sb.append("]");
				if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
					LogUtil.warn(this.work_type + "\t" + tableName + "." + sb.toString() + "_" + result.schemaWrapper.child_name);
				} else {
					LogUtil.warn(this.work_type + "\t" + tableName + "." + sb.toString());
				}
			}
		}
		return result;
	}

	// ComputeHash_Table_AddIndex ComputeHash_Table_RefreshIndex
	public ZillionHashWorker(SchemaRepositoryContainer container, ZillionTableSchema schema, String work_type,
			List<ZillionTableIndex> schema_indexList, List<SchemaWrapper> schemaWrapperList, int thread_count, AsyncJob asyncJob) {
		this.work_type = work_type;
		this.container = container;
		this.schema = schema;
		this.schema_indexList = schema_indexList;
		this.jobList = new ArrayList<ZillionHashWorkerItem>();
		for (SchemaWrapper schemaWrapper : schemaWrapperList) {
			this.jobList.add(new ZillionHashWorkerItem(schemaWrapper, this.schema));
		}
		this.thread_count = thread_count > this.jobList.size() ? this.jobList.size() : thread_count;
		this.asyncJob = asyncJob;
	}

	public ZillionHashWorker(SchemaRepositoryContainer container, ZillionTableSchema schema, List<SchemaWrapper> schemaWrapperList, int thread_count,
			AsyncJob asyncJob) {
		this.work_type = "RefreshHash";
		this.container = container;
		this.schema = schema;
		this.schema_indexList = new ArrayList<ZillionTableIndex>();
		this.jobList = new ArrayList<ZillionHashWorkerItem>();
		for (SchemaWrapper schemaWrapper : schemaWrapperList) {
			this.jobList.add(new ZillionHashWorkerItem(schemaWrapper, schema));
			for (ZillionTableIndex schema_index : schema.Indexes) {
				this.jobList.add(new ZillionHashWorkerItem(schemaWrapper, schema_index));
			}
		}
		this.thread_count = thread_count > this.jobList.size() ? this.jobList.size() : thread_count;
		this.asyncJob = asyncJob;
	}

	public HashObject Do() throws Exception {
		List<ZillionHashWorkerThread> threadList = new ArrayList<ZillionHashWorkerThread>();
		for (int index = 0; index < this.thread_count; index++) {
			ZillionHashWorkerThread thread = new ZillionHashWorkerThread(this);
			thread.start();
			threadList.add(thread);
		}

		while (true) {
			try {
				Thread.sleep(1L);
			} catch (InterruptedException e) {
				// e.printStackTrace();
			}

			boolean all_finish = true;
			for (ZillionHashWorkerThread thread : threadList) {
				if (thread.isAlive()) {
					all_finish = false;
					break;
				}
			}
			if (all_finish) {
				break;
			}
		}

		HashObject result = new HashObject();
		if (this.work_type.equals("RefreshHash")) {
			String namespace = schema.Namespace;
			String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
			String tableName = schema.table_name;

			{
				for (ZillionHashWorkerItem workItem : this.jobList) {
					if (!workItem.finish) {
						throw new MyException("ZillionHash: " + "some job not finish");
					}
					if (workItem.schema == null) {
						continue;
					}
					result.insert_count += workItem.hash.insert_count;
					result.hash_code = HashUtil.merge_hash_code(result.hash_code, workItem.hash.hash_code);
				}
				container.DatabaseImpl.meta_query_update_data(namespace, tableName, result);
				byte[] hash = result.toBytes();
				container.ZK.setData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName, hash, -1);
			}

			for (ZillionTableIndex schema_index : schema.Indexes) {
				String indexName = schema_index.index_name;

				HashObject hashObjectInner = new HashObject();
				for (ZillionHashWorkerItem workItem : this.jobList) {
					if (!workItem.finish) {
						throw new MyException("ZillionHash: " + "some job not finish");
					}
					if (workItem.schema_index == null || !workItem.schema_index.index_name.equals(indexName)) {
						continue;
					}
					hashObjectInner.insert_count += workItem.hash.insert_count;
					hashObjectInner.hash_code = HashUtil.merge_hash_code(hashObjectInner.hash_code, workItem.hash.hash_code);
				}
				container.DatabaseImpl.meta_query_update_index(namespace, tableName, indexName, hashObjectInner);
				byte[] hash = hashObjectInner.toBytes();
				container.ZK.setData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName), hash, -1);
			}
		} else {
			String namespace = schema.Namespace;
			String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
			String tableName = schema.table_name;

			for (ZillionHashWorkerItem workItem : this.jobList) {
				if (!workItem.finish) {
					throw new MyException("ZillionHash: " + "some job not finish");
				}
				result.insert_count += workItem.hash.insert_count;
				result.hash_code = HashUtil.merge_hash_code(result.hash_code, workItem.hash.hash_code);
			}

			{
				container.DatabaseImpl.meta_query_update_data(namespace, tableName, result);
				byte[] hash = result.toBytes();
				container.ZK.setData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName, hash, -1);
			}
			if (this.work_type.equals("AddIndex") || this.work_type.equals("RepairIndex")) {
				for (int index_index = 0; index_index < schema_indexList.size(); index_index++) {
					ZillionTableIndex schema_index = schema_indexList.get(index_index);
					String indexName = schema_index.index_name;
					container.DatabaseImpl.meta_query_update_index(namespace, tableName, indexName, result);
					byte[] hash = result.toBytes();
					container.ZK.setData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName), hash, -1);
				}
			}
		}

		return result;
	}
}
