package com.zillion.database.agent;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;

import com.alibaba.fastjson.JSONObject;
import com.zillion.database.async.AsyncUtil;
import com.zillion.database.util.DatabaseImpl_HBase;
import com.zillion.database.util.DatabaseImpl_Mix;
import com.zillion.database.util.TableWrapper_HBase;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.common.HashObject;
import com.zillion.util.common.HashUtil;
import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.HBaseAdminUtil;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.table.SchemaWrapper;
import com.zillion.util.table.ZillionTableColumn;
import com.zillion.util.table.ZillionTableColumnEnum;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;

public class ZillionHashWorkerThread extends Thread {
	ZillionHashWorker worker;

	public ZillionHashWorkerThread(ZillionHashWorker worker) {
		this.worker = worker;
	}

	public void run() {
		while (true) {
			ZillionHashWorkerItem workItem = worker.Pop();
			if (workItem == null) {
				break;
			}

			try {
				if (worker.work_type.equals("RefreshHash")) {
					if (workItem.schema_index != null) {
						this.Process_index(worker.container, worker.schema, workItem.schema_index, workItem);
					} else {
						this.Process_table(worker.container, worker.schema, new ArrayList<ZillionTableIndex>(), workItem);
					}
				} else {
					this.Process_table(worker.container, worker.schema, worker.schema_indexList, workItem);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private void Process_table(SchemaRepositoryContainer container, ZillionTableSchema schema, List<ZillionTableIndex> schema_indexList,
			ZillionHashWorkerItem workItem) throws Exception {
		DatabaseImpl_HBase databaseImpl;
		if (container.DatabaseImpl instanceof DatabaseImpl_HBase) {
			databaseImpl = (DatabaseImpl_HBase) container.DatabaseImpl;
		} else {
			databaseImpl = ((DatabaseImpl_Mix) container.DatabaseImpl).hbase;
		}
		String namespace = schema.Namespace;
		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		String tableName = schema.table_name;
		SchemaWrapper schemaWrapper = workItem.schemaWrapper;
		if (worker.work_type.equals("RepairIndex")) {
			for (int index_index = 0; index_index < schema_indexList.size(); index_index++) {
				ZillionTableIndex schema_index = schema_indexList.get(index_index);
				if (schema.table_type.startsWith("split_")) {
					HBaseAdminUtil.clearTable(databaseImpl.admin, databaseImpl.admin_bak, TableName.valueOf(namespace,
							ZillionConstant.prefix_index + (schema.table_name + "." + schema_index.index_name) + "_" + schemaWrapper.child_name));
				} else {
					HBaseAdminUtil.clearTable(databaseImpl.admin, databaseImpl.admin_bak,
							TableName.valueOf(namespace, ZillionConstant.prefix_index + (schema.table_name + "." + schema_index.index_name)));
				}
			}
		}
		HashObject resultInner = new HashObject();
		long count_wait_tmp = 0L;
		Scan scan = new Scan();
		scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
		if (worker.work_type.equals("AddIndex") || worker.work_type.equals("RepairIndex")) {
			for (String column_name : schema.OtherColumn) {
				scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name));
			}
		}
		TableWrapper_HBase table_wrapper = (TableWrapper_HBase) schemaWrapper.dataWrapper;
		ResultScanner scanner = table_wrapper.htable.getScanner(scan);
		List<List<Put>> batchList = new ArrayList<List<Put>>();
		for (int index_index = 0; index_index < schema_indexList.size(); index_index++) {
			batchList.add(new ArrayList<Put>());
		}
		while (true) {
			Result res = scanner.next();
			if (res == null) {
				break;
			}

			if (!HTableUtil.valid(schema, res.listCells())) {
				continue;
			}

			byte[] row_key_bytes = res.getRow();
			resultInner.insert_count++;
			resultInner.hash_code = HashUtil.merge_hash_code(resultInner.hash_code, row_key_bytes);
			count_wait_tmp++;
			if (count_wait_tmp >= AsyncUtil.count_wait) {
				worker.asyncJob.Add(count_wait_tmp);
				count_wait_tmp = 0L;
			}

			if (worker.work_type.equals("AddIndex") || worker.work_type.equals("RepairIndex")) {
				JSONObject InsertObject = HTableUtil.Convert(schema, res.getRow(), res.listCells());
				Map<String, byte[]> columnBytesMap = new HashMap<String, byte[]>();
				Iterator<String> iter = schema.ColumnMap.keySet().iterator();
				while (iter.hasNext()) {
					String column_name = iter.next();
					ZillionTableColumn column = schema.ColumnMap.get(column_name);
					if (InsertObject.containsKey(column_name)) {
						Object columnObject = InsertObject.get(column_name);
						byte[] column_bytes = HTableUtil.GenerateBytes(column, columnObject);
						columnBytesMap.put(column_name, column_bytes);
					}
				}

				for (int index_index = 0; index_index < schema_indexList.size(); index_index++) {
					ZillionTableIndex schema_index = schema_indexList.get(index_index);
					TableWrapper_HBase table_index = null;
					for (int index_ = 0; index_ < schema.Indexes.size(); index_++) {
						ZillionTableIndex zti = schema.Indexes.get(index_);
						if (zti.index_name.equals(schema_index.index_name)) {
							table_index = (TableWrapper_HBase) schemaWrapper.indexWrapperList.get(index_);
							break;
						}
					}
					List<Put> batch = batchList.get(index_index);

					byte[] index_row_bytes = HTableUtil.BuildKey(schema.ColumnMap, schema_index.Key, columnBytesMap);

					Put put_index = new Put(index_row_bytes);
					put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"), ByteToObject.toBytes("1"));
					put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("u"), ByteToObject.toBytes("1"));
					if (schema.table_type == null) {
					} else if (schema.table_type.equals("index_data") || (schema.table_type.startsWith("split_"))) {
						for (int ii = 0; ii < schema.OtherColumn.length; ii++) {
							String column_name = schema.OtherColumn[ii];
							if (!schema_index.KeyMap.containsKey(column_name)) {
								if (columnBytesMap.containsKey(column_name)) {
									byte[] column_bytes = columnBytesMap.get(column_name);
									put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name), column_bytes);
								}
							}
						}
					} else if (schema.table_type.equals("all_column")) {
						String[] Columns = schema.ColumnMap.keySet().toArray(new String[0]);
						for (int ii = 0; ii < Columns.length; ii++) {
							String column_name = Columns[ii];
							if (columnBytesMap.containsKey(column_name)) {
								byte[] column_bytes = columnBytesMap.get(column_name);
								put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name), column_bytes);
							}
						}
					}
					batch.add(put_index);
					if (batch.size() > 1000) {
						table_index.htable.put(batch);
						if (ZillionConstant.disable_AutoFlush) {
							table_index.htable.flushCommits();
						}
						batch.clear();
					}
				}
			}
		}
		if (worker.work_type.equals("AddIndex") || worker.work_type.equals("RepairIndex")) {
			for (int index_index = 0; index_index < schema_indexList.size(); index_index++) {
				ZillionTableIndex schema_index = schema_indexList.get(index_index);
				TableWrapper_HBase table_index = null;
				for (int index_ = 0; index_ < schema.Indexes.size(); index_++) {
					ZillionTableIndex zti = schema.Indexes.get(index_);
					if (zti.index_name.equals(schema_index.index_name)) {
						table_index = (TableWrapper_HBase) schemaWrapper.indexWrapperList.get(index_);
						break;
					}
				}
				List<Put> batch = batchList.get(index_index);

				if (batch.size() > 0) {
					table_index.htable.put(batch);
					if (ZillionConstant.disable_AutoFlush) {
						table_index.htable.flushCommits();
					}
					batch.clear();
				}
			}
		}
		scanner.close();
		try {
			Thread.sleep(1000L * 5);
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (count_wait_tmp > 0) {
			worker.asyncJob.Add(count_wait_tmp);
			count_wait_tmp = 0L;
		}
		{
			if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
				container.DatabaseImpl.meta_query_update_split_data(namespace, tableName, schemaWrapper.child_name, resultInner);
				byte[] hash = resultInner.toBytes();
				container.ZK.setData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName + "/" + schemaWrapper.child_name, hash, -1);
			}
		}
		if (worker.work_type.equals("AddIndex") || worker.work_type.equals("RepairIndex")) {
			for (int index_index = 0; index_index < schema_indexList.size(); index_index++) {
				ZillionTableIndex schema_index = schema_indexList.get(index_index);
				String indexName = schema_index.index_name;
				if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
					container.DatabaseImpl.meta_query_update_split_index(namespace, tableName, indexName, schemaWrapper.child_name, resultInner);
					byte[] hash = resultInner.toBytes();
					container.ZK.setData(
							zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName) + "/" + schemaWrapper.child_name,
							hash, -1);
				}
			}
		}

		workItem.hash = resultInner;
		workItem.finish = true;
	}

	private void Process_index(SchemaRepositoryContainer container, ZillionTableSchema schema, ZillionTableIndex schema_index,
			ZillionHashWorkerItem workItem) throws Exception {
		String namespace = schema.Namespace;
		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		String tableName = schema.table_name;
		String indexName = schema_index.index_name;
		SchemaWrapper schemaWrapper = workItem.schemaWrapper;
		TableWrapper_HBase table_index = null;
		for (int index_ = 0; index_ < schema.Indexes.size(); index_++) {
			ZillionTableIndex index = schema.Indexes.get(index_);
			TableWrapper_HBase indexWrapper = (TableWrapper_HBase) schemaWrapper.indexWrapperList.get(index_);
			if (index.index_name.equals(schema_index.index_name)) {
				table_index = indexWrapper;
				break;
			}
		}

		HashObject resultInner = new HashObject();

		Scan scan = new Scan();
		scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
		ResultScanner scanner = table_index.htable.getScanner(scan);
		while (true) {
			Result res = scanner.next();
			if (res == null) {
				break;
			}

			if (!HTableUtil.valid(schema, res.listCells())) {
				continue;
			}

			byte[] row_bytes_index = res.getRow();
			byte[] row_bytes;
			if (schema.table_type == null) {
				int offset = 0;
				for (int index_column = 0; index_column < schema_index.Key.length; index_column++) {
					String column_name = schema_index.Key[index_column];
					ZillionTableColumn ZillionTableColumn = schema.ColumnMap.get(column_name);
					if (row_bytes_index[offset] == 0x00) {
						offset++;
					} else {
						if (ZillionTableColumn.Type == ZillionTableColumnEnum.STRING) {
							offset++;
							while (row_bytes_index[offset] != 0x00) {
								offset++;
							}
							offset++;
						} else {
							offset++;
							offset += ByteToObject.lengthOfColumnEnum(ZillionTableColumn.Type);
						}
					}
				}
				row_bytes = new byte[row_bytes_index.length - offset];
				System.arraycopy(row_bytes_index, offset, row_bytes, 0, row_bytes.length);
			} else {
				Map<String, byte[]> key_bytesMap = new HashMap<String, byte[]>();
				int key_bytes_length = 0;
				int offset = 0;
				for (String column : schema_index.Key) {
					ZillionTableColumn ZillionTableColumn = schema.ColumnMap.get(column);
					int column_offset = offset;
					if (row_bytes_index[offset] == 0x00) {
						offset++;
					} else {
						if (ZillionTableColumn.Type == ZillionTableColumnEnum.STRING) {
							offset++;
							while (row_bytes_index[offset] != 0x00) {
								offset++;
							}
							offset++;
						} else {
							offset++;
							offset += ByteToObject.lengthOfColumnEnum(ZillionTableColumn.Type);
						}
					}
					int column_offset_next = offset;
					if (schema.KeyMap.containsKey(column)) {
						byte[] value = new byte[column_offset_next - column_offset];
						System.arraycopy(row_bytes_index, column_offset, value, 0, value.length);
						key_bytesMap.put(column, value);
						key_bytes_length += value.length;
					}
				}

				row_bytes = new byte[key_bytes_length];
				int copy_length = 0;
				for (String keyInner : schema.Key) {
					byte[] column_bytes = key_bytesMap.get(keyInner);
					System.arraycopy(column_bytes, 0, row_bytes, copy_length, column_bytes.length);
					copy_length += column_bytes.length;
				}
			}

			resultInner.insert_count++;
			resultInner.hash_code = HashUtil.merge_hash_code(resultInner.hash_code, row_bytes);
		}
		scanner.close();

		if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
			container.DatabaseImpl.meta_query_update_split_index(namespace, tableName, indexName, schemaWrapper.child_name, resultInner);
			byte[] hash = resultInner.toBytes();
			container.ZK.setData(
					zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName) + "/" + schemaWrapper.child_name, hash,
					-1);
		}

		workItem.hash = resultInner;
		workItem.finish = true;
	}
}
