package com.zillion.database.agent;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.hadoop.hbase.Cell;
import org.apache.hadoop.hbase.CellUtil;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter.RowRange;
import org.apache.hadoop.hbase.util.Bytes;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.util.DatabaseImpl_HBase;
import com.zillion.database.util.DatabaseImpl_Mix;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.common.SplitUtil;
import com.zillion.util.common.SplitWrapper;
import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;

public class ZillionMonitorUtil {
	public static void process(ZillionAgent agent, JSONObject queryJSON, JSONObject result) throws Exception {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
		String QueryType = (String) queryJSON.get("QueryType");

		DatabaseImpl_HBase impl = null;
		if (agent.container.DatabaseImpl instanceof DatabaseImpl_HBase) {
			impl = (DatabaseImpl_HBase) agent.container.DatabaseImpl;
		} else if (agent.container.DatabaseImpl instanceof DatabaseImpl_Mix) {
			impl = ((DatabaseImpl_Mix) agent.container.DatabaseImpl).hbase;
		}
		if (impl == null) {
			result.put("Reason", "monitor can't support MySQL");
			return;
		}

		String Database = (String) queryJSON.get("Database");
		String TimeFrom = (String) queryJSON.get("TimeFrom");
		String TimeTo = (String) queryJSON.get("TimeTo");
		String Caliber = (String) queryJSON.get("Caliber");
		Date time_from = sdf.parse(TimeFrom);
		Date time_to = sdf.parse(TimeTo);
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(time_from);
		calendar.set(Calendar.DAY_OF_MONTH, 1);
		calendar.set(Calendar.HOUR_OF_DAY, 0);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		List<String> monthList = new ArrayList<String>();
		Date tmp_time = calendar.getTime();
		while (tmp_time.getTime() < time_to.getTime()) {
			monthList.add(sdf.format(tmp_time).substring(0, 6));
			calendar.setTime(tmp_time);
			calendar.add(Calendar.MONTH, 1);
			tmp_time = calendar.getTime();
		}

		if (QueryType.equals("monitor_sum_curve") || QueryType.equals("monitor_table_total")) {
			stat_2(agent.container, impl, QueryType, Caliber, Database, TimeFrom, TimeTo, monthList, result);
		} else if (QueryType.equals("monitor_table_curve")) {
			String Datatable = (String) queryJSON.get("Datatable");
			stat_1(agent.container, impl, QueryType, Caliber, Database, Datatable, TimeFrom, TimeTo, monthList, result);
		}

		result.remove("Result");
		result.put("Result", "success");
	}

	private static void stat_1(SchemaRepositoryContainer container, DatabaseImpl_HBase impl, String QueryType, String Caliber, String Database,
			String Datatable, String TimeFrom, String TimeTo, List<String> monthList, JSONObject result) throws Exception {
		JSONObject content = new JSONObject();

		int index_out = Caliber.indexOf('_');
		String c1 = Caliber.substring(0, index_out);
		String c2 = Caliber.substring(index_out + 1);
		List<String> columnList = new ArrayList<String>();
		if (c2.equals("lines") || c2.equals("bytes")) {
			List<String> c1List = new ArrayList<String>();
			if (c1.equals("total")) {
				c1List.add("read");
				c1List.add("insert");
				c1List.add("update");
				c1List.add("delete");
			} else if (c1.equals("write")) {
				c1List.add("insert");
				c1List.add("update");
				c1List.add("delete");
			} else if (c1.equals("read")) {
				c1List.add("read");
			} else {
				columnList.add(Caliber);
			}
			for (String cc : c1List) {
				columnList.add(cc + "_" + c2);
			}
		} else {
			columnList.add(Caliber);
		}

		ZillionTableSchema schema = container.repositoryMap.get(Database).GetTable(Datatable);
		String tableName = Datatable;
		String query_month = null;
		if (schema == null) {
			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
			int index_ = Datatable.lastIndexOf("_");
			if (index_ != -1) {
				String prefix = Datatable.substring(0, index_);
				String suffix = Datatable.substring(index_ + 1);
				try {
					boolean is_split_table = false;
					SplitWrapper sw = new SplitWrapper(tableName, is_split_table, query_month, schema);
					SplitUtil.SplitParse(sdf, container.repositoryMap.get(Database), prefix, suffix, sw);
					tableName = sw.tableName;
					is_split_table = sw.is_split_table;
					query_month = sw.query_month;
					schema = sw.schema;
				} catch (ParseException e) {
					throw e;
				}
			}
		}

		if (query_month != null) {
			String child_name = query_month;
			JSONObject child_item = content;
			JSONArray split_curve = new JSONArray();
			child_item.put("total_curve", split_curve);
			{
				String table_real = ZillionConstant.prefix_data + tableName + "_" + child_name;
				JSONArray data_curve = null;
				if (schema.Indexes.size() > 0) {
					data_curve = new JSONArray();
					child_item.put("data_curve", data_curve);
				}
				stat_1_add(container, impl, Database, table_real, TimeFrom, TimeTo, monthList, columnList, null, split_curve, data_curve);
			}
			for (ZillionTableIndex schema_index : schema.Indexes) {
				String indexName = schema_index.index_name;
				String table_real = ZillionConstant.prefix_index + tableName + "." + indexName + "_" + child_name;
				JSONArray index_curve = new JSONArray();
				child_item.put("index_" + indexName + "_curve", index_curve);
				stat_1_add(container, impl, Database, table_real, TimeFrom, TimeTo, monthList, columnList, null, split_curve, index_curve);
			}
		} else {
			JSONArray total_curve = new JSONArray();
			content.put("total_curve", total_curve);
			if (schema.table_type != null && schema.table_type.startsWith("split_")) {
				String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + Database;
				List<String> child_nameList = container.ZK.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name,
						false);
				Collections.sort(child_nameList);
				JSONArray splitArray = new JSONArray();
				content.put("splitArray", splitArray);
				for (String child_name : child_nameList) {
					JSONObject child_item = new JSONObject();
					child_item.put("split", child_name);
					splitArray.add(child_item);
					JSONArray split_curve = new JSONArray();
					child_item.put("total_curve", split_curve);
					{
						String table_real = ZillionConstant.prefix_data + Datatable + "_" + child_name;
						JSONArray data_curve = null;
						if (schema.Indexes.size() > 0) {
							data_curve = new JSONArray();
							child_item.put("data_curve", data_curve);
						}
						stat_1_add(container, impl, Database, table_real, TimeFrom, TimeTo, monthList, columnList, total_curve, split_curve,
								data_curve);
					}
					for (ZillionTableIndex schema_index : schema.Indexes) {
						String indexName = schema_index.index_name;
						String table_real = ZillionConstant.prefix_index + Datatable + "." + indexName + "_" + child_name;
						JSONArray index_curve = new JSONArray();
						child_item.put("index_" + indexName + "_curve", index_curve);
						stat_1_add(container, impl, Database, table_real, TimeFrom, TimeTo, monthList, columnList, total_curve, split_curve,
								index_curve);
					}
				}

			} else {
				{
					String table_real = ZillionConstant.prefix_data + Datatable;
					JSONArray data_curve = null;
					if (schema.Indexes.size() > 0) {
						data_curve = new JSONArray();
						content.put("data_curve", data_curve);
					}
					stat_1_add(container, impl, Database, table_real, TimeFrom, TimeTo, monthList, columnList, total_curve, null, data_curve);
				}
				for (ZillionTableIndex schema_index : schema.Indexes) {
					String indexName = schema_index.index_name;
					String table_real = ZillionConstant.prefix_index + Datatable + "." + indexName;
					JSONArray index_curve = new JSONArray();
					content.put("index_" + indexName + "_curve", index_curve);
					stat_1_add(container, impl, Database, table_real, TimeFrom, TimeTo, monthList, columnList, total_curve, null, index_curve);
				}
			}
		}
		result.put("Content", content);
	}

	private static void stat_1_add(SchemaRepositoryContainer container, DatabaseImpl_HBase impl, String Database, String table_real, String TimeFrom,
			String TimeTo, List<String> monthList, List<String> columnList, JSONArray curve1, JSONArray curve2, JSONArray curve3) throws Exception {
		List<JSONArray> curveList = new ArrayList<JSONArray>();
		if (curve1 != null) {
			curveList.add(curve1);
		}
		if (curve2 != null) {
			curveList.add(curve2);
		}
		if (curve3 != null) {
			curveList.add(curve3);
		}
		for (String month : monthList) {
			TableName tableName = TableName.valueOf(Database, "zillion_meta_stat_1" + "_" + month);
			if (!impl.admin.tableExists(tableName)) {
				continue;
			}

			HTable table = (HTable) impl.connection.getTable(tableName);
			Scan scan = new Scan();
			for (String column : columnList) {
				scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column));
			}
			List<RowRange> rangeList = new ArrayList<RowRange>();
			rangeList.add(new RowRange(Bytes.toBytes(table_real + "," + TimeFrom), false, Bytes.toBytes(table_real + "," + TimeTo), false));
			MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
			scan.setFilter(filter);
			ResultScanner scanner = table.getScanner(scan);
			while (true) {
				Result res = scanner.next();
				if (res == null) {
					break;
				}

				String rowkey = Bytes.toString(res.getRow());
				int index_r1 = rowkey.indexOf(',');
				int index_r2 = rowkey.indexOf(',', index_r1 + 1);
				String time = rowkey.substring(index_r1 + 1, index_r2);

				List<Cell> cellList = res.listCells();
				long row_long_value = 0L;
				for (Cell cell : cellList) {
					byte[] cell_bytes = CellUtil.cloneValue(cell);
					row_long_value += Bytes.toLong(cell_bytes);
				}

				for (JSONArray curve : curveList) {
					int insert_index = FindInsertIndex(curve, time);
					JSONObject last_item = null;
					if (insert_index < 0) {
						last_item = (JSONObject) curve.get(-250 - insert_index);
					}
					if (last_item == null) {
						last_item = new JSONObject();
						last_item.put("time", time);
						last_item.put("count", row_long_value);
						curve.add(insert_index, last_item);
					} else {
						long count = (Long) last_item.get("count");
						last_item.remove("count");
						last_item.put("count", count + row_long_value);
					}
				}
			}
			scanner.close();
		}
	}

	public static int FindInsertIndex(JSONArray dataList, String time) {
		int beginIndex = 0;
		int endIndex = dataList.size() - 1;
		boolean lastMoveLeft = true;
		boolean find = false;
		int find_index = -1;
		while (beginIndex <= endIndex) {
			int tempIndex = (beginIndex + endIndex) / 2;

			JSONObject tmpData = (JSONObject) dataList.get(tempIndex);
			String tmpDate = (String) tmpData.get("time");

			int compareResult = tmpDate.compareTo(time);
			if (compareResult == 0) {
				find = true;
				find_index = tempIndex;
				break;
			} else if (compareResult < 0) {
				beginIndex = tempIndex + 1;
				lastMoveLeft = false;
			} else {
				endIndex = tempIndex - 1;
				lastMoveLeft = true;
			}
		}

		int insertIndex = 0;
		if (!find) {
			if (lastMoveLeft) {
				// endIndex+1(tempIndex)的数值大于searchDate
				insertIndex = endIndex + 1;
			} else {
				// beginIndex-1(tempIndex)的数值小于searchDate
				insertIndex = beginIndex;
			}
		} else {
			insertIndex = -250 - find_index;
		}
		return insertIndex;
	}

	private static void stat_2(SchemaRepositoryContainer container, DatabaseImpl_HBase impl, String QueryType, String Caliber, String Database,
			String TimeFrom, String TimeTo, List<String> monthList, JSONObject result) throws Exception {
		JSONArray content = new JSONArray();

		int index_ = Caliber.indexOf('_');
		String c1 = Caliber.substring(0, index_);
		String c2 = Caliber.substring(index_ + 1);
		List<String> columnList = new ArrayList<String>();
		if (c2.equals("lines") || c2.equals("bytes")) {
			List<String> c1List = new ArrayList<String>();
			if (c1.equals("total")) {
				c1List.add("read");
				c1List.add("insert");
				c1List.add("update");
				c1List.add("delete");
			} else if (c1.equals("write")) {
				c1List.add("insert");
				c1List.add("update");
				c1List.add("delete");
			} else if (c1.equals("read")) {
				c1List.add("read");
			} else {
				columnList.add(Caliber);
			}
			for (String cc : c1List) {
				columnList.add(cc + "_" + c2);
			}
		} else {
			columnList.add(Caliber);
		}

		JSONObject last_item = null;
		Map<String, JSONObject> tableMap = new HashMap<String, JSONObject>();
		for (String month : monthList) {
			TableName tableName = TableName.valueOf(Database, "zillion_meta_stat_2" + "_" + month);
			if (!impl.admin.tableExists(tableName)) {
				continue;
			}

			HTable table = (HTable) impl.connection.getTable(tableName);
			Scan scan = new Scan();
			for (String column : columnList) {
				scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column));
			}
			List<RowRange> rangeList = new ArrayList<RowRange>();
			rangeList.add(new RowRange(Bytes.toBytes(TimeFrom), false, Bytes.toBytes(TimeTo), false));
			MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
			scan.setFilter(filter);
			ResultScanner scanner = table.getScanner(scan);
			while (true) {
				Result res = scanner.next();
				if (res == null) {
					break;
				}

				String rowkey = Bytes.toString(res.getRow());
				int index_r1 = rowkey.indexOf(',');
				int index_r2 = rowkey.indexOf(',', index_r1 + 1);
				String time = rowkey.substring(0, index_r1);
				String table_real = rowkey.substring(index_r1 + 1, index_r2);
				String table_name = null;
				String index_name = null;
				String split = null;
				ZillionTableSchema schema = null;
				if (table_real.startsWith(ZillionConstant.prefix_data)) {
					String name2 = table_real.substring(ZillionConstant.prefix_data.length());
					schema = container.repositoryMap.get(Database).GetTable(name2);
					if (schema != null) {
						table_name = name2;
					} else {
						int indexInner = name2.lastIndexOf('_');
						String main_table = name2.substring(0, indexInner);
						String suffix = name2.substring(indexInner + 1);
						schema = container.repositoryMap.get(Database).GetTable(main_table);
						if (schema != null && schema.table_type != null && schema.table_type.startsWith("split_")) {
							table_name = main_table;
							split = suffix;
						}
					}
				} else {
					String name2 = table_real.substring(ZillionConstant.prefix_index.length());
					int index_dot = name2.indexOf('.');
					String table_nameInner = name2.substring(0, index_dot);
					String index_split = name2.substring(index_dot + 1);
					schema = container.repositoryMap.get(Database).GetTable(table_nameInner);
					if (schema != null) {
						table_name = table_nameInner;
						if (schema.table_type != null && schema.table_type.startsWith("split_")) {
							int index_2 = index_split.lastIndexOf('_');
							index_name = index_split.substring(0, index_2);
							split = index_split.substring(index_2 + 1);
						} else {
							index_name = index_split;
						}
					}
				}
				if (table_name == null) {
					continue;
				}

				List<Cell> cellList = res.listCells();
				long row_long_value = 0L;
				for (Cell cell : cellList) {
					byte[] cell_bytes = CellUtil.cloneValue(cell);
					row_long_value += Bytes.toLong(cell_bytes);
				}
				if (QueryType.equals("monitor_sum_curve")) {
					boolean time_change = false;
					if (last_item != null) {
						String time_last = (String) last_item.get("time");
						if (!time_last.equals(time)) {
							time_change = true;
						}
					}
					if (last_item == null || time_change) {
						last_item = new JSONObject();
						last_item.put("time", time);
						last_item.put("count", row_long_value);
						content.add(last_item);
					} else {
						long count = (Long) last_item.get("count");
						last_item.remove("count");
						last_item.put("count", count + row_long_value);
					}
				} else if (QueryType.equals("monitor_table_total")) {
					add2Map(tableMap, table_name, split, index_name, row_long_value);
				}
			}
			scanner.close();
		}
		if (QueryType.equals("monitor_table_total")) {
			String[] table_nameArray = tableMap.keySet().toArray(new String[0]);
			Arrays.sort(table_nameArray);
			for (String table_name : table_nameArray) {
				content.add(tableMap.get(table_name));
			}
		}
		result.put("Content", content);
	}

	private static void add2Map(Map<String, JSONObject> tableMap, String table_name, String split, String index_name, long row_long_value) {
		JSONObject table_item;
		if (!tableMap.containsKey(table_name)) {
			table_item = new JSONObject();
			table_item.put("table_name", table_name);
			tableMap.put(table_name, table_item);
		} else {
			table_item = tableMap.get(table_name);
		}

		if (split != null) {
			if (!table_item.containsKey("splits")) {
				table_item.put("splits", new JSONObject());
			}
			JSONObject splits = (JSONObject) table_item.get("splits");
			JSONObject split_item;
			if (!splits.containsKey(split)) {
				split_item = new JSONObject();
				splits.put(split, split_item);
			} else {
				split_item = (JSONObject) splits.get(split);
			}
			String key;
			if (index_name != null) {
				key = "index_" + index_name;
			} else {
				key = "data";
			}
			add2key(split_item, key, row_long_value);
			add2key(split_item, "total", row_long_value);
			add2key(table_item, key, row_long_value);
			add2key(table_item, "total", row_long_value);
		} else {
			String key;
			if (index_name != null) {
				key = "index_" + index_name;
			} else {
				key = "data";
			}
			add2key(table_item, key, row_long_value);
			add2key(table_item, "total", row_long_value);
		}
	}

	private static void add2key(JSONObject item, String key, long row_long_value) {
		if (item.containsKey(key)) {
			long exist_count = (Long) item.get(key);
			item.remove(key);
			item.put(key, exist_count + row_long_value);
		} else {
			item.put(key, row_long_value);
		}
	}
}
