package com.zillion.database.agent;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.hadoop.hbase.Cell;
import org.zillion.util.common.HexToByte;
import org.zillion.util.common.MyException;
import org.zillion.util.json.FastJsonUtil;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.ZillionTableColumn;
import com.zillion.util.table.ZillionTableColumnEnum;
import com.zillion.util.table.ZillionTableSchema;

public class ZillionQueryCheckUtil {
	// insertgetremoveظĶ
	public static void Check_batch(ZillionTableSchema schema, JSONArray InsertObjects) throws Exception {
		Map<String, Integer> keyMap = new HashMap<String, Integer>();
		for (int i = 0; i < InsertObjects.size(); i++) {
			JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
			byte[] row_key_bytes = HTableUtil.BuildKey(schema, InsertObject);
			String hexString = HexToByte.byteToHex(row_key_bytes);
			int exist_count = 0;
			if (keyMap.containsKey(hexString)) {
				exist_count = keyMap.get(hexString);
				keyMap.remove(hexString);
			}
			keyMap.put(hexString, exist_count + 1);
		}
		StringBuffer sb = new StringBuffer();
		Iterator<String> keyIter = keyMap.keySet().iterator();
		while (keyIter.hasNext()) {
			String key = keyIter.next();
			int count = keyMap.get(key);
			if (count > 1) {
				if (sb.length() > 0) {
					sb.append("\n");
				}
				JSONObject json = HTableUtil.Convert(schema, HexToByte.hexToByte(key), new ArrayList<Cell>());
				sb.append("this object appear " + count + " times:" + "\n");
				sb.append(FastJsonUtil.toFormatString(json));
			}
		}
		if (sb.length() > 0) {
			throw new MyException("ZillionQueryCheck: Key exist many times" + "\n" + sb.toString());
		}
	}

	public static void CheckInsert(ZillionTableSchema schema, JSONObject InsertObject) throws Exception {
		Iterator<String> iter = schema.ColumnMap.keySet().iterator();
		while (iter.hasNext()) {
			String Name = iter.next();
			ZillionTableColumn ztc = schema.ColumnMap.get(Name);
			Object columnValue = InsertObject.get(Name);
			CheckColumn(ztc, columnValue);
		}
	}

	public static void CheckKey(ZillionTableSchema schema, JSONObject Key) throws Exception {
		for (int i = 0; i < schema.Key.length; i++) {
			String Name = schema.Key[i];
			ZillionTableColumn ztc = schema.ColumnMap.get(Name);
			Object columnValue = Key.get(Name);
			CheckColumn(ztc, columnValue);
		}
	}

	public static void CheckCriteria(ZillionTableSchema schema, JSONObject Criteria) throws Exception {
		Iterator<String> iter = Criteria.keySet().iterator();
		while (iter.hasNext()) {
			String Name = iter.next();
			if (!schema.ColumnMap.containsKey(Name)) {
				throw new MyException("ZillionQueryCheck: Column not exist: " + Name);
			}
			ZillionTableColumn ztc = schema.ColumnMap.get(Name);
			if (ztc.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
				throw new MyException("ZillionQueryCheck: Criteria can't use CompressString: " + Name);
			}
			Object columnValue = Criteria.get(Name);
			if (columnValue instanceof JSONObject) {
				JSONObject columnValueJSON = (JSONObject) columnValue;
				if (columnValueJSON.containsKey("in")) {
					Object inObject = columnValueJSON.get("in");
					if (inObject instanceof JSONObject) {
					} else if (inObject instanceof JSONArray) {
						JSONArray inValueArray = (JSONArray) inObject;
						for (int i = 0; i < inValueArray.size(); i++) {
							Object columnValueInner = inValueArray.get(i);
							if (columnValueInner instanceof JSONArray || columnValueInner instanceof JSONObject) {
							} else {
								CheckColumn(ztc, columnValueInner);
							}
						}
					} else {
						throw new MyException("ZillionQueryCheck: in " + inObject.getClass().getName());
					}
				} else if (columnValueJSON.containsKey("$regex")) {
					String regex = (String) columnValueJSON.get("$regex");
					CheckColumn(ztc, regex);
				} else {
					Iterator<String> keyIter = columnValueJSON.keySet().iterator();
					while (keyIter.hasNext()) {
						String key = keyIter.next();
						Object columnValueInner = columnValueJSON.get(key);
						CheckColumn(ztc, columnValueInner);
					}
				}
			} else {
				CheckColumn(ztc, columnValue);
			}
		}
	}

	public static void CheckSet(ZillionTableSchema schema, JSONObject Set) throws Exception {
		Iterator<String> iter = Set.keySet().iterator();
		while (iter.hasNext()) {
			String Name = iter.next();
			if (!schema.ColumnMap.containsKey(Name)) {
				throw new MyException("ZillionQueryCheck: Column not exist: " + Name);
			}
			ZillionTableColumn ztc = schema.ColumnMap.get(Name);
			Object columnValue = Set.get(Name);
			CheckColumn(ztc, columnValue);
		}
	}

	private static void CheckColumn(ZillionTableColumn ztc, Object columnValue) throws Exception {
		if (ztc.NotNull && columnValue == null) {
			throw new MyException("ZillionQueryCheck: " + ztc.Name + "'s value cant be null ");
		}
		if (columnValue == null) {
			return;
		}

		if (ztc.Type == ZillionTableColumnEnum.STRING || ztc.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
			if (columnValue instanceof String) {
				String stringValue = (String) columnValue;
				if (stringValue.length() > ztc.max_length) {
					throw new MyException("ZillionQueryCheck: " + ztc.Name + " length " + stringValue.length() + " > " + ztc.max_length);
				}
			} else {
				throw new MyException("ZillionQueryCheck: " + ztc.Name + " value type error, value must be String");
			}
		} else {
			if (!HTableUtil.Can_Convert(ztc, columnValue)) {
				throw new MyException("ZillionQueryCheck: " + ztc.Name + " value type error");
			}
		}
	}
}
