package com.zillion.database.agent;

import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.zillion.util.common.ListUtil;
import org.zillion.util.common.MyException;
import org.zillion.util.json.FastJsonCompareUtil;
import org.zillion.util.json.FastJsonUtil;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.async.AsyncJob;
import com.zillion.database.second.SecondCriteria;
import com.zillion.database.util.ITableWrapper;
import com.zillion.util.common.JSONObjectComparator;
import com.zillion.util.common.SplitUtil;
import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.table.SchemaWrapper;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;

public class ZillionQueryUtil {
	public static JSONObject getData(SchemaRepositoryContainer container, ZillionTableSchema schema, JSONObject InsertObject, JSONArray content,
			JSONArray ReturnColumns) throws Exception {
		JSONObject result = null;
		JSONArray InsertObjectsInner = new JSONArray();
		InsertObjectsInner.add(InsertObject);
		JSONArray InsertObjects_realInner;
		if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
			String split_value = (String) InsertObject.get(schema.split_column);
			String child_name = split_value.substring(0, SplitUtil.split_type2child_length.get(schema.table_type));
			InsertObjects_realInner = getDatasInner(container, schema, child_name, InsertObjectsInner, content, ReturnColumns);
		} else {
			InsertObjects_realInner = getDatasInner(container, schema, null, InsertObjectsInner, content, ReturnColumns);
		}
		if (InsertObjects_realInner.size() > 0) {
			result = (JSONObject) InsertObjects_realInner.get(0);
		}
		return result;
	}

	public static JSONArray getDatas(SchemaRepositoryContainer container, ZillionTableSchema schema, JSONArray InsertObjects, JSONArray content,
			JSONArray ReturnColumns) throws Exception {
		if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
			Map<String, JSONArray> InsertObjectsMap = new HashMap<String, JSONArray>();
			for (int i = 0; i < InsertObjects.size(); i++) {
				JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
				String split_value = (String) InsertObject.get(schema.split_column);
				String child_name = split_value.substring(0, SplitUtil.split_type2child_length.get(schema.table_type));
				if (!InsertObjectsMap.containsKey(child_name)) {
					InsertObjectsMap.put(child_name, new JSONArray());
				}
				InsertObjectsMap.get(child_name).add(InsertObject);
			}
			JSONArray InsertObjects_real = new JSONArray();
			String[] child_nameArray = InsertObjectsMap.keySet().toArray(new String[0]);
			Arrays.sort(child_nameArray);
			for (String child_name : child_nameArray) {
				JSONArray InsertObjectsInner = InsertObjectsMap.get(child_name);
				JSONArray InsertObjects_realInner = getDatasInner(container, schema, child_name, InsertObjectsInner, content, ReturnColumns);
				InsertObjects_real.addAll(InsertObjects_realInner);
			}
			return InsertObjects_real;
		} else {
			JSONArray InsertObjects_real = getDatasInner(container, schema, null, InsertObjects, content, ReturnColumns);
			return InsertObjects_real;
		}
	}

	private static JSONArray getDatasInner(SchemaRepositoryContainer container, ZillionTableSchema schema, String child_name, JSONArray InsertObjects,
			JSONArray content, JSONArray ReturnColumns) throws Exception {
		Map<String, Boolean> ReturnColumnsMap = new HashMap<String, Boolean>();
		if (ReturnColumns == null) {
			Iterator<String> keyIter = schema.ColumnMap.keySet().iterator();
			while (keyIter.hasNext()) {
				String key = keyIter.next();
				ReturnColumnsMap.put(key, true);
			}
		} else {
			for (int index_rcs = 0; index_rcs < ReturnColumns.size(); index_rcs++) {
				String key = (String) ReturnColumns.get(index_rcs);
				ReturnColumnsMap.put(key, true);
			}
		}
		// NotNullStringmax_length
		if (ZillionConstant.check_query) {
			for (int i = 0; i < InsertObjects.size(); i++) {
				JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
				ZillionQueryCheckUtil.CheckKey(schema, InsertObject);
			}
		}

		JSONArray InsertObjects_real;
		ITableWrapper table_wrapper = schema.connect(container, child_name, false);
		if (table_wrapper != null) {
			InsertObjects_real = container.DatabaseImpl.batch_get(schema, table_wrapper, InsertObjects, content, ReturnColumnsMap);
			table_wrapper.unuse();
		} else {
			InsertObjects_real = InsertObjects;
		}

		return InsertObjects_real;
	}

	public static List<String> get_all_splits(ZillionTableSchema schema) throws Exception {
		List<String> result = new ArrayList<String>();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
		Date date_from = sdf.parse("20000101");
		Date date_to = sdf.parse("20310101");
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date_from);
		while (calendar.getTime().getTime() < date_to.getTime()) {
			String str = sdf.format(calendar.getTime());
			if (schema.table_type.equals("split_hour")) {
				result.add(str.substring(0, SplitUtil.split_type2child_length.get(schema.table_type)));
				calendar.add(Calendar.HOUR_OF_DAY, 1);
			} else if (schema.table_type.equals("split_day")) {
				result.add(str.substring(0, SplitUtil.split_type2child_length.get(schema.table_type)));
				calendar.add(Calendar.DAY_OF_MONTH, 1);
			} else if (schema.table_type.equals("split_month")) {
				result.add(str.substring(0, SplitUtil.split_type2child_length.get(schema.table_type)));
				calendar.add(Calendar.MONTH, 1);
			} else {
				result.add(str.substring(0, SplitUtil.split_type2child_length.get(schema.table_type)));
				calendar.add(Calendar.YEAR, 1);
			}
		}
		return result;
	}

	private static List<String> get_splits(ZillionTableSchema schema, List<String> child_nameList, List<JSONObject> CriteriaList) throws MyException {
		List<String> child_nameNeed = new ArrayList<String>();
		String suffix;
		if (schema.table_type.equals("split_hour")) {
			suffix = "0000";
		} else if (schema.table_type.equals("split_day")) {
			suffix = "000000";
		} else if (schema.table_type.equals("split_month")) {
			suffix = "01000000";
		} else if (schema.table_type.equals("split_year")) {
			suffix = "0101000000";
		} else {
			throw new MyException("get_splits error table_type:" + schema.table_type);
		}
		for (JSONObject CriteriaInner : CriteriaList) {
			if (CriteriaInner.containsKey(schema.split_column)) {
				Object split_valueObject = CriteriaInner.get(schema.split_column);
				if (split_valueObject == null) {
					throw new MyException("ZillionAgent: " + "split_column:" + schema.split_column + " can't be null");
				} else if (split_valueObject instanceof String) {
					String split_value = (String) split_valueObject;
					String child_name = split_value.substring(0, SplitUtil.split_type2child_length.get(schema.table_type));
					ListUtil.Merge(child_nameNeed, child_name);
				} else if (split_valueObject instanceof JSONObject) {
					JSONObject split_value = (JSONObject) split_valueObject;
					List<String> child_nameListInner = new ArrayList<String>();
					child_nameListInner.addAll(child_nameList);
					if (split_value.containsKey("$gt")) {
						String value = (String) split_value.get("$gt");
						String child_name = value;
						int child_length = SplitUtil.split_type2child_length.get(schema.table_type);
						if (value.length() > child_length) {
							child_name = value.substring(0, child_length);
						}
						for (int i = child_nameListInner.size() - 1; i >= 0; i--) {
							String month = child_nameListInner.get(i);
							if (month.compareTo(child_name) < 0) {
								child_nameListInner.remove(i);
							}
						}
					}
					if (split_value.containsKey("$gte")) {
						String value = (String) split_value.get("$gte");
						String child_name = value;
						int child_length = SplitUtil.split_type2child_length.get(schema.table_type);
						if (value.length() > child_length) {
							child_name = value.substring(0, child_length);
						}
						for (int i = child_nameListInner.size() - 1; i >= 0; i--) {
							String month = child_nameListInner.get(i);
							if (month.compareTo(child_name) < 0) {
								child_nameListInner.remove(i);
							}
						}
					}
					if (split_value.containsKey("$lt")) {
						String value = (String) split_value.get("$lt");
						for (int i = child_nameListInner.size() - 1; i >= 0; i--) {
							String child_name = child_nameListInner.get(i);
							if ((child_name + suffix).compareTo(value) >= 0) {
								child_nameListInner.remove(i);
							}
						}
					}
					if (split_value.containsKey("$lte")) {
						String value = (String) split_value.get("$lte");
						for (int i = child_nameListInner.size() - 1; i >= 0; i--) {
							String child_name = child_nameListInner.get(i);
							if ((child_name + suffix).compareTo(value) > 0) {
								child_nameListInner.remove(i);
							}
						}
					}
					ListUtil.MergeList(child_nameNeed, child_nameListInner);
				} else {
					throw new MyException(
							"ZillionAgent: " + "split_column:" + schema.split_column + " can't be " + split_valueObject.getClass().getName());
				}
			} else {
				ListUtil.MergeList(child_nameNeed, child_nameList);
				break;
			}
		}
		Collections.sort(child_nameNeed);
		return child_nameNeed;
	}

	public static List<SchemaWrapper> getSchemaWrapperList(SchemaRepositoryContainer container, ZillionTableSchema schema, String query_month,
			boolean create_split_table, List<JSONObject> CriteriaList, boolean has_data, boolean has_index, String index_name) throws Exception {
		List<SchemaWrapper> schemaWrapperList = new ArrayList<SchemaWrapper>();
		if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
			if (query_month != null) {
				ITableWrapper dataWrapper = null;
				if (has_data) {
					dataWrapper = schema.connect(container, query_month, create_split_table);
				}
				List<ITableWrapper> wrapperIndexs = new ArrayList<ITableWrapper>();
				if (has_index) {
					for (int i = 0; i < schema.Indexes.size(); i++) {
						ZillionTableIndex index = schema.Indexes.get(i);
						ITableWrapper indexWrapper;
						if (index_name == null || index.index_name.equals(index_name)) {
							indexWrapper = index.connect(container, query_month, create_split_table);
						} else {
							indexWrapper = null;
						}
						wrapperIndexs.add(indexWrapper);
					}
				}
				SchemaWrapper schemaWrapper = new SchemaWrapper(query_month, dataWrapper, wrapperIndexs);
				if (schemaWrapper.valid()) {
					schemaWrapperList.add(schemaWrapper);
				}
			} else {
				String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + schema.Namespace;
				List<String> child_nameList = container.ZK.getChildren(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name,
						false);
				List<String> child_nameNeed = get_splits(schema, child_nameList, CriteriaList);
				for (String child_name : child_nameNeed) {
					ITableWrapper dataWrapper = null;
					if (has_data) {
						dataWrapper = schema.connect(container, child_name, create_split_table);
					}
					List<ITableWrapper> wrapperIndexs = new ArrayList<ITableWrapper>();
					if (has_index) {
						for (int i = 0; i < schema.Indexes.size(); i++) {
							ZillionTableIndex index = schema.Indexes.get(i);
							ITableWrapper indexWrapper;
							if (index_name == null || index.index_name.equals(index_name)) {
								indexWrapper = index.connect(container, child_name, create_split_table);
							} else {
								indexWrapper = null;
							}
							wrapperIndexs.add(indexWrapper);
						}
					}
					SchemaWrapper schemaWrapper = new SchemaWrapper(child_name, dataWrapper, wrapperIndexs);
					if (schemaWrapper.valid()) {
						schemaWrapperList.add(schemaWrapper);
					}
				}
			}
		} else {
			ITableWrapper dataWrapper = null;
			if (has_data) {
				dataWrapper = schema.connect(container, null, false);
			}
			List<ITableWrapper> wrapperIndexs = new ArrayList<ITableWrapper>();
			if (has_index) {
				for (int i = 0; i < schema.Indexes.size(); i++) {
					ZillionTableIndex index = schema.Indexes.get(i);
					ITableWrapper indexWrapper;
					if (index_name == null || index.index_name.equals(index_name)) {
						indexWrapper = index.connect(container, null, false);
					} else {
						indexWrapper = null;
					}
					wrapperIndexs.add(indexWrapper);
				}
			}
			SchemaWrapper schemaWrapper = new SchemaWrapper(null, dataWrapper, wrapperIndexs);
			if (schemaWrapper.valid()) {
				schemaWrapperList.add(schemaWrapper);
			}
		}
		return schemaWrapperList;
	}

	public static Set<Object> parse_set(SchemaRepositoryContainer container, Object inObject) throws Exception {
		if (inObject instanceof JSONObject) {
			JSONObject inObjectJSON = (JSONObject) inObject;
			if (inObjectJSON.containsKey("QueryType")) {
				JSONObject queryResultInner = container.agent.Query(inObjectJSON);
				JSONArray valueArray = (JSONArray) queryResultInner.get("Content");
				Set<Object> result = FastJsonUtil.ValueArray2ValueSet(valueArray);
				return result;
			} else if (inObjectJSON.containsKey("SetOperator")) {
				String SetOperator = (String) inObjectJSON.get("SetOperator");
				if (SetOperator.equals("or") || SetOperator.equals("and")) {
					List<Set<Object>> valueSetList = new ArrayList<Set<Object>>();
					JSONArray SetArray = (JSONArray) inObjectJSON.get("SetArray");
					for (int i = 0; i < SetArray.size(); i++) {
						Object SetArrayItem = SetArray.get(i);
						Set<Object> innerSet = parse_set(container, SetArrayItem);
						valueSetList.add(innerSet);
					}
					if (SetOperator.equals("or")) {
						Set<Object> result = FastJsonUtil.ValueSet_or(valueSetList);
						return result;
					} else {
						Set<Object> result = FastJsonUtil.ValueSet_and(valueSetList);
						return result;
					}
				} else if (SetOperator.equals("sub")) {
					Set<Object> innerSet1 = parse_set(container, inObjectJSON.get("Set1"));
					Set<Object> innerSet2 = parse_set(container, inObjectJSON.get("Set2"));
					Set<Object> result = FastJsonUtil.ValueSet_sub(innerSet1, innerSet2);
					return result;
				} else {
					throw new MyException("SetOperator cant be " + SetOperator);
				}
			} else {
				throw new MyException("QueryType or SetOperator not exist");
			}
		} else if (inObject instanceof JSONArray) {
			JSONArray inObjectArray = (JSONArray) inObject;
			Set<Object> valueSet = FastJsonUtil.ValueArray2ValueSet(inObjectArray);
			return valueSet;
		} else {
			throw new MyException("seperate_in: " + inObject);
		}
	}

	private static List<JSONObject> seperate_in(SchemaRepositoryContainer container, JSONObject Criteria) throws Exception {
		List<JSONObject> result = new ArrayList<JSONObject>();
		List<String> inColumnList = new ArrayList<String>();
		List<JSONArray> inValueArrayList = new ArrayList<JSONArray>();
		int zuhe = 1;
		{
			Iterator<String> keyIter = Criteria.keySet().iterator();
			while (keyIter.hasNext()) {
				String key = keyIter.next();
				Object value = Criteria.get(key);
				if (value != null && value instanceof JSONObject) {
					JSONObject valueJSON = (JSONObject) value;
					if (valueJSON.containsKey("in")) {
						Object inObject = valueJSON.get("in");
						Set<Object> valueSet = parse_set(container, inObject);
						JSONArray valueArray = FastJsonUtil.ValueSet2ValueArray(valueSet);
						inColumnList.add(key);
						inValueArrayList.add(valueArray);
						zuhe *= valueArray.size();
					}
				}
			}
		}
		if (inColumnList.size() == 0) {
			result.add(Criteria);
		} else {
			for (int index_zuhe = 0; index_zuhe < zuhe; index_zuhe++) {
				int tmp = index_zuhe;
				int[] indexArray = new int[inColumnList.size()];
				for (int ii = 0; ii < inColumnList.size(); ii++) {
					JSONArray valueArray = inValueArrayList.get(ii);
					indexArray[ii] = tmp % valueArray.size();
					tmp /= valueArray.size();
				}
				JSONObject CriteriaInner = new JSONObject();
				for (int ii = 0; ii < inColumnList.size(); ii++) {
					String column = inColumnList.get(ii);
					JSONArray valueArray = inValueArrayList.get(ii);
					CriteriaInner.put(column, valueArray.get(indexArray[ii]));
				}
				Iterator<String> keyIter = Criteria.keySet().iterator();
				while (keyIter.hasNext()) {
					String key = keyIter.next();
					Object value = Criteria.get(key);
					boolean exist = false;
					for (String inColumn : inColumnList) {
						if (inColumn.equals(key)) {
							exist = true;
							break;
						}
					}
					if (!exist) {
						CriteriaInner.put(key, value);
					}
				}
				result.add(CriteriaInner);
			}
		}

		return result;
	}

	public static ZillionQueryResult query(SchemaRepositoryContainer container, ZillionTableSchema schema, String query_month, String QueryType,
			JSONArray ReturnColumns, JSONObject Criteria, JSONArray Criterias, JSONArray SecondCriteria, JSONArray OrderBy, JSONObject Limit,
			JSONObject Set, String ExportZip, String Hint, AsyncJob job, JSONArray ConflictArray) throws Exception {
		boolean update_check = true;
		if (update_check && QueryType.equals("update")) {
			if (schema.table_name.equals("work_order") || schema.table_name.equals("table_index_data")) {
				String uuid = UUID.randomUUID().toString();
				AsyncJob jobInner = new AsyncJob(uuid, null);
				LogUtil.warn("update_check_sql:" + schema.table_name + (query_month == null ? "" : "_" + query_month) + "\n" + "Criteria:"
						+ (Criteria != null ? FastJsonUtil.toFormatString(Criteria) : FastJsonUtil.toFormatString(Criterias)) + "\n" + "Set:"
						+ FastJsonUtil.toFormatString(Set));
				ZillionQueryResult resultInner = query(container, schema, query_month, "select", null, null, Criteria, Criterias, SecondCriteria,
						null, null, null, null, Hint, jobInner, ConflictArray);
				LogUtil.warn("update_check_old:" + resultInner.content.size() + "\t" + FastJsonUtil.toFormatString(resultInner.content));
			}
		}
		ZillionQueryResult result = query(container, schema, query_month, QueryType, null, ReturnColumns, Criteria, Criterias, SecondCriteria,
				OrderBy, Limit, Set, ExportZip, Hint, job, ConflictArray);
		if (update_check && QueryType.equals("update")) {
			if (schema.table_name.equals("work_order") || schema.table_name.equals("table_index_data")) {
				String uuid = UUID.randomUUID().toString();
				AsyncJob jobInner = new AsyncJob(uuid, null);
				ZillionQueryResult resultInner = query(container, schema, query_month, "select", null, null, Criteria, Criterias, SecondCriteria,
						null, null, null, null, Hint, jobInner, ConflictArray);
				LogUtil.warn("update_check_new:" + resultInner.content.size() + "\t" + FastJsonUtil.toFormatString(resultInner.content));
				boolean match = true;
				for (int i = 0; i < resultInner.content.size(); i++) {
					JSONObject item = (JSONObject) resultInner.content.get(i);
					if (!FastJsonCompareUtil.Instance().Contain(item, Set)) {
						match = false;
						break;
					}
				}
				LogUtil.warn("update_check_result:" + match);
			}
		}
		return result;
	}

	public static ZillionQueryResult query(SchemaRepositoryContainer container, ZillionTableSchema schema, String query_month, String QueryType,
			String UniqueReturnColumn, JSONArray ReturnColumns, JSONObject Criteria, JSONArray Criterias, JSONArray SecondCriteria, JSONArray OrderBy,
			JSONObject Limit, JSONObject Set, String ExportZip, String Hint, AsyncJob job, JSONArray ConflictArray) throws Exception {
		// NotNullStringmax_length
		if (ZillionConstant.check_query) {
			if (Criteria != null) {
				ZillionQueryCheckUtil.CheckCriteria(schema, Criteria);
			} else {
				for (int i = 0; i < Criterias.size(); i++) {
					JSONObject CriteriaInner = (JSONObject) Criterias.get(i);
					ZillionQueryCheckUtil.CheckCriteria(schema, CriteriaInner);
				}
			}
		}

		if (QueryType.equals("update")) {
			boolean contain_key = false;
			StringBuffer key_sb = new StringBuffer();
			Iterator<String> iter = Set.keySet().iterator();
			while (iter.hasNext()) {
				String column = iter.next();
				if (schema.KeyMap.containsKey(column)) {
					contain_key = true;
					if (key_sb.length() > 0) {
						key_sb.append(" ");
					}
					key_sb.append(column);
				}
			}
			if (contain_key) {
				throw new MyException("ZillionQueryCheck: you can't update key:" + key_sb.toString());
			}

			if (ZillionConstant.check_query) {
				ZillionQueryCheckUtil.CheckSet(schema, Set);
			}
		}

		// ϲб
		List<JSONObject> CriteriaList = new ArrayList<JSONObject>();
		if (Criteria != null) {
			CriteriaList.addAll(seperate_in(container, Criteria));
		} else {
			for (int i = 0; i < Criterias.size(); i++) {
				JSONObject CriteriaInner = (JSONObject) Criterias.get(i);
				CriteriaList.addAll(seperate_in(container, CriteriaInner));
			}
		}
		SecondCriteria scObject = null;
		if (SecondCriteria != null) {
			scObject = new SecondCriteria(container, schema, SecondCriteria);
		}

		List<ZillionCriteriaGroup> groupList = ZillionCriteriaMatchUtil.groupList(schema, CriteriaList, scObject, Hint);
		for (ZillionCriteriaGroup group : groupList) {
			// ӡgroupϢ
			if (ZillionConstant.printCriteriaGroup) {
				print(schema, group);
			}
		}

		long Final_Limit_Skip = 0;
		long Final_Limit_Count = Integer.MAX_VALUE;
		long Limit_Skip = 0;
		long Limit_Count = Integer.MAX_VALUE;
		if (QueryType.equals("select")) {
			if (Limit != null) {
				Long Skip_tmp = FastJsonUtil.getLong(Limit, "Skip");
				Long Count_tmp = FastJsonUtil.getLong(Limit, "Count");
				if (Skip_tmp != null) {
					Final_Limit_Skip = Skip_tmp;
				}
				if (Count_tmp != null) {
					Final_Limit_Count = Count_tmp;
				}
			}
		}

		// Ƿݿ
		JSONArray OrderByReal = null;
		Boolean SortInDB;
		boolean SortAsc = true;
		if (!QueryType.equals("select")) {
			SortInDB = false;
		} else {
			if (groupList.size() > 1) {
				SortInDB = false;
			} else if (OrderBy == null || OrderBy.size() == 0) {
				SortInDB = true;
			} else {
				// һ£ͷҳпܲͨڴ㶨
				ZillionCriteriaGroup group = groupList.get(0);
				String[] keys;
				if (group.index_index == -1) {
					keys = schema.Key;
				} else {
					keys = schema.Indexes.get(group.index_index).Key;
				}

				// ѯֱ
				List<String> splits = new ArrayList<String>();
				if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
					if (query_month == null) {
						List<String> all_splits = get_all_splits(schema);
						splits = get_splits(schema, all_splits, group.CriteriaList);
					} else {
						splits.add(query_month);
					}
				}

				// Сϸƥ
				int max_equal_small = 0;
				JSONObject criteriaInner = group.CriteriaList.get(0);
				for (int index_key = 0; index_key < keys.length; index_key++) {
					String keyItem = keys[index_key];
					if (criteriaInner.containsKey(keyItem)) {
						Object value = criteriaInner.get(keyItem);
						if (value != null && (value instanceof JSONObject || value instanceof JSONArray)) {
							break;
						} else {
							max_equal_small = index_key + 1;
						}
					} else {
						break;
					}
				}

				if (schema.table_type != null && (schema.table_type.startsWith("split_")) && splits.size() > 1) {
					// ֱͣҿԽֿ
					if (group.CriteriaList.size() > 1) {
						SortInDB = false;
					} else {
						int index_split = -1;
						for (int index_key = 0; index_key < keys.length; index_key++) {
							String keyItem = keys[index_key];
							if (keyItem.equals(schema.split_column)) {
								index_split = index_key;
								break;
							}
						}
						if (max_equal_small < index_split) {
							SortInDB = false;
						} else {
							JSONArray OrderBy_tmp = new JSONArray();
							if (!ZillionOrderByUtil.Match(keys, max_equal_small, OrderBy, OrderBy_tmp)) {
								SortInDB = false;
							} else {
								SortInDB = true;
								OrderByReal = OrderBy_tmp;
							}
						}
					}
				} else {
					// Ƿֱ߷ֱֻһֿ
					if (group.CriteriaList.size() > 1) {
						// ϸ
						if (!ZillionOrderByUtil.Match(keys, OrderBy)) {
							SortInDB = false;
						} else {
							JSONObject item0 = (JSONObject) OrderBy.get(0);
							boolean AscTmp = (Boolean) item0.get("Asc");
							if (!AscTmp) {
								SortInDB = false;
							} else {
								SortInDB = true;
							}
						}
					} else {
						JSONArray OrderBy_tmp = new JSONArray();
						if (!ZillionOrderByUtil.Match(keys, max_equal_small, OrderBy, OrderBy_tmp)) {
							SortInDB = false;
						} else {
							SortInDB = true;
							OrderByReal = OrderBy_tmp;
						}
					}
				}
			}
		}

		// ߱ݿ
		if (SortInDB) {
			if (OrderByReal != null && OrderByReal.size() > 0) {
				JSONObject item0 = (JSONObject) OrderByReal.get(0);
				SortAsc = (Boolean) item0.get("Asc");
			}
			Limit_Skip = Final_Limit_Skip;
			Limit_Count = Final_Limit_Count;
		}
		if (ZillionConstant.printCriteriaGroup) {
			LogUtil.warn("SortInDB:" + schema.Namespace + " " + schema.table_name + (query_month != null ? "_" + query_month : "") + "\n"
					+ (Criteria != null ? FastJsonUtil.toString(Criteria) : FastJsonUtil.toString(Criterias)) + "\n"
					+ (OrderBy != null ? FastJsonUtil.toString(OrderBy) : "") + "\n" + "SortInDB:" + SortInDB + "\n" + "SortAsc:" + SortAsc);
		}

		// QueryType
		if (QueryType.equals("update") || QueryType.equals("delete")) {
			// groupûҪƣdeleteغûҪupdateSetûֵҲõĸݵһ޸ĺڶ
		} else if (QueryType.equals("select_count") || QueryType.equals("export")) {
			if (groupList.size() > 1) {
				throw new MyException("ZillionAgent: " + QueryType + " group can't be " + groupList.size());
			}
		} else if (QueryType.equals("select")) {
			if (job.brs != null) {
				if (groupList.size() > 1) {
					throw new MyException("ZillionAgent: " + QueryType + " buffered group can't be " + groupList.size());
				}
				if (!SortInDB) {
					throw new MyException("ZillionAgent: " + QueryType + " buffered group can't order by in memory");
				}
			}
		}

		ZillionQueryResult result = new ZillionQueryResult();
		JSONArray content = new JSONArray();

		ZipArchiveOutputStream out = null;
		if (QueryType.equals("export")) {
			out = new ZipArchiveOutputStream(new FileOutputStream(ExportZip));
		}
		for (int index_group = 0; index_group < groupList.size(); index_group++) {
			ZillionCriteriaGroup group = groupList.get(index_group);
			JSONArray contentOfGroup = new JSONArray();
			List<SchemaWrapper> schemaWrapperList;
			if (group.index_index == -1) {
				if (QueryType.equals("delete") || QueryType.equals("update")) {
					schemaWrapperList = getSchemaWrapperList(container, schema, query_month, false, CriteriaList, true, true, null);
				} else {
					schemaWrapperList = getSchemaWrapperList(container, schema, query_month, false, CriteriaList, true, false, null);
				}
			} else {
				ZillionTableIndex schema_index = schema.Indexes.get(group.index_index);
				if (QueryType.equals("delete") || QueryType.equals("update")) {
					schemaWrapperList = getSchemaWrapperList(container, schema, query_month, false, CriteriaList, true, true, null);
				} else {
					schemaWrapperList = getSchemaWrapperList(container, schema, query_month, false, CriteriaList, false, true,
							schema_index.index_name);
				}
			}
			// 
			if (SortInDB && !SortAsc) {
				Collections.reverse(schemaWrapperList);
			}

			ZillionQueryTmp zqt = new ZillionQueryTmp();
			try {
				for (SchemaWrapper schemaWrapper : schemaWrapperList) {
					container.DatabaseImpl.scan(container, schema, schemaWrapper, ReturnColumns, group, QueryType, contentOfGroup, zqt, out, job,
							SortInDB, OrderByReal, Limit_Skip, Limit_Count, result, Set, ConflictArray);
					if (zqt.scanned_finish) {
						break;
					}
				}
			} catch (Exception exp) {
				print(schema, group);
				throw exp;
			} finally {
				for (SchemaWrapper schemaWrapper : schemaWrapperList) {
					schemaWrapper.unuse();
				}

				if (QueryType.equals("export")) {
					out.finish();
					out.close();
				}
			}

			if (QueryType.equals("select")) {
				// ϲ
				if (groupList.size() == 1) {
					content = contentOfGroup;
				} else {
					for (int ii = 0; ii < contentOfGroup.size(); ii++) {
						JSONObject item = (JSONObject) contentOfGroup.get(ii);
						if (FastJsonCompareUtil.Instance().Contain(content, item, schema.Key)) {
							continue;
						}
						content.add(item);
					}
				}
			}
		}

		if (QueryType.equals("select")) {
			if (!SortInDB && OrderBy != null && OrderBy.size() > 0) {
				List<JSONObject> contentList = new ArrayList<JSONObject>();
				for (int index_ = 0; index_ < content.size(); index_++) {
					contentList.add((JSONObject) content.get(index_));
				}
				Collections.sort(contentList, new JSONObjectComparator(OrderBy));
				JSONArray content_all = new JSONArray();
				for (int index_ = (int) Limit_Skip; index_ < contentList.size() && index_ < Limit_Skip + Limit_Count; index_++) {
					content_all.add(contentList.get(index_));
				}
				content = new JSONArray();
				for (int index_ = (int) Final_Limit_Skip; index_ < content_all.size()
						&& index_ < (int) (Final_Limit_Skip + Final_Limit_Count); index_++) {
					content.add(content_all.get(index_));
				}
			}
			content = cut(content, UniqueReturnColumn, ReturnColumns);

			if (job.brs != null) {
				job.brs.Finish(true, null);
			}
		}

		result.content = content;
		if (QueryType.equals("select")) {
			result.count = result.content.size();
		}
		return result;
	}

	public static JSONArray cut(JSONArray content, String UniqueReturnColumn, JSONArray ReturnColumns) {
		JSONArray result = content;
		if (UniqueReturnColumn != null) {
			JSONArray content_cut = new JSONArray();
			for (int index_ = 0; index_ < content.size(); index_++) {
				JSONObject contentItem = (JSONObject) content.get(index_);
				content_cut.add(contentItem.get(UniqueReturnColumn));
			}
			result = content_cut;
		} else if (ReturnColumns != null) {
			JSONArray content_cut = new JSONArray();
			for (int index_ = 0; index_ < content.size(); index_++) {
				JSONObject contentItem = (JSONObject) content.get(index_);
				JSONObject tmpObject = new JSONObject();
				for (int index_rc = 0; index_rc < ReturnColumns.size(); index_rc++) {
					String rc_name = (String) ReturnColumns.get(index_rc);
					if (contentItem.containsKey(rc_name)) {
						tmpObject.put(rc_name, contentItem.get(rc_name));
					}
				}
				content_cut.add(tmpObject);
			}
			result = content_cut;
		}
		return result;
	}

	public static void batch_put_insert(SchemaRepositoryContainer container, ZillionTableSchema schema, JSONArray InsertObjects, boolean is_put,
			JSONArray ConflictArray) throws Exception {
		// NotNullStringmax_length
		if (ZillionConstant.check_query) {
			for (int i = 0; i < InsertObjects.size(); i++) {
				JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
				ZillionQueryCheckUtil.CheckInsert(schema, InsertObject);
			}
		}

		if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
			Map<String, JSONArray> InsertObjectsMap = new HashMap<String, JSONArray>();
			for (int i = 0; i < InsertObjects.size(); i++) {
				JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
				String split_value = (String) InsertObject.get(schema.split_column);
				String child_name = split_value.substring(0, SplitUtil.split_type2child_length.get(schema.table_type));
				if (!InsertObjectsMap.containsKey(child_name)) {
					InsertObjectsMap.put(child_name, new JSONArray());
				}
				InsertObjectsMap.get(child_name).add(InsertObject);
			}
			String[] child_nameArray = InsertObjectsMap.keySet().toArray(new String[0]);
			Arrays.sort(child_nameArray);
			for (String child_name : child_nameArray) {
				JSONArray InsertObjectsInner = InsertObjectsMap.get(child_name);
				ITableWrapper dataWrapper = schema.connect(container, child_name, true);
				List<ITableWrapper> wrapperIndexs = new ArrayList<ITableWrapper>();
				for (int i = 0; i < schema.Indexes.size(); i++) {
					ZillionTableIndex index = schema.Indexes.get(i);
					ITableWrapper indexWrapper = index.connect(container, child_name, true);
					wrapperIndexs.add(indexWrapper);
				}
				SchemaWrapper schemaWrapper = new SchemaWrapper(child_name, dataWrapper, wrapperIndexs);
				try {
					container.DatabaseImpl.batch_process(container, schema, schemaWrapper, "insert", InsertObjectsInner, null, is_put, ConflictArray);
				} finally {
					schemaWrapper.unuse();
				}
			}
		} else {
			ITableWrapper dataWrapper = schema.connect(container, null, false);
			List<ITableWrapper> wrapperIndexs = new ArrayList<ITableWrapper>();
			for (int i = 0; i < schema.Indexes.size(); i++) {
				ZillionTableIndex index = schema.Indexes.get(i);
				ITableWrapper indexWrapper = index.connect(container, null, false);
				wrapperIndexs.add(indexWrapper);
			}
			SchemaWrapper schemaWrapper = new SchemaWrapper(null, dataWrapper, wrapperIndexs);
			try {
				container.DatabaseImpl.batch_process(container, schema, schemaWrapper, "insert", InsertObjects, null, is_put, ConflictArray);
			} finally {
				schemaWrapper.unuse();
			}
		}
	}

	private static String toSQLString(String columnName, Object columnCondition) {
		StringBuffer sbInner = new StringBuffer();
		if (columnCondition instanceof JSONObject) {
			JSONObject columnRangeJSON = (JSONObject) columnCondition;
			if (columnRangeJSON.containsKey("$regex")) {
				String regex = (String) columnRangeJSON.get("$regex");
				sbInner.append(columnName + " like " + "\"" + regex + "\"");
			} else {
				Iterator<String> keyIterInner = columnRangeJSON.keySet().iterator();
				while (keyIterInner.hasNext()) {
					String fuhao = keyIterInner.next();
					Object columnObject = columnRangeJSON.get(fuhao);
					if (sbInner.length() > 0) {
						sbInner.append(" and ");
					}
					if (fuhao.equals("$lt")) {
						sbInner.append(columnName + "<" + columnObject);
					} else if (fuhao.equals("$lte")) {
						sbInner.append(columnName + "<=" + columnObject);
					} else if (fuhao.equals("$gt")) {
						sbInner.append(columnName + ">" + columnObject);
					} else if (fuhao.equals("$gte")) {
						sbInner.append(columnName + ">=" + columnObject);
					}
				}
			}
		} else {
			if (sbInner.length() > 0) {
				sbInner.append(" and ");
			}
			sbInner.append(columnName + "=" + columnCondition);
		}
		return sbInner.toString();
	}

	public static void batch_remove(SchemaRepositoryContainer container, ZillionTableSchema schema, JSONArray InsertObjects, JSONArray ConflictArray)
			throws Exception {
		// NotNullStringmax_length
		if (ZillionConstant.check_query) {
			for (int i = 0; i < InsertObjects.size(); i++) {
				JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
				ZillionQueryCheckUtil.CheckKey(schema, InsertObject);
			}
		}

		if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
			Map<String, JSONArray> InsertObjectsMap = new HashMap<String, JSONArray>();
			for (int i = 0; i < InsertObjects.size(); i++) {
				JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
				String split_value = (String) InsertObject.get(schema.split_column);
				String child_name = split_value.substring(0, SplitUtil.split_type2child_length.get(schema.table_type));
				if (!InsertObjectsMap.containsKey(child_name)) {
					InsertObjectsMap.put(child_name, new JSONArray());
				}
				InsertObjectsMap.get(child_name).add(InsertObject);
			}
			String[] child_nameArray = InsertObjectsMap.keySet().toArray(new String[0]);
			Arrays.sort(child_nameArray);
			for (String child_name : child_nameArray) {
				JSONArray InsertObjectsInner = InsertObjectsMap.get(child_name);
				ITableWrapper dataWrapper = schema.connect(container, child_name, false);
				if (dataWrapper != null) {
					List<ITableWrapper> wrapperIndexs = new ArrayList<ITableWrapper>();
					for (int i = 0; i < schema.Indexes.size(); i++) {
						ZillionTableIndex index = schema.Indexes.get(i);
						ITableWrapper indexWrapper = index.connect(container, child_name, false);
						wrapperIndexs.add(indexWrapper);
					}
					SchemaWrapper schemaWrapper = new SchemaWrapper(child_name, dataWrapper, wrapperIndexs);
					try {
						container.DatabaseImpl.batch_process(container, schema, schemaWrapper, "delete", InsertObjectsInner, null, false,
								ConflictArray);
					} finally {
						schemaWrapper.unuse();
					}
				}
			}
		} else {
			ITableWrapper dataWrapper = schema.connect(container, null, false);
			List<ITableWrapper> wrapperIndexs = new ArrayList<ITableWrapper>();
			for (int i = 0; i < schema.Indexes.size(); i++) {
				ZillionTableIndex index = schema.Indexes.get(i);
				ITableWrapper indexWrapper = index.connect(container, null, false);
				wrapperIndexs.add(indexWrapper);
			}
			SchemaWrapper schemaWrapper = new SchemaWrapper(null, dataWrapper, wrapperIndexs);
			try {
				container.DatabaseImpl.batch_process(container, schema, schemaWrapper, "delete", InsertObjects, null, false, ConflictArray);
			} finally {
				schemaWrapper.unuse();
			}
		}
	}

	private static void print(ZillionTableSchema schema, ZillionCriteriaGroup group) {
		StringBuffer sbKeyAll = new StringBuffer();
		String[] keysOfGroup;
		if (group.index_index == -1) {
			keysOfGroup = schema.Key;
			sbKeyAll.append("Key");
		} else {
			ZillionTableIndex schema_index = schema.Indexes.get(group.index_index);
			keysOfGroup = schema_index.Key;
			sbKeyAll.append(schema_index.index_name);
		}
		StringBuffer sbKey = new StringBuffer();
		for (String keyOfGroup : keysOfGroup) {
			if (sbKey.length() > 0) {
				sbKey.append(",");
			}
			sbKey.append(keyOfGroup);
		}
		sbKeyAll.append("[" + sbKey.toString() + "]");

		StringBuffer sbKeyCondition = new StringBuffer();
		for (int i = 0; i < group.itemListOfGroup.size(); i++) {
			ZillionCriteriaMatchItem itemInner = group.itemListOfGroup.get(i);
			StringBuffer sbInner = new StringBuffer();
			for (int ii = 0; ii < itemInner.matched; ii++) {
				String columnName = keysOfGroup[ii];
				Object columnCondition = group.CriteriaList.get(i).get(columnName);
				String SQLString = toSQLString(columnName, columnCondition);
				if (sbInner.length() > 0) {
					sbInner.append(" and ");
				}
				sbInner.append(SQLString);
			}
			if (sbKeyCondition.length() > 0) {
				sbKeyCondition.append(" or ");
			}
			sbKeyCondition.append("(" + sbInner.toString() + ")");
		}

		StringBuffer sb = new StringBuffer();
		sb.append("select from " + "`" + schema.Namespace + "." + schema.table_name + "`");
		sb.append(" use " + sbKeyAll.toString());
		sb.append(" where ");
		if (group.otherCriteria.size() > 0) {
			sb.append("(" + sbKeyCondition.toString() + ")");
			Iterator<String> keyIterInner = group.otherCriteria.keySet().iterator();
			while (keyIterInner.hasNext()) {
				String otherKey = keyIterInner.next();
				Object otherValue = group.otherCriteria.get(otherKey);
				String SQLString = toSQLString(otherKey, otherValue);
				if (sb.length() > 0) {
					sb.append(" and ");
				}
				sb.append(SQLString);
			}
		} else {
			sb.append(sbKeyCondition.toString());
		}
		LogUtil.warn(sb.toString());

		StringBuffer sbKeyCondition_json = new StringBuffer();
		for (JSONObject condition : group.CriteriaList) {
			if (sbKeyCondition_json.length() > 0) {
				sbKeyCondition_json.append(",");
			}
			sbKeyCondition_json.append(FastJsonUtil.toFormatString(condition));
		}
		LogUtil.warn("Key Condition:" + sbKeyCondition_json.toString());
		if (group.otherCriteria != null) {
			StringBuffer sbOtherCondition_json = new StringBuffer();
			sbOtherCondition_json.append(FastJsonUtil.toFormatString(group.otherCriteria));
			LogUtil.warn("Other Condition:" + sbOtherCondition_json.toString());
		}
	}
}
