package com.zillion.database.async;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.zillion.util.common.MyException;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

public class AsyncJobRepository {
	static Map<String, AsyncJob> jobMap = new HashMap<String, AsyncJob>();
	static Map<String, AsyncJobThread> threadMap = new HashMap<String, AsyncJobThread>();

	public static synchronized void Add(AsyncJob job, AsyncJobThread thread) throws Exception {
		if (jobMap.containsKey(job.uuid)) {
			throw new MyException("UUID exist:" + job.uuid);
		}
		jobMap.put(job.uuid, job);
		threadMap.put(job.uuid, thread);
	}

	public static synchronized JSONObject QueryJobStatus(String uuid) throws Exception {
		if (!jobMap.containsKey(uuid)) {
			throw new MyException("UUID not exist:" + uuid);
		}

		JSONObject result = new JSONObject();
		AsyncJob job = jobMap.get(uuid);
		result.put("finish", job.IsFinish());
		result.put("success", job.success);
		result.put("exception", job.exception);
		result.put("count", job.count);
		result.put("queryResult", job.queryResult);
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		result.put("time_start", sdf.format(job.time_start));
		if (job.time_end != null) {
			result.put("time_end", sdf.format(job.time_end));
		}

		return result;
	}

	public static synchronized void ClearJob(String uuid) throws Exception {
		if (!jobMap.containsKey(uuid)) {
			throw new MyException("UUID not exist:" + uuid);
		}
		AsyncJob job = jobMap.get(uuid);
		if (!job.IsFinish()) {
			throw new MyException("job not finish:" + uuid);
		}

		jobMap.remove(uuid);
		threadMap.remove(uuid);
	}

	public static synchronized JSONArray GetJobList() throws Exception {
		JSONArray result = new JSONArray();

		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Iterator<String> keyIter = jobMap.keySet().iterator();
		while (keyIter.hasNext()) {
			String uuid = keyIter.next();
			AsyncJob job = jobMap.get(uuid);

			JSONObject item = new JSONObject();
			item.put("uuid", job.uuid);
			item.put("finish", job.IsFinish());
			item.put("success", job.success);
			item.put("exception", job.exception);
			item.put("count", job.count);
			item.put("time_start", sdf.format(job.time_start));
			if (job.time_end != null) {
				item.put("time_end", sdf.format(job.time_end));
			}
			result.add(item);
		}

		return result;
	}

	public static synchronized JSONArray RemoveTimeoutJob() throws Exception {
		JSONArray result = new JSONArray();

		Date currentTime = new Date();
		String[] uuids = jobMap.keySet().toArray(new String[0]);
		for (int i = 0; i < uuids.length; i++) {
			String uuid = uuids[i];
			AsyncJob job = jobMap.get(uuid);
			if (job.IsFinish() && currentTime.getTime() - job.time_end.getTime() > 1000L * 60 * 5) {
				result.add(job.uuid);

				jobMap.remove(uuid);
				threadMap.remove(uuid);
			}
		}

		return result;
	}

	public static synchronized AsyncResultSet GetBufferedResultSet(String uuid) throws Exception {
		if (!jobMap.containsKey(uuid)) {
			throw new MyException("UUID not exist:" + uuid);
		}

		return jobMap.get(uuid).brs;
	}
}
