package com.zillion.database.consistency;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.hadoop.hbase.Cell;
import org.apache.hadoop.hbase.CellUtil;
import org.apache.hadoop.hbase.client.Delete;
import org.apache.hadoop.hbase.client.Get;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.util.Bytes;
import org.zillion.util.common.HexToByte;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.util.DatabaseImpl_HBase;
import com.zillion.database.util.DatabaseImpl_Mix;
import com.zillion.database.util.ITableWrapper;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.table.SchemaWrapper;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;
import com.zillion.util.zk.ZKLockBatch;
import com.zillion.util.zk.ZKNodeBatch;
import com.zillion.util.zk.ZKNodeUseLock;

import redis.clients.jedis.ShardedJedis;

public class ZillionConsistencyThread extends Thread {
	boolean needStop = false;
	SchemaRepositoryContainer container;

	public ZillionConsistencyThread(SchemaRepositoryContainer container) {
		this.container = container;
	}

	public void RequestStop() {
		this.needStop = true;
	}

	@Override
	public void run() {
		long cache_cycle_count = 1000;
		Date last_cache = new Date();
		while (!this.needStop) {
			try {
				Thread.sleep(100L);
			} catch (Exception e) {
				e.printStackTrace();
			}

			Date curr_time = new Date();
			if (curr_time.getTime() - last_cache.getTime() >= cache_cycle_count) {
				last_cache = curr_time;
				try {
					this.consistency(this.container);
				} catch (Exception e) {
					LogUtil.warn(e);
				}
			}
		}
	}

	public void consistency(SchemaRepositoryContainer container) {
		DatabaseImpl_HBase impl = null;
		if (container.DatabaseImpl instanceof DatabaseImpl_HBase) {
			impl = (DatabaseImpl_HBase) container.DatabaseImpl;
		} else if (container.DatabaseImpl instanceof DatabaseImpl_Mix) {
			impl = ((DatabaseImpl_Mix) container.DatabaseImpl).hbase;
		}
		if (impl == null) {
			return;
		}

		List<String> namespaceList = container.namespaceList;
		for (String namespace : namespaceList) {
			ShardedJedis jedis = null;
			HTable htable = null;
			try {
				if (ZillionConstant.strong_consistency_cache_redis && container.redisConfig != null) {
					jedis = container.redisPool.getResource();
				} else {
					htable = impl.Get_table_meta_cache(namespace);
				}
				if (ZillionConstant.strong_consistency_cache_redis && container.redisConfig != null) {
					List<String> tableList = container.repositoryMap.get(namespace).TableList();
					for (String table : tableList) {
						ZillionTableSchema schema = container.repositoryMap.get(namespace).GetTable(table);
						if (schema.strong_consistency) {
							Map<String, String> consistency_cache = jedis.hgetAll(namespace + ":" + table);
							Iterator<String> keyIter = consistency_cache.keySet().iterator();
							while (keyIter.hasNext()) {
								String key = keyIter.next();
								String value = consistency_cache.get(key);
								JSONObject cacheObject = JSON.parseObject(value);
								String QueryType = (String) cacheObject.get("QueryType");
								JSONObject InsertObject = (JSONObject) cacheObject.get("InsertObject");
								JSONObject Set = null;
								if (QueryType.equals("update")) {
									Set = (JSONObject) cacheObject.get("Set");
								}
								this.consistencyOne(impl, container, schema, jedis, htable, QueryType, InsertObject, Set, false);
							}
						}
					}
				} else {
					if (htable == null) {
						continue;
					}

					Scan scan = new Scan();
					scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("QueryType"));
					scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("InsertObject"));
					scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("Set"));
					ResultScanner scanner = htable.getScanner(scan);
					while (true) {
						Result res = scanner.next();
						if (res == null) {
							break;
						}
						byte[] real_bytes = res.getRow();
						int index_ = -1;
						for (int i = 0; i < real_bytes.length; i++) {
							if (real_bytes[i] == 0 || real_bytes[i] == 1) {
								index_ = i;
								break;
							}
						}
						String table_name = new String(real_bytes, 0, index_);
						ZillionTableSchema schema = container.repositoryMap.get(namespace).GetTable(table_name);
						String QueryType = null;
						JSONObject InsertObject = null;
						JSONObject Set = null;
						for (Cell cell : res.listCells()) {
							byte[] column_name_bytes = CellUtil.cloneQualifier(cell);
							String column_name = ByteToObject.toString(column_name_bytes);
							byte[] cell_bytes = CellUtil.cloneValue(cell);
							if (column_name.equals("QueryType")) {
								QueryType = ByteToObject.toString(cell_bytes);
							} else if (column_name.equals("InsertObject")) {
								InsertObject = JSON.parseObject(ByteToObject.toString(cell_bytes));
							} else if (column_name.equals("Set")) {
								Set = JSON.parseObject(ByteToObject.toString(cell_bytes));
							}
						}

						this.consistencyOne(impl, container, schema, jedis, htable, QueryType, InsertObject, Set, false);
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				if (jedis != null) {
					try {
						jedis.close();
					} catch (Exception e) {
						// e.printStackTrace();
					}
				}
			}
		}
	}

	private void consistencyOne(DatabaseImpl_HBase impl, SchemaRepositoryContainer container, ZillionTableSchema schema, ShardedJedis jedis,
			HTable htable, String QueryType, JSONObject InsertObject_outer, JSONObject Set, boolean is_put) throws Exception {
		// LogUtil.warn("consistencyOne" + ":" + QueryType + "\t" + InsertObject_outer.toJSONString());
		SchemaWrapper schemaWrapper;
		if (schema.table_type != null && (schema.table_type.startsWith("split_"))) {
			String split_value = (String) InsertObject_outer.get(schema.split_column);
			String table_type = schema.table_type;
			int split_length = table_type.equals("split_year") ? 4 : (table_type.equals("split_month") ? 6 : 8);
			String child_name = split_value.substring(0, split_length);
			ITableWrapper dataWrapper = schema.connect(container, child_name, true);
			List<ITableWrapper> wrapperIndexs = new ArrayList<ITableWrapper>();
			for (int i = 0; i < schema.Indexes.size(); i++) {
				ZillionTableIndex index = schema.Indexes.get(i);
				ITableWrapper indexWrapper = index.connect(container, child_name, true);
				wrapperIndexs.add(indexWrapper);
			}
			schemaWrapper = new SchemaWrapper(child_name, dataWrapper, wrapperIndexs);
		} else {
			ITableWrapper dataWrapper = schema.connect(container, null, false);
			List<ITableWrapper> wrapperIndexs = new ArrayList<ITableWrapper>();
			for (int i = 0; i < schema.Indexes.size(); i++) {
				ZillionTableIndex index = schema.Indexes.get(i);
				ITableWrapper indexWrapper = index.connect(container, null, false);
				wrapperIndexs.add(indexWrapper);
			}
			schemaWrapper = new SchemaWrapper(null, dataWrapper, wrapperIndexs);
		}
		try {
			JSONArray InsertObjects_lock = new JSONArray();
			InsertObjects_lock.add(InsertObject_outer);
			// 
			List<ZKNodeUseLock> nodeList = new ArrayList<ZKNodeUseLock>();
			for (int index_object = 0; index_object < InsertObjects_lock.size(); index_object++) {
				JSONObject InsertObject = (JSONObject) InsertObjects_lock.get(index_object);
				byte[] keyBytes = HTableUtil.BuildKey(schema, InsertObject);
				String node = HexToByte.byteToHex(keyBytes);
				ZKNodeUseLock ZKLockNode = new ZKNodeUseLock(node);
				nodeList.add(ZKLockNode);
			}
			ZKNodeBatch ZKNodeBatch = new ZKNodeBatch(schema.Namespace, schema.table_name, nodeList);
			// ȡmeta
			ZKLockBatch.Create(container, schema.Namespace, schema.table_name, ZKNodeBatch);
			List<ZKNodeUseLock> success_pathList = new ArrayList<ZKNodeUseLock>();
			List<ZKNodeUseLock> failure_pathList = new ArrayList<ZKNodeUseLock>();
			try {
				ZKLockBatch.WaitOnce(container, schema.Namespace, schema.table_name, ZKNodeBatch);

				// ס
				JSONArray InsertObjects_locked = new JSONArray();
				for (int index_object = 0; index_object < InsertObjects_lock.size(); index_object++) {
					JSONObject InsertObject = (JSONObject) InsertObjects_lock.get(index_object);
					ZKNodeUseLock node = nodeList.get(index_object);
					if (node.success) {
						InsertObjects_locked.add(InsertObject);
						success_pathList.add(node);
					} else {
						failure_pathList.add(node);
					}
				}

				if (InsertObjects_locked.size() > 0) {
					// ȷݵԻ
					JSONArray InsertObjects = new JSONArray();
					List<byte[]> real_bytesList_process = new ArrayList<byte[]>();
					if (ZillionConstant.strong_consistency_cache_redis && container.redisConfig != null) {
						String[] keys = new String[InsertObjects_locked.size()];
						for (int i = 0; i < InsertObjects_locked.size(); i++) {
							JSONObject InsertObject = (JSONObject) InsertObjects_locked.get(i);
							byte[] row_bytes = HTableUtil.BuildKey(schema, InsertObject);
							keys[i] = HexToByte.byteToHex(row_bytes);
						}
						List<String> rsmap = jedis.hmget(schema.Namespace + ":" + schema.table_name, keys);
						for (int i = 0; i < InsertObjects_locked.size(); i++) {
							JSONObject InsertObject = (JSONObject) InsertObjects_locked.get(i);
							if (rsmap.get(i) != null) {
								InsertObjects.add(InsertObject);
							}
						}
					} else {
						List<byte[]> real_bytesList = new ArrayList<byte[]>();
						List<Get> batch = new ArrayList<Get>();
						for (int i = 0; i < InsertObjects_locked.size(); i++) {
							JSONObject InsertObject = (JSONObject) InsertObjects_locked.get(i);
							byte[] table_bytes = ByteToObject.toBytes(schema.table_name);
							byte[] row_bytes = HTableUtil.BuildKey(schema, InsertObject);
							byte[] real_bytes = new byte[table_bytes.length + row_bytes.length];
							System.arraycopy(table_bytes, 0, real_bytes, 0, table_bytes.length);
							System.arraycopy(row_bytes, 0, real_bytes, table_bytes.length, row_bytes.length);
							real_bytesList.add(real_bytes);
							Get get = new Get(real_bytes);
							get.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("QueryType"));
							batch.add(get);
						}
						Result[] results = htable.get(batch);
						for (int i = 0; i < InsertObjects_locked.size(); i++) {
							JSONObject InsertObject = (JSONObject) InsertObjects_locked.get(i);
							byte[] real_bytes = real_bytesList.get(i);
							Result res = results[i];
							if (res != null && res.getRow() != null) {
								InsertObjects.add(InsertObject);
								real_bytesList_process.add(real_bytes);
							}
						}
					}

					if (InsertObjects.size() > 0) {
						// ִݵԲ
						impl.batch_processInner(container, schema, schemaWrapper, QueryType, InsertObjects, Set, is_put, false);

						// ɾݵԲ
						if (ZillionConstant.strong_consistency_cache_redis && container.redisConfig != null) {
							String[] keys = new String[InsertObjects.size()];
							for (int i = 0; i < InsertObjects.size(); i++) {
								JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
								byte[] row_bytes = HTableUtil.BuildKey(schema, InsertObject);
								keys[i] = HexToByte.byteToHex(row_bytes);
							}
							jedis.hdel(schema.Namespace + ":" + schema.table_name, keys);
						} else {
							List<Delete> batch = new ArrayList<Delete>();
							for (int index_object = 0; index_object < InsertObjects.size(); index_object++) {
								byte[] real_bytes = real_bytesList_process.get(index_object);
								Delete put = new Delete(real_bytes);
								put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("QueryType"));
								put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("InsertObject"));
								if (Set != null) {
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("Set"));
								}
								batch.add(put);
							}
							htable.delete(batch);
							if (ZillionConstant.disable_AutoFlush) {
								htable.flushCommits();
							}
						}
					}
				}
			} catch (Exception e) {
				throw e;
			} finally {
				// ͷmeta
				ZKNodeBatch.time = new Date();
				ZKNodeBatch.finish = true;
			}
		} finally {
			schemaWrapper.unuse();
		}
	}
}
