package com.zillion.database.consistency;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.zk.ZKNodeBatch;
import com.zillion.util.zk.ZKNodeUseLock;

public class ZillionRedissonLockThread extends Thread {
	boolean needStop = false;
	SchemaRepositoryContainer container;

	public ZillionRedissonLockThread(SchemaRepositoryContainer container) {
		this.container = container;
	}

	public void RequestStop() {
		this.needStop = true;
	}

	@Override
	public void run() {
		while (!this.needStop) {
			try {
				Thread.sleep(1L);

				while (!this.needStop) {
					ZKNodeBatch item = container.ThreadLocalRepository.myQueue.poll();
					if (item == null) {
						break;
					}
					synchronized (item) {
						for (ZKNodeUseLock node : item.nodeList) {
							String lockName = "ZKLockNode:" + item.Namespace + ":" + item.table_name + ":" + node.key;
							node.redisson_lock = container.redisson.getLock(lockName);
							if (!node.redisson_lock.isLocked()) {
								boolean success = node.redisson_lock.tryLock(0, 30, TimeUnit.SECONDS);
								node.success = success;
								if (success) {
									// LogUtil.info(node.key + "\t" + "tryLock" + "\t" + lockName + "\t" + node.success);
								}
							} else {
								// LogUtil.info(node.key + "\t" + "tryLock" + "\t" + lockName + "\t" + node.success);
							}
						}
						item.processed = true;
						container.ThreadLocalRepository.add(item);
						item.notify();
					}
				}

				List<ZKNodeBatch> itemList = container.ThreadLocalRepository.get();
				Date curr_time = new Date();
				for (int i = itemList.size() - 1; i >= 0; i--) {
					ZKNodeBatch item = itemList.get(i);
					if (item.finish && item.time.getTime() + ZillionConstant.strong_consistency_lock_delete_delay <= curr_time.getTime()) {
						for (int ii = 0; ii < item.nodeList.size(); ii++) {
							ZKNodeUseLock node = item.nodeList.get(ii);
							if (node.success) {
								node.redisson_lock.unlock();
								// String lockName = "ZKLockNode:" + item.Namespace + ":" + item.table_name + ":" + node.key;
								// LogUtil.info(node.key + "\t" + "unlock" + "\t" + lockName);
							}
						}
						itemList.remove(i);
					}
				}
			} catch (InterruptedException e) {
				e.printStackTrace();
			}

		}
	}
}
