package com.zillion.database.second;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.zillion.util.common.MyException;
import org.zillion.util.json.FastJsonUtil;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.agent.ZillionQueryUtil;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.table.ZillionTableColumn;
import com.zillion.util.table.ZillionTableSchema;

public class SecondCriteria {
	String CriteriaOperator;
	JSONArray CriteriaArray;
	String Operator;
	String Column;
	Object Value;

	List<SecondCriteria> children;
	ISecondComparable comparable;

	public SecondCriteria(SchemaRepositoryContainer container, ZillionTableSchema schema, Object param) throws Exception {
		if (param instanceof JSONObject) {
			JSONObject paramJSON = (JSONObject) param;
			if (paramJSON.containsKey("CriteriaOperator")) {
				CriteriaOperator = (String) paramJSON.get("CriteriaOperator");
				CriteriaArray = (JSONArray) paramJSON.get("CriteriaArray");
				children = new ArrayList<SecondCriteria>();
				for (int i = 0; i < CriteriaArray.size(); i++) {
					Object item = CriteriaArray.get(i);
					SecondCriteria child = new SecondCriteria(container, schema, item);
					children.add(child);
				}
			} else if (paramJSON.containsKey("Operator")) {
				Operator = (String) paramJSON.get("Operator");
				Column = (String) paramJSON.get("Column");
				ZillionTableColumn tc = schema.ColumnMap.get(Column);
				if (Operator.equals("in") || Operator.equals("outof")) {
					Object inObject = paramJSON.get("Value");
					if (inObject instanceof JSONObject) {
						Set<Object> valueSet = ZillionQueryUtil.parse_set(container, inObject);
						JSONArray valueArray = FastJsonUtil.ValueSet2ValueArray(valueSet);
						Value = valueArray;
					} else {
						JSONArray valueArray = new JSONArray();
						JSONArray valueArray_ori = (JSONArray) inObject;
						for (int i = 0; i < valueArray_ori.size(); i++) {
							valueArray.add(HTableUtil.ConvertValue(tc, valueArray_ori.get(i)));
						}
						Value = valueArray;
					}
				} else {
					Value = HTableUtil.ConvertValue(tc, paramJSON.get("Value"));
				}
				comparable = SecondComparableManager.Instance().Generate(Operator, Column, Value);
			}
		} else if (param instanceof JSONArray) {
			JSONArray paramArray = (JSONArray) param;
			CriteriaOperator = "and";
			CriteriaArray = paramArray;
			children = new ArrayList<SecondCriteria>();
			for (int i = 0; i < CriteriaArray.size(); i++) {
				Object item = CriteriaArray.get(i);
				SecondCriteria child = new SecondCriteria(container, schema, item);
				children.add(child);
			}
		}
	}

	public boolean match(JSONObject jsonObject) throws Exception {
		if (CriteriaOperator != null) {
			if (CriteriaOperator.equals("and")) {
				for (SecondCriteria child : children) {
					if (!child.match(jsonObject)) {
						return false;
					}
				}
				return true;
			} else if (CriteriaOperator.equals("or")) {
				for (SecondCriteria child : children) {
					if (child.match(jsonObject)) {
						return true;
					}
				}
				return false;
			} else if (CriteriaOperator.equals("not")) {
				for (SecondCriteria child : children) {
					if (!child.match(jsonObject)) {
						return true;
					}
				}
				return false;
			} else {
				throw new MyException("CriteriaOperator error " + CriteriaOperator);
			}
		} else {
			return comparable.match(jsonObject);
		}
	}
}
