package com.zillion.database.util;

import java.io.FileInputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HTableDescriptor;
import org.apache.hadoop.hbase.NamespaceDescriptor;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.apache.hadoop.hbase.client.Delete;
import org.apache.hadoop.hbase.client.Get;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.filter.Filter;
import org.apache.hadoop.hbase.filter.FilterList;
import org.apache.hadoop.hbase.filter.FilterList.Operator;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter;
import org.apache.hadoop.hbase.io.compress.Compression;
import org.apache.hadoop.hbase.util.Bytes;
import org.apache.hadoop.security.UserGroupInformation;
import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.data.Stat;
import org.zillion.util.common.FileUtil;
import org.zillion.util.common.HexToByte;
import org.zillion.util.common.MyException;
import org.zillion.util.format.FormatStaticUtil;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.zillion.database.agent.ZillionCriteriaGroup;
import com.zillion.database.agent.ZillionQueryResult;
import com.zillion.database.agent.ZillionQueryTmp;
import com.zillion.database.async.AsyncJob;
import com.zillion.database.async.AsyncUtil;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.common.ElasticSearchUtil;
import com.zillion.util.common.HashObject;
import com.zillion.util.common.HashUtil;
import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.HBaseAdminUtil;
import com.zillion.util.table.HBaseConnectionWrapper;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.table.SchemaWrapper;
import com.zillion.util.table.ZillionTableColumn;
import com.zillion.util.table.ZillionTableColumnEnum;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;
import com.zillion.util.zk.ZKHelper;
import com.zillion.util.zk.ZKLockBatch;
import com.zillion.util.zk.ZKNodeBatch;
import com.zillion.util.zk.ZKNodeUseLock;
import com.zillion.util.zk.ZKWrapper;

import redis.clients.jedis.ShardedJedis;

public class DatabaseImpl_HBase implements IDatabase {
	public Configuration configuration;
	public Configuration configuration_bak;

	public Connection connection;
	public Admin admin;
	public Connection connection_bak;
	public Admin admin_bak;
	public HBaseConnectionWrapper ConnectionWrapper;
	public Map<String, HTable> table_meta_dataMap;
	public Map<String, HTable> table_meta_indexMap;
	public Map<String, HTable> table_meta_splitMap;
	public Map<String, HTable> table_meta_cacheMap;

	public DatabaseImpl_HBase(Configuration configuration, Configuration configuration_bak) {
		this.configuration = configuration;
		this.configuration_bak = configuration_bak;
	}

	public void Start() throws Exception {
		{
			String zillion_kerberos_txt = configuration.get("zillion.kerberos.txt");
			String zillion_kerberos_keytab = configuration.get("zillion.kerberos.keytab");
			if (zillion_kerberos_txt != null && zillion_kerberos_keytab != null) {
				byte[] bytes = FileUtil.read_file_bytes(new FileInputStream(zillion_kerberos_txt));
				String base64String = new String(bytes, "UTF-8");
				String base64Header = "data:application/octet-stream;base64,";
				if (base64String.startsWith(base64Header)) {
					base64String = base64String.substring(base64Header.length());
				}
				byte[] keytab_bytes = Base64.getDecoder().decode(base64String);
				FileUtil.write_file(keytab_bytes, zillion_kerberos_keytab);
			}

			String principal = configuration.get("kerberos.principal");
			String file = configuration.get("keytab.file");
			if (principal != null && file != null) {
				String jaas_conf_path = configuration.get("zillion.jaas.conf");
				String krb5_conf_path = configuration.get("zillion.krb5.conf");
				System.setProperty("java.security.auth.login.config", jaas_conf_path);
				System.setProperty("java.security.krb5.conf", krb5_conf_path);
				String new_principal = get_new_principal(principal);
				if (new_principal != null) {
					principal = new_principal;
					configuration.set("kerberos.principal", principal);
				}
				// String master_principal =
				// configuration.get("hbase.master.kerberos.principal");
				// if (master_principal != null) {
				// String new_master_principal =
				// get_new_principal(master_principal);
				// if (new_master_principal != null) {
				// configuration.set("hbase.master.kerberos.principal",
				// new_master_principal);
				// }
				// }
				// String regionserver_principal =
				// configuration.get("hbase.regionserver.kerberos.principal");
				// if (regionserver_principal != null) {
				// String new_regionserver_principal =
				// get_new_principal(regionserver_principal);
				// if (new_regionserver_principal != null) {
				// configuration.set("hbase.regionserver.kerberos.principal",
				// new_regionserver_principal);
				// }
				// }

				UserGroupInformation.setConfiguration(configuration);
				try {
					UserGroupInformation.loginUserFromKeytab(principal, file);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		connection = ConnectionFactory.createConnection(configuration);
		admin = connection.getAdmin();
		if (configuration_bak != null) {
			connection_bak = ConnectionFactory.createConnection(configuration_bak);
			admin_bak = connection_bak.getAdmin();
		}

		ConnectionWrapper = new HBaseConnectionWrapper(configuration);
		ConnectionWrapper.InitializeConnection(connection);
		this.table_meta_dataMap = new HashMap<String, HTable>();
		this.table_meta_indexMap = new HashMap<String, HTable>();
		this.table_meta_splitMap = new HashMap<String, HTable>();
		this.table_meta_cacheMap = new HashMap<String, HTable>();
	}

	private String get_new_principal(String principal) {
		String result = null;
		try {
			int index_ = principal.indexOf("_HOST");
			if (index_ != -1) {
				String prefix = principal.substring(0, index_);
				String suffix = principal.substring(index_ + "_HOST".length());
				if (prefix.charAt(prefix.length() - 1) == '/' && suffix.charAt(0) == '@') {
					result = prefix + InetAddress.getLocalHost().getHostName() + suffix;
				}
			}
		} catch (Exception e) {
			LogUtil.error(e.getMessage());
		}
		return result;
	}

	public void Stop() throws Exception {
		this.connection.close();
	}

	public HTable Get_table_meta_data(String namespace) throws Exception {
		if (!this.table_meta_dataMap.containsKey(namespace)) {
			HTable table_meta_data = (HTable) this.connection.getTable(TableName.valueOf(namespace, "zillion_meta_data"));
			this.table_meta_dataMap.put(namespace, table_meta_data);
		}
		return this.table_meta_dataMap.get(namespace);
	}

	public HTable Get_table_meta_split(String namespace) throws Exception {
		if (!this.table_meta_splitMap.containsKey(namespace)) {
			if (this.admin.tableExists(TableName.valueOf(namespace, "zillion_meta_split"))) {
				HTable table_meta_split = (HTable) this.connection.getTable(TableName.valueOf(namespace, "zillion_meta_split"));
				this.table_meta_splitMap.put(namespace, table_meta_split);
			}
		}
		return this.table_meta_splitMap.get(namespace);
	}

	public HTable Get_table_meta_cache(String namespace) throws Exception {
		if (!this.table_meta_cacheMap.containsKey(namespace)) {
			if (this.admin.tableExists(TableName.valueOf(namespace, "zillion_meta_cache"))) {
				HTable table_meta_cache = (HTable) this.connection.getTable(TableName.valueOf(namespace, "zillion_meta_cache"));
				this.table_meta_cacheMap.put(namespace, table_meta_cache);
			}
		}
		return this.table_meta_cacheMap.get(namespace);
	}

	public HTable Get_table_meta_index(String namespace) throws Exception {
		if (!this.table_meta_indexMap.containsKey(namespace)) {
			HTable table_meta_index = (HTable) this.connection.getTable(TableName.valueOf(namespace, "zillion_meta_index"));
			this.table_meta_indexMap.put(namespace, table_meta_index);
		}
		return this.table_meta_indexMap.get(namespace);
	}

	public List<String> ori_namespaceList() throws Exception {
		List<String> result = new ArrayList<String>();
		NamespaceDescriptor[] namespaces = admin.listNamespaceDescriptors();
		for (NamespaceDescriptor namespace : namespaces) {
			String namespace_name = namespace.getName();
			if (ZillionConstant.ReservedNamespaceMap.containsKey(namespace_name)) {
				continue;
			}
			result.add(namespace_name);
		}
		return result;
	}

	public List<String> ori_tableList(String namespace) throws Exception {
		List<String> result = new ArrayList<String>();
		HTableDescriptor[] descs = this.admin.listTableDescriptorsByNamespace(namespace);
		for (HTableDescriptor desc : descs) {
			String table_name = desc.getTableName().getNameAsString().substring(namespace.length() + 1);
			result.add(table_name);
		}
		return result;
	}

	public boolean ori_tableExists(String namespace, String ori_tableName) throws Exception {
		return admin.tableExists(TableName.valueOf(namespace, ori_tableName));
	}

	public void ori_drop_table(String namespace, String ori_tableName) throws Exception {
		HBaseAdminUtil.deleteTable(this.admin, this.admin_bak, TableName.valueOf(namespace, ori_tableName));
	}

	public void namespace_create(String namespace) throws Exception {
		HBaseAdminUtil.CreateNamespace(this.admin, this.admin_bak, namespace);
	}

	public void namespace_drop(String namespace) throws Exception {
		HBaseAdminUtil.DeleteNamespace(this.admin, this.admin_bak, namespace);
	}

	public void meta_manage_create_data(String namespace) throws Exception {
		HBaseAdminUtil.createTable(this.admin, this.admin_bak, TableName.valueOf(namespace, "zillion_meta_data"), ZillionConstant.familyNames, null,
				false, Compression.Algorithm.NONE);
	}

	public void meta_manage_create_index(String namespace) throws Exception {
		HBaseAdminUtil.createTable(this.admin, this.admin_bak, TableName.valueOf(namespace, "zillion_meta_index"), ZillionConstant.familyNames, null,
				false, Compression.Algorithm.NONE);
	}

	public void meta_manage_make_sure_split(String namespace) throws Exception {
		TableName zillion_meta_split = TableName.valueOf(namespace, "zillion_meta_split");
		if (!HBaseAdminUtil.tableExists(this.admin, zillion_meta_split)) {
			HBaseAdminUtil.createTable(this.admin, this.admin_bak, zillion_meta_split, ZillionConstant.familyNames, null, false,
					Compression.Algorithm.NONE);
		}
	}

	public void meta_manage_make_sure_cache(String namespace) throws Exception {
		TableName zillion_meta_cache = TableName.valueOf(namespace, "zillion_meta_cache");
		if (!HBaseAdminUtil.tableExists(this.admin, zillion_meta_cache)) {
			HBaseAdminUtil.createTable(this.admin, this.admin_bak, zillion_meta_cache, ZillionConstant.familyNames, null, false,
					Compression.Algorithm.NONE);
		}
	}

	public void meta_manage_drop_data(String namespace) throws Exception {
		HBaseAdminUtil.deleteTable(this.admin, this.admin_bak, TableName.valueOf(namespace, "zillion_meta_data"));
	}

	public void meta_manage_drop_index(String namespace) throws Exception {
		HBaseAdminUtil.deleteTable(this.admin, this.admin_bak, TableName.valueOf(namespace, "zillion_meta_index"));
	}

	public void meta_manage_drop_split(String namespace) throws Exception {
		HBaseAdminUtil.deleteTable(this.admin, this.admin_bak, TableName.valueOf(namespace, "zillion_meta_split"));
	}

	public void meta_query_insert_data(String namespace, String tableName, JSONObject content, HashObject hashObject) throws Exception {
		HTable table_meta_data = this.Get_table_meta_data(namespace);
		Put put_table = new Put(ByteToObject.toBytes(tableName));
		put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("q"),
				ByteToObject.toBytes(JSON.toJSONString(content, SerializerFeature.WriteMapNullValue)));
		put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.insert_count)));
		put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.delete_count)));
		put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"), hashObject.hash_code);
		table_meta_data.put(put_table);
	}

	public void meta_query_insert_index(String namespace, String tableName, String indexName, JSONArray indexColumns, HashObject hashObject)
			throws Exception {
		HTable table_meta_index = this.Get_table_meta_index(namespace);
		Put put_index = new Put(ByteToObject.toBytes(tableName + "." + indexName));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("q"),
				ByteToObject.toBytes(JSON.toJSONString(indexColumns, SerializerFeature.WriteMapNullValue)));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.insert_count)));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.delete_count)));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"), hashObject.hash_code);
		table_meta_index.put(put_index);
	}

	public void meta_query_insert_split(String namespace, String tableName, HashObject hashObject) throws Exception {
		HTable table_meta_split = this.Get_table_meta_split(namespace);
		Put put_table = new Put(ByteToObject.toBytes(tableName));
		put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.insert_count)));
		put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.delete_count)));
		put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"), hashObject.hash_code);
		table_meta_split.put(put_table);
	}

	public void meta_query_delete_data(String namespace, String tableName) throws Exception {
		HTable table_meta_data = this.Get_table_meta_data(namespace);
		Delete delete_table = new Delete(ByteToObject.toBytes(tableName));
		delete_table.addFamily(ByteToObject.toBytes("f"));
		table_meta_data.delete(delete_table);
	}

	public void meta_query_delete_index(String namespace, String tableName, String indexName) throws Exception {
		HTable table_meta_index = this.Get_table_meta_index(namespace);
		Delete delete_index = new Delete(ByteToObject.toBytes(tableName + "." + indexName));
		delete_index.addFamily(ByteToObject.toBytes("f"));
		table_meta_index.delete(delete_index);
	}

	public void meta_query_delete_split(String namespace, String tableName) throws Exception {
		Delete delete_table = new Delete(ByteToObject.toBytes(tableName));
		delete_table.addFamily(ByteToObject.toBytes("f"));
		HTable table_meta_split = this.Get_table_meta_split(namespace);
		table_meta_split.delete(delete_table);
	}

	public void meta_query_update_data(String namespace, String tableName, JSONObject content) throws Exception {
		HTable table_meta_data = this.Get_table_meta_data(namespace);
		Put put_table = new Put(ByteToObject.toBytes(tableName));
		put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("q"),
				ByteToObject.toBytes(JSON.toJSONString(content, SerializerFeature.WriteMapNullValue)));
		table_meta_data.put(put_table);
	}

	public void meta_query_update_data(String namespace, String tableName, HashObject hashObject) throws Exception {
		HTable table_meta_data = this.Get_table_meta_data(namespace);
		Put put_table = new Put(ByteToObject.toBytes(tableName));
		put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.insert_count)));
		put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.delete_count)));
		put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"), hashObject.hash_code);
		table_meta_data.put(put_table);
	}

	public void meta_query_update_index(String namespace, String tableName, String indexName, HashObject hashObject) throws Exception {
		HTable table_meta_index = this.Get_table_meta_index(namespace);
		Put put_index = new Put(ByteToObject.toBytes(tableName + "." + indexName));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.insert_count)));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.delete_count)));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"), hashObject.hash_code);
		table_meta_index.put(put_index);
	}

	public void meta_query_update_split_data(String namespace, String tableName, String child_name, HashObject hashObject) throws Exception {
		HTable table_meta_split = this.Get_table_meta_split(namespace);
		Put put_index = new Put(ByteToObject.toBytes(tableName + "_" + child_name));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.insert_count)));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.delete_count)));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"), hashObject.hash_code);
		table_meta_split.put(put_index);
	}

	public void meta_query_update_split_index(String namespace, String tableName, String indexName, String child_name, HashObject hashObject)
			throws Exception {
		HTable table_meta_split = this.Get_table_meta_split(namespace);
		Put put_index = new Put(ByteToObject.toBytes(tableName + "." + indexName + "_" + child_name));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.insert_count)));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"),
				ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.delete_count)));
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"), hashObject.hash_code);
		table_meta_split.put(put_index);
	}

	public boolean table_split_index_exist(String namespace, String table_name, String index_name, String child_name) throws Exception {
		String tableName = ZillionConstant.prefix_index + (table_name + "." + index_name) + "_" + child_name;
		return this.ori_tableExists(namespace, tableName);
	}

	public void table_manage_create_data(String namespace, ZillionTableSchema schema) throws Exception {
		String tableName = schema.table_name;
		boolean inMemory = false;
		if (schema.resident_memory) {
			inMemory = true;
		}
		HBaseAdminUtil.createTable(this.admin, this.admin_bak, TableName.valueOf(namespace, ZillionConstant.prefix_data + tableName),
				ZillionConstant.familyNames, schema.splitKeys, inMemory, schema.compress ? Compression.Algorithm.LZ4 : Compression.Algorithm.NONE);
	}

	public void table_manage_create_index(String namespace, String tableName, ZillionTableIndex schema_index) throws Exception {
		String indexName = schema_index.index_name;
		boolean inMemory = false;
		if (schema_index.schema.resident_memory) {
			inMemory = true;
		}
		HBaseAdminUtil.createTable(this.admin, this.admin_bak,
				TableName.valueOf(namespace, ZillionConstant.prefix_index + (tableName + "." + indexName)), ZillionConstant.familyNames, null,
				inMemory, schema_index.schema.compress ? Compression.Algorithm.LZ4 : Compression.Algorithm.NONE);
	}

	public void table_manage_create_split_data(String namespace, String tableName, String child_name, ZillionTableSchema schema) throws Exception {
		String ori_tableName = ZillionConstant.prefix_data + tableName + "_" + child_name;
		TableName tableNameObject = TableName.valueOf(namespace, ori_tableName);
		boolean inMemory = false;
		if (schema.resident_memory) {
			inMemory = true;
		}
		HBaseAdminUtil.createTable(this.admin, this.admin_bak, tableNameObject, ZillionConstant.familyNames, schema.splitKeys, inMemory,
				schema.compress ? Compression.Algorithm.LZ4 : Compression.Algorithm.NONE);
	}

	public void table_manage_create_split_index(String namespace, String tableName, String indexName, String child_name,
			ZillionTableIndex schema_index) throws Exception {
		String ori_tableName = ZillionConstant.prefix_index + (tableName + "." + indexName) + "_" + child_name;
		TableName tableNameObject = TableName.valueOf(namespace, ori_tableName);
		boolean inMemory = false;
		if (schema_index.schema.resident_memory) {
			inMemory = true;
		}
		HBaseAdminUtil.createTable(this.admin, this.admin_bak, tableNameObject, ZillionConstant.familyNames, null, inMemory,
				schema_index.schema.compress ? Compression.Algorithm.LZ4 : Compression.Algorithm.NONE);
	}

	public void table_manage_drop_data(String namespace, String tableName) throws Exception {
		HBaseAdminUtil.deleteTable(this.admin, this.admin_bak, TableName.valueOf(namespace, ZillionConstant.prefix_data + tableName));
	}

	public void table_manage_drop_index(String namespace, String tableName, String indexName) throws Exception {
		HBaseAdminUtil.deleteTable(this.admin, this.admin_bak,
				TableName.valueOf(namespace, ZillionConstant.prefix_index + (tableName + "." + indexName)));
	}

	public void table_manage_drop_split_data(String namespace, String tableName, String child_name) throws Exception {
		String tableNameString = ZillionConstant.prefix_data + tableName + "_" + child_name;
		TableName tableNameObject = TableName.valueOf(namespace, tableNameString);
		HBaseAdminUtil.deleteTable(this.admin, this.admin_bak, tableNameObject);
	}

	public void table_manage_drop_split_index(String namespace, String tableName, String indexName, String child_name) throws Exception {
		String tableNameString = ZillionConstant.prefix_index + (tableName + "." + indexName) + "_" + child_name;
		TableName tableNameObject = TableName.valueOf(namespace, tableNameString);
		HBaseAdminUtil.deleteTable(this.admin, this.admin_bak, tableNameObject);
	}

	public void table_manage_alter_add_column(String namespace, String ori_tableName, List<ZillionTableColumn> ztcList) throws Exception {
		// TODO
	}

	public void table_manage_alter_drop_column(String namespace, String ori_tableName, List<String> columnList) throws Exception {
		// TODO
	}

	public void table_manage_alter_compression(String namespace, String ori_tableName, boolean enable) throws Exception {
		HBaseAdminUtil.modifyColumn_compression(admin, admin_bak, TableName.valueOf(namespace, ori_tableName), ZillionConstant.familyNames,
				enable ? Compression.Algorithm.LZ4 : Compression.Algorithm.NONE);
	}

	public void table_manage_alter_in_memory(String namespace, String ori_tableName, boolean enable) throws Exception {
		HBaseAdminUtil.modifyColumn_in_memory(admin, admin_bak, TableName.valueOf(namespace, ori_tableName), ZillionConstant.familyNames, enable);
	}

	public void meta_read(String namespace, List<String> tableNameList, List<String> tableContentList, List<HashObject> tableHashList,
			List<String> indexNameList, List<String> indexContentList, List<HashObject> indexHashList, List<String> splitNameList,
			List<HashObject> splitHashList) throws Exception {
		HTable table_meta_data = (HTable) connection.getTable(TableName.valueOf(namespace, "zillion_meta_data"));
		HTable table_meta_index = (HTable) connection.getTable(TableName.valueOf(namespace, "zillion_meta_index"));

		table_meta_dataMap.put(namespace, table_meta_data);
		table_meta_indexMap.put(namespace, table_meta_index);

		{
			Scan scan = new Scan();
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("q"));
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"));
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"));
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"));
			ResultScanner scanner = table_meta_data.getScanner(scan);
			while (true) {
				Result res = scanner.next();
				if (res == null) {
					break;
				}
				String row = ByteToObject.toString(res.getRow());
				String q = ByteToObject.toString(res.getValue(ByteToObject.toBytes("f"), ByteToObject.toBytes("q")));
				HashObject hashObject = new HashObject();
				hashObject.insert_count = Long.parseLong(ByteToObject.toString(res.getValue(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"))));
				hashObject.delete_count = Long.parseLong(ByteToObject.toString(res.getValue(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"))));
				hashObject.hash_code = res.getValue(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"));
				tableNameList.add(row);
				tableContentList.add(q);
				tableHashList.add(hashObject);
			}
			scanner.close();
		}
		{
			Scan scan = new Scan();
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("q"));
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"));
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"));
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"));
			ResultScanner scanner = table_meta_index.getScanner(scan);
			while (true) {
				Result res = scanner.next();
				if (res == null) {
					break;
				}
				String row = ByteToObject.toString(res.getRow());
				String q = ByteToObject.toString(res.getValue(ByteToObject.toBytes("f"), ByteToObject.toBytes("q")));
				HashObject hashObject = new HashObject();
				hashObject.insert_count = Long.parseLong(ByteToObject.toString(res.getValue(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"))));
				hashObject.delete_count = Long.parseLong(ByteToObject.toString(res.getValue(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"))));
				hashObject.hash_code = res.getValue(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"));
				indexNameList.add(row);
				indexContentList.add(q);
				indexHashList.add(hashObject);
			}
			scanner.close();
		}
		if (this.admin.tableExists(TableName.valueOf(namespace, "zillion_meta_split"))) {
			HTable table_meta_split = (HTable) connection.getTable(TableName.valueOf(namespace, "zillion_meta_split"));
			table_meta_splitMap.put(namespace, table_meta_split);
			Scan scan = new Scan();
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"));
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"));
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"));
			ResultScanner scanner = table_meta_split.getScanner(scan);
			while (true) {
				Result res = scanner.next();
				if (res == null) {
					break;
				}
				String row = ByteToObject.toString(res.getRow());
				HashObject hashObject = new HashObject();
				hashObject.insert_count = Long.parseLong(ByteToObject.toString(res.getValue(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"))));
				hashObject.delete_count = Long.parseLong(ByteToObject.toString(res.getValue(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"))));
				hashObject.hash_code = res.getValue(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"));
				splitNameList.add(row);
				splitHashList.add(hashObject);
			}
			scanner.close();
		}
	}

	public int clear_table_split(String namespace, ZillionTableSchema schema, List<String> query_monthList, ZKWrapper ZK) throws Exception {
		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		Stat stat = new Stat();
		int count = 0;
		for (int index_split = 0; index_split < query_monthList.size(); index_split++) {
			String query_month = query_monthList.get(index_split);
			byte[] hash = ZK.getData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name + "/" + query_month, false, stat);
			HashObject hashObject = HashUtil.Parse(hash);
			hashObject.delete_count = hashObject.insert_count - hashObject.delete_count;
			hashObject.insert_count = 0;
			count += hashObject.delete_count;
			ZK.create(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + schema.table_name + "/" + query_month + "/" + "tmp-",
					hashObject.toBytes(), CreateMode.PERSISTENT_SEQUENTIAL);
			for (ZillionTableIndex schema_index : schema.Indexes) {
				String indexName = schema_index.index_name;
				ZK.create(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (schema.table_name + "." + indexName) + "/" + query_month + "/"
						+ "tmp-", hashObject.toBytes(), CreateMode.PERSISTENT_SEQUENTIAL);
			}
		}
		for (int index_wait = 0; index_wait < 15; index_wait++) {
			try {
				Thread.sleep(1000L);
			} catch (Exception e) {
				e.printStackTrace();
			}
			boolean hash_tmp_finish = true;
			for (int index_split = 0; index_split < query_monthList.size(); index_split++) {
				String query_month = query_monthList.get(index_split);
				List<String> children = ZK.getChildren(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + schema.table_name + "/" + query_month,
						false);
				if (children.size() > 0) {
					hash_tmp_finish = false;
				}
				for (ZillionTableIndex schema_index : schema.Indexes) {
					String indexName = schema_index.index_name;
					children = ZK.getChildren(
							zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (schema.table_name + "." + indexName) + "/" + query_month, false);
					if (children.size() > 0) {
						hash_tmp_finish = false;
						break;
					}
				}
				if (!hash_tmp_finish) {
					break;
				}
			}
			if (hash_tmp_finish) {
				try {
					Thread.sleep(1000L);
				} catch (Exception e) {
					e.printStackTrace();
				}
				break;
			}
		}
		for (int index_split = 0; index_split < query_monthList.size(); index_split++) {
			String query_month = query_monthList.get(index_split);
			ZKHelper.recursiveChildren(ZK, zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + schema.table_name + "/" + query_month);
			for (ZillionTableIndex schema_index : schema.Indexes) {
				String indexName = schema_index.index_name;
				ZKHelper.recursiveChildren(ZK,
						zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (schema.table_name + "." + indexName) + "/" + query_month);
			}
		}

		HashObject hashObject = new HashObject();
		byte[] hash = hashObject.toBytes();
		for (int index_split = 0; index_split < query_monthList.size(); index_split++) {
			String query_month = query_monthList.get(index_split);

			ZK.setData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name + "/" + query_month, hash, -1);
			HTable table_meta_split = this.Get_table_meta_split(namespace);
			Put put_table = new Put(ByteToObject.toBytes(schema.table_name + "_" + query_month));
			put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"),
					ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.insert_count)));
			put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"),
					ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.delete_count)));
			put_table.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"), hashObject.hash_code);
			table_meta_split.put(put_table);
			HBaseAdminUtil.clearTable(this.admin, this.admin_bak,
					TableName.valueOf(namespace, ZillionConstant.prefix_data + schema.table_name + "_" + query_month));
			for (ZillionTableIndex schema_index : schema.Indexes) {
				String indexName = schema_index.index_name;
				ZK.setData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (schema.table_name + "." + indexName) + "/" + query_month, hash,
						-1);
				Put put_index = new Put(ByteToObject.toBytes((schema.table_name + "." + indexName) + "_" + query_month));
				put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hi"),
						ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.insert_count)));
				put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hd"),
						ByteToObject.toBytes(FormatStaticUtil.toString(hashObject.delete_count)));
				put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("hc"), hashObject.hash_code);
				table_meta_split.put(put_index);
				HBaseAdminUtil.clearTable(this.admin, this.admin_bak,
						TableName.valueOf(namespace, ZillionConstant.prefix_index + (schema.table_name + "." + indexName) + "_" + query_month));
			}
		}
		return count;
	}

	public int clear_table_not_split(String namespace, ZillionTableSchema schema, ZKWrapper ZK) throws Exception {
		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		Stat stat = new Stat();
		int count = 0;
		{
			byte[] hash = ZK.getData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name, false, stat);
			HashObject hashObject = HashUtil.Parse(hash);
			hashObject.delete_count = hashObject.insert_count - hashObject.delete_count;
			hashObject.insert_count = 0;
			count += hashObject.delete_count;
			ZK.create(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + schema.table_name + "/" + "tmp-", hashObject.toBytes(),
					CreateMode.PERSISTENT_SEQUENTIAL);
			for (ZillionTableIndex schema_index : schema.Indexes) {
				String indexName = schema_index.index_name;
				ZK.create(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (schema.table_name + "." + indexName) + "/" + "tmp-",
						hashObject.toBytes(), CreateMode.PERSISTENT_SEQUENTIAL);
			}
		}
		for (int index_wait = 0; index_wait < 15; index_wait++) {
			try {
				Thread.sleep(1000L);
			} catch (Exception e) {
				e.printStackTrace();
			}
			boolean hash_tmp_finish = true;
			List<String> children = ZK.getChildren(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + schema.table_name, false);
			if (children.size() > 0) {
				hash_tmp_finish = false;
			}
			for (ZillionTableIndex schema_index : schema.Indexes) {
				String indexName = schema_index.index_name;
				children = ZK.getChildren(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (schema.table_name + "." + indexName), false);
				if (children.size() > 0) {
					hash_tmp_finish = false;
					break;
				}
			}
			if (hash_tmp_finish) {
				try {
					Thread.sleep(1000L);
				} catch (Exception e) {
					e.printStackTrace();
				}
				break;
			}
		}
		{
			ZKHelper.recursiveChildren(ZK, zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + schema.table_name);
			for (ZillionTableIndex schema_index : schema.Indexes) {
				String indexName = schema_index.index_name;
				ZKHelper.recursiveChildren(ZK, zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (schema.table_name + "." + indexName));
			}
		}

		{
			HashObject hashObject = new HashObject();
			byte[] hash = hashObject.toBytes();
			ZK.setData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name, hash, -1);
			HBaseAdminUtil.clearTable(this.admin, this.admin_bak, TableName.valueOf(namespace, ZillionConstant.prefix_data + schema.table_name));
			for (ZillionTableIndex schema_index : schema.Indexes) {
				String indexName = schema_index.index_name;
				ZK.setData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (schema.table_name + "." + indexName), hash, -1);
				HBaseAdminUtil.clearTable(this.admin, this.admin_bak,
						TableName.valueOf(namespace, ZillionConstant.prefix_index + (schema.table_name + "." + indexName)));
			}
		}
		return count;
	}

	private static void generate_ScanColumns(ZillionTableSchema schema, JSONObject Criteria, Map<String, Boolean> ScanColumnsMap) {
		Iterator<String> keyIter = Criteria.keySet().iterator();
		while (keyIter.hasNext()) {
			String key = keyIter.next();
			ScanColumnsMap.put(key, true);
		}
	}

	public void scan(SchemaRepositoryContainer container, ZillionTableSchema schema, SchemaWrapper schemaWrapper, JSONArray ReturnColumns,
			ZillionCriteriaGroup group, String QueryType, JSONArray content, ZillionQueryTmp zqt, ZipArchiveOutputStream out, AsyncJob job,
			boolean SortInDB, JSONArray OrderBy, long Limit_Skip, long Limit_Count, ZillionQueryResult result, JSONObject Set,
			JSONArray ConflictArray) throws Exception {
		// ScanColumnsMapСֶµ
		Map<String, Boolean> ScanColumnsMap = new HashMap<String, Boolean>();
		generate_ScanColumns(schema, group.otherCriteria, ScanColumnsMap);
		boolean SortAsc = true;
		if (QueryType.equals("delete")) {
			for (ZillionTableIndex index_schama : schema.Indexes) {
				for (String keyItem : index_schama.Key) {
					ScanColumnsMap.put(keyItem, true);
				}
			}
		} else if (QueryType.equals("update")) {
			boolean update_index = false;
			for (ZillionTableIndex index_schama : schema.Indexes) {
				for (String keyItem : index_schama.Key) {
					if (Set.containsKey(keyItem)) {
						update_index = true;
						break;
					}
				}
				if (update_index) {
					break;
				}
			}
			if (update_index) {
				for (int i = 0; i < schema.OtherColumn.length; i++) {
					String column_name = schema.OtherColumn[i];
					ScanColumnsMap.put(column_name, true);
				}
			} else {
				for (ZillionTableIndex index_schama : schema.Indexes) {
					for (String keyItem : index_schama.Key) {
						ScanColumnsMap.put(keyItem, true);
					}
				}
			}
		} else if (QueryType.equals("select") || QueryType.equals("export")) {
			if (ReturnColumns == null) {
				ReturnColumns = new JSONArray();
				Iterator<String> keyIter = schema.ColumnMap.keySet().iterator();
				while (keyIter.hasNext()) {
					String key = keyIter.next();
					ReturnColumns.add(key);
				}
			}
			String[] keysInner;
			if (group.index_index == -1) {
				keysInner = schema.Key;
			} else {
				keysInner = schema.Indexes.get(group.index_index).Key;
			}
			for (int index_rcs = 0; index_rcs < ReturnColumns.size(); index_rcs++) {
				String key = (String) ReturnColumns.get(index_rcs);
				boolean key_exist = false;
				for (String keyInner : keysInner) {
					if (keyInner.equals(key)) {
						key_exist = true;
						break;
					}
				}
				if (!key_exist) {
					ScanColumnsMap.put(key, true);
				}
			}
			if (!SortInDB && OrderBy != null && OrderBy.size() > 0) {
				for (int i = 0; i < OrderBy.size(); i++) {
					JSONObject item = (JSONObject) OrderBy.get(i);
					String key = (String) item.get("Column");
					boolean key_exist = false;
					for (String keyInner : keysInner) {
						if (keyInner.equals(key)) {
							key_exist = true;
							break;
						}
					}
					if (!key_exist) {
						ScanColumnsMap.put(key, true);
					}
				}
			}
			if (SortInDB) {
				if (OrderBy != null && OrderBy.size() > 0) {
					JSONObject item0 = (JSONObject) OrderBy.get(0);
					SortAsc = (Boolean) item0.get("Asc");
				}
			}
		}

		Scan scan = new Scan();
		scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
		if (schema.table_type == null || schema.table_type.equals("index_data") || (schema.table_type.startsWith("split_"))) {
			for (int i = 0; i < schema.OtherColumn.length; i++) {
				String column_name = schema.OtherColumn[i];
				if (ScanColumnsMap.containsKey(column_name)) {
					scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name));
				}
			}
		} else if (schema.table_type.equals("all_column")) {
			String[] Columns = schema.ColumnMap.keySet().toArray(new String[0]);
			for (int ii = 0; ii < Columns.length; ii++) {
				String column_name = Columns[ii];
				scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name));
			}
		}
		if (!SortAsc) {
			scan.setReversed(true);
			scan.setStartRow(group.itemListOfGroup.get(0).bytes_lt);
			scan.setStopRow(group.itemListOfGroup.get(0).bytes_gt);
			if (group.valueFilterList.size() > 0) {
				FilterList filterList = new FilterList(Operator.MUST_PASS_ALL);
				for (Filter filter : group.valueFilterList) {
					filterList.addFilter(filter);
				}
				scan.setFilter(filterList);
			}
		} else {
			Filter rowFilter = new MultiRowRangeFilter(group.rangeList_new);
			if (group.valueFilterList.size() > 0) {
				FilterList filterList = new FilterList(Operator.MUST_PASS_ALL);
				filterList.addFilter(rowFilter);
				for (Filter filter : group.valueFilterList) {
					filterList.addFilter(filter);
				}
				scan.setFilter(filterList);
			} else {
				scan.setFilter(rowFilter);
			}
		}

		ITableWrapper tableWrapper;
		if (group.index_index == -1) {
			tableWrapper = schemaWrapper.dataWrapper;
		} else {
			tableWrapper = schemaWrapper.indexWrapperList.get(group.index_index);
		}
		ResultScanner scanner = ((TableWrapper_HBase) tableWrapper).htable.getScanner(scan);
		JSONArray contentInner = new JSONArray();
		try {
			while (true) {
				Result res = scanner.next();
				if (res == null) {
					break;
				}

				if (HTableUtil.valid(schema, res.listCells())) {
					JSONObject resultObject = null;
					if (group.scObject != null) {
						if (group.index_index == -1) {
							resultObject = HTableUtil.Convert(schema, res.getRow(), res.listCells());
						} else {
							ZillionTableIndex schema_index = schema.Indexes.get(group.index_index);
							resultObject = HTableUtil.Convert_index_data(schema, schema_index, res.getRow(), res.listCells());
						}
						if (!group.scObject.match(resultObject)) {
							continue;
						}
					}

					zqt.scanned_count++;
					zqt.count_wait_tmp++;
					if (zqt.count_wait_tmp >= AsyncUtil.count_wait) {
						job.Add(zqt.count_wait_tmp);
						zqt.count_wait_tmp = 0L;
					}
					if (zqt.scanned_count > Limit_Skip && zqt.scanned_count <= Limit_Skip + Limit_Count) {
						result.count++;
						if (QueryType.equals("select_count")) {
						} else {
							if (resultObject == null) {
								if (group.index_index == -1) {
									resultObject = HTableUtil.Convert(schema, res.getRow(), res.listCells());
								} else {
									ZillionTableIndex schema_index = schema.Indexes.get(group.index_index);
									resultObject = HTableUtil.Convert_index_data(schema, schema_index, res.getRow(), res.listCells());
								}
							}
							if (QueryType.equals("select")) {
								if (job.brs != null) {
									job.brs.TryPush(resultObject);
								} else {
									content.add(resultObject);
								}
							} else {
								contentInner.add(resultObject);
							}
						}

						if (QueryType.equals("delete")) {
							if (contentInner.size() >= ZillionConstant.query_group_size) {
								batch_process(container, schema, schemaWrapper, QueryType, contentInner, Set, false, ConflictArray);
								contentInner.clear();
							}
						}
						if (QueryType.equals("export")) {
							if (contentInner.size() >= ZillionConstant.export_batch_size) {
								out.putArchiveEntry(new ZipArchiveEntry(zqt.df.format(zqt.zip_index++) + ".json"));
								byte[] bytes = JSON.toJSONString(contentInner, SerializerFeature.WriteMapNullValue).getBytes("UTF-8");
								out.write(bytes);
								out.closeArchiveEntry();
								contentInner.clear();
							}
						}
					} else if (zqt.scanned_count > Limit_Skip + Limit_Count) {
						zqt.scanned_finish = true;
						break;
					}
				}
			}
			scanner.close();
		} catch (Exception e) {
			LogUtil.warn(e);
			try {
				scanner.close();
			} catch (Exception e1) {
				// e1.printStackTrace();
			}
			if (e instanceof java.lang.NullPointerException) {
			} else {
				throw e;
			}
		}
		if (zqt.count_wait_tmp > 0) {
			job.Add(zqt.count_wait_tmp);
			zqt.count_wait_tmp = 0L;
		}
		if (QueryType.equals("delete") || QueryType.equals("update")) {
			if (contentInner.size() > 0) {
				batch_process(container, schema, schemaWrapper, QueryType, contentInner, Set, false, ConflictArray);
				contentInner.clear();
			}
		}
		if (QueryType.equals("export")) {
			if (contentInner.size() > 0) {
				out.putArchiveEntry(new ZipArchiveEntry(zqt.df.format(zqt.zip_index++) + ".json"));
				byte[] bytes = JSON.toJSONString(contentInner, SerializerFeature.WriteMapNullValue).getBytes("UTF-8");
				out.write(bytes);
				out.closeArchiveEntry();
				contentInner.clear();
			}
		}
	}

	public void batch_process(SchemaRepositoryContainer container, ZillionTableSchema schema, SchemaWrapper wrapper, String QueryType,
			JSONArray InsertObjects_all, JSONObject Set, boolean is_put, JSONArray ConflictArray) throws Exception {
		if (!QueryType.equals("insert") && !QueryType.equals("delete") && !QueryType.equals("update")) {
			return;
		}
		if (InsertObjects_all.size() == 0) {
			return;
		}

		if (schema.strong_consistency) {
			JSONArray InsertObjects_lock = new JSONArray();
			ShardedJedis jedis = null;
			HTable htable = null;
			if (ZillionConstant.strong_consistency_cache_redis && container.redisConfig != null) {
				jedis = container.redisPool.getResource();
			} else {
				htable = this.Get_table_meta_cache(schema.Namespace);
			}
			// ݵԻ
			if (ZillionConstant.strong_consistency_cache_redis && container.redisConfig != null) {
				String[] keys = new String[InsertObjects_all.size()];
				for (int i = 0; i < InsertObjects_all.size(); i++) {
					JSONObject InsertObject = (JSONObject) InsertObjects_all.get(i);
					byte[] row_bytes = HTableUtil.BuildKey(schema, InsertObject);
					keys[i] = HexToByte.byteToHex(row_bytes);
				}
				List<String> rsmap = jedis.hmget(schema.Namespace + ":" + schema.table_name, keys);
				for (int i = 0; i < InsertObjects_all.size(); i++) {
					JSONObject InsertObject = (JSONObject) InsertObjects_all.get(i);
					if (rsmap.get(i) != null) {
						ConflictArray.add(InsertObject);
					} else {
						InsertObjects_lock.add(InsertObject);
					}
				}
			} else {
				List<Get> batch = new ArrayList<Get>();
				for (int i = 0; i < InsertObjects_all.size(); i++) {
					JSONObject InsertObject = (JSONObject) InsertObjects_all.get(i);
					byte[] table_bytes = ByteToObject.toBytes(schema.table_name);
					byte[] row_bytes = HTableUtil.BuildKey(schema, InsertObject);
					byte[] real_bytes = new byte[table_bytes.length + row_bytes.length];
					System.arraycopy(table_bytes, 0, real_bytes, 0, table_bytes.length);
					System.arraycopy(row_bytes, 0, real_bytes, table_bytes.length, row_bytes.length);
					Get get = new Get(real_bytes);
					get.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("QueryType"));
					batch.add(get);
				}
				Result[] results = htable.get(batch);
				for (int i = 0; i < InsertObjects_all.size(); i++) {
					JSONObject InsertObject = (JSONObject) InsertObjects_all.get(i);
					Result res = results[i];
					if (res != null && res.getRow() != null) {
						ConflictArray.add(InsertObject);
					} else {
						InsertObjects_lock.add(InsertObject);
					}
				}
			}

			if (InsertObjects_lock.size() > 0) {
				// 
				List<ZKNodeUseLock> nodeList = new ArrayList<ZKNodeUseLock>();
				for (int index_object = 0; index_object < InsertObjects_lock.size(); index_object++) {
					JSONObject InsertObject = (JSONObject) InsertObjects_lock.get(index_object);
					byte[] keyBytes = HTableUtil.BuildKey(schema, InsertObject);
					String node = HexToByte.byteToHex(keyBytes);
					ZKNodeUseLock ZKLockNode = new ZKNodeUseLock(node);
					nodeList.add(ZKLockNode);
				}
				ZKNodeBatch ZKNodeBatch = new ZKNodeBatch(schema.Namespace, schema.table_name, nodeList);
				// ȡmeta
				ZKLockBatch.Create(container, schema.Namespace, schema.table_name, ZKNodeBatch);
				List<ZKNodeUseLock> success_pathList = new ArrayList<ZKNodeUseLock>();
				List<ZKNodeUseLock> failure_pathList = new ArrayList<ZKNodeUseLock>();
				try {
					ZKLockBatch.WaitOnce(container, schema.Namespace, schema.table_name, ZKNodeBatch);
					if (ZKNodeBatch.processed) {
						// ס
						JSONArray InsertObjects = new JSONArray();
						for (int index_object = 0; index_object < InsertObjects_lock.size(); index_object++) {
							JSONObject InsertObject = (JSONObject) InsertObjects_lock.get(index_object);
							ZKNodeUseLock node = nodeList.get(index_object);
							if (node.success) {
								InsertObjects.add(InsertObject);
								success_pathList.add(node);
							} else {
								ConflictArray.add(InsertObject);
								failure_pathList.add(node);
							}
						}

						if (InsertObjects.size() > 0) {
							List<byte[]> real_bytesList = new ArrayList<byte[]>();
							// ݵԲ
							if (ZillionConstant.strong_consistency_cache_redis && container.redisConfig != null) {
								Map<String, String> map = new HashMap<String, String>();
								for (int i = 0; i < InsertObjects.size(); i++) {
									JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
									byte[] row_bytes = HTableUtil.BuildKey(schema, InsertObject);
									String key = HexToByte.byteToHex(row_bytes);
									JSONObject put = new JSONObject();
									put.put("QueryType", QueryType);
									put.put("InsertObject", InsertObject);
									if (Set != null) {
										put.put("Set", Set);
									}
									map.put(key, JSON.toJSONString(put, SerializerFeature.WriteMapNullValue));
								}
								jedis.hmset(schema.Namespace + ":" + schema.table_name, map);
							} else {
								{
									List<Put> batch = new ArrayList<Put>();
									for (int index_object = 0; index_object < InsertObjects.size(); index_object++) {
										JSONObject InsertObject = (JSONObject) InsertObjects.get(index_object);
										byte[] table_bytes = ByteToObject.toBytes(schema.table_name);
										byte[] row_bytes = HTableUtil.BuildKey(schema, InsertObject);
										byte[] real_bytes = new byte[table_bytes.length + row_bytes.length];
										System.arraycopy(table_bytes, 0, real_bytes, 0, table_bytes.length);
										System.arraycopy(row_bytes, 0, real_bytes, table_bytes.length, row_bytes.length);
										real_bytesList.add(real_bytes);
										Put put = new Put(real_bytes);
										put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("QueryType"), Bytes.toBytes(QueryType));
										put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("InsertObject"),
												Bytes.toBytes(JSON.toJSONString(InsertObject, SerializerFeature.WriteMapNullValue)));
										if (Set != null) {
											put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("Set"),
													Bytes.toBytes(JSON.toJSONString(Set, SerializerFeature.WriteMapNullValue)));
										}
										batch.add(put);
									}
									htable.put(batch);
									if (ZillionConstant.disable_AutoFlush) {
										htable.flushCommits();
									}
								}
							}

							// ִݵԲ
							this.batch_processInner(container, schema, wrapper, QueryType, InsertObjects, Set, is_put, true);

							// ɾݵԲ
							if (ZillionConstant.strong_consistency_cache_redis && container.redisConfig != null) {
								String[] keys = new String[InsertObjects.size()];
								for (int i = 0; i < InsertObjects.size(); i++) {
									JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
									byte[] row_bytes = HTableUtil.BuildKey(schema, InsertObject);
									keys[i] = HexToByte.byteToHex(row_bytes);
								}
								jedis.hdel(schema.Namespace + ":" + schema.table_name, keys);
							} else {
								List<Delete> batch = new ArrayList<Delete>();
								for (int index_object = 0; index_object < InsertObjects.size(); index_object++) {
									byte[] real_bytes = real_bytesList.get(index_object);
									Delete put = new Delete(real_bytes);
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("QueryType"));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("InsertObject"));
									if (Set != null) {
										put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("Set"));
									}
									batch.add(put);
								}
								htable.delete(batch);
								if (ZillionConstant.disable_AutoFlush) {
									htable.flushCommits();
								}
							}
						}
					}
				} catch (Exception e) {
					throw e;
				} finally {
					// ͷmeta
					ZKNodeBatch.time = new Date();
					ZKNodeBatch.finish = true;
					if (jedis != null) {
						try {
							jedis.close();
						} catch (Exception e) {
							// e.printStackTrace();
						}
					}
				}
			}
		} else {
			this.batch_processInner(container, schema, wrapper, QueryType, InsertObjects_all, Set, is_put, true);
		}
	}

	public void batch_processInner(SchemaRepositoryContainer container, ZillionTableSchema schema, SchemaWrapper wrapper, String QueryType,
			JSONArray InsertObjects, JSONObject Set, boolean is_put, boolean allow_random_exception) throws Exception {
		if (!QueryType.equals("insert") && !QueryType.equals("delete") && !QueryType.equals("update")) {
			return;
		}
		if (InsertObjects.size() == 0) {
			return;
		}

		Random rand = null;
		if (ZillionConstant.random_sleep_enable || (allow_random_exception && ZillionConstant.random_exception_enable)) {
			rand = new Random();
		}

		Map<String, byte[]> columnBytesMap_new = new HashMap<String, byte[]>();
		if (QueryType.equals("update")) {
			for (int i = 0; i < schema.OtherColumn.length; i++) {
				String column_name = schema.OtherColumn[i];
				ZillionTableColumn column = schema.ColumnMap.get(column_name);
				if (Set.containsKey(column_name)) {
					Object columnObject = Set.get(column_name);
					byte[] column_bytes = HTableUtil.GenerateBytes(column, columnObject);
					columnBytesMap_new.put(column_name, column_bytes);
				}
			}
		}

		List<Delete> batch_delete = new ArrayList<Delete>();
		List<Put> batch_delete_UsePutV = new ArrayList<Put>();
		List<Put> batch_insert = new ArrayList<Put>();
		List<List<Delete>> index_deleteListList = new ArrayList<List<Delete>>();
		List<List<Put>> index_deleteListList_UsePutV = new ArrayList<List<Put>>();
		List<List<Put>> index_insertListList = new ArrayList<List<Put>>();
		for (int i = 0; i < schema.Indexes.size(); i++) {
			index_deleteListList.add(new ArrayList<Delete>());
			index_deleteListList_UsePutV.add(new ArrayList<Put>());
			index_insertListList.add(new ArrayList<Put>());
		}
		for (int index_object = 0; index_object < InsertObjects.size(); index_object++) {
			JSONObject InsertObject = (JSONObject) InsertObjects.get(index_object);
			Map<String, byte[]> columnBytesMap = new HashMap<String, byte[]>();
			Iterator<String> iter = schema.ColumnMap.keySet().iterator();
			while (iter.hasNext()) {
				String column_name = iter.next();
				ZillionTableColumn column = schema.ColumnMap.get(column_name);
				if (InsertObject.containsKey(column_name)) {
					Object columnObject = InsertObject.get(column_name);
					byte[] column_bytes = HTableUtil.GenerateBytes(column, columnObject);
					columnBytesMap.put(column_name, column_bytes);
				}
			}

			Map<String, byte[]> columnBytesMap_tmp;
			if (QueryType.equals("update")) {
				columnBytesMap_tmp = new HashMap<String, byte[]>();
				columnBytesMap_tmp.putAll(columnBytesMap);
				Iterator<String> keyIter = columnBytesMap_new.keySet().iterator();
				while (keyIter.hasNext()) {
					String key = keyIter.next();
					columnBytesMap_tmp.remove(key);
					columnBytesMap_tmp.put(key, columnBytesMap_new.get(key));
				}
			} else {
				columnBytesMap_tmp = columnBytesMap;
			}

			byte[] row_key_bytes = HTableUtil.BuildKey(schema.ColumnMap, schema.Key, columnBytesMap);
			byte[] row_key_bytes_new = HTableUtil.BuildKey(schema.ColumnMap, schema.Key, columnBytesMap_tmp);
			if (QueryType.equals("delete")) {
				if (schema.delete_UsePutV) {
					Put delete_UsePutV = build_data_delete_UsePutV(schema, row_key_bytes);
					batch_delete_UsePutV.add(delete_UsePutV);
				} else {
					Delete delete = new Delete(row_key_bytes);
					delete.addFamily(ByteToObject.toBytes("f"));
					batch_delete.add(delete);
				}
			} else if (QueryType.equals("insert")) {
				Put put = build_data_insert(schema, row_key_bytes, columnBytesMap_tmp, false);
				batch_insert.add(put);
			} else if (QueryType.equals("update")) {
				int compare = Bytes.compareTo(row_key_bytes, row_key_bytes_new);
				if (compare == 0) {
					Put put = build_data_insert(schema, row_key_bytes_new, columnBytesMap_tmp, true);
					batch_insert.add(put);
				} else {
					if (schema.delete_UsePutV) {
						Put delete_UsePutV = build_data_delete_UsePutV(schema, row_key_bytes);
						batch_delete_UsePutV.add(delete_UsePutV);
					} else {
						Delete delete = new Delete(row_key_bytes);
						delete.addFamily(ByteToObject.toBytes("f"));
						batch_delete.add(delete);
					}

					Put put = build_data_insert(schema, row_key_bytes_new, columnBytesMap_tmp, false);
					batch_insert.add(put);
				}
			}

			for (int i = 0; i < schema.Indexes.size(); i++) {
				ZillionTableIndex index = schema.Indexes.get(i);
				List<Delete> index_deleteList = index_deleteListList.get(i);
				List<Put> index_deleteList_UsePutV = index_deleteListList_UsePutV.get(i);
				List<Put> index_insertList = index_insertListList.get(i);
				byte[] index_row_bytes;
				if (schema.table_type == null) {
					byte[] row_key_bytes_index = HTableUtil.BuildKey(schema.ColumnMap, index.Key, columnBytesMap);

					index_row_bytes = new byte[row_key_bytes_index.length + row_key_bytes.length];
					System.arraycopy(row_key_bytes_index, 0, index_row_bytes, 0, row_key_bytes_index.length);
					System.arraycopy(row_key_bytes, 0, index_row_bytes, row_key_bytes_index.length, row_key_bytes.length);
				} else {
					index_row_bytes = HTableUtil.BuildKey(schema.ColumnMap, index.Key, columnBytesMap);
				}

				if (QueryType.equals("delete")) {
					if (schema.delete_UsePutV) {
						Put delete_UsePutV = build_index_delete_UsePutV(schema, index, index_row_bytes);
						index_deleteList_UsePutV.add(delete_UsePutV);
					} else {
						Delete delete_index = new Delete(index_row_bytes);
						delete_index.addFamily(ByteToObject.toBytes("f"));
						index_deleteList.add(delete_index);
					}
				} else if (QueryType.equals("insert")) {
					Put put_index = build_index_insert(schema, index, index_row_bytes, columnBytesMap_tmp, false);
					index_insertList.add(put_index);
				} else if (QueryType.equals("update")) {
					byte[] index_row_bytes_new;
					if (schema.table_type == null) {
						byte[] row_key_bytes_index = HTableUtil.BuildKey(schema.ColumnMap, index.Key, columnBytesMap_tmp);

						index_row_bytes_new = new byte[row_key_bytes_index.length + row_key_bytes_new.length];
						System.arraycopy(row_key_bytes_index, 0, index_row_bytes_new, 0, row_key_bytes_index.length);
						System.arraycopy(row_key_bytes_new, 0, index_row_bytes_new, row_key_bytes_index.length, row_key_bytes_new.length);
					} else {
						index_row_bytes_new = HTableUtil.BuildKey(schema.ColumnMap, index.Key, columnBytesMap_tmp);
					}

					int compare = Bytes.compareTo(index_row_bytes, index_row_bytes_new);
					if (compare == 0) {
						Put put_index = build_index_insert(schema, index, index_row_bytes_new, columnBytesMap_tmp, true);
						index_insertList.add(put_index);
					} else {
						if (schema.delete_UsePutV) {
							Put delete_UsePutV = build_index_delete_UsePutV(schema, index, index_row_bytes);
							index_deleteList_UsePutV.add(delete_UsePutV);
						} else {
							Delete delete_index = new Delete(index_row_bytes);
							delete_index.addFamily(ByteToObject.toBytes("f"));
							index_deleteList.add(delete_index);
						}

						Put put_index = build_index_insert(schema, index, index_row_bytes_new, columnBytesMap_tmp, false);
						index_insertList.add(put_index);
					}
				}
			}
		}

		if (schema.delete_UsePutV) {
			if (batch_delete_UsePutV.size() > 0) {
				if (ZillionConstant.random_sleep_enable) {
					Thread.sleep(rand.nextInt(ZillionConstant.random_sleep_bound));
				}
				if (allow_random_exception && ZillionConstant.random_exception_enable && rand.nextInt(ZillionConstant.random_exception_bound) == 0) {
					throw new MyException("random_exception");
				}
				((TableWrapper_HBase) wrapper.dataWrapper).htable.put(batch_delete_UsePutV);
				if (ZillionConstant.disable_AutoFlush) {
					((TableWrapper_HBase) wrapper.dataWrapper).htable.flushCommits();
				}
			}
		} else {
			if (batch_delete.size() > 0) {
				if (ZillionConstant.random_sleep_enable) {
					Thread.sleep(rand.nextInt(ZillionConstant.random_sleep_bound));
				}
				if (allow_random_exception && ZillionConstant.random_exception_enable && rand.nextInt(ZillionConstant.random_exception_bound) == 0) {
					throw new MyException("random_exception");
				}
				((TableWrapper_HBase) wrapper.dataWrapper).htable.delete(batch_delete);
				if (ZillionConstant.disable_AutoFlush) {
					((TableWrapper_HBase) wrapper.dataWrapper).htable.flushCommits();
				}
			}
		}
		if (batch_insert.size() > 0) {
			if (ZillionConstant.random_sleep_enable) {
				Thread.sleep(rand.nextInt(ZillionConstant.random_sleep_bound));
			}
			if (allow_random_exception && ZillionConstant.random_exception_enable && rand.nextInt(ZillionConstant.random_exception_bound) == 0) {
				throw new MyException("random_exception");
			}
			((TableWrapper_HBase) wrapper.dataWrapper).htable.put(batch_insert);
			if (ZillionConstant.disable_AutoFlush) {
				((TableWrapper_HBase) wrapper.dataWrapper).htable.flushCommits();
			}
		}

		if (schema.delete_UsePutV) {
			for (int i = 0; i < schema.Indexes.size(); i++) {
				ITableWrapper indexTable = wrapper.indexWrapperList.get(i);
				List<Put> index_deleteList_UsePutV = index_deleteListList_UsePutV.get(i);
				if (index_deleteList_UsePutV.size() > 0) {
					if (ZillionConstant.random_sleep_enable) {
						Thread.sleep(rand.nextInt(ZillionConstant.random_sleep_bound));
					}
					if (allow_random_exception && ZillionConstant.random_exception_enable
							&& rand.nextInt(ZillionConstant.random_exception_bound) == 0) {
						throw new MyException("random_exception");
					}
					((TableWrapper_HBase) indexTable).htable.put(index_deleteList_UsePutV);
					if (ZillionConstant.disable_AutoFlush) {
						((TableWrapper_HBase) indexTable).htable.flushCommits();
					}
				}
			}
		} else {
			for (int i = 0; i < schema.Indexes.size(); i++) {
				ITableWrapper indexTable = wrapper.indexWrapperList.get(i);
				List<Delete> index_deleteList = index_deleteListList.get(i);
				if (index_deleteList.size() > 0) {
					if (ZillionConstant.random_sleep_enable) {
						Thread.sleep(rand.nextInt(ZillionConstant.random_sleep_bound));
					}
					if (allow_random_exception && ZillionConstant.random_exception_enable
							&& rand.nextInt(ZillionConstant.random_exception_bound) == 0) {
						throw new MyException("random_exception");
					}
					((TableWrapper_HBase) indexTable).htable.delete(index_deleteList);
					if (ZillionConstant.disable_AutoFlush) {
						((TableWrapper_HBase) indexTable).htable.flushCommits();
					}
				}
			}
		}
		for (int i = 0; i < schema.Indexes.size(); i++) {
			ITableWrapper indexTable = wrapper.indexWrapperList.get(i);
			List<Put> index_insertList = index_insertListList.get(i);
			if (index_insertList.size() > 0) {
				if (ZillionConstant.random_sleep_enable) {
					Thread.sleep(rand.nextInt(ZillionConstant.random_sleep_bound));
				}
				if (allow_random_exception && ZillionConstant.random_exception_enable && rand.nextInt(ZillionConstant.random_exception_bound) == 0) {
					throw new MyException("random_exception");
				}
				((TableWrapper_HBase) indexTable).htable.put(index_insertList);
				if (ZillionConstant.disable_AutoFlush) {
					((TableWrapper_HBase) indexTable).htable.flushCommits();
				}
			}
		}

		if (schema.elastic_search) {
			this.batch_processES(container, schema, wrapper, QueryType, InsertObjects, Set, is_put, allow_random_exception);
		}
	}

	public void batch_processES(SchemaRepositoryContainer container, ZillionTableSchema schema, SchemaWrapper wrapper, String QueryType,
			JSONArray InsertObjects, JSONObject Set, boolean is_put, boolean allow_random_exception) throws Exception {
		if (QueryType.equals("insert")) {
			ElasticSearchUtil.insert(container.esClient, schema.Namespace, schema.table_name, InsertObjects);
		} else if (QueryType.equals("delete")) {
			ElasticSearchUtil.delete(schema, container.esClient, schema.Namespace, schema.table_name, InsertObjects);
		} else if (QueryType.equals("update")) {
			ElasticSearchUtil.update(schema, container.esClient, schema.Namespace, schema.table_name, InsertObjects, Set);
		}
	}

	private static Put build_data_delete_UsePutV(ZillionTableSchema schema, byte[] row_key_bytes) {
		Put delete_UsePutV = new Put(row_key_bytes);
		delete_UsePutV.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"), ByteToObject.toBytes("0"));
		if (schema.table_type == null || schema.table_type.equals("index_data") || (schema.table_type.startsWith("split_"))) {
			for (int i = 0; i < schema.OtherColumn.length; i++) {
				String column_name = schema.OtherColumn[i];
				delete_UsePutV.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name), null);
			}
		} else if (schema.table_type.equals("all_column")) {
			String[] Columns = schema.ColumnMap.keySet().toArray(new String[0]);
			for (int ii = 0; ii < Columns.length; ii++) {
				String column_name = Columns[ii];
				delete_UsePutV.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name), null);
			}
		}
		return delete_UsePutV;
	}

	private static Put build_data_insert(ZillionTableSchema schema, byte[] row_key_bytes_new, Map<String, byte[]> columnBytesMap_tmp,
			boolean is_update) {
		Put put = new Put(row_key_bytes_new);
		if (is_update) {
			put.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("u"), ByteToObject.toBytes("1"));
		}
		put.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"), ByteToObject.toBytes("1"));
		if (schema.table_type == null || schema.table_type.equals("index_data") || (schema.table_type.startsWith("split_"))) {
			for (int ii = 0; ii < schema.OtherColumn.length; ii++) {
				String column_name = schema.OtherColumn[ii];
				ZillionTableColumn column_def = schema.ColumnMap.get(column_name);
				if (columnBytesMap_tmp.containsKey(column_name)) {
					byte[] column_bytes = columnBytesMap_tmp.get(column_name);
					if (column_def.Type == ZillionTableColumnEnum.STRING || column_def.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
						if (column_bytes != null && column_bytes.length == 0) {
							byte[] column_bytesInner = new byte[1];
							column_bytesInner[0] = 0x00;
							column_bytes = column_bytesInner;
						}
					}
					put.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name), column_bytes);
				}
			}
		} else if (schema.table_type.equals("all_column")) {
			String[] Columns = schema.ColumnMap.keySet().toArray(new String[0]);
			for (int ii = 0; ii < Columns.length; ii++) {
				String column_name = Columns[ii];
				ZillionTableColumn column_def = schema.ColumnMap.get(column_name);
				if (columnBytesMap_tmp.containsKey(column_name)) {
					byte[] column_bytes = columnBytesMap_tmp.get(column_name);
					if (column_def.Type == ZillionTableColumnEnum.STRING || column_def.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
						if (column_bytes != null && column_bytes.length == 0) {
							byte[] column_bytesInner = new byte[1];
							column_bytesInner[0] = 0x00;
							column_bytes = column_bytesInner;
						}
					}
					put.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name), column_bytes);
				}
			}
		}
		return put;
	}

	private static Put build_index_delete_UsePutV(ZillionTableSchema schema, ZillionTableIndex index, byte[] index_row_bytes) {
		Put delete_UsePutV = new Put(index_row_bytes);
		delete_UsePutV.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"), ByteToObject.toBytes("0"));
		if (schema.table_type == null) {
		} else if (schema.table_type.equals("index_data") || (schema.table_type.startsWith("split_"))) {
			for (int ii = 0; ii < schema.OtherColumn.length; ii++) {
				String column_name = schema.OtherColumn[ii];
				if (!index.KeyMap.containsKey(column_name)) {
					delete_UsePutV.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name), null);
				}
			}
		} else if (schema.table_type.equals("all_column")) {
			String[] Columns = schema.ColumnMap.keySet().toArray(new String[0]);
			for (int ii = 0; ii < Columns.length; ii++) {
				String column_name = Columns[ii];
				delete_UsePutV.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name), null);
			}
		}
		return delete_UsePutV;
	}

	private static Put build_index_insert(ZillionTableSchema schema, ZillionTableIndex index, byte[] index_row_bytes_new,
			Map<String, byte[]> columnBytesMap_tmp, boolean is_update) {
		Put put_index = new Put(index_row_bytes_new);
		if (is_update) {
			put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("u"), ByteToObject.toBytes("1"));
		}
		put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"), ByteToObject.toBytes("1"));
		if (schema.table_type == null) {
		} else if (schema.table_type.equals("index_data") || (schema.table_type.startsWith("split_"))) {
			for (int ii = 0; ii < schema.OtherColumn.length; ii++) {
				String column_name = schema.OtherColumn[ii];
				ZillionTableColumn column_def = schema.ColumnMap.get(column_name);
				if (!index.KeyMap.containsKey(column_name)) {
					if (columnBytesMap_tmp.containsKey(column_name)) {
						byte[] column_bytes = columnBytesMap_tmp.get(column_name);
						if (column_def.Type == ZillionTableColumnEnum.STRING || column_def.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
							if (column_bytes != null && column_bytes.length == 0) {
								byte[] column_bytesInner = new byte[1];
								column_bytesInner[0] = 0x00;
								column_bytes = column_bytesInner;
							}
						}
						put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name), column_bytes);
					}
				}
			}
		} else if (schema.table_type.equals("all_column")) {
			String[] Columns = schema.ColumnMap.keySet().toArray(new String[0]);
			for (int ii = 0; ii < Columns.length; ii++) {
				String column_name = Columns[ii];
				ZillionTableColumn column_def = schema.ColumnMap.get(column_name);
				if (columnBytesMap_tmp.containsKey(column_name)) {
					byte[] column_bytes = columnBytesMap_tmp.get(column_name);
					if (column_def.Type == ZillionTableColumnEnum.STRING || column_def.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
						if (column_bytes != null && column_bytes.length == 0) {
							byte[] column_bytesInner = new byte[1];
							column_bytesInner[0] = 0x00;
							column_bytes = column_bytesInner;
						}
					}
					put_index.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name), column_bytes);
				}
			}
		}
		return put_index;
	}

	public JSONArray batch_get(ZillionTableSchema schema, ITableWrapper table_wrapper, JSONArray InsertObjects, JSONArray content,
			Map<String, Boolean> ReturnColumnsMap) throws Exception {
		byte[][] row_bytes_array = new byte[InsertObjects.size()][];
		List<Get> batch = new ArrayList<Get>();
		for (int i = 0; i < InsertObjects.size(); i++) {
			JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
			byte[] row_bytes = HTableUtil.BuildKey(schema, InsertObject);
			row_bytes_array[i] = row_bytes;
			Get get = new Get(row_bytes);
			get.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
			for (String column_name : schema.OtherColumn) {
				if (ReturnColumnsMap.containsKey(column_name)) {
					get.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name));
				}
			}
			batch.add(get);
		}
		List<byte[]> exist_row_bytesList = new ArrayList<byte[]>();
		Result[] results = ((TableWrapper_HBase) table_wrapper).htable.get(batch);
		for (Result res : results) {
			if (res != null && res.getRow() != null) {
				if (HTableUtil.valid(schema, res.listCells())) {
					exist_row_bytesList.add(res.getRow());
					JSONObject resultObject = HTableUtil.Convert(schema, res.getRow(), res.listCells());
					content.add(resultObject);
				}
			}
		}

		JSONArray InsertObjects_real = new JSONArray();
		for (int i = 0; i < InsertObjects.size(); i++) {
			JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
			byte[] row_bytes = row_bytes_array[i];
			boolean exist = false;
			for (byte[] exist_row_bytes : exist_row_bytesList) {
				if (Bytes.compareTo(exist_row_bytes, row_bytes) == 0) {
					exist = true;
					break;
				}
			}
			if (!exist) {
				InsertObjects_real.add(InsertObject);
			}
		}
		return InsertObjects_real;
	}

	public ITableWrapper generateTableWrapper(String namespace, String ori_tableName, int exist_count) throws Exception {
		TableName tableNameObject = TableName.valueOf(namespace, ori_tableName);
		return this.ConnectionWrapper.getTable(exist_count, tableNameObject);
	}
}
