package com.zillion.database.util;

import java.lang.management.ManagementFactory;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.management.JMX;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.sql.DataSource;

import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import com.zaxxer.hikari.HikariPoolMXBean;
import com.zillion.database.agent.ZillionCriteriaGroup;
import com.zillion.database.agent.ZillionQueryResult;
import com.zillion.database.agent.ZillionQueryTmp;
import com.zillion.database.async.AsyncJob;
import com.zillion.util.common.HashObject;
import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.table.SchemaWrapper;
import com.zillion.util.table.ZillionTableColumn;
import com.zillion.util.table.ZillionTableColumnEnum;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;
import com.zillion.util.zk.ZKWrapper;

public class DatabaseImpl_MySQL implements IDatabase {
	// public static String DEFAULT_CHARACTER_SET = "gbk";
	// public static String DEFAULT_CHARACTER_SET_CI= "gbk_chinese_ci";
	// public static String DEFAULT_CHARACTER_SET = "utf8";
	// public static String DEFAULT_CHARACTER_SET_CI= "utf8_general_ci";
	public static String DEFAULT_CHARACTER_SET = "utf8mb4";
	public static String DEFAULT_CHARACTER_SET_CI = "utf8mb4_general_ci";

	// INNODB/егȲܳ767ܳȲܳ3072
	// public static String DEFAULT_STORAGE_ENGINE = "INNODB";
	// public static int DEFAULT_VARCHAR_LENGTH = 383;

	// MYISAM/ܳȲܳ1000
	public static String DEFAULT_STORAGE_ENGINE = "MYISAM";
	public static int DEFAULT_VARCHAR_LENGTH = 125;

	static {
		try {
			// Class.forName("com.mysql.jdbc.Driver").newInstance();
			Class.forName("com.mysql.cj.jdbc.Driver").newInstance();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}

	Configuration_MySQL configuration;

	DataSource ds;
	String poolName = UUID.randomUUID().toString();
	HikariPoolMXBean poolProxy;

	public DatabaseImpl_MySQL(Configuration_MySQL configuration) {
		this.configuration = configuration;
	}

	public String getDataSourceStatus() throws SQLException {
		if (ds instanceof HikariDataSource) {
			StringBuffer sb = new StringBuffer();
			sb.append("\t" + "NumActive:" + poolProxy.getActiveConnections());
			sb.append("\t" + "NumIdle:" + poolProxy.getIdleConnections());
			return sb.toString();
		}
		return null;
	}

	public void Start() throws Exception {
		if (configuration.config instanceof DataSourceConfig_HikariCP) {
			DataSourceConfig_HikariCP config_c3p0 = (DataSourceConfig_HikariCP) configuration.config;
			HikariConfig config = new HikariConfig();
			config.setJdbcUrl(config_c3p0.jdbcUrl);
			config.setUsername(config_c3p0.user);
			config.setPassword(config_c3p0.password);
			config.setMaximumPoolSize(config_c3p0.maxPoolSize);
			config.setIdleTimeout(config_c3p0.maxIdleTime * 1000L);
			config.addDataSourceProperty("cachePrepStmts", "true");
			config.addDataSourceProperty("prepStmtCacheSize", config_c3p0.maxStatements);
			config.addDataSourceProperty("prepStmtCacheSqlLimit", "2048");
			config.setPoolName(poolName);
			config.setRegisterMbeans(true);

			MBeanServer mBeanServer = ManagementFactory.getPlatformMBeanServer();
			ObjectName poolNameObject = new ObjectName("com.zaxxer.hikari:type=Pool (" + poolName + ")");
			poolProxy = JMX.newMXBeanProxy(mBeanServer, poolNameObject, HikariPoolMXBean.class);

			HikariDataSource hds = new HikariDataSource(config);
			ds = hds;
		} else {
			throw new Exception("Unknowd DataSourceConfig:" + configuration.config.name);
		}

		LogUtil.warn("DatabaseImpl_MySQL.Start() begin");
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement("select version()");
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				String version = rs.getString(1);
				LogUtil.warn("MySQL " + version);
			}
			rs.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
		LogUtil.warn("DatabaseImpl_MySQL.Start() end");
	}

	public void Stop() throws Exception {
		if (ds instanceof HikariDataSource) {
			HikariDataSource bds = (HikariDataSource) ds;
			bds.close();
		}
	}

	private Connection getConnection() throws Exception {
		Connection conn = this.ds.getConnection();
		return conn;
	}

	public void namespace_create(String namespace) throws Exception {
		String stringSQL = "create database " + namespace + " DEFAULT CHARSET=" + DEFAULT_CHARACTER_SET + " COLLATE " + DEFAULT_CHARACTER_SET_CI;
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.execute();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void namespace_drop(String namespace) throws Exception {
		String stringSQL = "drop database " + namespace;
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.execute();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void meta_manage_create_data(String namespace) throws Exception {
		String stringSQL = "CREATE TABLE " + namespace + "." + "`zillion_meta_data`"
				+ "(`tableName` varchar(250) NOT NULL,`q` text NOT NULL,`hi` varchar(500) DEFAULT NULL,`hd` varchar(500) DEFAULT NULL,`hc` varchar(500) DEFAULT NULL,PRIMARY KEY (`tableName`)) ENGINE="
				+ DEFAULT_STORAGE_ENGINE + " DEFAULT CHARSET=" + DEFAULT_CHARACTER_SET + ";";
		Connection conn = this.getConnection();
		try {
			MySQLUtil.ori_execute(conn, stringSQL);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void meta_manage_create_index(String namespace) throws Exception {
		String stringSQL = "CREATE TABLE " + namespace + "." + "`zillion_meta_index`"
				+ "(`tableName` varchar(250) NOT NULL,`q` text NOT NULL,`hi` varchar(500) DEFAULT NULL,`hd` varchar(500) DEFAULT NULL,`hc` varchar(500) DEFAULT NULL,PRIMARY KEY (`tableName`)) ENGINE="
				+ DEFAULT_STORAGE_ENGINE + " DEFAULT CHARSET=" + DEFAULT_CHARACTER_SET + ";";
		Connection conn = this.getConnection();
		try {
			MySQLUtil.ori_execute(conn, stringSQL);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void meta_manage_make_sure_split(String namespace) throws Exception {
		if (this.ori_tableExists(namespace, "zillion_meta_split")) {
			return;
		}
		String stringSQL = "CREATE TABLE " + namespace + "." + "`zillion_meta_split`"
				+ "(`tableName` varchar(250) NOT NULL,`hi` varchar(500) DEFAULT NULL,`hd` varchar(500) DEFAULT NULL,`hc` varchar(500) DEFAULT NULL,PRIMARY KEY (`tableName`)) ENGINE="
				+ DEFAULT_STORAGE_ENGINE + " DEFAULT CHARSET=" + DEFAULT_CHARACTER_SET + ";";
		Connection conn = this.getConnection();
		try {
			MySQLUtil.ori_execute(conn, stringSQL);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void meta_manage_make_sure_cache(String namespace) throws Exception {
		// TODO
	}

	public void meta_manage_drop_data(String namespace) throws Exception {
		this.ori_drop_table(namespace, "zillion_meta_data");
	}

	public void meta_manage_drop_index(String namespace) throws Exception {
		this.ori_drop_table(namespace, "zillion_meta_index");
	}

	public void meta_manage_drop_split(String namespace) throws Exception {
		this.ori_drop_table(namespace, "zillion_meta_split");
	}

	public void meta_query_insert_data(String namespace, String tableName, JSONObject content, HashObject hashObject) throws Exception {
		String stringSQL = "insert into " + namespace + "." + "`zillion_meta_data` (tableName,q) VALUES(?,?)";
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.setString(1, tableName);
			stmt.setString(2, JSON.toJSONString(content, SerializerFeature.WriteMapNullValue));
			stmt.execute();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void meta_query_insert_index(String namespace, String tableName, String indexName, JSONArray indexColumns, HashObject hashObject)
			throws Exception {
		String stringSQL = "insert into " + namespace + "." + "`zillion_meta_index` (tableName,q) VALUES(?,?)";
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.setString(1, tableName + "." + indexName);
			stmt.setString(2, JSON.toJSONString(indexColumns, SerializerFeature.WriteMapNullValue));
			stmt.execute();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void meta_query_insert_split(String namespace, String tableName, HashObject hashObject) throws Exception {
		String stringSQL = "insert into " + namespace + "." + "`zillion_meta_split` (tableName) VALUES(?)";
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.setString(1, tableName);
			stmt.execute();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void meta_query_delete_data(String namespace, String tableName) throws Exception {
		String stringSQL = "delete from " + namespace + "." + "`zillion_meta_data` where tableName=?";
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.setString(1, tableName);
			stmt.execute();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void meta_query_delete_index(String namespace, String tableName, String indexName) throws Exception {
		String stringSQL = "delete from " + namespace + "." + "`zillion_meta_index` where tableName=?";
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.setString(1, tableName + "." + indexName);
			stmt.execute();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void meta_query_delete_split(String namespace, String tableName) throws Exception {
		String stringSQL = "delete from " + namespace + "." + "`zillion_meta_split` where tableName=?";
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.setString(1, tableName);
			stmt.execute();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void meta_query_update_data(String namespace, String tableName, JSONObject content) throws Exception {
		String stringSQL = "update " + namespace + "." + "`zillion_meta_data` set `q`=? where tableName='" + tableName + "'";
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.setString(1, JSON.toJSONString(content, SerializerFeature.WriteMapNullValue));
			stmt.execute();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void meta_query_update_data(String namespace, String tableName, HashObject hashObject) throws Exception {
		// TODO
	}

	public void meta_query_update_index(String namespace, String tableName, String indexName, HashObject hashObject) throws Exception {
		// TODO
	}

	public void meta_query_update_split_data(String namespace, String tableName, String child_name, HashObject hashObject) throws Exception {
		// TODO
	}

	public void meta_query_update_split_index(String namespace, String tableName, String indexName, String child_name, HashObject hashObject)
			throws Exception {
		// TODO
	}

	public boolean table_split_index_exist(String namespace, String table_name, String index_name, String child_name) throws Exception {
		String tableName = ZillionConstant.prefix_data + (table_name) + "_" + child_name;
		return this.ori_tableExists(namespace, tableName);
	}

	public void table_manage_create_data(String namespace, ZillionTableSchema schema) throws Exception {
		String tableName = schema.table_name;
		String ori_tableName = ZillionConstant.prefix_data + tableName;
		ori_create_table_MySQL(namespace, schema, ori_tableName);
	}

	public void table_manage_create_index(String namespace, String tableName, ZillionTableIndex schema_index) throws Exception {
		ZillionTableSchema schema = schema_index.schema;
		String ori_tableName = ZillionConstant.prefix_data + tableName;
		ori_create_index(namespace, schema, schema_index, ori_tableName);
	}

	public void table_manage_create_split_data(String namespace, String tableName, String child_name, ZillionTableSchema schema) throws Exception {
		String ori_tableName = ZillionConstant.prefix_data + tableName + "_" + child_name;
		ori_create_table_MySQL(namespace, schema, ori_tableName);
	}

	public void table_manage_create_split_index(String namespace, String tableName, String indexName, String child_name,
			ZillionTableIndex schema_index) throws Exception {
		String ori_tableName = ZillionConstant.prefix_data + tableName + "_" + child_name;
		ZillionTableSchema schema = schema_index.schema;
		ori_create_index(namespace, schema, schema_index, ori_tableName);
	}

	public void table_manage_drop_data(String namespace, String tableName) throws Exception {
		Connection conn = this.getConnection();
		try {
			ori_drop_table(namespace, ZillionConstant.prefix_data + tableName);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void table_manage_drop_index(String namespace, String tableName, String indexName) throws Exception {
		Connection conn = this.getConnection();
		try {
			ori_drop_index(namespace, ZillionConstant.prefix_data + tableName, indexName);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void table_manage_drop_split_data(String namespace, String tableName, String child_name) throws Exception {
		Connection conn = this.getConnection();
		try {
			ori_drop_table(namespace, ZillionConstant.prefix_data + tableName + "_" + child_name);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void table_manage_drop_split_index(String namespace, String tableName, String indexName, String child_name) throws Exception {
		Connection conn = this.getConnection();
		try {
			ori_drop_index(namespace, ZillionConstant.prefix_data + tableName + "_" + child_name, indexName);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void table_manage_alter_add_column(String namespace, String ori_tableName, List<ZillionTableColumn> ztcList) throws Exception {
		StringBuffer sb_table = new StringBuffer();
		for (ZillionTableColumn column : ztcList) {
			this.generateColumn_MySQL(sb_table, column);
		}
		String stringSQL = "ALTER TABLE " + namespace + "." + "`" + ori_tableName + "`" + " ADD COLUMN (" + sb_table.toString() + ")";
		Connection conn = this.getConnection();
		try {
			MySQLUtil.ori_execute(conn, stringSQL);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void table_manage_alter_drop_column(String namespace, String ori_tableName, List<String> columnList) throws Exception {
		for (String column : columnList) {
			String stringSQL = "ALTER TABLE " + namespace + "." + "`" + ori_tableName + "`" + " DROP COLUMN " + "`" + column + "`";
			Connection conn = this.getConnection();
			try {
				MySQLUtil.ori_execute(conn, stringSQL);
			} catch (Exception et) {
				throw et;
			} finally {
				conn.close();
			}
		}
	}

	public void table_manage_alter_compression(String namespace, String ori_tableName, boolean enable) throws Exception {
		// TODO
	}

	public void table_manage_alter_in_memory(String namespace, String ori_tableName, boolean enable) throws Exception {
		// TODO
	}

	public void meta_read(String namespace, List<String> tableNameList, List<String> tableContentList, List<HashObject> tableHashList,
			List<String> indexNameList, List<String> indexContentList, List<HashObject> indexHashList, List<String> splitNameList,
			List<HashObject> splitHashList) throws Exception {
		Connection conn = this.getConnection();
		try {
			{
				String stringSQL = "select * from " + namespace + ".`" + "zillion_meta_data" + "`";
				List<Integer> typeList = new ArrayList<Integer>();
				List<Object> params = new ArrayList<Object>();
				ResultSet_MySQL results = MySQLUtil.ori_select(conn, stringSQL, typeList, params);
				for (int i = 0; i < results.rowList.size(); i++) {
					Object[] columns = results.rowList.get(i);
					String row = (String) columns[0];
					String q = (String) columns[1];
					HashObject hashObject = new HashObject();
					tableNameList.add(row);
					tableContentList.add(q);
					tableHashList.add(hashObject);
				}
			}
			{
				String stringSQL = "select * from " + namespace + ".`" + "zillion_meta_index" + "`";
				List<Integer> typeList = new ArrayList<Integer>();
				List<Object> params = new ArrayList<Object>();
				ResultSet_MySQL results = MySQLUtil.ori_select(conn, stringSQL, typeList, params);
				for (int i = 0; i < results.rowList.size(); i++) {
					Object[] columns = results.rowList.get(i);
					String row = (String) columns[0];
					String q = (String) columns[1];
					HashObject hashObject = new HashObject();
					indexNameList.add(row);
					indexContentList.add(q);
					indexHashList.add(hashObject);
				}
			}
			if (this.ori_tableExists(namespace, "zillion_meta_split")) {
				String stringSQL = "select * from " + namespace + ".`" + "zillion_meta_split" + "`";
				List<Integer> typeList = new ArrayList<Integer>();
				List<Object> params = new ArrayList<Object>();
				ResultSet_MySQL results = MySQLUtil.ori_select(conn, stringSQL, typeList, params);
				for (int i = 0; i < results.rowList.size(); i++) {
					Object[] columns = results.rowList.get(i);
					String row = (String) columns[0];
					HashObject hashObject = new HashObject();
					splitNameList.add(row);
					splitHashList.add(hashObject);
				}
			}
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public int clear_table_split(String namespace, ZillionTableSchema schema, List<String> query_monthList, ZKWrapper ZK) throws Exception {
		int count = 0;
		Connection conn = this.getConnection();
		try {
			String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
			HashObject hashObject = new HashObject();
			byte[] hash = hashObject.toBytes();
			for (int index_split = 0; index_split < query_monthList.size(); index_split++) {
				String query_month = query_monthList.get(index_split);

				ZK.setData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name + "/" + query_month, hash, -1);
				count = MySQLUtil.ori_executeQuery(conn,
						"delete from " + namespace + ".`" + ZillionConstant.prefix_data + schema.table_name + "_" + query_month + "`");
			}
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
		return count;
	}

	public int clear_table_not_split(String namespace, ZillionTableSchema schema, ZKWrapper ZK) throws Exception {
		int count = 0;
		Connection conn = this.getConnection();
		try {
			String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
			{
				HashObject hashObject = new HashObject();
				byte[] hash = hashObject.toBytes();
				ZK.setData(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + schema.table_name, hash, -1);
				count = MySQLUtil.ori_executeQuery(conn, "delete from " + namespace + ".`" + ZillionConstant.prefix_data + schema.table_name + "`");
			}
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
		return count;
	}

	public void scan(SchemaRepositoryContainer container, ZillionTableSchema schema, SchemaWrapper schemaWrapper, JSONArray ReturnColumns,
			ZillionCriteriaGroup group, String QueryType, JSONArray content, ZillionQueryTmp zqt, ZipArchiveOutputStream out, AsyncJob job,
			boolean SortInDB, JSONArray OrderBy, long Limit_Skip, long Limit_Count, ZillionQueryResult result, JSONObject Set,
			JSONArray ConflictArray) throws Exception {
		Connection conn = this.getConnection();
		try {
			String ori_tableName = ZillionConstant.prefix_data + schema.table_name
					+ (schemaWrapper.child_name == null ? "" : "_" + schemaWrapper.child_name);
			String ori_tableName_full = schema.Namespace + ".`" + ori_tableName + "`";
			String[] keysInner;
			if (group.index_index == -1) {
				keysInner = schema.Key;
			} else {
				keysInner = schema.Indexes.get(group.index_index).Key;
			}
			MySQLUtil.scan(conn, ori_tableName_full, container, schema, schemaWrapper, ReturnColumns, group.CriteriaList, QueryType, content, zqt,
					out, job, keysInner, SortInDB, OrderBy, Limit_Skip, Limit_Count, result, Set);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void batch_process(SchemaRepositoryContainer container, ZillionTableSchema schema, SchemaWrapper wrapper, String QueryType,
			JSONArray InsertObjects, JSONObject Set, boolean is_put, JSONArray ConflictArray) throws Exception {
		Connection conn = this.getConnection();
		try {
			String ori_tableName = ZillionConstant.prefix_data + schema.table_name + (wrapper.child_name == null ? "" : "_" + wrapper.child_name);
			String ori_tableName_full = schema.Namespace + ".`" + ori_tableName + "`";
			MySQLUtil.batch_process(conn, ori_tableName_full, schema, QueryType, InsertObjects, is_put);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public JSONArray batch_get(ZillionTableSchema schema, ITableWrapper table_wrapper, JSONArray InsertObjects, JSONArray content,
			Map<String, Boolean> ReturnColumnsMap) throws Exception {
		JSONArray result = new JSONArray();
		Connection conn = this.getConnection();
		try {
			TableWrapper_MySQL TableWrapper_MySQL = (TableWrapper_MySQL) table_wrapper;
			String ori_tableName_full = schema.Namespace + "." + "`" + TableWrapper_MySQL.tableName + "`";
			result = MySQLUtil.batch_get(conn, ori_tableName_full, schema, InsertObjects, content, ReturnColumnsMap);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}

		return result;
	}

	public ITableWrapper generateTableWrapper(String namespace, String ori_tableName, int exist_count) throws Exception {
		TableWrapper_MySQL wrapper = new TableWrapper_MySQL(exist_count, ori_tableName);
		return wrapper;
	}

	private void ori_create_index(String namespace, ZillionTableSchema schema, ZillionTableIndex schema_index, String ori_tableName)
			throws Exception {
		StringBuffer sb_key = new StringBuffer();
		for (String key : schema_index.Key) {
			ZillionTableColumn column = schema.ColumnMap.get(key);
			if (sb_key.length() != 0) {
				sb_key.append(",");
			}
			sb_key.append("`");
			sb_key.append(column.Name);
			sb_key.append("`");
		}
		String stringSQL = "ALTER TABLE " + namespace + "." + "`" + ori_tableName + "`" + " ADD INDEX " + schema_index.index_name + "("
				+ sb_key.toString() + ")";
		Connection conn = this.getConnection();
		try {
			MySQLUtil.ori_execute(conn, stringSQL);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	private void ori_create_table_MySQL(String namespace, ZillionTableSchema schema, String ori_tableName) throws Exception {
		StringBuffer sb_table = new StringBuffer();
		StringBuffer sb_key = new StringBuffer();
		for (String key : schema.Key) {
			ZillionTableColumn column = schema.ColumnMap.get(key);
			generateColumn_MySQL(sb_table, column);
			if (sb_key.length() != 0) {
				sb_key.append(",");
			}
			sb_key.append("`");
			sb_key.append(column.Name);
			sb_key.append("`");
		}
		for (String key : schema.OtherColumn) {
			ZillionTableColumn column = schema.ColumnMap.get(key);
			generateColumn_MySQL(sb_table, column);
		}
		String stringSQL = "CREATE TABLE " + namespace + "." + "`" + ori_tableName + "`" + "(" + sb_table.toString() + ",PRIMARY KEY ("
				+ sb_key.toString() + ")) ENGINE=" + DEFAULT_STORAGE_ENGINE + " DEFAULT CHARSET=" + DEFAULT_CHARACTER_SET + ";";
		Connection conn = this.getConnection();
		try {
			MySQLUtil.ori_execute(conn, stringSQL);
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	private void ori_drop_index(String namespace, String ori_tableName, String index_name) throws Exception {
		String stringSQL = "alter table " + namespace + "." + "`" + ori_tableName + "` drop index `" + index_name + "`";
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.execute();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	public void ori_drop_table(String namespace, String ori_tableName) throws Exception {
		String stringSQL = "drop table " + namespace + "." + "`" + ori_tableName + "`";
		Connection conn = this.getConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.execute();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
	}

	private void generateColumn_MySQL(StringBuffer sb_table, ZillionTableColumn column) {
		if (sb_table.length() != 0) {
			sb_table.append(",");
		}
		sb_table.append("`");
		sb_table.append(column.Name);
		sb_table.append("`");
		sb_table.append(" ");
		if (column.Type == ZillionTableColumnEnum.LONG) {
			sb_table.append("bigint");
		} else if (column.Type == ZillionTableColumnEnum.DOUBLE) {
			sb_table.append("double");
		} else if (column.Type == ZillionTableColumnEnum.STRING || column.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
			int max_length_real = column.max_length == Integer.MAX_VALUE ? DEFAULT_VARCHAR_LENGTH : column.max_length;
			if (max_length_real <= 255) {
				sb_table.append("varchar(" + max_length_real + ")");
			} else if (max_length_real <= 65535) {
				sb_table.append("text");
			} else if (max_length_real <= 16777215) {
				sb_table.append("mediumtext");
			} else {
				sb_table.append("longtext");
			}
			if (column.ASCIIString) {
				sb_table.append(" CHARACTER SET latin1 COLLATE latin1_general_ci");
			}
		} else if (column.Type == ZillionTableColumnEnum.BOOLEAN) {
			sb_table.append("int");
		}
		if (column.NotNull || (!column.NotNull && column.NullValueConvert)) {
			sb_table.append(" NOT NULL");
		} else {
			sb_table.append(" DEFAULT NULL");
		}
	}

	public List<String> ori_namespaceList() throws Exception {
		List<String> result = new ArrayList<String>();
		Connection conn = this.getConnection();
		try {
			String stringSQL = "select * from information_schema.schemata";
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				String SCHEMA_NAME = rs.getString("SCHEMA_NAME");
				if (SCHEMA_NAME.equals("information_schema") || SCHEMA_NAME.equals("mysql")) {
					continue;
				}
				result.add(SCHEMA_NAME);
			}
			rs.close();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
		return result;
	}

	public boolean ori_tableExists(String namespace, String ori_tableName) throws Exception {
		boolean result = false;
		Connection conn = this.getConnection();
		try {
			String stringSQL = "select * from information_schema.tables where table_schema=? and table_name=?";
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.setString(1, namespace);
			stmt.setString(2, ori_tableName);
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				result = true;
			}
			rs.close();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
		return result;
	}

	public List<String> ori_tableList(String namespace) throws Exception {
		List<String> result = new ArrayList<String>();
		Connection conn = this.getConnection();
		try {
			String stringSQL = "select * from information_schema.tables WHERE table_schema=?";
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			stmt.setString(1, namespace);
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				String TABLE_NAME = rs.getString("TABLE_NAME");
				result.add(TABLE_NAME);
			}
			rs.close();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
		return result;
	}

	public int ori_table_count(String namespace, String ori_tableName) throws Exception {
		int count = 0;
		Connection conn = this.getConnection();
		try {
			String stringSQL = "select count(*) from " + namespace + "." + "`" + ori_tableName + "`";
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				count = rs.getInt(1);
			}
			rs.close();
			stmt.close();
		} catch (Exception et) {
			throw et;
		} finally {
			conn.close();
		}
		return count;
	}
}
