package com.zillion.database.util;

import java.util.List;
import java.util.Map;

import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.agent.ZillionCriteriaGroup;
import com.zillion.database.agent.ZillionQueryResult;
import com.zillion.database.agent.ZillionQueryTmp;
import com.zillion.database.async.AsyncJob;
import com.zillion.util.common.HashObject;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.table.SchemaWrapper;
import com.zillion.util.table.ZillionTableColumn;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;
import com.zillion.util.zk.ZKWrapper;

public interface IDatabase {
	void Start() throws Exception;

	void Stop() throws Exception;

	List<String> ori_namespaceList() throws Exception;

	List<String> ori_tableList(String namespace) throws Exception;

	boolean ori_tableExists(String namespace, String ori_tableName) throws Exception;

	void ori_drop_table(String namespace, String ori_tableName) throws Exception;

	void namespace_create(String namespace) throws Exception;

	void namespace_drop(String namespace) throws Exception;

	void meta_manage_create_data(String namespace) throws Exception;

	void meta_manage_create_index(String namespace) throws Exception;

	void meta_manage_make_sure_split(String namespace) throws Exception;

	void meta_manage_make_sure_cache(String namespace) throws Exception;

	void meta_manage_drop_data(String namespace) throws Exception;

	void meta_manage_drop_index(String namespace) throws Exception;

	void meta_manage_drop_split(String namespace) throws Exception;

	void meta_query_insert_data(String namespace, String tableName, JSONObject content, HashObject hashObject) throws Exception;

	void meta_query_insert_index(String namespace, String tableName, String indexName, JSONArray indexColumns, HashObject hashObject)
			throws Exception;

	void meta_query_insert_split(String namespace, String tableName, HashObject hashObject) throws Exception;

	void meta_query_delete_data(String namespace, String tableName) throws Exception;

	void meta_query_delete_index(String namespace, String tableName, String indexName) throws Exception;

	void meta_query_delete_split(String namespace, String tableName) throws Exception;

	void meta_query_update_data(String namespace, String tableName, JSONObject content) throws Exception;

	void meta_query_update_data(String namespace, String tableName, HashObject hashObject) throws Exception;

	void meta_query_update_index(String namespace, String tableName, String indexName, HashObject hashObject) throws Exception;

	void meta_query_update_split_data(String namespace, String tableName, String child_name, HashObject hashObject) throws Exception;

	void meta_query_update_split_index(String namespace, String tableName, String indexName, String child_name, HashObject hashObject)
			throws Exception;

	boolean table_split_index_exist(String namespace, String table_name, String index_name, String child_name) throws Exception;

	void table_manage_create_data(String namespace, ZillionTableSchema schema) throws Exception;

	void table_manage_create_index(String namespace, String tableName, ZillionTableIndex schema_index) throws Exception;

	void table_manage_create_split_data(String namespace, String tableName, String child_name, ZillionTableSchema schema) throws Exception;

	void table_manage_create_split_index(String namespace, String tableName, String indexName, String child_name, ZillionTableIndex schema_index)
			throws Exception;

	void table_manage_drop_data(String namespace, String tableName) throws Exception;

	void table_manage_drop_index(String namespace, String tableName, String indexName) throws Exception;

	void table_manage_drop_split_data(String namespace, String tableName, String child_name) throws Exception;

	void table_manage_drop_split_index(String namespace, String tableName, String indexName, String child_name) throws Exception;

	void table_manage_alter_add_column(String namespace, String ori_tableName, List<ZillionTableColumn> ztcList) throws Exception;

	void table_manage_alter_drop_column(String namespace, String ori_tableName, List<String> columnList) throws Exception;

	void table_manage_alter_compression(String namespace, String ori_tableName, boolean enable) throws Exception;

	void table_manage_alter_in_memory(String namespace, String ori_tableName, boolean enable) throws Exception;

	void meta_read(String namespace, List<String> tableNameList, List<String> tableContentList, List<HashObject> tableHashList,
			List<String> indexNameList, List<String> indexContentList, List<HashObject> indexHashList, List<String> splitNameList,
			List<HashObject> splitHashList) throws Exception;

	int clear_table_split(String namespace, ZillionTableSchema schema, List<String> query_monthList, ZKWrapper ZK) throws Exception;

	int clear_table_not_split(String namespace, ZillionTableSchema schema, ZKWrapper ZK) throws Exception;

	void scan(SchemaRepositoryContainer container, ZillionTableSchema schema, SchemaWrapper schemaWrapper, JSONArray ReturnColumns,
			ZillionCriteriaGroup group, String QueryType, JSONArray content, ZillionQueryTmp zqt, ZipArchiveOutputStream out, AsyncJob job,
			boolean SortInDB, JSONArray OrderBy, long Limit_Skip, long Limit_Count, ZillionQueryResult result, JSONObject Set,
			JSONArray ConflictArray) throws Exception;

	void batch_process(SchemaRepositoryContainer container, ZillionTableSchema schema, SchemaWrapper wrapper, String QueryType,
			JSONArray InsertObjects, JSONObject Set, boolean is_put, JSONArray ConflictArray) throws Exception;

	JSONArray batch_get(ZillionTableSchema schema, ITableWrapper table_wrapper, JSONArray InsertObjects, JSONArray content,
			Map<String, Boolean> ReturnColumnsMap) throws Exception;

	ITableWrapper generateTableWrapper(String namespace, String ori_tableName, int exist_count) throws Exception;
}
