package com.zillion.database.util;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.zillion.util.common.MyException;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.zillion.database.agent.ZillionQueryResult;
import com.zillion.database.agent.ZillionQueryTmp;
import com.zillion.database.async.AsyncJob;
import com.zillion.database.async.AsyncUtil;
import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.SchemaRepositoryContainer;
import com.zillion.util.table.SchemaWrapper;
import com.zillion.util.table.ZillionTableColumn;
import com.zillion.util.table.ZillionTableColumnEnum;
import com.zillion.util.table.ZillionTableSchema;

public class MySQLUtil {

	public static void scan(Connection conn, String ori_tableName_full, SchemaRepositoryContainer container, ZillionTableSchema schema,
			SchemaWrapper schemaWrapper, JSONArray ReturnColumns, List<JSONObject> CriteriaList, String QueryType, JSONArray content,
			ZillionQueryTmp zqt, ZipArchiveOutputStream out, AsyncJob job, String[] keys, boolean SortInDB, JSONArray OrderBy, long Limit_Skip,
			long Limit_Count, ZillionQueryResult result, JSONObject Set) throws Exception {
		List<Integer> typeList = new ArrayList<Integer>();
		List<Object> paramList = new ArrayList<Object>();

		StringBuffer sb_return = new StringBuffer();
		if (QueryType.equals("select") || QueryType.equals("export")) {
			if (ReturnColumns != null) {
				for (int i = 0; i < ReturnColumns.size(); i++) {
					String key = (String) ReturnColumns.get(i);
					if (sb_return.length() > 0) {
						sb_return.append(",");
					}
					sb_return.append("`" + key + "`");
				}
			} else {
				sb_return.append(" * ");
			}
		}

		StringBuffer sb_set = new StringBuffer();
		if (QueryType.equals("update")) {
			generateSQL_Set(schema.ColumnMap, Set, sb_set, typeList, paramList);
		}

		StringBuffer sb_condition = new StringBuffer();
		generateSQL(schema.ColumnMap, CriteriaList, sb_condition, typeList, paramList);

		StringBuffer sb_order_by = new StringBuffer();
		if (SortInDB && OrderBy != null && OrderBy.size() > 0) {
			for (int i = 0; i < OrderBy.size(); i++) {
				JSONObject item0 = (JSONObject) OrderBy.get(i);
				String Column = (String) item0.get("Column");
				boolean Asc = (Boolean) item0.get("Asc");
				if (i > 0) {
					sb_order_by.append(",");
				}
				sb_order_by.append("`" + Column + "`" + " " + (Asc ? "asc" : "desc"));
			}
		} else {
			for (String keyItem : keys) {
				if (sb_order_by.length() > 0) {
					sb_order_by.append(",");
				}
				sb_order_by.append("`" + keyItem + "`");
			}
		}

		String sb_limit = " limit " + Limit_Skip + "," + Limit_Count;

		JSONArray contentInner = new JSONArray();
		String stringSQL;
		String sql_where = sb_condition.toString().length() > 2 ? " where " + sb_condition.toString() : " ";
		String sql_order_by = " order by " + sb_order_by.toString();
		if (QueryType.equals("select_count")) {
			stringSQL = "select count(*) from " + ori_tableName_full + sql_where + " " + sb_limit;
			result.count += ori_execute(conn, stringSQL, typeList, paramList);
		} else if (QueryType.equals("select") || QueryType.equals("export")) {
			stringSQL = "select " + sb_return + " from " + ori_tableName_full + sql_where + " " + sql_order_by + " " + sb_limit;
			JSONArray selectContent = ori_select(conn, stringSQL, typeList, paramList, schema);
			for (int index_row = 0; index_row < selectContent.size(); index_row++) {
				JSONObject contentItem = (JSONObject) selectContent.get(index_row);

				zqt.scanned_count++;
				zqt.count_wait_tmp++;
				if (zqt.count_wait_tmp >= AsyncUtil.count_wait) {
					job.Add(zqt.count_wait_tmp);
					zqt.count_wait_tmp = 0L;
				}
				if (zqt.scanned_count > Limit_Skip && zqt.scanned_count <= Limit_Skip + Limit_Count) {
					result.count++;
					if (QueryType.equals("select")) {
						if (job.brs != null) {
							job.brs.TryPush(contentItem);
						} else {
							content.add(contentItem);
						}
					} else {
						contentInner.add(contentItem);
					}
					if (QueryType.equals("export")) {
						if (contentInner.size() >= ZillionConstant.export_batch_size) {
							out.putArchiveEntry(new ZipArchiveEntry(zqt.df.format(zqt.zip_index++) + ".json"));
							byte[] bytes = JSON.toJSONString(contentInner, SerializerFeature.WriteMapNullValue).getBytes("UTF-8");
							out.write(bytes);
							out.closeArchiveEntry();
							contentInner.clear();
						}
					}
				} else if (zqt.scanned_count > Limit_Skip + Limit_Count) {
					zqt.scanned_finish = true;
					break;
				}
			}
			if (zqt.count_wait_tmp > 0) {
				job.Add(zqt.count_wait_tmp);
				zqt.count_wait_tmp = 0L;
			}
			if (QueryType.equals("export")) {
				if (contentInner.size() > 0) {
					out.putArchiveEntry(new ZipArchiveEntry(zqt.df.format(zqt.zip_index++) + ".json"));
					byte[] bytes = JSON.toJSONString(contentInner, SerializerFeature.WriteMapNullValue).getBytes("UTF-8");
					out.write(bytes);
					out.closeArchiveEntry();
					contentInner.clear();
				}
			}
		} else if (QueryType.equals("delete")) {
			stringSQL = "delete from " + ori_tableName_full + sql_where;
			result.count += ori_executeUpdate(conn, stringSQL, typeList, paramList);
		} else if (QueryType.equals("update")) {
			stringSQL = "update " + ori_tableName_full + " set " + sb_set.toString() + sql_where;
			result.count += ori_executeUpdate(conn, stringSQL, typeList, paramList);
		} else {
			throw new MyException("");
		}
	}

	public static void batch_process(Connection conn, String ori_tableName_full, ZillionTableSchema schema, String QueryType, JSONArray InsertObjects,
			boolean is_put) throws Exception {
		if (!QueryType.equals("insert") && !QueryType.equals("delete")) {
			return;
		}
		if (InsertObjects.size() == 0) {
			return;
		}

		if (QueryType.equals("insert")) {
			JSONArray object_insert = new JSONArray();
			JSONArray object_update = new JSONArray();
			if (is_put) {
				for (int i = 0; i < InsertObjects.size(); i++) {
					JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
					JSONObject Criteria = new JSONObject();
					for (String key : schema.Key) {
						Criteria.put(key, InsertObject.get(key));
					}
					List<Integer> typeList = new ArrayList<Integer>();
					List<Object> paramList = new ArrayList<Object>();
					StringBuffer sb_condition = new StringBuffer();
					generateSQL(schema.ColumnMap, Criteria, sb_condition, typeList, paramList);
					String stringSQL = "select * from " + ori_tableName_full + " where " + sb_condition.toString();
					JSONArray selectContent = ori_select(conn, stringSQL, typeList, paramList, schema);
					if (selectContent.size() == 0) {
						object_insert.add(InsertObject);
					} else {
						object_update.add(InsertObject);
					}
				}
			} else {
				object_insert.addAll(InsertObjects);
			}

			for (int i = 0; i < object_update.size(); i++) {
				JSONObject InsertObject = (JSONObject) object_update.get(i);
				List<Integer> typeList = new ArrayList<Integer>();
				List<Object> paramList = new ArrayList<Object>();
				StringBuffer sb_set = new StringBuffer();
				JSONObject CriteriaInner = new JSONObject();
				JSONObject SetInner = new JSONObject();
				for (String key : schema.Key) {
					CriteriaInner.put(key, InsertObject.get(key));
				}
				List<JSONObject> CriteriaInnerList = new ArrayList<JSONObject>();
				CriteriaInnerList.add(CriteriaInner);
				for (String key : schema.OtherColumn) {
					SetInner.put(key, InsertObject.get(key));
				}
				generateSQL_Set(schema.ColumnMap, SetInner, sb_set, typeList, paramList);

				StringBuffer sb_condition = new StringBuffer();
				generateSQL(schema.ColumnMap, CriteriaInnerList, sb_condition, typeList, paramList);

				String sql_where = sb_condition.toString().length() > 2 ? " where " + sb_condition.toString() : " ";
				String stringSQL = "update " + ori_tableName_full + " set " + sb_set.toString() + sql_where;
				ori_executeUpdate(conn, stringSQL, typeList, paramList);
			}
			{
				StringBuffer sb_col = new StringBuffer();
				StringBuffer sb_wenhao = new StringBuffer();
				for (String col : schema.Key) {
					if (sb_col.length() > 0) {
						sb_col.append(",");
					}
					sb_col.append("`" + col + "`");

					if (sb_wenhao.length() > 0) {
						sb_wenhao.append(",");
					}
					sb_wenhao.append("?");
				}
				for (String col : schema.OtherColumn) {
					if (sb_col.length() > 0) {
						sb_col.append(",");
					}
					sb_col.append("`" + col + "`");

					if (sb_wenhao.length() > 0) {
						sb_wenhao.append(",");
					}
					sb_wenhao.append("?");
				}
				String stringSQL = "insert into " + ori_tableName_full + "(" + sb_col + ") values(" + sb_wenhao + ")";
				PreparedStatement stmt = conn.prepareStatement(stringSQL);
				for (int i = 0; i < object_insert.size(); i++) {
					JSONObject InsertObject = (JSONObject) object_insert.get(i);
					int parameterIndex = 1;
					String[][] columnArrayArray = new String[2][];
					columnArrayArray[0] = schema.Key;
					columnArrayArray[1] = schema.OtherColumn;
					for (String[] columnArray : columnArrayArray) {
						for (String col : columnArray) {
							ZillionTableColumn ztc = schema.ColumnMap.get(col);
							Object columnObject = InsertObject.get(col);
							if (columnObject == null && ztc.NullValueConvert) {
								columnObject = "";
							}
							int type = Types.VARCHAR;
							if (ztc.Type == ZillionTableColumnEnum.LONG) {
								type = Types.BIGINT;
							} else if (ztc.Type == ZillionTableColumnEnum.DOUBLE) {
								type = Types.DOUBLE;
							} else if (ztc.Type == ZillionTableColumnEnum.BOOLEAN) {
								type = Types.INTEGER;
							}
							if (columnObject == null) {
								stmt.setNull(parameterIndex, type);
							} else {
								if (type == Types.INTEGER) {
									stmt.setInt(parameterIndex, param_to_int(columnObject));
								} else if (type == Types.BIGINT) {
									stmt.setBigDecimal(parameterIndex, new BigDecimal(param_to_long(columnObject)));
								} else if (type == Types.DOUBLE) {
									stmt.setDouble(parameterIndex, param_to_double(columnObject));
								} else if (type == Types.VARCHAR) {
									stmt.setString(parameterIndex, (String) columnObject);
								}
							}
							parameterIndex++;
						}
					}
					stmt.addBatch();
				}
				stmt.executeBatch();
				stmt.close();
			}
		} else if (QueryType.equals("delete")) {
			StringBuffer sb = new StringBuffer();
			for (String col : schema.Key) {
				if (sb.length() > 0) {
					sb.append(",");
				}
				sb.append("`" + col + "`=?");
			}
			String stringSQL = "delete from " + ori_tableName_full + " where " + sb.toString();
			PreparedStatement stmt = conn.prepareStatement(stringSQL);
			for (int i = 0; i < InsertObjects.size(); i++) {
				JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
				int parameterIndex = 1;
				for (String col : schema.Key) {
					ZillionTableColumn ztc = schema.ColumnMap.get(col);
					Object columnObject = InsertObject.get(col);
					if (columnObject == null && ztc.NullValueConvert) {
						columnObject = "";
					}
					int type = Types.VARCHAR;
					if (ztc.Type == ZillionTableColumnEnum.LONG) {
						type = Types.BIGINT;
					} else if (ztc.Type == ZillionTableColumnEnum.DOUBLE) {
						type = Types.DOUBLE;
					} else if (ztc.Type == ZillionTableColumnEnum.BOOLEAN) {
						type = Types.INTEGER;
					}
					if (columnObject == null) {
						stmt.setNull(parameterIndex, type);
					} else {
						if (type == Types.INTEGER) {
							stmt.setInt(parameterIndex, param_to_int(columnObject));
						} else if (type == Types.BIGINT) {
							stmt.setBigDecimal(parameterIndex, new BigDecimal(param_to_long(columnObject)));
						} else if (type == Types.DOUBLE) {
							stmt.setDouble(parameterIndex, param_to_double(columnObject));
						} else if (type == Types.VARCHAR) {
							stmt.setString(parameterIndex, (String) columnObject);
						}
					}
					parameterIndex++;
				}
				stmt.addBatch();
			}
			stmt.execute();
			stmt.executeBatch();
			stmt.close();
		}
	}

	public static JSONArray batch_get(Connection conn, String ori_tableName_full, ZillionTableSchema schema, JSONArray InsertObjects,
			JSONArray content, Map<String, Boolean> ReturnColumnsMap) throws Exception {
		JSONArray InsertObjects_real = new JSONArray();

		List<Integer> typeList = new ArrayList<Integer>();
		List<Object> paramList = new ArrayList<Object>();
		for (int i = 0; i < InsertObjects.size(); i++) {
			typeList.clear();
			paramList.clear();
			StringBuffer sb_condition = new StringBuffer();

			JSONObject InsertObject = (JSONObject) InsertObjects.get(i);
			JSONObject Criteria = new JSONObject();
			for (String key : schema.Key) {
				Criteria.put(key, InsertObject.get(key));
			}
			generateSQL(schema.ColumnMap, Criteria, sb_condition, typeList, paramList);
			String stringSQL = "select * from " + ori_tableName_full + " where " + sb_condition.toString();
			JSONArray selectContent = ori_select(conn, stringSQL, typeList, paramList, schema);
			if (selectContent.size() > 0) {
				content.addAll(selectContent);
			} else {
				InsertObjects_real.add(InsertObject);
			}
		}

		return InsertObjects_real;
	}

	public static void generateSQL(Map<String, ZillionTableColumn> ColumnMap, List<JSONObject> CriteriaList, StringBuffer sb, List<Integer> typeList,
			List<Object> paramList) {
		{
			JSONObject Criteria = CriteriaList.get(0);
			StringBuffer sb1 = new StringBuffer();
			generateSQL(ColumnMap, Criteria, sb1, typeList, paramList);
			sb.append("(" + sb1.toString() + ")");
		}
		for (int i = 1; i < CriteriaList.size(); i++) {
			JSONObject CriteriaInner = (JSONObject) CriteriaList.get(i);
			StringBuffer sb1 = new StringBuffer();
			generateSQL(ColumnMap, CriteriaInner, sb1, typeList, paramList);
			sb.append(" or ");
			sb.append("(" + sb1.toString() + ")");
		}
	}

	public static void generateSQL(Map<String, ZillionTableColumn> ColumnMap, JSONObject Criteria, StringBuffer sb, List<Integer> typeList,
			List<Object> paramList) {
		Iterator<String> keyIter = Criteria.keySet().iterator();
		while (keyIter.hasNext()) {
			String key = keyIter.next();
			Object columnObject = Criteria.get(key);
			ZillionTableColumn columnDef = ColumnMap.get(key);
			int type = Types.VARCHAR;
			if (columnDef.Type == ZillionTableColumnEnum.LONG) {
				type = Types.BIGINT;
			} else if (columnDef.Type == ZillionTableColumnEnum.DOUBLE) {
				type = Types.DOUBLE;
			} else if (columnDef.Type == ZillionTableColumnEnum.BOOLEAN) {
				type = Types.INTEGER;
			}
			if (sb.length() > 0) {
				sb.append(" and ");
			}
			if (columnObject instanceof JSONObject) {
				JSONObject columnJSON = (JSONObject) columnObject;
				StringBuffer sbInner = new StringBuffer();
				Iterator<String> keyIterInner = columnJSON.keySet().iterator();
				while (keyIterInner.hasNext()) {
					if (sbInner.length() > 0) {
						sbInner.append(" and ");
					}
					String fuhao = keyIterInner.next();
					Object columnObjectInner = columnJSON.get(fuhao);
					if (columnObjectInner == null && columnDef.NullValueConvert) {
						columnObjectInner = "";
					}
					if (columnObjectInner == null) {
						if (fuhao.equals("$lt")) {
							sbInner.append("1=0");
						} else if (fuhao.equals("$lte")) {
							sbInner.append("`" + key + "` is null");
						} else if (fuhao.equals("$gt")) {
							sbInner.append("`" + key + "` is not null");
						} else if (fuhao.equals("$gte")) {
							sbInner.append("1=1");
						}
					} else {
						if (fuhao.equals("$lt")) {
							if (columnDef.NullValueConvert) {
								sbInner.append("`" + key + "`<?");
							} else {
								sbInner.append("(`" + key + "`<? or " + key + " is null)");
							}
							typeList.add(type);
							paramList.add(columnObjectInner);
						} else if (fuhao.equals("$lte")) {
							if (columnDef.NullValueConvert) {
								sbInner.append("`" + key + "`<=?");
							} else {
								sbInner.append("(`" + key + "`<=? or " + key + " is null)");
							}
							typeList.add(type);
							paramList.add(columnObjectInner);
						} else if (fuhao.equals("$gt")) {
							sbInner.append("`" + key + "`>?");
							typeList.add(type);
							paramList.add(columnObjectInner);
						} else if (fuhao.equals("$gte")) {
							sbInner.append("`" + key + "`>=?");
							typeList.add(type);
							paramList.add(columnObjectInner);
						}
					}
				}
				sb.append(sbInner.toString());
			} else {
				if (columnObject == null && columnDef.NullValueConvert) {
					columnObject = "";
				}
				if (columnObject == null) {
					sb.append("`" + key + "` is null");
				} else {
					sb.append("`" + key + "`=?");
					typeList.add(type);
					paramList.add(columnObject);
				}
			}
		}
	}

	public static void generateSQL_Set(Map<String, ZillionTableColumn> ColumnMap, JSONObject Set, StringBuffer sb_set, List<Integer> typeList,
			List<Object> paramList) {
		{
			Iterator<String> keyIter = Set.keySet().iterator();
			while (keyIter.hasNext()) {
				String key = keyIter.next();
				Object columnObject = Set.get(key);
				ZillionTableColumn columnDef = ColumnMap.get(key);
				int type = Types.VARCHAR;
				if (columnDef.Type == ZillionTableColumnEnum.LONG) {
					type = Types.BIGINT;
				} else if (columnDef.Type == ZillionTableColumnEnum.DOUBLE) {
					type = Types.DOUBLE;
				} else if (columnDef.Type == ZillionTableColumnEnum.BOOLEAN) {
					type = Types.INTEGER;
				}
				if (sb_set.length() > 0) {
					sb_set.append(",");
				}
				sb_set.append("`" + key + "`=?");
				if (columnObject == null && columnDef.NullValueConvert) {
					columnObject = "";
				}
				typeList.add(type);
				paramList.add(columnObject);
			}
		}
	}

	public static boolean ori_execute(Connection conn, String stringSQL) throws Exception {
		PreparedStatement stmt = conn.prepareStatement(stringSQL);
		boolean result = stmt.execute();
		stmt.close();
		return result;
	}

	public static int ori_execute(Connection conn, String stringSQL, List<Integer> typeList, List<Object> params) throws Exception {
		int result = 0;
		PreparedStatement stmt = conn.prepareStatement(stringSQL);
		for (int i = 0; i < params.size(); i++) {
			int parameterIndex = i + 1;
			Object param = params.get(i);
			int ttttt = typeList.get(i);
			if (param == null) {
				stmt.setNull(parameterIndex, ttttt);
			} else {
				if (ttttt == Types.INTEGER) {
					stmt.setInt(parameterIndex, param_to_int(param));
				} else if (ttttt == Types.BIGINT) {
					stmt.setBigDecimal(parameterIndex, new BigDecimal(param_to_long(param)));
				} else if (ttttt == Types.DOUBLE) {
					stmt.setDouble(parameterIndex, param_to_double(param));
				} else if (ttttt == Types.VARCHAR) {
					stmt.setString(parameterIndex, (String) param);
				}
			}
		}
		ResultSet rs = stmt.executeQuery();
		if (rs.next()) {
			result = rs.getInt(1);
		}
		rs.close();
		stmt.close();
		return result;
	}

	public static int ori_executeQuery(Connection conn, String stringSQL) throws Exception {
		int result = 0;
		PreparedStatement stmt = conn.prepareStatement(stringSQL);
		ResultSet rs = stmt.executeQuery();
		if (rs.next()) {
			result = rs.getInt(1);
		}
		rs.close();
		stmt.close();
		return result;
	}

	public static int ori_executeUpdate(Connection conn, String stringSQL, List<Integer> typeList, List<Object> params) throws Exception {
		int result = 0;
		PreparedStatement stmt = conn.prepareStatement(stringSQL);
		for (int i = 0; i < params.size(); i++) {
			int parameterIndex = i + 1;
			Object param = params.get(i);
			int ttttt = typeList.get(i);
			if (param == null) {
				stmt.setNull(parameterIndex, ttttt);
			} else {
				if (ttttt == Types.INTEGER) {
					stmt.setInt(parameterIndex, param_to_int(param));
				} else if (ttttt == Types.BIGINT) {
					stmt.setBigDecimal(parameterIndex, new BigDecimal(param_to_long(param)));
				} else if (ttttt == Types.DOUBLE) {
					stmt.setDouble(parameterIndex, param_to_double(param));
				} else if (ttttt == Types.VARCHAR) {
					stmt.setString(parameterIndex, (String) param);
				}
			}
		}
		result = stmt.executeUpdate();
		stmt.close();
		return result;
	}

	public static ResultSet_MySQL ori_select(Connection conn, String stringSQL, List<Integer> typeList, List<Object> params) throws Exception {
		ResultSet_MySQL result = new ResultSet_MySQL();

		PreparedStatement stmt = conn.prepareStatement(stringSQL);
		for (int i = 0; i < params.size(); i++) {
			int parameterIndex = i + 1;
			Object param = params.get(i);
			int ttttt = typeList.get(i);
			if (param == null) {
				stmt.setNull(parameterIndex, ttttt);
			} else {
				if (ttttt == Types.INTEGER) {
					stmt.setInt(parameterIndex, param_to_int(param));
				} else if (ttttt == Types.BIGINT) {
					stmt.setBigDecimal(parameterIndex, new BigDecimal(param_to_long(param)));
				} else if (ttttt == Types.DOUBLE) {
					stmt.setDouble(parameterIndex, param_to_double(param));
				} else if (ttttt == Types.VARCHAR) {
					stmt.setString(parameterIndex, (String) param);
				}
			}
		}
		ResultSet rs = stmt.executeQuery();
		ResultSetMetaData rsmd = rs.getMetaData();
		String[] columnLabels = new String[rsmd.getColumnCount()];
		int[] columnTypes = new int[rsmd.getColumnCount()];
		for (int i = 1; i <= rsmd.getColumnCount(); i++) {
			columnLabels[i - 1] = rsmd.getColumnLabel(i);
			columnTypes[i - 1] = rsmd.getColumnType(i);
		}
		result.columnLabels = columnLabels;
		result.columnTypes = columnTypes;
		while (rs.next()) {
			Object[] columns = new Object[columnLabels.length];
			for (int i = 0; i < columnLabels.length; i++) {
				int columnType = columnTypes[i];
				if (columnType == Types.INTEGER) {
					columns[i] = rs.getInt(columnLabels[i]);
				} else if (columnType == Types.BIGINT) {
					try {
						columns[i] = rs.getBigDecimal(columnLabels[i]);
					} catch (Exception e) {
						columns[i] = rs.getLong(columnLabels[i]);
					}
				} else if (columnType == Types.DOUBLE) {
					columns[i] = rs.getDouble(columnLabels[i]);
				} else if (columnType == Types.VARCHAR) {
					columns[i] = rs.getString(columnLabels[i]);
				} else if (columnType == Types.LONGVARCHAR) {
					columns[i] = rs.getString(columnLabels[i]);
				} else {
					throw new MyException("");
				}
				if (rs.wasNull()) {
					columns[i] = null;
				}
			}
			result.rowList.add(columns);
		}
		rs.close();
		stmt.close();

		return result;
	}

	public static JSONArray ori_select(Connection conn, String stringSQL, List<Integer> typeList, List<Object> params, ZillionTableSchema schema)
			throws Exception {
		JSONArray result = new JSONArray();

		PreparedStatement stmt = conn.prepareStatement(stringSQL);
		for (int i = 0; i < params.size(); i++) {
			int parameterIndex = i + 1;
			Object param = params.get(i);
			int ttttt = typeList.get(i);
			if (param == null) {
				stmt.setNull(parameterIndex, ttttt);
			} else {
				if (ttttt == Types.INTEGER) {
					stmt.setInt(parameterIndex, param_to_int(param));
				} else if (ttttt == Types.BIGINT) {
					stmt.setBigDecimal(parameterIndex, new BigDecimal(param_to_long(param)));
				} else if (ttttt == Types.DOUBLE) {
					stmt.setDouble(parameterIndex, param_to_double(param));
				} else if (ttttt == Types.VARCHAR) {
					stmt.setString(parameterIndex, (String) param);
				}
			}
		}
		ResultSet rs = stmt.executeQuery();
		ResultSetMetaData rsmd = rs.getMetaData();
		String[] columnLabels = new String[rsmd.getColumnCount()];
		int[] columnTypes = new int[rsmd.getColumnCount()];
		for (int i = 1; i <= rsmd.getColumnCount(); i++) {
			columnLabels[i - 1] = rsmd.getColumnLabel(i);
			columnTypes[i - 1] = rsmd.getColumnType(i);
		}
		while (rs.next()) {
			JSONObject contentItem = new JSONObject();
			for (int i = 0; i < columnLabels.length; i++) {
				String columnLabel = columnLabels[i];
				int columnType = columnTypes[i];
				ZillionTableColumn columnDef = schema.ColumnMap.get(columnLabel);
				if (columnDef == null) {
					Iterator<String> keyIter_def = schema.ColumnMap.keySet().iterator();
					while (keyIter_def.hasNext()) {
						String key_def = keyIter_def.next();
						if (key_def.equalsIgnoreCase(columnLabel)) {
							columnDef = schema.ColumnMap.get(key_def);
							break;
						}
					}
				}

				Object columnObject = null;
				if (columnType == Types.INTEGER) {
					columnObject = rs.getInt(columnLabels[i]);
				} else if (columnType == Types.BIGINT) {
					try {
						columnObject = rs.getBigDecimal(columnLabels[i]);
					} catch (Exception e) {
						columnObject = rs.getLong(columnLabels[i]);
					}
				} else if (columnType == Types.DOUBLE) {
					columnObject = rs.getDouble(columnLabels[i]);
				} else if (columnType == Types.VARCHAR) {
					columnObject = rs.getString(columnLabels[i]);
				} else if (columnType == Types.LONGVARCHAR) {
					columnObject = rs.getString(columnLabels[i]);
				} else {
					throw new MyException("");
				}
				if (rs.wasNull()) {
					columnObject = null;
				}

				String columnName = columnDef.Name;
				Object columnValue = null;
				if (columnObject != null) {
					if (columnDef.Type == ZillionTableColumnEnum.LONG) {
						if (columnObject instanceof Long) {
							columnValue = ((Long) columnObject).longValue();
						} else if (columnObject instanceof BigDecimal) {
							columnValue = ((BigDecimal) columnObject).longValue();
						} else {
							throw new Exception("ZillionTableColumnEnum.LONG:" + columnObject.getClass().getName());
						}
					} else if (columnDef.Type == ZillionTableColumnEnum.DOUBLE) {
						columnValue = ((Double) columnObject).doubleValue();
					} else if (columnDef.Type == ZillionTableColumnEnum.STRING || columnDef.Type == ZillionTableColumnEnum.COMPRESSSTRING) {
						if (columnObject.equals("") && columnDef.NullValueConvert) {
							columnObject = null;
						}
						columnValue = ((String) columnObject);
					} else if (columnDef.Type == ZillionTableColumnEnum.BOOLEAN) {
						columnValue = ((Integer) columnObject) == 1 ? true : false;
					}
				}
				contentItem.put(columnName, columnValue);
			}
			result.add(contentItem);
		}
		rs.close();
		stmt.close();

		return result;
	}

	public static double param_to_double(Object columnObject) throws Exception {
		double result;
		if (columnObject instanceof Integer) {
			result = ((Integer) columnObject).doubleValue();
		} else if (columnObject instanceof Long) {
			result = ((Long) columnObject).doubleValue();
		} else if (columnObject instanceof BigInteger) {
			result = ((BigInteger) columnObject).doubleValue();
		} else if (columnObject instanceof Float) {
			result = ((Float) columnObject).doubleValue();
		} else if (columnObject instanceof Double) {
			result = ((Double) columnObject).doubleValue();
		} else if (columnObject instanceof BigDecimal) {
			result = ((BigDecimal) columnObject).doubleValue();
		} else {
			throw new MyException("");
		}
		return result;
	}

	public static int param_to_int(Object columnObject) throws Exception {
		int result;
		if (columnObject instanceof Integer) {
			result = ((Integer) columnObject).intValue();
		} else if (columnObject instanceof Long) {
			result = ((Long) columnObject).intValue();
		} else if (columnObject instanceof BigInteger) {
			result = ((BigInteger) columnObject).intValue();
		} else if (columnObject instanceof Boolean) {
			result = ((Boolean) columnObject) ? 1 : 0;
		} else {
			throw new MyException("");
		}
		return result;
	}

	public static long param_to_long(Object columnObject) throws Exception {
		long result;
		if (columnObject instanceof Integer) {
			result = ((Integer) columnObject).longValue();
		} else if (columnObject instanceof Long) {
			result = ((Long) columnObject).longValue();
		} else if (columnObject instanceof BigInteger) {
			result = ((BigInteger) columnObject).longValue();
		} else if (columnObject instanceof BigDecimal) {
			result = ((BigDecimal) columnObject).longValue();
		} else if (columnObject instanceof Boolean) {
			result = ((Boolean) columnObject) ? 1 : 0;
		} else {
			throw new MyException("");
		}
		return result;
	}
}
