package com.zillion.util.common;

import java.nio.charset.Charset;

import org.apache.hadoop.hbase.util.Bytes;

import com.zillion.util.table.ZillionTableColumnEnum;

public class ByteToObject {
	public static final int SIZEOF_INT = Integer.SIZE / Byte.SIZE;
	public static final int SIZEOF_LONG = Long.SIZE / Byte.SIZE;
	public static final int SIZEOF_FLOAT = Float.SIZE / Byte.SIZE;
	public static final int SIZEOF_DOUBLE = Double.SIZE / Byte.SIZE;
	public static final int SIZEOF_BOOLEAN = Byte.SIZE / Byte.SIZE;

	public static final Charset UTF8_CHARSET = Charset.forName("UTF-8");

	public static int lengthOfColumnEnum(ZillionTableColumnEnum ColumnEnum) {
		if (ColumnEnum == ZillionTableColumnEnum.INT) {
			return SIZEOF_LONG;
		} else if (ColumnEnum == ZillionTableColumnEnum.LONG) {
			return SIZEOF_LONG;
		} else if (ColumnEnum == ZillionTableColumnEnum.FLOAT) {
			return SIZEOF_DOUBLE;
		} else if (ColumnEnum == ZillionTableColumnEnum.DOUBLE) {
			return SIZEOF_DOUBLE;
		} else if (ColumnEnum == ZillionTableColumnEnum.BOOLEAN) {
			return SIZEOF_BOOLEAN;
		} else {
			return 0;
		}
	}

	public static void putInt(byte[] byteArray, int offset, int value) {
		Bytes.putInt(byteArray, offset, value);
		process_Long(byteArray, offset);
	}

	public static void putLong(byte[] byteArray, int offset, long value) {
		Bytes.putLong(byteArray, offset, value);
		process_Long(byteArray, offset);
	}

	public static void putFloat(byte[] byteArray, int offset, float value) {
		Bytes.putFloat(byteArray, offset, value);
		process_Double_toBytes(byteArray, offset, Bytes.SIZEOF_FLOAT);
	}

	public static void putDouble(byte[] byteArray, int offset, double value) {
		Bytes.putDouble(byteArray, offset, value);
		process_Double_toBytes(byteArray, offset, Bytes.SIZEOF_DOUBLE);
	}

	public static void putBoolean(byte[] byteArray, int offset, boolean value) {
		byteArray[offset] = (value ? (byte) -1 : (byte) 0);
	}

	public static byte[] toBytes(int value) {
		byte[] result = Bytes.toBytes(value);
		process_Long(result, 0);
		return result;
	}

	public static byte[] toBytes(long value) {
		byte[] result = Bytes.toBytes(value);
		process_Long(result, 0);
		return result;
	}

	public static byte[] toBytes(float value) {
		byte[] result = Bytes.toBytes(value);
		process_Double_toBytes(result, 0, Bytes.SIZEOF_FLOAT);
		return result;
	}

	public static byte[] toBytes(double value) {
		byte[] result = Bytes.toBytes(value);
		process_Double_toBytes(result, 0, Bytes.SIZEOF_DOUBLE);
		return result;
	}

	public static byte[] toBytes(String value) {
		return value.getBytes(UTF8_CHARSET);
	}

	public static byte[] toBytes(boolean value) {
		return new byte[] { value ? (byte) -1 : (byte) 0 };
	}

	public static int toInt(byte[] value) {
		return toInt(value, 0);
	}

	public static int toInt(byte[] value, int offset) {
		byte[] result = new byte[Bytes.SIZEOF_INT];
		System.arraycopy(value, offset, result, 0, result.length);
		process_Long(result, 0);
		return Bytes.toInt(result);
	}

	public static long toLong(byte[] value) {
		return toLong(value, 0);
	}

	public static long toLong(byte[] value, int offset) {
		byte[] result = new byte[Bytes.SIZEOF_LONG];
		System.arraycopy(value, offset, result, 0, result.length);
		process_Long(result, 0);
		return Bytes.toLong(result);
	}

	public static float toFloat(byte[] value) {
		return toFloat(value, 0);
	}

	public static float toFloat(byte[] value, int offset) {
		byte[] result = new byte[Bytes.SIZEOF_FLOAT];
		System.arraycopy(value, offset, result, 0, result.length);
		process_Double_toDouble(result, 0, result.length);
		return Bytes.toFloat(result);
	}

	public static double toDouble(byte[] value) {
		return toDouble(value, 0);
	}

	public static double toDouble(byte[] value, int offset) {
		byte[] result = new byte[Bytes.SIZEOF_DOUBLE];
		System.arraycopy(value, offset, result, 0, result.length);
		process_Double_toDouble(result, 0, result.length);
		return Bytes.toDouble(result);
	}

	public static String toString(byte[] value) {
		return toString(value, 0, value.length);
	}

	public static String toString(byte[] value, int offset, int len) {
		return new String(value, offset, len, UTF8_CHARSET);
	}

	public static boolean toBoolean(byte[] value) {
		return toBoolean(value, 0);
	}

	public static boolean toBoolean(byte[] value, int offset) {
		return value[offset] != (byte) 0;
	}

	public static void process_Long(byte[] result, int offset) {
		int first = 0xFF & result[offset];
		if (first >= 0x80) {
			first -= 0x80;
		} else {
			first += 0x80;
		}
		result[offset] = (byte) first;
	}

	public static void process_Double_toBytes(byte[] result, int offset, int len) {
		int first = 0xFF & result[offset];
		if (first >= 0x80) {// Ҫλλȡλ룬β
			int tmp;
			for (int i = offset; i < offset + len; i++) {
				tmp = 0xFF & result[i];
				tmp = 0xFF - tmp;
				result[i] = (byte) tmp;
			}
		} else {// ܼ
			first += 0x80;
			result[offset] = (byte) first;
		}
	}

	public static void process_Double_toDouble(byte[] result, int offset, int len) {
		int first = 0xFF & result[offset];
		if (first < 0x80) {// Ҫλλȡλ룬β
			int tmp;
			for (int i = offset; i < offset + len; i++) {
				tmp = 0xFF & result[i];
				tmp = 0xFF - tmp;
				result[i] = (byte) tmp;
			}
		} else {// ܼ
			first -= 0x80;
			result[offset] = (byte) first;
		}
	}

	public static byte[] lastBytes(byte[] bytes_gt) {
		byte[] result;
		int index_not_0 = -1;
		for (int i = bytes_gt.length - 1; i >= 0; i--) {
			if (bytes_gt[i] != 0x00) {
				index_not_0 = i;
				break;
			}
		}
		if (index_not_0 == -1) {
			result = new byte[0];

			result = new byte[bytes_gt.length - 1];
			System.arraycopy(bytes_gt, 0, result, 0, bytes_gt.length - 1);
		} else {
			result = new byte[bytes_gt.length];
			System.arraycopy(bytes_gt, 0, result, 0, index_not_0 + 1);
			int a = result[index_not_0];
			int b = a - 1;
			result[index_not_0] = (byte) b;
			for (int i = index_not_0 + 1; i < bytes_gt.length; i++) {
				result[i] = -1;
			}
		}
		return result;
	}
}
