package com.zillion.util.table;

import org.apache.hadoop.hbase.HColumnDescriptor;
import org.apache.hadoop.hbase.HTableDescriptor;
import org.apache.hadoop.hbase.NamespaceDescriptor;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.io.compress.Compression;

import com.zillion.util.common.ZillionConstant;

public class HBaseAdminUtil {
	public static boolean tableExists(Admin admin, TableName tableName) throws Exception {
		return admin.tableExists(tableName);
	}

	public static void CreateNamespace(Admin admin, Admin admin_bak, String namespace) throws Exception {
		if (admin_bak != null) {
			NamespaceDescriptor namespaceDesc = NamespaceDescriptor.create(namespace).build();
			admin_bak.createNamespace(namespaceDesc);
		}
		{
			NamespaceDescriptor namespaceDesc = NamespaceDescriptor.create(namespace).build();
			admin.createNamespace(namespaceDesc);
		}
	}

	public static void DeleteNamespace(Admin admin, Admin admin_bak, String namespace) throws Exception {
		{
			admin.deleteNamespace(namespace);
		}
		if (admin_bak != null) {
			admin_bak.deleteNamespace(namespace);
		}
	}

	public static void createTable(Admin admin, Admin admin_bak, TableName tableName, String[] familyNames, byte[][] splitKeys, boolean inMemory,
			Compression.Algorithm algo) throws Exception {
		if (admin_bak != null) {
			HTableDescriptor desc = new HTableDescriptor(tableName);
			for (int i = 0; i < familyNames.length; i++) {
				String familyName = familyNames[i];
				HColumnDescriptor coldef = new HColumnDescriptor(familyName);
				coldef.setMaxVersions(ZillionConstant.maxVersions);
				coldef.setInMemory(inMemory);
				if (algo != Compression.Algorithm.NONE) {
					coldef.setCompressionType(algo);
					coldef.setCompactionCompressionType(algo);
				}
				desc.addFamily(coldef);
			}
			if (splitKeys != null) {
				admin_bak.createTable(desc, splitKeys);
			} else {
				admin_bak.createTable(desc);
			}
		}
		{
			HTableDescriptor desc = new HTableDescriptor(tableName);
			for (int i = 0; i < familyNames.length; i++) {
				String familyName = familyNames[i];
				HColumnDescriptor coldef = new HColumnDescriptor(familyName);
				coldef.setMaxVersions(ZillionConstant.maxVersions);
				coldef.setInMemory(inMemory);
				if (algo != Compression.Algorithm.NONE) {
					coldef.setCompressionType(algo);
					coldef.setCompactionCompressionType(algo);
				}
				if (admin_bak != null) {
					coldef.setScope(1);
				}
				desc.addFamily(coldef);
			}
			if (splitKeys != null) {
				admin.createTable(desc, splitKeys);
			} else {
				admin.createTable(desc);
			}
		}
	}

	public static void modifyColumn_compression(Admin admin, Admin admin_bak, TableName tableName, String[] familyNames, Compression.Algorithm algo)
			throws Exception {
		if (admin_bak != null) {
			for (int i = 0; i < familyNames.length; i++) {
				String familyName = familyNames[i];
				HColumnDescriptor coldef = new HColumnDescriptor(familyName);
				coldef.setCompressionType(algo);
				coldef.setCompactionCompressionType(algo);
				admin_bak.modifyColumn(tableName, coldef);
			}
			if (ZillionConstant.alter_compress_major_compact) {
				admin_bak.majorCompact(tableName);
			}
		}
		{
			for (int i = 0; i < familyNames.length; i++) {
				String familyName = familyNames[i];
				HColumnDescriptor coldef = new HColumnDescriptor(familyName);
				coldef.setCompressionType(algo);
				coldef.setCompactionCompressionType(algo);
				admin.modifyColumn(tableName, coldef);
			}
			if (ZillionConstant.alter_compress_major_compact) {
				admin.majorCompact(tableName);
			}
		}
	}

	public static void modifyColumn_in_memory(Admin admin, Admin admin_bak, TableName tableName, String[] familyNames, boolean enable)
			throws Exception {
		if (admin_bak != null) {
			for (int i = 0; i < familyNames.length; i++) {
				String familyName = familyNames[i];
				HColumnDescriptor coldef = new HColumnDescriptor(familyName);
				coldef.setInMemory(enable);
				admin_bak.modifyColumn(tableName, coldef);
			}
		}
		{
			for (int i = 0; i < familyNames.length; i++) {
				String familyName = familyNames[i];
				HColumnDescriptor coldef = new HColumnDescriptor(familyName);
				coldef.setInMemory(enable);
				admin.modifyColumn(tableName, coldef);
			}
		}
	}

	public static void deleteTable(Admin admin, Admin admin_bak, TableName tableName) throws Exception {
		{
			if (admin.tableExists(tableName)) {
				if (admin.isTableEnabled(tableName)) {
					admin.disableTable(tableName);
				}
				admin.deleteTable(tableName);
			}
		}
		if (admin_bak != null) {
			if (admin_bak.tableExists(tableName)) {
				if (admin_bak.isTableEnabled(tableName)) {
					admin_bak.disableTable(tableName);
				}
				admin_bak.deleteTable(tableName);
			}
		}
	}

	public static void clearTable(Admin admin, Admin admin_bak, TableName tableName) throws Exception {
		{
			if (admin.tableExists(tableName)) {
				if (admin.isTableEnabled(tableName)) {
					admin.disableTable(tableName);
				}
				admin.truncateTable(tableName, true);
			}
		}
		if (admin_bak != null) {
			if (admin_bak.tableExists(tableName)) {
				if (admin_bak.isTableEnabled(tableName)) {
					admin_bak.disableTable(tableName);
				}
				admin_bak.truncateTable(tableName, true);
			}
		}
	}

}
