package com.zillion.util.table;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.zillion.util.common.HexToByte;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

public class SchemaParser {
	public static void ParseIndexSchema(ZillionTableIndex result, ZillionTableSchema schema, JSONArray Columns) {
		Map<String, Boolean> columnExistMap = new HashMap<String, Boolean>();
		List<String> keyList = new ArrayList<String>();
		for (int i = 0; i < Columns.size(); i++) {
			String key_name = (String) Columns.get(i);

			if (columnExistMap.containsKey(key_name)) {
				continue;
			}
			columnExistMap.put(key_name, true);

			keyList.add(key_name);
		}
		if (schema.table_type == null) {
		} else {
			for (int i = 0; i < schema.Key.length; i++) {
				String key_name = schema.Key[i];

				if (columnExistMap.containsKey(key_name)) {
					continue;
				}
				columnExistMap.put(key_name, true);

				keyList.add(key_name);
			}
		}
		result.Key = keyList.toArray(new String[0]);
		result.KeyMap = new HashMap<String, Boolean>();
		for (String column : result.Key) {
			result.KeyMap.put(column, true);
		}
	}

	public static ZillionTableIndex ParseIndexSchema(ZillionTableSchema schema, JSONObject index_schema) {
		ZillionTableIndex result = new ZillionTableIndex(schema, (String) index_schema.get("Name"));
		JSONArray Columns = (JSONArray) index_schema.get("Columns");
		ParseIndexSchema(result, schema, Columns);
		return result;
	}

	public static void ParseTableSchema(ZillionTableSchema schema, JSONObject table_schema) {
		JSONArray Columns = (JSONArray) table_schema.get("Columns");
		JSONArray Key = (JSONArray) table_schema.get("Key");

		if (table_schema.containsKey("delete_UsePutV")) {
			schema.delete_UsePutV = (Boolean) table_schema.get("delete_UsePutV");
		}
		if (table_schema.containsKey("compress")) {
			schema.compress = (Boolean) table_schema.get("compress");
		}
		if (table_schema.containsKey("resident_memory")) {
			schema.resident_memory = (Boolean) table_schema.get("resident_memory");
		}
		if (table_schema.containsKey("store_in_minor_db")) {
			schema.store_in_minor_db = (Boolean) table_schema.get("store_in_minor_db");
		}
		if (table_schema.containsKey("strong_consistency")) {
			schema.strong_consistency = (Boolean) table_schema.get("strong_consistency");
		}
		if (table_schema.containsKey("elastic_search")) {
			schema.elastic_search = (Boolean) table_schema.get("elastic_search");
		}
		if (table_schema.containsKey("table_type")) {
			schema.table_type = (String) table_schema.get("table_type");
		}
		if (table_schema.containsKey("split_column")) {
			schema.split_column = (String) table_schema.get("split_column");
		}

		if (table_schema.containsKey("region_keys")) {
			JSONArray region_keys = (JSONArray) table_schema.get("region_keys");
			schema.splitKeys = new byte[region_keys.size()][];
			for (int index_key = 0; index_key < region_keys.size(); index_key++) {
				String region_key = (String) region_keys.get(index_key);
				schema.splitKeys[index_key] = HexToByte.hexToByte(region_key);
			}
		}

		schema.Key = new String[Key.size()];
		schema.KeyMap = new HashMap<String, Boolean>();
		for (int i = 0; i < Key.size(); i++) {
			schema.Key[i] = (String) Key.get(i);
			schema.KeyMap.put(schema.Key[i], true);
		}

		List<String> other_column = new ArrayList<String>();
		for (int i = 0; i < Columns.size(); i++) {
			JSONObject column = (JSONObject) Columns.get(i);
			ZillionTableColumn ztc = ParseColumn(column);
			schema.ColumnMap.put(ztc.Name, ztc);

			boolean iskey = false;
			for (int index_key = 0; index_key < schema.Key.length; index_key++) {
				String KeyItem = schema.Key[index_key];
				if (KeyItem.equals(ztc.Name)) {
					iskey = true;
					break;
				}
			}
			if (!iskey) {
				other_column.add(ztc.Name);
			}
		}
		schema.OtherColumn = other_column.toArray(new String[0]);
	}

	public static ZillionTableColumn ParseColumn(JSONObject column) {
		String Name = (String) column.get("Name");
		String Type = (String) column.get("Type");
		ZillionTableColumn ztc = new ZillionTableColumn();
		ztc.Name = Name;
		if (Type.startsWith("String")) {
			ztc.Type = ZillionTableColumnEnum.STRING;
			if (Type.length() > "String".length()) {
				ztc.max_length = Integer.parseInt(Type.substring("String".length() + 1, Type.length() - 1));
			}
		} else if (Type.startsWith("ASCIIString")) {
			ztc.Type = ZillionTableColumnEnum.STRING;
			ztc.ASCIIString = true;
			if (Type.length() > "ASCIIString".length()) {
				ztc.max_length = Integer.parseInt(Type.substring("ASCIIString".length() + 1, Type.length() - 1));
			}
		} else if (Type.startsWith("CompressString")) {
			ztc.Type = ZillionTableColumnEnum.COMPRESSSTRING;
			if (Type.length() > "CompressString".length()) {
				ztc.max_length = Integer.parseInt(Type.substring("CompressString".length() + 1, Type.length() - 1));
			}
		} else if (Type.equals("Int")) {
			ztc.Type = ZillionTableColumnEnum.INT;
		} else if (Type.equals("Long")) {
			ztc.Type = ZillionTableColumnEnum.LONG;
		} else if (Type.equals("Float")) {
			ztc.Type = ZillionTableColumnEnum.FLOAT;
		} else if (Type.equals("Double")) {
			ztc.Type = ZillionTableColumnEnum.DOUBLE;
		} else if (Type.equals("Boolean")) {
			ztc.Type = ZillionTableColumnEnum.BOOLEAN;
		}
		if (column.containsKey("NotNull")) {
			ztc.NotNull = (Boolean) column.get("NotNull");
		}
		if (ztc.Type == ZillionTableColumnEnum.STRING && !ztc.NotNull) {
			if (column.containsKey("NullValueConvert")) {
				ztc.NullValueConvert = (Boolean) column.get("NullValueConvert");
			}
		}

		return ztc;
	}
}
