package com.zillion.util.table;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.hadoop.conf.Configuration;
import org.apache.http.HttpHost;
import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.data.Stat;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;
import org.redisson.Redisson;
import org.redisson.config.Config;
import org.zillion.util.common.MyException;
import org.zillion.util.json.FastJsonReaderUtil;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.util.Configuration_ElasticSearch;
import com.zillion.database.util.Configuration_MySQL;
import com.zillion.database.util.Configuration_Redis;
import com.zillion.database.util.DatabaseImpl_HBase;
import com.zillion.database.util.DatabaseImpl_Mix;
import com.zillion.database.util.DatabaseImpl_MySQL;
import com.zillion.database.util.IDatabase;
import com.zillion.util.common.HashObject;
import com.zillion.util.common.ZillionConstant;
import com.zillion.util.zk.ZKLockRW;
import com.zillion.util.zk.ZKWrapper;

import redis.clients.jedis.JedisPoolConfig;
import redis.clients.jedis.JedisShardInfo;
import redis.clients.jedis.ShardedJedisPool;

public class SchemaRepositoryHelper {
	String full_name;

	public SchemaRepositoryHelper(String full_name) {
		this.full_name = full_name;
	}

	public void Initialize(SchemaRepositoryContainer container, String database_type, Configuration configuration, Configuration configuration_bak,
			Configuration_MySQL configuration_MySQL, Configuration_ElasticSearch configuration_ElasticSearch, Configuration_Redis configuration_Redis,
			boolean print, boolean observer) throws Exception {
		String zookeeper_connectionAddress = null;
		int zookeeper_sessionTimeout = 30000;
		ZKWrapper ZK;
		if (database_type.equals("HBase")) {
			zookeeper_connectionAddress = configuration.get("hbase.zookeeper.quorum");
			zookeeper_sessionTimeout = configuration.getInt("zookeeper.session.timeout", 30000);
			if (configuration.get("zillion.zookeeper.quorum") != null && configuration.get("zillion.zookeeper.session.timeout") != null) {
				zookeeper_connectionAddress = configuration.get("zillion.zookeeper.quorum");
				zookeeper_sessionTimeout = configuration.getInt("zillion.zookeeper.session.timeout", 30000);
			}
			container.DatabaseImpl = new DatabaseImpl_HBase(configuration, configuration_bak);
			container.DatabaseImpl.Start();
		} else if (database_type.equals("MySQL")) {
			zookeeper_connectionAddress = configuration_MySQL.zookeeper_connectionAddress;
			zookeeper_sessionTimeout = configuration_MySQL.zookeeper_sessionTimeout;
			container.DatabaseImpl = new DatabaseImpl_MySQL(configuration_MySQL);
			container.DatabaseImpl.Start();
		} else if (database_type.equals("Mix")) {
			zookeeper_connectionAddress = configuration.get("hbase.zookeeper.quorum");
			zookeeper_sessionTimeout = configuration.getInt("zookeeper.session.timeout", 30000);
			if (configuration.get("zillion.zookeeper.quorum") != null && configuration.get("zillion.zookeeper.session.timeout") != null) {
				zookeeper_connectionAddress = configuration.get("zillion.zookeeper.quorum");
				zookeeper_sessionTimeout = configuration.getInt("zillion.zookeeper.session.timeout", 30000);
			}
			container.DatabaseImpl = new DatabaseImpl_Mix(configuration, configuration_bak, configuration_MySQL, container);
			container.DatabaseImpl.Start();
		} else {
			throw new MyException("Database type error:" + database_type);
		}

		if (configuration_ElasticSearch != null) {
			container.esClient = new RestHighLevelClient(RestClient.builder(
					new HttpHost(configuration_ElasticSearch.hostname, configuration_ElasticSearch.port, configuration_ElasticSearch.scheme)));
		}

		if (configuration_Redis != null) {
			container.redisConfig = new JedisPoolConfig();
			container.redisConfig.setMaxTotal(100);
			container.redisConfig.setMaxIdle(50);
			container.redisConfig.setMaxWaitMillis(3000);
			container.redisConfig.setTestOnBorrow(true);
			container.redisConfig.setTestOnReturn(true);
			JedisShardInfo jedisShardInfo1 = new JedisShardInfo(configuration_Redis.hostname, configuration_Redis.port);
			jedisShardInfo1.setPassword(configuration_Redis.password);
			container.redisServerList = new LinkedList<JedisShardInfo>();
			container.redisServerList.add(jedisShardInfo1);
			container.redisPool = new ShardedJedisPool(container.redisConfig, container.redisServerList);

			container.redissonConfig = new Config();
			container.redissonConfig.useSingleServer().setAddress("redis://" + configuration_Redis.hostname + ":" + configuration_Redis.port)
					.setPassword(configuration_Redis.password);
			// container.redissonConfigconfig.useClusterServers().addNodeAddress("redis://192.168.3.222:6379").setPassword("foobared");
			container.redisson = Redisson.create(container.redissonConfig);
		}

		LogUtil.warn(" ********************************** " + "SchemaRepository Initialize" + "\t" + "begin");

		if ((database_type.equals("HBase") || database_type.equals("Mix"))
				&& (configuration.get("kerberos.principal") != null && configuration.get("keytab.file") != null)) {
			String jaas_conf_path = configuration.get("zillion.jaas.conf");
			String krb5_conf_path = configuration.get("zillion.krb5.conf");
			if (configuration.get("zillion.zookeeper.quorum") != null && configuration.get("zillion.zookeeper.session.timeout") != null) {
				ZK = new ZKWrapper(zookeeper_connectionAddress, zookeeper_sessionTimeout);
			} else {
				ZK = new ZKWrapper(zookeeper_connectionAddress, zookeeper_sessionTimeout, jaas_conf_path, krb5_conf_path);
			}
		} else {
			ZK = new ZKWrapper(zookeeper_connectionAddress, zookeeper_sessionTimeout);
		}

		Stat stat = ZK.exists(ZillionConstant.zk_root, false);
		if (stat == null) {
			try {// Դڵ
				ZK.create(ZillionConstant.zk_root, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
			} catch (Exception e) {
				// e.printStackTrace();
			}
		}

		String status = "begin";
		while (true) {
			Thread.sleep(1L);

			if (status.equals("begin")) {
				stat = ZK.exists(ZillionConstant.zk_root + ZillionConstant._initbegin, false);
				if (stat != null) {
					status = "check_begin_Y";
				} else {
					status = "check_begin_N";
				}
			} else if (status.equals("check_begin_Y")) {
				stat = ZK.exists(ZillionConstant.zk_root + ZillionConstant._initend, false);
				if (stat != null) {
					status = "check_end_Y";
				} else {
					status = "check_end_N";
				}
			} else if (status.equals("check_begin_N")) {
				try {// Դbeginڵ
					ZK.create(ZillionConstant.zk_root + ZillionConstant._initbegin, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
					status = "process";
				} catch (Exception e) {
					// e.printStackTrace();
					status = "begin";
				}
			} else if (status.equals("check_end_Y")) {
				status = "ready";
			} else if (status.equals("check_end_N")) {
				status = "check_begin_Y";
			} else if (status.equals("process")) {
				ZK.create(ZillionConstant.zk_root + ZillionConstant._ns, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
				ZK.create(ZillionConstant.zk_root + ZillionConstant._nslock, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);

				List<String> namespaceList = container.DatabaseImpl.ori_namespaceList();
				Map<String, Boolean> namespaceWhiteMap = new HashMap<String, Boolean>();
				if (configuration.get("zillion.namespace.list") != null) {
					String whiteListString = configuration.get("zillion.namespace.list");
					String[] splits = whiteListString.trim().split(",");
					for (String split : splits) {
						namespaceWhiteMap.put(split, true);
					}
				}
				for (String namespace : namespaceList) {
					// namespace
					if (namespaceWhiteMap.size() > 0 && !namespaceWhiteMap.containsKey(namespace)) {
						continue;
					}
					// listtablenamespace
					try {
						List<String> tableList = container.DatabaseImpl.ori_tableList(namespace);
						// пտԶԪݱɹʼzk
						if (tableList.size() == 0) {
							try {
								container.DatabaseImpl.meta_manage_create_data(namespace);
								container.DatabaseImpl.meta_manage_create_index(namespace);
								//
								InitializeNamespace(ZK, container.DatabaseImpl, namespace);
							} catch (Exception e) {
								LogUtil.warn(e);
							}
						} else {
							InitializeNamespace(ZK, container.DatabaseImpl, namespace);
						}
					} catch (Exception e) {
						LogUtil.warn(e);
					}
				}

				ZK.create(ZillionConstant.zk_root + ZillionConstant._initend, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
				status = "ready";
			} else if (status.equals("ready")) {
				break;
			}
		}

		container.repositoryMap = new HashMap<String, SchemaRepository>();
		container.ZK = ZK;
		this.ReadMeta(container, print, observer);

		SchemaRepositoryThread thread = new SchemaRepositoryThread(container, print, observer, this);
		thread.start();

		container.thread = thread;

		container.initialized = true;

		LogUtil.warn(" ********************************** " + "SchemaRepository Initialize" + "\t" + "end");
	}

	public JSONArray InitializeNamespace(ZKWrapper ZK, IDatabase DatabaseImpl, String namespace) throws Exception {
		JSONArray result = new JSONArray();
		String method_name = "InitializeNamespace";
		if (!DatabaseImpl.ori_tableExists(namespace, "zillion_meta_data") || !DatabaseImpl.ori_tableExists(namespace, "zillion_meta_index")) {
			return result;
		}

		List<String> tableNameList = new ArrayList<String>();
		List<String> tableContentList = new ArrayList<String>();
		List<HashObject> tableHashList = new ArrayList<HashObject>();
		List<String> indexNameList = new ArrayList<String>();
		List<String> indexContentList = new ArrayList<String>();
		List<HashObject> indexHashList = new ArrayList<HashObject>();
		List<String> splitNameList = new ArrayList<String>();
		List<HashObject> splitHashList = new ArrayList<HashObject>();
		DatabaseImpl.meta_read(namespace, tableNameList, tableContentList, tableHashList, indexNameList, indexContentList, indexHashList,
				splitNameList, splitHashList);

		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		ZK.create(zkpath_namespace, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
		ZK.create(zkpath_namespace + ZillionConstant.zk_metalock, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);

		String parent_path_meta = zkpath_namespace + ZillionConstant.zk_metalock;
		String lock_type_meta = "w";
		String node_path_meta = ZKLockRW.Create(ZK, parent_path_meta, lock_type_meta, this.full_name);
		LogUtil.info(" ********************************** " + method_name + " metalock:" + " begin " + node_path_meta);
		try {
			ZK.create(zkpath_namespace + ZillionConstant.zk_meta_data, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
			ZK.create(zkpath_namespace + ZillionConstant.zk_meta_index, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
			ZK.create(zkpath_namespace + ZillionConstant.zk_tablelocks, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
			ZK.create(zkpath_namespace + ZillionConstant.zk_cachelocks, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
			ZK.create(zkpath_namespace + ZillionConstant.zk_hash_exist, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
			ZK.create(zkpath_namespace + ZillionConstant.zk_hash_tmp, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
			ZK.create(zkpath_namespace + ZillionConstant.zk_hashlocks, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
			for (int i = 0; i < tableNameList.size(); i++) {
				String tableName = tableNameList.get(i);
				String content = tableContentList.get(i);
				HashObject hashObject = tableHashList.get(i);
				byte[] hash = hashObject.toBytes();
				ZK.create(zkpath_namespace + ZillionConstant.zk_meta_data + "/" + tableName, content.getBytes("UTF-8"), CreateMode.PERSISTENT);
				ZK.create(zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + tableName, "c".getBytes("UTF-8"), CreateMode.PERSISTENT);
				ZK.create(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName, hash, CreateMode.PERSISTENT);
				ZK.create(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + tableName, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
				ZK.create(zkpath_namespace + ZillionConstant.zk_hashlocks + "/" + tableName, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
				result.add(tableName);
			}
			for (int i = 0; i < indexNameList.size(); i++) {
				String name = indexNameList.get(i);
				String[] splits = name.split("\\.");
				String tableName = splits[0];
				String indexName = splits[1];
				String content = indexContentList.get(i);
				HashObject hashObject = indexHashList.get(i);
				byte[] hash = hashObject.toBytes();
				ZK.create(zkpath_namespace + ZillionConstant.zk_meta_index + "/" + (tableName + "." + indexName), content.getBytes("UTF-8"),
						CreateMode.PERSISTENT);
				ZK.create(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + (tableName + "." + indexName), hash, CreateMode.PERSISTENT);
				ZK.create(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + (tableName + "." + indexName), "true".getBytes("UTF-8"),
						CreateMode.PERSISTENT);
			}
			for (int i = 0; i < splitNameList.size(); i++) {
				String splitName = splitNameList.get(i);
				int index_ = splitName.lastIndexOf('_');
				String tableName = splitName.substring(0, index_);
				if (tableName.startsWith(ZillionConstant.prefix_data)) {
					continue;
				}
				String child_name = splitName.substring(index_ + 1);
				HashObject hashObject = splitHashList.get(i);
				byte[] hash = hashObject.toBytes();
				ZK.create(zkpath_namespace + ZillionConstant.zk_hash_exist + "/" + tableName + "/" + child_name, hash, CreateMode.PERSISTENT);
				ZK.create(zkpath_namespace + ZillionConstant.zk_hash_tmp + "/" + tableName + "/" + child_name, "true".getBytes("UTF-8"),
						CreateMode.PERSISTENT);
			}

			ZK.create(zkpath_namespace + ZillionConstant.zk_observer, "true".getBytes("UTF-8"), CreateMode.PERSISTENT);
		} catch (Exception e) {
			throw e;
		} finally {
			// ͷmeta
			ZKLockRW.Finish(ZK, node_path_meta);
			LogUtil.info(" ********************************** " + method_name + " metalock:" + " finish " + node_path_meta);
		}

		return result;
	}

	public void ReadMeta(SchemaRepositoryContainer container, boolean print, boolean observer) throws Exception {
		Stat stat;
		// ʼ-ReadMeta
		stat = container.ZK.exists(ZillionConstant.zk_root + ZillionConstant._initend, false);
		if (stat == null) {
			return;
		}

		Map<String, Boolean> namespaceWhiteMap = new HashMap<String, Boolean>();
		Configuration configuration = container.agent.configuration;
		if (configuration.get("zillion.namespace.list") != null) {
			String whiteListString = configuration.get("zillion.namespace.list");
			String[] splits = whiteListString.trim().split(",");
			for (String split : splits) {
				namespaceWhiteMap.put(split, true);
			}
		}

		List<String> children = container.ZK.getChildren(ZillionConstant.zk_root + ZillionConstant._ns, false);
		container.namespaceList = children;

		List<String> namespaceList = container.namespaceList;
		for (String namespace : namespaceList) {
			// namespace
			if (namespaceWhiteMap.size() > 0 && !namespaceWhiteMap.containsKey(namespace)) {
				continue;
			}
			SchemaRepository repository;
			if (container.repositoryMap.containsKey(namespace)) {
				repository = container.repositoryMap.get(namespace);
			} else {
				repository = new SchemaRepository();
				repository.namespace = namespace;
				container.repositoryMap.put(namespace, repository);
			}

			ReadMetaNamespace(container, namespace, repository, print, observer);
		}
	}

	private boolean ReadMetaNamespace(SchemaRepositoryContainer container, String namespace, SchemaRepository repository, boolean print,
			boolean observer) throws Exception {
		String method_name = this.full_name + "\t" + "ReadMetaNamespace" + "\t" + namespace;
		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;

		Stat stat;
		// 汾
		stat = new Stat();
		container.ZK.getData(zkpath_namespace + ZillionConstant.zk_tablelocks, false, stat);
		int repo_version = stat.getVersion();
		int exist_repo_version = repository.GetVersion();
		if (exist_repo_version != repo_version) {
			LogUtil.warn(" ********************************** ReadMetaNamespace " + namespace + " old version: " + exist_repo_version
					+ " new version: " + repo_version);
			if (container.without_lock) {
				this.ReadMetaNamespace_without_lock(container, namespace, repository, print);
			} else {
				String parent_path_meta = zkpath_namespace + ZillionConstant.zk_metalock;
				String lock_type_meta = "r";
				// ȡmeta
				String node_path_meta = ZKLockRW.Create(container.ZK, parent_path_meta, lock_type_meta, this.full_name);
				if (print) {
					LogUtil.debug(" ********************************** " + method_name + " metalock:" + " WaitFor " + node_path_meta);
				}
				try {
					boolean wait_success = ZKLockRW.WaitOnce(container.ZK, parent_path_meta, node_path_meta);
					if (print) {
						LogUtil.debug(" ********************************** " + method_name + " metalock:" + " WaitFor " + wait_success + " "
								+ node_path_meta);
					}
					if (!wait_success) {
						return false;
					}

					this.ReadMetaNamespace_without_lock(container, namespace, repository, print);

					if (print) {
						LogUtil.info(" ********************************** " + method_name + " work success ");
					}
					// -ReadMeta

					if (observer) {
						SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
						String observer_content = "" + repository.GetVersion() + "_" + sdf.format(new Date()) + "_" + this.full_name;
						if (repository.node_path == null) {
							repository.node_path = container.ZK.create(zkpath_namespace + ZillionConstant.zk_observer + "/" + "observer",
									observer_content.getBytes("UTF-8"), CreateMode.EPHEMERAL_SEQUENTIAL);
							if (print) {
								LogUtil.info(" ********************************** " + method_name + " " + zkpath_namespace
										+ ZillionConstant.zk_observer + " create data " + repository.GetVersion());
							}
						} else {
							container.ZK.setData(repository.node_path, observer_content.getBytes("UTF-8"), -1);
							if (print) {
								LogUtil.info(" ********************************** " + method_name + " " + zkpath_namespace
										+ ZillionConstant.zk_observer + " update data " + repository.GetVersion());
							}
						}
					}

					return true;
				} catch (Exception e) {
					throw e;
				} finally {
					// ͷmeta
					ZKLockRW.Finish(container.ZK, node_path_meta);
					if (print) {
						LogUtil.debug(" ********************************** " + method_name + " metalock:" + " finish " + node_path_meta);
					}
				}
			}
		} else {
			for (String tableName : repository.TableList()) {
				ZillionTableSchema schema = repository.GetTable(tableName);
				schema.timeout_unuse(container);
				schema.RefreshTableWrapper(container);
			}
		}
		return false;
	}

	private void ReadMetaNamespace_without_lock(SchemaRepositoryContainer container, String namespace, SchemaRepository repository, boolean print)
			throws Exception {
		String method_name = this.full_name + "\t" + "ReadMetaNamespace" + "\t" + namespace;
		String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + namespace;
		Stat stat = new Stat();

		Map<String, Boolean> validMap = new HashMap<String, Boolean>();
		List<String> indexFullNameList = container.ZK.getChildren(zkpath_namespace + ZillionConstant.zk_meta_index, false);
		List<String> tableNameList = container.ZK.getChildren(zkpath_namespace + ZillionConstant.zk_tablelocks, false);
		for (String tableName : tableNameList) {
			String parent_path_table = zkpath_namespace + ZillionConstant.zk_tablelocks + "/" + tableName;
			stat = new Stat();
			byte[] data_bytes = container.ZK.getData(parent_path_table, false, stat);
			int table_version = stat.getVersion();
			String data_String = new String(data_bytes, "UTF-8");
			if (data_String.equals("c")) {
				validMap.put(tableName, true);

				ZillionTableSchema exist_schema = repository.GetTable(tableName);
				if (exist_schema == null || exist_schema.version != table_version) {
					if (print) {
						LogUtil.info(" ********************************** " + method_name + " work read " + tableName);
					}
					stat = new Stat();
					data_bytes = container.ZK.getData(zkpath_namespace + ZillionConstant.zk_meta_data + "/" + tableName, false, stat);
					JSONObject content = (JSONObject) FastJsonReaderUtil.Instance().parse(new String(data_bytes, "UTF-8"));
					ZillionTableSchema schema = new ZillionTableSchema(namespace, tableName);
					schema.JSON = content;
					schema.version = table_version;
					SchemaParser.ParseTableSchema(schema, content);
					for (String indexFullName : indexFullNameList) {
						if (indexFullName.startsWith(tableName + ".")) {
							Stat statInner = new Stat();
							byte[] data_bytesInner = container.ZK.getData(zkpath_namespace + ZillionConstant.zk_meta_index + "/" + indexFullName,
									false, statInner);
							JSONArray index_Columns = (JSONArray) FastJsonReaderUtil.Instance().parse(new String(data_bytesInner, "UTF-8"));
							ZillionTableIndex table_index = new ZillionTableIndex(schema, indexFullName.substring(tableName.length() + 1));
							SchemaParser.ParseIndexSchema(table_index, schema, index_Columns);
							schema.AddIndex(table_index);
						}
					}
					if (exist_schema != null) {
						repository.DropTable(tableName);

						exist_schema.unuseTable(container.ZK);
					}
					repository.AddTable(schema);
					repository.SetTableVersion(tableName, table_version);
				}
			}
		}
		List<String> tableList = repository.TableList();
		for (String tableName : tableList) {
			if (!validMap.containsKey(tableName)) {
				ZillionTableSchema schema = repository.DropTable(tableName);
				schema.unuseTable(container.ZK);
			}
		}

		stat = new Stat();
		container.ZK.getData(zkpath_namespace + ZillionConstant.zk_tablelocks, false, stat);
		repository.SetVersion(stat.getVersion());
	}
}
