package com.zillion.util.zk;

import java.util.List;

import org.apache.zookeeper.KeeperException.NoNodeException;
import org.apache.zookeeper.KeeperException.NotEmptyException;
import org.apache.zookeeper.data.Stat;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSONArray;

public class ZKHelper {

	public static void recursive(ZKWrapper zk, String path) throws Exception {
		recursive(zk, path, true, null);
	}

	public static void recursive(ZKWrapper zk, String path, JSONArray result) throws Exception {
		recursive(zk, path, true, result);
	}

	public static void recursiveChildren(ZKWrapper zk, String path) throws Exception {
		recursiveChildren(zk, path, null);
	}

	public static void recursiveChildren(ZKWrapper zk, String path, JSONArray result) throws Exception {
		List<String> children = zk.getChildren(path, false);
		for (String child : children) {
			String parentPath = path.length() == 1 ? "/" : path + "/";
			String childPath = parentPath + child;

			recursive(zk, childPath, false, result);
		}
	}

	private static void recursive(ZKWrapper zk, String path, boolean not_exist_throw, JSONArray result) throws Exception {
		Stat stat = zk.exists(path, false);
		if (stat == null) {
			return;
		}

		if (result != null) {
			result.add(path);
		}

		List<String> children = zk.getChildren(path, false);
		for (String child : children) {
			String parentPath = path.length() == 1 ? "/" : path + "/";
			String childPath = parentPath + child;

			recursive(zk, childPath, false, result);
		}

		try {
			zk.delete(path, -1);
		} catch (NoNodeException e) {
			if (not_exist_throw) {
				LogUtil.warn(e.getMessage());
				throw e;
			}
		} catch (NotEmptyException e) {
			Thread.sleep(500L);
			try {
				zk.delete(path, -1);
			} catch (Exception e1) {
				LogUtil.warn(e.getMessage());
				throw e;
			}
		} catch (Exception e) {
			LogUtil.warn(e.getMessage());
			throw e;
		}
	}
}
