package com.zillion.util.zk;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.data.Stat;

import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.SchemaRepositoryContainer;

public class ZKLockBatch {
	public static void Create(SchemaRepositoryContainer container, String Namespace, String table_name, ZKNodeBatch nodeBatch) throws Exception {
		if (container.redissonConfig != null) {
		} else {
			ZKWrapper zk = container.ZK;
			String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + Namespace;
			String parent_path = zkpath_namespace + ZillionConstant.zk_cachelocks + "/" + table_name;

			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
			byte[] data = (sdf.format(new Date())).getBytes("UTF-8");
			for (ZKNodeUseLock node : nodeBatch.nodeList) {
				node.zk_path = zk.create(parent_path + "/" + node + "-", data, CreateMode.EPHEMERAL_SEQUENTIAL);
				int index_ = node.zk_path.lastIndexOf('-');
				node.zk_sequence = node.zk_path.substring(index_ + 1);
			}
		}
	}

	public static void WaitOnce(SchemaRepositoryContainer container, String Namespace, String table_name, ZKNodeBatch nodeBatch) throws Exception {
		if (container.redissonConfig != null) {
			synchronized (nodeBatch) {
				container.ThreadLocalRepository.myQueue.offer(nodeBatch);
				nodeBatch.wait(1000L);
				return;
			}
		} else {
			ZKWrapper zk = container.ZK;
			String zkpath_namespace = ZillionConstant.zk_root + ZillionConstant._ns + "/" + Namespace;
			String parent_path = zkpath_namespace + ZillionConstant.zk_cachelocks + "/" + table_name;

			List<String> children = zk.getChildren(parent_path, false);
			for (int i = 0; i < nodeBatch.nodeList.size(); i++) {
				ZKNodeUseLock node = nodeBatch.nodeList.get(i);
				boolean success = true;
				for (String child : children) {
					int index_ = child.lastIndexOf('-');
					String nodeInner = child.substring(0, index_);
					String sequenceInner = child.substring(index_ + 1);
					if (nodeInner.equals(node.key) && sequenceInner.compareTo(node.zk_sequence) < 0) {
						success = false;
						break;
					}
				}
				node.success = success;
			}
			nodeBatch.processed = true;
		}
	}

	public static void Finish(SchemaRepositoryContainer container, ZKNodeBatch nodeBatch) throws Exception {
		if (container.redissonConfig != null) {
			nodeBatch.time = new Date();
			nodeBatch.finish = true;
		} else {
			ZKWrapper zk = container.ZK;
			for (int i = 0; i < nodeBatch.nodeList.size(); i++) {
				ZKNodeUseLock node = nodeBatch.nodeList.get(i);
				try {
					Stat stat = zk.exists(node.zk_path, false);
					if (stat != null) {
						zk.delete(node.zk_path, -1);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}
}
