package com.zillion.util.zk;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.data.Stat;

public class ZKLockRW {
	public static String Create(ZKWrapper zk, String parent_path, String lock_type, String lock_content) throws Exception {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
		byte[] data = (sdf.format(new Date()) + "_" + lock_content).getBytes("UTF-8");
		String node_path = zk.create(parent_path + "/" + lock_type + "-", data, CreateMode.EPHEMERAL_SEQUENTIAL);
		return node_path;
	}

	public static boolean WaitFor(ZKWrapper zk, String parent_path, String node_path, int seconds) throws Exception {
		boolean result = false;
		Date time_from = new Date();
		while (true) {
			boolean wait_success = WaitOnce(zk, parent_path, node_path);
			if (wait_success) {
				result = true;
				break;
			}

			Thread.sleep(1000L);

			Date currentTime = new Date();
			if (currentTime.getTime() - time_from.getTime() > 1000L * seconds) {
				break;
			}
		}
		return result;
	}

	public static boolean WaitOnce(ZKWrapper zk, String parent_path, String node_path) throws Exception {
		char lock_type = node_path.charAt(parent_path.length() + 1);
		int sequence_id = Integer.parseInt(node_path.substring(parent_path.length() + 3));

		String exist = null;

		exist = null;
		List<String> children = zk.getChildren(parent_path, false);
		if (lock_type == 'r') {
			for (String child : children) {
				char tmp_lock_type = child.charAt(0);
				int tmp_id = Integer.parseInt(child.substring(2));
				if (tmp_lock_type == 'w' && tmp_id < sequence_id) {
					exist = child;
					break;
				}
			}
		} else {
			for (String child : children) {
				int tmp_id = Integer.parseInt(child.substring(2));
				if (tmp_id < sequence_id) {
					exist = child;
					break;
				}
			}
		}

		if (exist != null) {
			return false;
		} else {
			return true;
		}
	}

	@SuppressWarnings("unused")
	private static void Wait(ZKWrapper zk, String parent_path, String node_path) throws Exception {
		char lock_type = node_path.charAt(parent_path.length() + 1);
		int sequence_id = Integer.parseInt(node_path.substring(parent_path.length() + 3));

		String status = "begin";
		String exist = null;
		ZKLockSingal singal = null;
		while (true) {
			Thread.sleep(1L);

			if (status.equals("begin")) {
				exist = null;
				List<String> children = zk.getChildren(parent_path, false);
				if (lock_type == 'r') {
					for (String child : children) {
						char tmp_lock_type = child.charAt(0);
						int tmp_id = Integer.parseInt(child.substring(2));
						if (tmp_lock_type == 'w' && tmp_id < sequence_id) {
							exist = child;
							break;
						}
					}
				} else {
					for (String child : children) {
						int tmp_id = Integer.parseInt(child.substring(2));
						if (tmp_id < sequence_id) {
							exist = child;
							break;
						}
					}
				}

				if (exist != null) {
					status = "exist_Y";
				} else {
					status = "exist_N";
				}
			} else if (status.equals("exist_Y")) {
				singal = new ZKLockSingal();
				Stat stat = zk.exists(parent_path + "/" + exist, new ZKLockWatcher(singal));
				if (stat == null) {
					status = "begin";
				} else {
					status = "wait";
				}
			} else if (status.equals("wait")) {
				if (singal.deleted) {
					status = "begin";
				}
			} else if (status.equals("exist_N")) {
				status = "success";
			} else if (status.equals("success")) {
				break;
			}
		}
	}

	public static void Finish(ZKWrapper zk, String node_path) throws Exception {
		Stat stat = zk.exists(node_path, false);
		if (stat != null) {
			zk.delete(node_path, -1);
		}
	}
}
