package org.zillion.util.common;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Random;
import java.util.zip.Deflater;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;
import java.util.zip.Inflater;

public class CompressUtil {
	public static String algo = "gzip";// gzip,deflate
	public static int deflate_level = 1;

	public static byte[] compress(byte[] bytes) throws IOException {
		if (bytes == null || bytes.length == 0) {
			return bytes;
		}

		if (algo.equals("gzip")) {
			return gzip_compress(bytes);
		} else if (algo.equals("deflate")) {
			return deflate_compress(bytes, deflate_level);
		} else {
			return null;
		}
	}

	public static byte[] decompress(byte[] encrypted) throws IOException {
		if (encrypted == null || encrypted.length == 0) {
			return encrypted;
		}

		if (algo.equals("gzip")) {
			return gzip_decompress(encrypted);
		} else if (algo.equals("deflate")) {
			return deflate_decompress(encrypted);
		} else {
			return null;
		}
	}

	private static byte[] gzip_compress(byte[] bytes) throws IOException {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			GZIPOutputStream gzip = new GZIPOutputStream(out);
			gzip.write(bytes);
			gzip.close();
			return out.toByteArray();
		} finally {
			try {
				out.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private static byte[] gzip_decompress(byte[] encrypted) throws IOException {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		ByteArrayInputStream in = new ByteArrayInputStream(encrypted);
		try {
			GZIPInputStream gunzip = new GZIPInputStream(in);
			byte[] buffer = new byte[1024];
			int n;
			while ((n = gunzip.read(buffer)) >= 0) {
				out.write(buffer, 0, n);
			}
			return out.toByteArray();
		} finally {
			try {
				in.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			try {
				out.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private static byte[] deflate_compress(byte[] data, int level) {
		byte[] output = new byte[0];

		Deflater compresser = new Deflater(level);

		compresser.reset();
		compresser.setInput(data);
		compresser.finish();
		ByteArrayOutputStream bos = new ByteArrayOutputStream(data.length);
		try {
			byte[] buf = new byte[1024];
			while (!compresser.finished()) {
				int i = compresser.deflate(buf);
				bos.write(buf, 0, i);
			}
			output = bos.toByteArray();
		} catch (Exception e) {
			output = data;
			e.printStackTrace();
		} finally {
			try {
				bos.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		compresser.end();
		return output;
	}

	private static byte[] deflate_decompress(byte[] data) {
		byte[] output = new byte[0];

		Inflater decompresser = new Inflater();
		decompresser.reset();
		decompresser.setInput(data);

		ByteArrayOutputStream o = new ByteArrayOutputStream(data.length);
		try {
			byte[] buf = new byte[1024];
			while (!decompresser.finished()) {
				int i = decompresser.inflate(buf);
				o.write(buf, 0, i);
			}
			output = o.toByteArray();
		} catch (Exception e) {
			output = data;
			e.printStackTrace();
		} finally {
			try {
				o.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		decompresser.end();
		return output;
	}

	public static byte[] encode(byte[] bytes, Byte[] forbid_bytes) throws IOException {
		Byte n1 = null;
		Byte n2 = null;
		for (int i = -128; i < 128; i++) {
			byte tmp = (byte) i;
			boolean exist = false;
			for (byte tmp2 : forbid_bytes) {
				if (tmp2 == tmp) {
					exist = true;
					break;
				}
			}
			if (!exist) {
				if (n1 == null) {
					n1 = tmp;
				} else if (n2 == null) {
					n2 = tmp;
					break;
				}
			}
		}

		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			for (byte tmp : bytes) {
				if (tmp == n1) {
					out.write(n1);
					out.write(n2);
				} else {
					int index = -1;
					for (int i = 0; i < forbid_bytes.length; i++) {
						if (forbid_bytes[i] == tmp) {
							index = i;
							break;
						}
					}
					if (index == -1) {
						out.write(tmp);
					} else {
						for (int i = 0; i < index + 2; i++) {
							out.write(n1);
						}
						out.write(n2);
					}
				}
			}
			return out.toByteArray();
		} finally {
			try {
				out.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public static byte[] decode(byte[] encrypted, Byte[] forbid_bytes) throws Exception {
		Byte n1 = null;
		Byte n2 = null;
		for (int i = -128; i < 128; i++) {
			byte tmp = (byte) i;
			boolean exist = false;
			for (byte tmp2 : forbid_bytes) {
				if (tmp2 == tmp) {
					exist = true;
					break;
				}
			}
			if (!exist) {
				if (n1 == null) {
					n1 = tmp;
				} else if (n2 == null) {
					n2 = tmp;
					break;
				}
			}
		}

		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			int index = 0;
			while (index < encrypted.length) {
				byte tmp = encrypted[index];
				if (tmp == n1) {
					int index_n2 = -1;
					for (int i = index + 1; i < encrypted.length; i++) {
						if (encrypted[i] == n2) {
							index_n2 = i;
							break;
						}
					}
					if (index_n2 == -1) {
						throw new Exception();
					}
					int n1_count = index_n2 - index;
					if (n1_count == 1) {
						out.write(n1);
					} else {
						out.write(forbid_bytes[n1_count - 2]);
					}
					index = index_n2 + 1;
				} else {
					out.write(tmp);
					index++;
				}
			}
			return out.toByteArray();
		} finally {
			try {
				out.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public static void main(String[] args) throws Exception {
		Byte[] forbid_bytes = { 0x00, 0x01 };
		String a = "1101070037;1;report;20160101000000;123456;1000;1;101;33.440&1101070037;1;report;20160101000001;123456;1001;1;101;33.441&1101070037;1;report;20160101000002;123456;1002;1;101;33.442&1101070037;1;report;20160101000003;123456;1003;1;101;33.443&1101070037;1;report;20160101000004;123456;1004;1;101;33.444&1101070037;1;report;20160101000005;123456;1005;1;101;33.445&1101070037;1;report;20160101000006;123456;1006;1;101;33.446&1101070037;1;report;20160101000007;123456;1007;1;101;33.447&1101070037;1;report;20160101000008;123456;1008;1;101;33.448&1101070037;1;report;20160101000009;123456;1009;1;101;33.449&1101070037;1;report;20160101000010;123456;1000;1;101;33.4400&1101070037;1;report;20160101000011;123456;1001;1;101;33.4411&1101070037;1;report;20160101000012;123456;1002;1;101;33.4422&1101070037;1;report;20160101000013;123456;1003;1;101;33.4433&1101070037;1;report;20160101000014;123456;1004;1;101;33.4444&1101070037;1;report;20160101000015;123456;1005;1;101;33.4455&1101070037;1;report;20160101000016;123456;1006;1;101;33.4466&1101070037;1;report;20160101000017;123456;1007;1;101;33.4477&1101070037;1;report;20160101000018;123456;1008;1;101;33.4488&1101070037;1;report;20160101000019;123456;1009;1;101;33.4499";
		for (int i = 0; i < 11; i++) {
			byte[] ab = a.getBytes();
			byte[] e = CompressUtil.compress(ab);
			byte[] ee = CompressUtil.encode(e, forbid_bytes);
			byte[] ed = CompressUtil.decode(ee, forbid_bytes);
			byte[] d = CompressUtil.decompress(ed);
			System.out.println(new String(d));
			System.out.println(ab.length + "\t" + ee.length);
			a = a.substring(1);
		}

		byte[][] ori_bytes_array = new byte[100][];
		int ori_bytes_length = 0;
		byte[][] zip_bytes_array = new byte[ori_bytes_array.length][];
		int zip_bytes_length = 0;
		byte[][] result_bytes_array = new byte[ori_bytes_array.length][];
		for (int i = 0; i < ori_bytes_array.length; i++) {
			StringBuffer sb = new StringBuffer();
			Random rand = new Random();
			for (int ii = 0; ii < 1000000; ii++) {
				char c = (char) ('a' + rand.nextInt(26));
				sb.append(c);
			}
			ori_bytes_array[i] = sb.toString().getBytes();
			ori_bytes_length += ori_bytes_array[i].length;
		}
		// LogUtil.warn(sb.toString());

		{
			zip_bytes_length = 0;
			long t1 = (new Date()).getTime();
			for (int i = 0; i < ori_bytes_array.length; i++) {
				zip_bytes_array[i] = gzip_compress(ori_bytes_array[i]);
				zip_bytes_length += zip_bytes_array[i].length;
			}
			long t2 = (new Date()).getTime();
			for (int i = 0; i < ori_bytes_array.length; i++) {
				result_bytes_array[i] = gzip_decompress(zip_bytes_array[i]);
			}
			long t3 = (new Date()).getTime();
			System.out.println("gzip:" + 1.0 * zip_bytes_length / ori_bytes_length + "\t" + (t2 - t1) + "\t" + (t3 - t2));
		}

		for (int level = 1; level <= 9; level++) {
			zip_bytes_length = 0;
			long t1 = (new Date()).getTime();
			for (int i = 0; i < ori_bytes_array.length; i++) {
				zip_bytes_array[i] = deflate_compress(ori_bytes_array[i], level);
				zip_bytes_length += zip_bytes_array[i].length;
			}
			long t2 = (new Date()).getTime();
			for (int i = 0; i < ori_bytes_array.length; i++) {
				result_bytes_array[i] = deflate_decompress(zip_bytes_array[i]);
			}
			long t3 = (new Date()).getTime();
			System.out.println("deflate " + level + ":" + 1.0 * zip_bytes_length / ori_bytes_length + "\t" + (t2 - t1) + "\t" + (t3 - t2));
		}
	}
}