package org.zillion.util.common;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.util.common.JSONObjectComparator;

public class MemoryQueryUtil {
	private static MemoryQueryUtil instance = new MemoryQueryUtil();

	public static MemoryQueryUtil Instance() {
		return instance;
	}

	private MemoryQueryUtil() {

	}

	public JSONArray Delete(List<JSONObject> objectList, JSONObject criteria) {
		JSONArray Criterias = new JSONArray();
		Criterias.add(criteria);
		return this.query_inner(objectList, Criterias, null, true);
	}

	public JSONArray Select(List<JSONObject> objectList, JSONObject criteria, JSONArray OrderBy) {
		JSONArray Criterias = new JSONArray();
		Criterias.add(criteria);
		return this.query_inner(objectList, Criterias, OrderBy, false);
	}

	public JSONArray Select(List<JSONObject> objectList, JSONArray Criterias, JSONArray OrderBy) {
		return this.query_inner(objectList, Criterias, OrderBy, false);
	}

	private JSONArray query_inner(List<JSONObject> objectList, JSONArray Criterias, JSONArray OrderBy, boolean need_remove) {
		List<JSONObject> content = new ArrayList<JSONObject>();

		List<Map<String, Pattern>> PatternMapList = new ArrayList<Map<String, Pattern>>();
		{
			for (int ii = 0; ii < Criterias.size(); ii++) {
				Map<String, Pattern> PatternMap = new HashMap<String, Pattern>();
				JSONObject criteria = (JSONObject) Criterias.get(ii);
				Iterator<String> keyIter = criteria.keySet().iterator();
				while (keyIter.hasNext()) {
					String key = keyIter.next();
					Object condition = criteria.get(key);
					if (condition instanceof JSONObject) {
						JSONObject conditionJSON = (JSONObject) condition;
						if (conditionJSON.containsKey("$regex")) {
							String regex = (String) conditionJSON.get("$regex");
							Pattern pattern = Pattern.compile(regex);
							PatternMap.put(key, pattern);
						}
					}
				}
				PatternMapList.add(PatternMap);
			}
		}
		for (int i = objectList.size() - 1; i >= 0; i--) {
			JSONObject object = objectList.get(i);
			boolean has_all_match = false;
			for (int ii = 0; ii < Criterias.size(); ii++) {
				JSONObject criteria = (JSONObject) Criterias.get(ii);
				Map<String, Pattern> PatternMap = PatternMapList.get(ii);
				boolean all_match = true;
				Iterator<String> keyIter = criteria.keySet().iterator();
				while (keyIter.hasNext()) {
					String key = keyIter.next();
					Object itemValue = object.get(key);
					Object condition = criteria.get(key);
					Pattern Pattern = PatternMap.get(key);
					if (!this.Match(Pattern, itemValue, condition)) {
						all_match = false;
						break;
					}
				}
				if (all_match) {
					has_all_match = true;
					break;
				}
			}
			if (has_all_match) {
				content.add(object);
				if (need_remove) {
					objectList.remove(i);
				}
			}
		}
		if (OrderBy != null) {
			Collections.sort(content, new JSONObjectComparator(OrderBy));
		}

		JSONArray result = new JSONArray();
		for (JSONObject item : content) {
			result.add(item);
		}
		return result;
	}

	private boolean Match(Pattern Pattern, Object itemValue, Object condition) {
		if (condition == null) {
			if (itemValue == null) {
				return true;
			} else {
				return false;
			}
		} else {
			if (condition instanceof JSONObject) {
				JSONObject conditionJSON = (JSONObject) condition;
				boolean all_match = true;
				Iterator<String> keyIter = conditionJSON.keySet().iterator();
				while (keyIter.hasNext()) {
					String key = keyIter.next();
					Object valueInner = conditionJSON.get(key);
					if (!this.MatchItem(Pattern, itemValue, key, valueInner)) {
						all_match = false;
						break;
					}
				}
				if (all_match) {
					return true;
				} else {
					return false;
				}
			} else {
				return this.MatchItem(null, itemValue, "$e", condition);
			}
		}
	}

	private boolean MatchItem(Pattern Pattern, Object itemValue, String operator, Object valueInner) {
		if (operator.equals("in")) {
			boolean in_exist = false;
			JSONArray valueArray = (JSONArray) valueInner;
			for (int i = 0; i < valueArray.size(); i++) {
				Object valueObject = valueArray.get(i);
				if (this.MatchItem(Pattern, itemValue, "$e", valueObject)) {
					in_exist = true;
					break;
				}
			}
			if (in_exist) {
				return true;
			} else {
				return false;
			}
		}

		if (operator.equals("$regex")) {
			if (itemValue == null) {
				return false;
			} else {
				Matcher matcher = Pattern.matcher((String) itemValue);
				if (matcher.matches()) {
					return true;
				} else {
					return false;
				}
			}
		}

		if (itemValue == null || valueInner == null) {
			if (operator.equals("$e")) {
				if (itemValue == null && valueInner == null) {
					return true;
				} else {
					return false;
				}
			} else if (operator.equals("$lt")) {
				if (itemValue == null && valueInner != null) {
					return true;
				} else {
					return false;
				}
			} else if (operator.equals("$lte")) {
				if (itemValue == null) {
					return true;
				} else {
					return false;
				}
			} else if (operator.equals("$gt")) {
				if (itemValue != null) {
					return true;
				} else {
					return false;
				}
			} else if (operator.equals("$gte")) {
				if (valueInner == null) {
					return true;
				} else {
					return false;
				}
			} else {
				return false;
			}
		}

		if (itemValue instanceof Boolean && valueInner instanceof Boolean) {
			long v1 = ((Boolean) itemValue) ? 1 : 0;
			long v2 = ((Boolean) valueInner) ? 1 : 0;
			if (operator.equals("$e")) {
				return v1 == v2;
			} else if (operator.equals("$lt")) {
				return v1 < v2;
			} else if (operator.equals("$lte")) {
				return v1 <= v2;
			} else if (operator.equals("$gt")) {
				return v1 > v2;
			} else if (operator.equals("$gte")) {
				return v1 >= v2;
			} else {
				return false;
			}
		} else if (itemValue instanceof Long && valueInner instanceof Long) {
			long v1 = (Long) itemValue;
			long v2 = (Long) valueInner;
			if (operator.equals("$e")) {
				return v1 == v2;
			} else if (operator.equals("$lt")) {
				return v1 < v2;
			} else if (operator.equals("$lte")) {
				return v1 <= v2;
			} else if (operator.equals("$gt")) {
				return v1 > v2;
			} else if (operator.equals("$gte")) {
				return v1 >= v2;
			} else {
				return false;
			}
		} else if (itemValue instanceof Double && valueInner instanceof Double) {
			double v1 = (Double) itemValue;
			double v2 = (Double) valueInner;
			if (operator.equals("$e")) {
				return v1 == v2;
			} else if (operator.equals("$lt")) {
				return v1 < v2;
			} else if (operator.equals("$lte")) {
				return v1 <= v2;
			} else if (operator.equals("$gt")) {
				return v1 > v2;
			} else if (operator.equals("$gte")) {
				return v1 >= v2;
			} else {
				return false;
			}
		} else if (itemValue instanceof String && valueInner instanceof String) {
			String v1 = (String) itemValue;
			String v2 = (String) valueInner;
			if (operator.equals("$e")) {
				return v1.equals(v2);
			} else if (operator.equals("$lt")) {
				return v1.compareTo(v2) < 0;
			} else if (operator.equals("$lte")) {
				return v1.compareTo(v2) <= 0;
			} else if (operator.equals("$gt")) {
				return v1.compareTo(v2) > 0;
			} else if (operator.equals("$gte")) {
				return v1.compareTo(v2) >= 0;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

	public void SetColumn(JSONObject object, String Name, String Multiplicity, List<?> objectListInner) {
		if (Multiplicity.equals("1") || Multiplicity.equals("0..1")) {
			if (objectListInner.size() > 0) {
				object.put(Name, objectListInner.get(0));
			} else {
				object.put(Name, null);
			}
		} else if (Multiplicity.equals("0..n")) {
			JSONArray objectItems = new JSONArray();
			for (int index_objectSub = 0; index_objectSub < objectListInner.size(); index_objectSub++) {
				objectItems.add(objectListInner.get(index_objectSub));
			}
			object.put(Name, objectItems);
		}
	}
}
