package org.zillion.util.json;

import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import com.sagacloud.json.JSONArray;
import com.sagacloud.json.JSONObject;

public class JSONUtil {
	private static JSONUtil instance = new JSONUtil();

	public static JSONUtil Instance() {
		return instance;
	}

	private JSONUtil() {

	}

	public void Set_JSON(Object entity, JSONObject json) throws Exception {
		Class<?> targetClass = entity.getClass();

		Method[] targetMethodArray = targetClass.getMethods();
		for (int i = 0; i < targetMethodArray.length; i++) {
			Method method = targetMethodArray[i];
			int modifiers = method.getModifiers();
			String methodName = method.getName();
			if (modifiers == 1 && methodName.startsWith("get")) {
				Class<?>[] parameterTypes = method.getParameterTypes();
				if (parameterTypes.length == 0) {
					String fieldName = methodName.substring(3);
					fieldName = fieldName.substring(0, 1).toLowerCase() + fieldName.substring(1);
					if (json.containsKey(fieldName)) {
						json.remove(fieldName);
					}
					Object value = method.invoke(entity, new Object[] {});
					if (value != null) {
						String valueClassName = value.getClass().getName();
						Object put_value = null;
						if (valueClassName.equals("java.lang.String")) {
							put_value = value;
						} else if (valueClassName.equals("java.util.Date")) {
							SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
							put_value = sdf.format((Date) value);
						} else if (valueClassName.equals("java.lang.Long") || valueClassName.equals("long")) {
							put_value = value;
						} else if (valueClassName.equals("java.lang.Double") || valueClassName.equals("double")) {
							put_value = value;
						} else if (valueClassName.equals("java.lang.Boolean") || valueClassName.equals("boolean")) {
							put_value = value;
						} else {
							put_value = this.DeepCopyByObject(value);
						}

						json.put(fieldName, put_value);
					}
				}
			}
		}
	}

	public void Set_JavaObject(JSONObject json, Object entity) throws Exception {
		Class<?> targetClass = entity.getClass();

		if (json == null) {
			return;
		}

		Method[] targetMethodArray = targetClass.getMethods();
		for (int i = 0; i < targetMethodArray.length; i++) {
			Method method = targetMethodArray[i];
			int modifiers = method.getModifiers();
			String methodName = method.getName();
			if (modifiers == 1 && methodName.startsWith("set")) {
				Class<?>[] parameterTypes = method.getParameterTypes();
				if (parameterTypes.length == 1) {
					String fieldName = methodName.substring(3);
					fieldName = fieldName.substring(0, 1).toLowerCase() + fieldName.substring(1);
					Class<?> paramClass = parameterTypes[0];
					if (json.containsKey(fieldName)) {
						Object sourceFieldValue = json.get(fieldName);
						Object targetFieldValue = To_JavaObject(sourceFieldValue, paramClass);

						method.invoke(entity, new Object[] { targetFieldValue });
					}
				}
			}
		}
	}

	public Object DeepCopyByObject(Object source) throws Exception {
		Class<?> sourceClass = source.getClass();
		if (sourceClass.isArray()) {
			JSONArray result = new JSONArray();
			int array_length = Array.getLength(source);
			for (int i = 0; i < array_length; i++) {
				Object item = Array.get(source, i);
				result.add(this.DeepCopyByObject(item));
			}
			return result;
		} else {
			JSONObject result = new JSONObject();
			Method[] meethodArray = sourceClass.getMethods();
			for (int i = 0; i < meethodArray.length; i++) {
				Method method = meethodArray[i];
				int modifiers = method.getModifiers();
				String methodName = method.getName();
				if (modifiers == 1 && methodName.startsWith("get")) {
					Class<?>[] parameterTypes = method.getParameterTypes();
					if (parameterTypes.length == 0) {
						String fieldName = methodName.substring(3);
						fieldName = fieldName.substring(0, 1).toLowerCase() + fieldName.substring(1);
						Object value = method.invoke(source, new Object[] {});
						Object put_value;
						if (value == null) {
							put_value = null;
						} else {
							String valueClassName = value.getClass().getName();
							if (valueClassName.equals("java.lang.String")) {
								put_value = value;
							} else if (valueClassName.equals("java.util.Date")) {
								SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
								put_value = sdf.format(value);
							} else if (valueClassName.equals("java.lang.Long") || valueClassName.equals("long")) {
								put_value = value;
							} else if (valueClassName.equals("java.lang.Double") || valueClassName.equals("double")) {
								put_value = value;
							} else if (valueClassName.equals("java.lang.Boolean") || valueClassName.equals("boolean")) {
								put_value = value;
							} else {
								put_value = this.DeepCopyByObject(value);
							}
						}
						result.put(fieldName, put_value);
					}
				}
			}
			return result;
		}
	}

	public Object To_JavaObject(Object json, Class<?> entityClass) throws Exception {
		if (json == null) {
			return null;
		}

		String targetClassName = entityClass.getName();
		if (json instanceof JSONArray && entityClass.isArray()) {
			JSONArray sourceEntity = (JSONArray) json;
			int sourceLength = sourceEntity.size();

			Class<?> targetComponentType = entityClass.getComponentType();
			Object targetObjectArray = Array.newInstance(targetComponentType, sourceLength);

			for (int i = 0; i < sourceLength; i++) {
				Object sourceObject = sourceEntity.get(i);

				Object targetObject = To_JavaObject(sourceObject, targetComponentType);
				Array.set(targetObjectArray, i, targetObject);
			}

			return targetObjectArray;
		} else if (json instanceof JSONObject) {
			JSONObject sourceEntity = (JSONObject) json;

			Constructor<?> constructorMethod = entityClass.getConstructor(new Class<?>[] {});
			Object targetObject = constructorMethod.newInstance(new Object[] {});
			Method[] methodArray = entityClass.getMethods();
			for (int i = 0; i < methodArray.length; i++) {
				Method method = methodArray[i];
				int modifiers = method.getModifiers();
				String methodName = method.getName();
				if (modifiers == 1 && methodName.startsWith("set")) {
					Class<?>[] parameterTypes = method.getParameterTypes();
					if (parameterTypes.length == 1) {
						String fieldName = methodName.substring(3);
						fieldName = fieldName.substring(0, 1).toLowerCase() + fieldName.substring(1);
						Class<?> paramClass = parameterTypes[0];
						if (sourceEntity.containsKey(fieldName)) {
							Object sourceFieldValue = sourceEntity.get(fieldName);
							Object targetFieldValue = To_JavaObject(sourceFieldValue, paramClass);

							method.invoke(targetObject, new Object[] { targetFieldValue });
						}
					}
				}
			}

			return targetObject;
		} else if (json instanceof String && targetClassName.equals("java.lang.String")) {
			return (String) json;
		} else if (json instanceof String && targetClassName.equals("java.util.Date")) {
			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
			return sdf.parse((String) json);
		} else if (targetClassName.equals("java.lang.Long") || targetClassName.equals("long")) {
			if (json instanceof String) {
				Long value = Long.parseLong((String) json);
				return value;
			} else if (json instanceof Long) {
				Long value = (Long) json;
				return value;
			} else {
				return null;
			}
		} else if (targetClassName.equals("java.lang.Double") || targetClassName.equals("double")) {
			if (json instanceof String) {
				Double value = Double.parseDouble((String) json);
				return value;
			} else if (json instanceof Long) {
				Double value = ((Long) json).doubleValue();
				return value;
			} else if (json instanceof Double) {
				Double value = (Double) json;
				return value;
			} else {
				return null;
			}
		} else if (targetClassName.equals("java.lang.Boolean") || targetClassName.equals("boolean")) {
			if (json instanceof String) {
				Boolean value = ((String) json).equalsIgnoreCase("true") ? true : false;
				return value;
			} else if (json instanceof Boolean) {
				Boolean value = (Boolean) json;
				return value;
			} else {
				return null;
			}
		} else {
			return null;
		}
	}

	public Object Clone_JSON(Object source) {
		if (source instanceof JSONObject) {
			JSONObject result = new JSONObject();
			JSONObject sourceJSON = (JSONObject) source;
			List<String> keyList = sourceJSON.keySet();
			for (int i = 0; i < keyList.size(); i++) {
				String key = keyList.get(i);
				Object value = sourceJSON.get(key);
				result.put(key, Clone_JSON(value));
			}
			return result;
		} else if (source instanceof JSONArray) {
			JSONArray result = new JSONArray();
			JSONArray sourceJSON = (JSONArray) source;
			for (int i = 0; i < sourceJSON.size(); i++) {
				result.add(Clone_JSON(sourceJSON.get(i)));
			}
			return result;
		} else if (source instanceof String) {
			return source;
		} else if (source instanceof Double) {
			return source;
		} else if (source instanceof Long) {
			return source;
		} else if (source instanceof Boolean) {
			return source;
		}

		return null;
	}
}
