package org.zillion.util.log;

import java.io.File;
import java.net.URL;
import java.net.URLDecoder;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

public class LogUtil {
	private static Logger logger;

	static {
		try {
			String resource = "/log4j.properties";
			URL resourceUrl = LogUtil.class.getResource(resource);
			String resourcePath = null;
			if (resourceUrl == null) {
				resourcePath = System.getProperty("user.dir") + resource;
			} else {
				resourcePath = resourceUrl.getPath();
			}
			String GetPath = resourcePath.substring(0, resourcePath.length() - resource.length());
			String filepathLog4j = GetPath + "/log4j.properties";
			File fileLog4j = new File(filepathLog4j);
			if (fileLog4j.exists()) {
				PropertyConfigurator.configure(URLDecoder.decode(filepathLog4j, "UTF-8"));
				logger = Logger.getLogger(LogUtil.class);
			} else {
				PropertyConfigurator.configure(LogUtil.class.getResourceAsStream("/log4j.properties"));
				logger = Logger.getLogger(LogUtil.class);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void debug(String message) {
		if (logger != null) {
			logger.debug(message);
		} else {
			System.out.println("debug " + message);
		}
	}

	public static void info(String message) {
		if (logger != null) {
			logger.info(message);
		} else {
			System.out.println("info " + message);
		}
	}

	public static void warn(String message) {
		if (logger != null) {
			logger.warn(message);
		} else {
			System.out.println("warn " + message);
		}
	}

	public static void error(String message) {
		if (logger != null) {
			logger.error(message);
		} else {
			System.out.println("error " + message);
		}
	}

	public static void fatal(String message) {
		if (logger != null) {
			logger.fatal(message);
		} else {
			System.out.println("fatal " + message);
		}
	}

	public synchronized static void debug(Exception e) {
		String message = GetExceptionStackTrace(e);
		if (logger != null) {
			logger.debug(message);
		} else {
			System.out.println("debug " + message);
		}
	}

	public synchronized static void info(Exception e) {
		String message = GetExceptionStackTrace(e);
		if (logger != null) {
			logger.info(message);
		} else {
			System.out.println("info " + message);
		}
	}

	public synchronized static void warn(Exception e) {
		String message = GetExceptionStackTrace(e);
		if (logger != null) {
			logger.warn(message);
		} else {
			System.out.println("warn " + message);
		}
	}

	public synchronized static void error(Exception e) {
		String message = GetExceptionStackTrace(e);
		if (logger != null) {
			logger.error(message);
		} else {
			System.out.println("error " + message);
		}
	}

	public synchronized static void fatal(Exception e) {
		String message = GetExceptionStackTrace(e);
		if (logger != null) {
			logger.fatal(message);
		} else {
			System.out.println("fatal " + message);
		}
	}

	public static String GetExceptionStackTrace(Exception e) {
		StringBuffer sb = new StringBuffer();
		sb.append("Exception: ");
		sb.append("\n");
		sb.append(e.getClass().getName() + ": " + e.getMessage());
		sb.append("\n");
		StackTraceElement[] elements = e.getStackTrace();
		for (int i = 0; i < elements.length; i++) {
			StackTraceElement element = elements[i];
			sb.append("\t" + element.getClassName() + "." + element.getMethodName() + "(" + element.getFileName() + ":" + element.getLineNumber()
					+ ")");
			sb.append("\n");
		}
		Throwable cause = e.getCause();
		if (cause != null) {
			sb.append("Caused by: " + cause.getClass().getName() + ": " + cause.getMessage());
			sb.append("\n");
			StackTraceElement[] elements2 = cause.getStackTrace();
			for (int i = 0; i < elements2.length; i++) {
				StackTraceElement element = elements2[i];
				sb.append("\t" + element.getClassName() + "." + element.getMethodName() + "(" + element.getFileName() + ":" + element.getLineNumber()
						+ ")");
				sb.append("\n");
			}
		}
		return sb.toString();
	}
}
