import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.HTableDescriptor;
import org.apache.hadoop.hbase.NamespaceDescriptor;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;

import com.zillion.util.zk.ZKWrapper;

public class AgentResetWithoutCoprocessor {

	public static void main(String[] args) throws Exception {
		try {
			ZKWrapper zk = new ZKWrapper("zookeeper1:2181,zookeeper2:2181,zookeeper3:2181", 10000);
			recursive(zk, "/zillion");
			zk.close();
		} catch (Exception e) {
			e.printStackTrace();
		}

		try {
			Configuration configuration = HBaseConfiguration.create();
			Connection connection = ConnectionFactory.createConnection(configuration);
			Admin admin = connection.getAdmin();

			NamespaceDescriptor[] nss = admin.listNamespaceDescriptors();
			for (NamespaceDescriptor ns : nss) {
				if (ns.getName().equals("default") || ns.getName().equals("hbase")) {
					continue;
				}
				HTableDescriptor[] descs = admin.listTableDescriptorsByNamespace(ns.getName());
				for (HTableDescriptor desc : descs) {
					String namespace = desc.getTableName().getNamespaceAsString();
					String tableName_tmp = desc.getTableName().getNameAsString();
					System.out.println(namespace + "\t" + tableName_tmp);
					try {
						admin.disableTable(TableName.valueOf(tableName_tmp));
					} catch (Exception e) {
						e.printStackTrace();
					}
					try {
						admin.deleteTable(TableName.valueOf(tableName_tmp));
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
				try {
					admin.deleteNamespace(ns.getName());
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			admin.close();
			connection.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void recursive(ZKWrapper zk, String path) throws Exception {
		List<String> children = zk.getChildren(path, false);
		for (String child : children) {
			String parentPath = path.length() == 1 ? "/" : path + "/";
			String childPath = parentPath + child;

			recursive(zk, childPath);
		}

		zk.delete(path, -1);
		System.out.println("delete:" + path);
	}

	public static void recursiveChildren(ZKWrapper zk, String path) throws Exception {
		List<String> children = zk.getChildren(path, false);
		for (String child : children) {
			String parentPath = path.length() == 1 ? "/" : path + "/";
			String childPath = parentPath + child;

			recursive(zk, childPath);
		}
	}

}
