import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.apache.hadoop.hbase.client.Get;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter.RowRange;
import org.zillion.util.common.HexToByte;
import org.zillion.util.json.FastJsonReaderUtil;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.agent.ZillionAgent;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.SchemaParser;
import com.zillion.util.table.ZillionTableColumn;
import com.zillion.util.table.ZillionTableColumnEnum;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;

public class AgentTest1 {
	static boolean rebuildtable = true;

	static int size_Long = 100;
	static int size_String = 100;
	static int size_Boolean = 2;
	static int size_Double = 50;

	static String database_name = "database_1";
	static String table_name = "table_1";

	static String def_path = "etc/base1/def.json";
	// static String def_path = "etc/base1/def_delete_UsePutV.json";

	static JSONObject table_JSON;
	static JSONArray index_JSON;
	static ZillionTableSchema table_schema;

	static Configuration configuration = HBaseConfiguration.create();

	public static void main(String[] args) throws Exception {
		ZillionAgent agent = new ZillionAgent(configuration);
		agent.Start();

		agent.NDL_AddDatabase(database_name);

		// ʼ
		JSONObject def = FastJsonReaderUtil.Instance().ReadJSONObject(new File(def_path));
		table_schema = new ZillionTableSchema(database_name, table_name);
		table_schema.JSON = def;
		SchemaParser.ParseTableSchema(table_schema, table_schema.JSON);

		JSONArray Columns = (JSONArray) def.get("Columns");
		JSONArray Key = (JSONArray) def.get("Key");
		index_JSON = (JSONArray) def.get("Indexes");
		table_JSON = new JSONObject();
		table_JSON.put("Columns", Columns);
		table_JSON.put("Key", Key);
		if (def.containsKey("delete_UsePutV")) {
			table_JSON.put("delete_UsePutV", (Boolean) def.get("delete_UsePutV"));
		}

		List<List<Long>> timeListList = new ArrayList<List<Long>>();
		for (int i = 0; i < 1; i++) {
			Aztec(agent, timeListList);
		}

		agent.Stop();
		System.out.println();
	}

	static void Aztec(ZillionAgent agent, List<List<Long>> timeListList) throws Exception {
		List<Date> dateList = new ArrayList<Date>();
		dateList.add(new Date());

		if (rebuildtable) {
			boolean exist = false;
			JSONObject wrapperObject = new JSONObject();
			wrapperObject.put("QueryType", "table_list");
			wrapperObject.put("Database", database_name);
			JSONObject result = agent.Query(wrapperObject);
			JSONArray array = (JSONArray) result.get("Content");
			for (int ii = 0; ii < array.size(); ii++) {
				String tableNameInner = (String) array.get(ii);
				if (tableNameInner.equals(table_name)) {
					exist = true;
					break;
				}
			}
			if (exist) {
				agent.DDL_DropTable(database_name, table_name);
			}
			agent.DDL_AddTable(database_name, table_name, table_JSON, index_JSON);

			insert(agent);
			dateList.add(new Date());
		}

		// get_Long(agent);
		// dateList.add(new Date());
		// get_Long_onebyone(agent);
		// dateList.add(new Date());
		// get_String(agent);
		// dateList.add(new Date());
		// get_Double(agent);
		// dateList.add(new Date());
		select(agent);
		dateList.add(new Date());
		// select_index1_ori(agent);
		// dateList.add(new Date());
		select_index1(agent);
		dateList.add(new Date());
		select_index2(agent);
		dateList.add(new Date());

		// delete(agent);
		// dateList.add(new Date());
		// delete_index1(agent);
		// dateList.add(new Date());
		// delete_index2(agent);
		// dateList.add(new Date());
		// select(agent);
		// dateList.add(new Date());

		List<Long> timeList = new ArrayList<Long>();
		for (int ii = 1; ii < dateList.size(); ii++) {
			timeList.add(dateList.get(ii).getTime() - dateList.get(ii - 1).getTime());
		}
		timeListList.add(timeList);
		for (List<Long> timeListInner : timeListList) {
			StringBuffer sb = new StringBuffer();
			for (Long time : timeListInner) {
				sb.append("\t" + time);
			}
			LogUtil.info(sb.toString());
		}

		agent.container.repositoryMap.get(database_name).GetTable(table_name).unuseTable(agent.container.ZK);

	}

	static void insert(ZillionAgent agent) throws Exception {
		Random rand = new Random();
		LogUtil.info("insert begin:" + size_Long);
		for (int i = 0; i < size_Long; i++) {
			for (int ii = 0; ii < size_String; ii++) {
				for (int iii = 0; iii < size_Boolean; iii++) {
					JSONArray InsertObjects = new JSONArray();
					for (int iiii = 0; iiii < size_Double; iiii++) {
						int index = ((i * size_String + ii) * 2 + iii) * size_Double + iiii;
						JSONObject InsertObject = new JSONObject();
						InsertObject.put("KeyLong", (long) i);
						InsertObject.put("KeyString", "n" + ii);
						InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
						InsertObject.put("KeyDouble", 1.0 * iiii);
						InsertObject.put("ValueLong", (long) index);
						InsertObject.put("ValueString", "n" + ii);
						InsertObject.put("ValueBoolean", iii % 2 == 0 ? true : false);
						InsertObject.put("ValueDouble", 1.0 * iiii);
						int bytes_count = 1 + rand.nextInt(10);
						byte[] bytes = new byte[bytes_count];
						rand.nextBytes(bytes);
						InsertObject.put("ValueBytes", HexToByte.byteToHex(bytes));
						InsertObjects.add(InsertObject);
					}
					JSONObject wrapperObject = new JSONObject();
					wrapperObject.put("QueryType", "batch_put");
					wrapperObject.put("InsertObjects", InsertObjects);
					wrapperObject.put("Database", database_name);
					wrapperObject.put("Datatable", table_name);
					agent.Query(wrapperObject);
				}
			}

			if ((i + 1) % (size_Long / 10) == 0) {
				LogUtil.info("doing:" + (i + 1) + "/" + size_Long);
			}
		}
		LogUtil.info("finish:" + size_Long);
	}

	static void get_Long(ZillionAgent agent) throws Exception {
		LogUtil.info("get_Long begin:" + size_Long);
		for (int i = 0; i < size_Long; i++) {
			for (int ii = 0; ii < size_String; ii++) {
				JSONArray InsertObjects = new JSONArray();
				for (int iii = 0; iii < size_Boolean; iii++) {
					for (int iiii = 0; iiii < size_Double; iiii++) {
						JSONObject InsertObject = new JSONObject();
						InsertObject.put("KeyLong", (long) i);
						InsertObject.put("KeyString", "n" + ii);
						InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
						InsertObject.put("KeyDouble", 1.0 * iiii);
						InsertObjects.add(InsertObject);
					}
				}
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "batch_get");
				wrapperObject.put("Keys", InsertObjects);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				agent.Query(wrapperObject);
			}

			if ((i + 1) % (size_Long / size_Long) == 0) {
				LogUtil.info("doing:" + (i + 1) + "/" + size_Long);
			}
		}
		LogUtil.info("finish:" + size_Long);
	}

	static void get_Long_onebyone(ZillionAgent agent) throws Exception {
		LogUtil.info("get_Long_onebyone begin:" + size_Long);
		for (int i = 0; i < size_Long; i++) {
			for (int ii = 0; ii < size_String; ii++) {
				for (int iii = 0; iii < size_Boolean; iii++) {
					for (int iiii = 0; iiii < size_Double; iiii++) {
						JSONObject InsertObject = new JSONObject();
						InsertObject.put("KeyLong", (long) i);
						InsertObject.put("KeyString", "n" + ii);
						InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
						InsertObject.put("KeyDouble", 1.0 * iiii);

						JSONObject wrapperObject = new JSONObject();
						wrapperObject.put("QueryType", "get");
						wrapperObject.put("Key", InsertObject);
						wrapperObject.put("Database", database_name);
						wrapperObject.put("Datatable", table_name);
						agent.Query(wrapperObject);
					}
				}
			}

			if ((i + 1) % (size_Long / 10) == 0) {
				LogUtil.info("doing:" + (i + 1) + "/" + size_Long);
			}
		}
		LogUtil.info("finish:" + size_Long);
	}

	static void get_String(ZillionAgent agent) throws Exception {
		LogUtil.info("get_String begin:" + size_String);
		for (int ii = 0; ii < size_String; ii++) {
			for (int i = 0; i < size_Long; i++) {
				JSONArray InsertObjects = new JSONArray();
				for (int iii = 0; iii < size_Boolean; iii++) {
					for (int iiii = 0; iiii < size_Double; iiii++) {
						JSONObject InsertObject = new JSONObject();
						InsertObject.put("KeyLong", (long) i);
						InsertObject.put("KeyString", "n" + ii);
						InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
						InsertObject.put("KeyDouble", 1.0 * iiii);
						InsertObjects.add(InsertObject);
					}
				}
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "batch_get");
				wrapperObject.put("Keys", InsertObjects);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				agent.Query(wrapperObject);
			}

			if ((ii + 1) % (size_String / 10) == 0) {
				LogUtil.info("doing:" + (ii + 1) + "/" + size_String);
			}
		}
		LogUtil.info("finish:" + size_String);
	}

	static void get_Double(ZillionAgent agent) throws Exception {
		LogUtil.info("get_Double begin:" + size_Double);
		for (int iiii = 0; iiii < size_Double; iiii++) {
			for (int i = 0; i < size_Long; i++) {
				JSONArray InsertObjects = new JSONArray();
				for (int ii = 0; ii < size_String; ii++) {
					for (int iii = 0; iii < size_Boolean; iii++) {
						JSONObject InsertObject = new JSONObject();
						InsertObject.put("KeyLong", (long) i);
						InsertObject.put("KeyString", "n" + ii);
						InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
						InsertObject.put("KeyDouble", 1.0 * iiii);
						InsertObjects.add(InsertObject);
					}
				}
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "batch_get");
				wrapperObject.put("Keys", InsertObjects);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				agent.Query(wrapperObject);
			}

			if ((iiii + 1) % (size_Double / 10) == 0) {
				LogUtil.info("doing:" + (iiii + 1) + "/" + size_Double);
			}
		}
		LogUtil.info("finish:" + size_Double);
	}

	static void select(ZillionAgent agent) throws Exception {
		LogUtil.info("select begin:" + size_Long);
		int count = 0;
		for (int i = 0; i < size_Long; i++) {
			// for (int ii = 0; ii < size_String; ii++) {
			JSONObject Criteria = new JSONObject();
			Criteria.put("KeyLong", (long) i);
			// Criteria.put("KeyString", "n" + ii);
			JSONObject wrapperObject = new JSONObject();
			wrapperObject.put("QueryType", "select");
			wrapperObject.put("Criteria", Criteria);
			wrapperObject.put("Database", database_name);
			wrapperObject.put("Datatable", table_name);
			JSONObject queryResult = agent.Query(wrapperObject);
			JSONArray resultContent = (JSONArray) queryResult.get("Content");
			count += resultContent.size();
			// }
			if ((i + 1) % (size_Long / 10) == 0) {
				LogUtil.info("doing:" + (i + 1) + "/" + size_Long + "\t" + count);
			}
		}
		LogUtil.info("finish:" + size_Long + "\t" + count);
	}

	static void select_index1_ori(ZillionAgent agent) throws Exception {
		ZillionTableIndex schema_index = SchemaParser.ParseIndexSchema(table_schema, (JSONObject) index_JSON.get(0));
		Connection connection = ConnectionFactory.createConnection(configuration);
		HTable table_data = (HTable) connection.getTable(TableName.valueOf(database_name, "zillion_data_" + table_name));
		HTable table_index = (HTable) connection.getTable(TableName.valueOf(database_name, "zillion_index_" + table_name + "." + "index_1"));

		LogUtil.info("select_index1_ori begin:" + size_Double);
		int count = 0;
		for (int iiii = 0; iiii < size_Double; iiii++) {
			byte[] index_key_head = ByteToObject.toBytes(1.0 * iiii);
			List<byte[]> row_bytesList = new ArrayList<byte[]>();
			Scan scan = new Scan();
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
			byte[] bytes_gt = new byte[1 + index_key_head.length + 1];
			byte[] bytes_lt = new byte[1 + index_key_head.length + 1];
			bytes_gt[0] = 0x01;
			bytes_lt[0] = 0x01;
			System.arraycopy(index_key_head, 0, bytes_gt, 1, index_key_head.length);
			System.arraycopy(index_key_head, 0, bytes_lt, 1, index_key_head.length);
			bytes_gt[bytes_gt.length - 1] = 0x00;
			bytes_lt[bytes_lt.length - 1] = 0x02;
			List<RowRange> rangeList = new ArrayList<RowRange>();
			rangeList.add(new RowRange(bytes_gt, false, bytes_lt, false));
			MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
			scan.setFilter(filter);
			ResultScanner scanner = table_index.getScanner(scan);
			JSONArray resultContent = new JSONArray();
			while (true) {
				Result res = scanner.next();
				if (res == null) {
					break;
				}
				if (HTableUtil.valid(table_schema, res.listCells())) {
					byte[] row_bytes_index = res.getRow();
					int offset = 0;
					for (int index_column = 0; index_column < schema_index.Key.length; index_column++) {
						String column_name = schema_index.Key[index_column];
						ZillionTableColumn ZillionTableColumn = table_schema.ColumnMap.get(column_name);
						if (row_bytes_index[offset] == 0x00) {
							offset++;
						} else {
							if (ZillionTableColumn.Type == ZillionTableColumnEnum.STRING) {
								offset++;
								while (row_bytes_index[offset] != 0x00) {
									offset++;
								}
								offset++;
							} else {
								offset++;
								offset += ByteToObject.lengthOfColumnEnum(ZillionTableColumn.Type);
							}
						}
					}
					byte[] row_bytes = new byte[row_bytes_index.length - offset];
					System.arraycopy(row_bytes_index, offset, row_bytes, 0, row_bytes.length);
					row_bytesList.add(row_bytes);

					if (row_bytesList.size() >= 200) {
						List<Get> batch_get = new ArrayList<Get>();
						for (byte[] row_bytes_inner : row_bytesList) {
							Get get = new Get(row_bytes_inner);
							for (String column_name : table_schema.OtherColumn) {
								get.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name));
							}
							batch_get.add(get);
						}
						Result[] results_inner = table_data.get(batch_get);
						for (Result result_inner : results_inner) {
							JSONObject resultObject = HTableUtil.Convert(table_schema, result_inner.getRow(), result_inner.listCells());
							resultContent.add(resultObject);
						}
						row_bytesList.clear();
					}
				}
			}
			if (row_bytesList.size() > 0) {
				List<Get> batch_get = new ArrayList<Get>();
				for (byte[] row_bytes_inner : row_bytesList) {
					Get get = new Get(row_bytes_inner);
					for (String column_name : table_schema.OtherColumn) {
						get.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name));
					}
					batch_get.add(get);
				}
				Result[] results_inner = table_data.get(batch_get);
				for (Result result_inner : results_inner) {
					JSONObject resultObject = HTableUtil.Convert(table_schema, result_inner.getRow(), result_inner.listCells());
					resultContent.add(resultObject);
				}
				row_bytesList.clear();
			}
			scanner.close();
			count += row_bytesList.size();
			////////////////////////////////////////////////////////////////
			// for (int i = 0; i < size_Long; i++) {
			// List<byte[]> row_bytesListInner = new ArrayList<byte[]>();
			// for (int ii = 0; ii < size_String; ii++) {
			// for (int iii = 0; iii < size_Boolean; iii++) {
			// JSONObject InsertObject = new JSONObject();
			// InsertObject.put("KeyLong", (long) i);
			// InsertObject.put("KeyString", "n" + ii);
			// InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
			// InsertObject.put("KeyDouble", 1.0 * iiii);
			// byte[] row_bytes_inner = HTableUtil.BuildKey(table_schema,
			// InsertObject);
			// row_bytesListInner.add(row_bytes_inner);
			// }
			// }
			// List<Get> batch_get = new ArrayList<Get>();
			// for (byte[] row_bytes_inner : row_bytesListInner) {
			// Get get = new Get(row_bytes_inner);
			// for (String column_name : table_schema.OtherColumn) {
			// get.addColumn(ByteToObject.toBytes("f"),
			// ByteToObject.toBytes(column_name));
			// }
			// batch_get.add(get);
			// }
			// Result[] results_inner = table_data.get(batch_get);
			// for (Result result_inner : results_inner) {
			// JSONObject resultObject = HTableUtil.Convert(table_schema,
			// result_inner.getRow(), result_inner.listCells());
			// resultContent.add(resultObject);
			// }
			// }
			////////////////////////////////////////////////////////////////
			count += resultContent.size();

			if ((iiii + 1) % (size_Double / size_Double) == 0) {
				LogUtil.info("doing:" + (iiii + 1) + "/" + size_Double + "\t" + count);
			}
		}
		LogUtil.info("finish:" + size_Double + "\t" + count);
	}

	static void select_index1(ZillionAgent agent) throws Exception {
		LogUtil.info("select_index1 begin:" + size_Double);
		int count = 0;
		for (int iiii = 0; iiii < size_Double; iiii++) {
			// for (int iii = 0; iii < size_Boolean; iii++) {
			JSONObject Criteria = new JSONObject();
			Criteria.put("KeyDouble", 1.0 * iiii);
			// Criteria.put("KeyBoolean", iii % 2 == 0 ? true : false);
			JSONObject wrapperObject = new JSONObject();
			wrapperObject.put("QueryType", "select");
			wrapperObject.put("Criteria", Criteria);
			wrapperObject.put("Database", database_name);
			wrapperObject.put("Datatable", table_name);
			JSONObject queryResult = agent.Query(wrapperObject);
			JSONArray resultContent = (JSONArray) queryResult.get("Content");
			count += resultContent.size();
			// }
			if ((iiii + 1) % (size_Double / 10) == 0) {
				LogUtil.info("doing:" + (iiii + 1) + "/" + size_Double + "\t" + count);
			}
		}
		LogUtil.info("finish:" + size_Double + "\t" + count);
	}

	static void select_index2(ZillionAgent agent) throws Exception {
		LogUtil.info("select_index2 begin:" + size_String);
		int count = 0;
		for (int ii = 0; ii < size_String; ii++) {
			{
				JSONObject Criteria = new JSONObject();
				Criteria.put("KeyBoolean", false);
				Criteria.put("KeyString", "n" + ii);
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "select");
				wrapperObject.put("Criteria", Criteria);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				JSONObject queryResult = agent.Query(wrapperObject);
				JSONArray resultContent = (JSONArray) queryResult.get("Content");
				count += resultContent.size();
			}
			{
				JSONObject Criteria = new JSONObject();
				Criteria.put("KeyBoolean", true);
				Criteria.put("KeyString", "n" + ii);
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "select");
				wrapperObject.put("Criteria", Criteria);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				JSONObject queryResult = agent.Query(wrapperObject);
				JSONArray resultContent = (JSONArray) queryResult.get("Content");
				count += resultContent.size();
			}
			if ((ii + 1) % (size_String / 10) == 0) {
				LogUtil.info("doing:" + (ii + 1) + "/" + size_String + "\t" + count);
			}
		}
		LogUtil.info("finish:" + size_String + "\t" + count);
	}

	static void delete(ZillionAgent agent) throws Exception {
		LogUtil.info("delete begin:" + (size_Long / 2) + "\t" + size_Long);
		int count = 0;
		for (int i = 0; i < (size_Long / 2); i++) {
			for (int ii = 0; ii < size_String; ii++) {
				JSONObject Criteria = new JSONObject();
				Criteria.put("KeyLong", (long) i);
				Criteria.put("KeyString", "n" + ii);
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "delete");
				wrapperObject.put("Criteria", Criteria);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				JSONObject queryResult = agent.Query(wrapperObject);
				JSONArray resultContent = (JSONArray) queryResult.get("Content");
				count += resultContent.size();
			}
		}
		LogUtil.info("finish:" + (size_Long / 2) + "\t" + size_Long + "\t" + count);
	}

	static void delete_index1(ZillionAgent agent) throws Exception {
		LogUtil.info("delete_index1 begin:" + (size_Double / 2) + "/" + size_Double);
		int count = 0;
		for (int iiii = 0; iiii < (size_Double / 2); iiii++) {
			for (int iii = 0; iii < size_Boolean; iii++) {
				JSONObject Criteria = new JSONObject();
				Criteria.put("KeyDouble", 1.0 * iiii);
				Criteria.put("KeyBoolean", iii % 2 == 0 ? true : false);
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "delete");
				wrapperObject.put("Criteria", Criteria);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				JSONObject queryResult = agent.Query(wrapperObject);
				JSONArray resultContent = (JSONArray) queryResult.get("Content");
				count += resultContent.size();
			}
		}
		LogUtil.info("finish:" + (size_Double / 2) + "/" + size_Double + "\t" + count);
	}

	static void delete_index2(ZillionAgent agent) throws Exception {
		LogUtil.info("delete_index2 begin:" + (size_String / 2) + "/" + size_String);
		int count = 0;
		for (int ii = 0; ii < (size_String / 2); ii++) {
			for (int i = 0; i < size_Long; i++) {
				JSONObject Criteria = new JSONObject();
				Criteria.put("KeyBoolean", false);
				Criteria.put("KeyString", "n" + ii);
				Criteria.put("KeyLong", (long) i);
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "delete");
				wrapperObject.put("Criteria", Criteria);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				JSONObject queryResult = agent.Query(wrapperObject);
				JSONArray resultContent = (JSONArray) queryResult.get("Content");
				count += resultContent.size();
			}
		}
		LogUtil.info("finish:" + (size_String / 2) + "/" + size_String + "\t" + count);
	}
}
