import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.filter.CompareFilter.CompareOp;
import org.apache.hadoop.hbase.filter.FilterList;
import org.apache.hadoop.hbase.filter.FilterList.Operator;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter.RowRange;
import org.apache.hadoop.hbase.filter.SingleColumnValueFilter;
import org.zillion.util.common.HexToByte;
import org.zillion.util.json.FastJsonReaderUtil;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.agent.ZillionAgent;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.SchemaParser;
import com.zillion.util.table.ZillionTableSchema;

public class AgentTest_1point3 {
	static boolean rebuildtable = false;

	static int size_Long = 10;
	static int size_String = 10;
	static int size_Boolean = 2;
	static int size_Double = 10;

	static String database_name = "database_1";
	static String[] table_names = { "table_1", "table_2", "table_3" };

	static String def_path = "etc/1point3/def.json";
	// static String def_path = "etc/1point3/def_delete_UsePutV.json";

	static JSONObject[] table_JSON;
	static JSONArray[] index_JSON;
	static ZillionTableSchema[] table_schema;
	static JSONObject[] defs = new JSONObject[3];

	static Configuration configuration = new Configuration();

	public static void main(String[] args) throws Exception {
		// Configuration configuration = HBaseConfiguration.create();
		configuration.set("hbase.cluster.distributed", "true");
		configuration.set("hbase.zookeeper.quorum", "zookeeper1,zookeeper2,zookeeper3");
		ZillionAgent agent = new ZillionAgent(configuration);
		agent.Start();

		// ʼ
		defs[0] = FastJsonReaderUtil.Instance().ReadJSONObject(new File(def_path));
		defs[1] = FastJsonReaderUtil.Instance().ReadJSONObject(new File(def_path));
		defs[2] = FastJsonReaderUtil.Instance().ReadJSONObject(new File(def_path));
		defs[0].put("table_type", null);
		defs[1].put("table_type", "index_data");
		defs[2].put("table_type", "all_column");
		table_JSON = new JSONObject[3];
		index_JSON = new JSONArray[3];
		table_schema = new ZillionTableSchema[3];
		for (int i = 0; i < defs.length; i++) {
			table_schema[i] = new ZillionTableSchema(database_name, table_names[i]);
			table_schema[i].JSON = defs[i];
			SchemaParser.ParseTableSchema(table_schema[i], table_schema[i].JSON);

			JSONArray Columns = (JSONArray) defs[i].get("Columns");
			JSONArray Key = (JSONArray) defs[i].get("Key");
			index_JSON[i] = (JSONArray) defs[i].get("Indexes");
			table_JSON[i] = new JSONObject();
			table_JSON[i].put("Columns", Columns);
			table_JSON[i].put("Key", Key);
			if (defs[i].containsKey("delete_UsePutV")) {
				table_JSON[i].put("delete_UsePutV", (Boolean) defs[i].get("delete_UsePutV"));
			}
			if (defs[i].containsKey("table_type")) {
				table_JSON[i].put("table_type", (String) defs[i].get("table_type"));
			}
		}
		table_schema[2].delete_UsePutV = false;

		if (rebuildtable) {
			for (int i = 0; i < defs.length; i++) {
				try {
					agent.DDL_DropTable(database_name, table_names[i]);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			for (int i = 0; i < defs.length; i++) {
				try {
					agent.DDL_AddTable(database_name, table_names[i], table_JSON[i], index_JSON[i]);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			for (int i = 0; i < defs.length; i++) {
				InsertOne(agent, table_names[i]);
				// insert(agent, table_names[i]);
			}
		}

		// select_all(agent);
		// select_2_3(agent);
		// select_3(agent);
		// select_3_ori(agent);

		update_all(agent);

		agent.Stop();
		System.out.println();
	}

	static void update_all(ZillionAgent agent) {
		int i = 0;
		int ii = 0;
		int iii = 0;
		int iiii = 0;
		int index = ((i * size_String + ii) * 2 + iii) * size_Double + iiii;

		JSONObject Criteria = new JSONObject();
		{
			Criteria.put("KeyLong", (long) i);
			// Criteria.put("KeyString", "n" + ii);
			// Criteria.put("KeyBoolean", iii % 2 == 0 ? true : false);
			// Criteria.put("KeyDouble", 1.0 * iiii);
			// Criteria.put("ValueLong", (long) index);
			// Criteria.put("ValueString", "n" + ii);
			// Criteria.put("ValueBoolean", iii % 2 == 0 ? true : false);
			// Criteria.put("ValueDouble", 1.0 * iiii);
		}
		JSONObject Set = new JSONObject();
		{
			Set.put("KeyLong", (long) i);
			// Set.put("KeyString", "n" + ii);
			// Set.put("KeyBoolean", iii % 2 == 0 ? true : false);
			Set.put("KeyDouble", 8.0);
			// Set.put("ValueLong", (long) index);
			// Set.put("ValueString", "n" + ii);
			// Set.put("ValueBoolean", iii % 2 == 0 ? true : false);
			Set.put("ValueDouble", 8.0);
		}

		StringBuffer sb = new StringBuffer();
		int[] sizes = new int[defs.length];
		for (int k = 0; k < defs.length; k++) {
			try {
				sizes[k] = update(agent, table_names[k], Criteria, Set);
			} catch (Exception e) {
				e.printStackTrace();
			}
			sb.append(sizes[k] + "\t");
		}
		LogUtil.warn(sb.toString());
	}

	static int update(ZillionAgent agent, String table_name, JSONObject Criteria, JSONObject Set) throws Exception {
		JSONObject wrapperObject = new JSONObject();
		wrapperObject.put("QueryType", "update");
		wrapperObject.put("Criteria", Criteria);
		wrapperObject.put("Set", Set);
		wrapperObject.put("Database", database_name);
		wrapperObject.put("Datatable", table_name);
		JSONObject queryResult = agent.Query(wrapperObject);
		long resultCount = (Long) queryResult.get("Count");
		return (int) resultCount;
		// LogUtil.warn(table_name + "\t" + resultContent.size());
	}

	static void select_3_ori(ZillionAgent agent) throws Exception {
		Connection connection = ConnectionFactory.createConnection(configuration);
		HTable table_data = (HTable) connection.getTable(TableName.valueOf(database_name, "zillion_data_" + "table_3"));
		HTable table_index = (HTable) connection.getTable(TableName.valueOf(database_name, "zillion_index_" + "table_3" + "." + "index_2"));

		Random rand = new Random();
		for (int ttt = 0; ttt < 1; ttt++) {
			int i = 1;
			int ii = 1;
			int iii = 1;
			int iiii = 1;
			int index = ((i * size_String + ii) * 2 + iii) * size_Double + iiii;

			List<JSONObject> InsertObjectList = new ArrayList<JSONObject>();
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyLong", (long) i);
				// InsertObject.put("KeyString", "n" + ii);
				// InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
				// InsertObject.put("KeyDouble", 1.0 * iiii);
				// InsertObject.put("ValueLong", (long) index);
				// InsertObject.put("ValueString", "n" + ii);
				InsertObject.put("ValueBoolean", iii % 2 == 0 ? true : false);
				InsertObject.put("ValueDouble", 1.0 * iiii);
				InsertObjectList.add(InsertObject);

				byte[] index_key_head = ByteToObject.toBytes((Long) InsertObject.get("KeyLong"));
				byte[] bytes_gt = new byte[1 + index_key_head.length + 1];
				byte[] bytes_lt = new byte[1 + index_key_head.length + 1];
				bytes_gt[0] = 0x01;
				bytes_lt[0] = 0x01;
				System.arraycopy(index_key_head, 0, bytes_gt, 1, index_key_head.length);
				System.arraycopy(index_key_head, 0, bytes_lt, 1, index_key_head.length);
				bytes_gt[bytes_gt.length - 1] = 0x00;
				bytes_lt[bytes_lt.length - 1] = 0x02;

				Scan scan = new Scan();
				scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("ValueLong"));
				scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("ValueString"));
				scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("ValueBoolean"));
				scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("ValueDouble"));
				List<RowRange> rangeList = new ArrayList<RowRange>();
				rangeList.add(new RowRange(bytes_gt, true, bytes_lt, false));
				MultiRowRangeFilter multiFilter = new MultiRowRangeFilter(rangeList);
				FilterList filterList = new FilterList(Operator.MUST_PASS_ALL);
				filterList.addFilter(multiFilter);
				filterList.addFilter(new SingleColumnValueFilter(ByteToObject.toBytes("f"), ByteToObject.toBytes("ValueBoolean"), CompareOp.EQUAL,
						ByteToObject.toBytes((Boolean) InsertObject.get("ValueBoolean"))));
				filterList.addFilter(new SingleColumnValueFilter(ByteToObject.toBytes("f"), ByteToObject.toBytes("ValueDouble"), CompareOp.EQUAL,
						ByteToObject.toBytes((Double) InsertObject.get("ValueDouble"))));
				scan.setFilter(filterList);
				ResultScanner scanner = table_data.getScanner(scan);
				int count = 0;
				while (true) {
					Result res = scanner.next();
					if (res == null) {
						break;
					}

					if (HTableUtil.valid(table_schema[2], res.listCells())) {
						count++;
					}
				}
				LogUtil.warn("" + count);
			}
		}
	}

	static JSONObject buildRange(Object gt, boolean gt_include, Object lt, boolean lt_include) {
		JSONObject result = new JSONObject();
		if (gt_include) {
			result.put("$gte", gt);
		} else {
			result.put("$gt", gt);
		}
		if (lt_include) {
			result.put("$lte", lt);
		} else {
			result.put("$lt", lt);
		}
		return result;
	}

	static void select_3(ZillionAgent agent) {
		Random rand = new Random();
		for (int ttt = 0; ttt < 10; ttt++) {
			// int i = rand.nextInt(size_Long / 2);
			// int ii = rand.nextInt(size_String);
			// int iii = rand.nextInt(size_Boolean);
			// int iiii = rand.nextInt(size_Double / 2);
			int i = 0;
			int ii = 0;
			int iii = 0;
			int iiii = 0;
			int index = ((i * size_String + ii) * 2 + iii) * size_Double + iiii;

			List<JSONObject> InsertObjectList = new ArrayList<JSONObject>();
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyLong", (long) i);
				// InsertObject.put("KeyString", "n" + ii);
				// InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
				// InsertObject.put("KeyDouble", 1.0 * iiii);
				// InsertObject.put("ValueLong", (long) index);
				// InsertObject.put("ValueString", "n" + ii);
				InsertObject.put("ValueBoolean", iii % 2 == 0 ? true : false);
				InsertObject.put("ValueDouble", 1.0 * iiii);
				InsertObjectList.add(InsertObject);
			}
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyLong", buildRange((long) i, true, (long) (i + 5), false));
				// InsertObject.put("KeyString", "n" + ii);
				// InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
				// InsertObject.put("KeyDouble", 1.0 * iiii);
				// InsertObject.put("ValueLong", (long) index);
				// InsertObject.put("ValueString", "n" + ii);
				InsertObject.put("ValueBoolean", buildRange(false, true, true, true));
				InsertObject.put("ValueDouble", buildRange((double) iiii, true, (double) (iiii + 5), false));
				InsertObjectList.add(InsertObject);
			}

			for (JSONObject InsertObject : InsertObjectList) {
				StringBuffer sb = new StringBuffer();
				int[] sizes = new int[defs.length];
				for (int k = 2; k < defs.length; k++) {
					try {
						sizes[k] = select(agent, table_names[k], InsertObject);
					} catch (Exception e) {
						e.printStackTrace();
					}
					sb.append(sizes[k] + "\t");
				}
				LogUtil.warn(sb.toString());
			}
		}
	}

	static void select_2_3(ZillionAgent agent) {
		Random rand = new Random();
		for (int ttt = 0; ttt < 10; ttt++) {
			int i = rand.nextInt(size_Long);
			int ii = rand.nextInt(size_String);
			int iii = rand.nextInt(size_Boolean);
			int iiii = rand.nextInt(size_Double);
			int index = ((i * size_String + ii) * 2 + iii) * size_Double + iiii;

			List<JSONObject> InsertObjectList = new ArrayList<JSONObject>();
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyLong", (long) i);
				InsertObject.put("KeyString", "n" + ii);
				InsertObject.put("ValueLong", (long) index);
				InsertObject.put("ValueString", "n" + ii);
				InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
				InsertObjectList.add(InsertObject);
			}
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyLong", (long) i);
				InsertObject.put("KeyString", "n" + ii);
				InsertObject.put("ValueLong", (long) index);
				InsertObject.put("ValueString", "n" + ii);
				InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
				InsertObject.put("KeyDouble", 1.0 * iiii);
				InsertObjectList.add(InsertObject);
			}

			for (JSONObject InsertObject : InsertObjectList) {
				StringBuffer sb = new StringBuffer();
				int[] sizes = new int[defs.length];
				for (int k = 1; k < defs.length; k++) {
					try {
						sizes[k] = select(agent, table_names[k], InsertObject);
					} catch (Exception e) {
						e.printStackTrace();
					}
					sb.append(sizes[k] + "\t");
				}
				LogUtil.warn(sb.toString());
			}
		}
	}

	static void select_all(ZillionAgent agent) {
		Random rand = new Random();
		for (int ttt = 0; ttt < 10; ttt++) {
			int i = rand.nextInt(size_Long);
			int ii = rand.nextInt(size_String);
			int iii = rand.nextInt(size_Boolean);
			int iiii = rand.nextInt(size_Double);
			int index = ((i * size_String + ii) * 2 + iii) * size_Double + iiii;

			List<JSONObject> InsertObjectList = new ArrayList<JSONObject>();
			{
				JSONObject InsertObject = new JSONObject();
				InsertObjectList.add(InsertObject);
			}
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyLong", (long) i);
				InsertObjectList.add(InsertObject);
			}
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyLong", (long) i);
				InsertObject.put("KeyString", "n" + ii);
				InsertObjectList.add(InsertObject);
			}
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyLong", (long) i);
				InsertObject.put("KeyString", "n" + ii);
				InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
				InsertObjectList.add(InsertObject);
			}
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyLong", (long) i);
				InsertObject.put("KeyString", "n" + ii);
				InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
				InsertObject.put("KeyDouble", 1.0 * iiii);
				InsertObjectList.add(InsertObject);
			}

			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyDouble", 1.0 * iiii);
				InsertObjectList.add(InsertObject);
			}
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyDouble", 1.0 * iiii);
				InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
				InsertObjectList.add(InsertObject);
			}
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyDouble", 1.0 * iiii);
				InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
				InsertObject.put("KeyString", "n" + ii);
				InsertObjectList.add(InsertObject);
			}

			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyLong", (long) i);
				InsertObject.put("KeyString", "n" + ii);
				InsertObject.put("ValueLong", (long) index);
				InsertObjectList.add(InsertObject);
			}
			{
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("KeyLong", (long) i);
				InsertObject.put("KeyString", "n" + ii);
				InsertObject.put("ValueLong", (long) index);
				InsertObject.put("ValueString", "n" + ii);
				InsertObjectList.add(InsertObject);
			}

			for (JSONObject InsertObject : InsertObjectList) {
				StringBuffer sb = new StringBuffer();
				int[] sizes = new int[defs.length];
				for (int k = 0; k < defs.length; k++) {
					try {
						sizes[k] = select(agent, table_names[k], InsertObject);
					} catch (Exception e) {
						e.printStackTrace();
					}
					sb.append(sizes[k] + "\t");
				}
				LogUtil.warn(sb.toString());
			}
		}
	}

	static int select(ZillionAgent agent, String table_name, JSONObject Criteria) throws Exception {
		JSONObject wrapperObject = new JSONObject();
		wrapperObject.put("QueryType", "select");
		wrapperObject.put("Criteria", Criteria);
		wrapperObject.put("Database", database_name);
		wrapperObject.put("Datatable", table_name);
		JSONObject queryResult = agent.Query(wrapperObject);
		JSONArray resultContent = (JSONArray) queryResult.get("Content");
		LogUtil.warn(resultContent.toJSONString());
		return resultContent.size();
		// LogUtil.warn(table_name + "\t" + resultContent.size());
	}

	static void insert(ZillionAgent agent, String table_name) throws Exception {
		Random rand = new Random();
		LogUtil.info("insert begin:" + size_Long);
		for (int i = 0; i < size_Long; i++) {
			for (int ii = 0; ii < size_String; ii++) {
				for (int iii = 0; iii < size_Boolean; iii++) {
					JSONArray InsertObjects = new JSONArray();
					for (int iiii = 0; iiii < size_Double; iiii++) {
						int index = ((i * size_String + ii) * 2 + iii) * size_Double + iiii;
						JSONObject InsertObject = new JSONObject();
						InsertObject.put("KeyLong", (long) i);
						InsertObject.put("KeyString", "n" + ii);
						InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
						InsertObject.put("KeyDouble", 1.0 * iiii);
						InsertObject.put("ValueLong", (long) index);
						InsertObject.put("ValueString", "n" + ii);
						InsertObject.put("ValueBoolean", iii % 2 == 0 ? true : false);
						InsertObject.put("ValueDouble", 1.0 * iiii);
						int bytes_count = 1 + rand.nextInt(10);
						byte[] bytes = new byte[bytes_count];
						rand.nextBytes(bytes);
						InsertObject.put("ValueBytes", HexToByte.byteToHex(bytes));
						InsertObjects.add(InsertObject);
					}
					JSONObject wrapperObject = new JSONObject();
					wrapperObject.put("QueryType", "batch_put");
					wrapperObject.put("InsertObjects", InsertObjects);
					wrapperObject.put("Database", database_name);
					wrapperObject.put("Datatable", table_name);
					agent.Query(wrapperObject);
				}
			}

			if ((i + 1) % (size_Long / 10) == 0) {
				LogUtil.info("doing:" + (i + 1) + "/" + size_Long);
			}
		}
		LogUtil.info("finish:" + size_Long);
	}

	static void InsertOne(ZillionAgent agent, String table_name) throws Exception {
		JSONArray InsertObjects = new JSONArray();
		int i = 0;
		int ii = 0;
		int iii = 0;
		int iiii = 0;
		Random rand = new Random();
		{
			int index = ((i * 100 + ii) * 2 + iii) * 100 + iiii;
			JSONObject InsertObject = new JSONObject();
			InsertObject.put("KeyLong", (long) i);
			InsertObject.put("KeyString", "n" + ii);
			InsertObject.put("KeyBoolean", iii % 2 == 0 ? true : false);
			InsertObject.put("KeyDouble", 1.0 * iiii);
			InsertObject.put("ValueLong", (long) index);
			InsertObject.put("ValueString", "n" + ii);
			InsertObject.put("ValueBoolean", iii % 2 == 0 ? true : false);
			InsertObject.put("ValueDouble", 1.0 * iiii);
			int bytes_count = 1 + rand.nextInt(10);
			byte[] bytes = new byte[bytes_count];
			rand.nextBytes(bytes);
			InsertObject.put("ValueBytes", HexToByte.byteToHex(bytes));
			InsertObjects.add(InsertObject);
		}
		JSONObject wrapperObject = new JSONObject();
		wrapperObject.put("QueryType", "batch_insert");
		wrapperObject.put("InsertObjects", InsertObjects);
		wrapperObject.put("Database", database_name);
		wrapperObject.put("Datatable", table_name);
		agent.Query(wrapperObject);
	}

}
