import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.agent.ZillionAgent;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.SchemaParser;
import com.zillion.util.table.ZillionTableSchema;

public class AgentTest_copytable {

	public static void main(String[] args) throws Exception {
		Configuration configuration = HBaseConfiguration.create();
		ZillionAgent agent = new ZillionAgent(configuration);
		agent.Start();

		Connection connection = ConnectionFactory.createConnection(configuration);

		String from_database = "db_public";
		String from_table = "originaltemppresent";
		String to_database = "db_public";
		String to_table = "originaldatapresent";

		int process_count = 0;
		int copy_count = 0;
		int exist_count = 0;

		JSONObject wrapperObject = new JSONObject();
		wrapperObject.put("QueryType", "table_detail");
		wrapperObject.put("Database", from_database);
		wrapperObject.put("Datatable", from_table);
		JSONObject queryResult = agent.Query(wrapperObject);
		ZillionTableSchema schema = new ZillionTableSchema(from_database, from_table);
		schema.JSON = (JSONObject) queryResult.get("Content");
		SchemaParser.ParseTableSchema(schema, schema.JSON);

		TableName from_TableName = TableName.valueOf(from_database, "zillion_data_" + from_table);
		HTable table = (HTable) connection.getTable(from_TableName);
		Scan scan = new Scan();
		if (schema.delete_UsePutV) {
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
		}
		for (String column_name : schema.OtherColumn) {
			scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name));
		}
		ResultScanner scanner = table.getScanner(scan);
		JSONArray InsertObjects = new JSONArray();
		while (true) {
			Result res = scanner.next();
			if (res == null) {
				break;
			}

			if (HTableUtil.valid(schema, res.listCells())) {
				JSONObject resultObject = HTableUtil.Convert(schema, res.getRow(), res.listCells());
				InsertObjects.add(resultObject);
			}

			if (InsertObjects.size() >= 10000) {
				wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "batch_insert");
				wrapperObject.put("InsertObjects", InsertObjects);
				wrapperObject.put("Database", to_database);
				wrapperObject.put("Datatable", to_table);
				queryResult = agent.Query(wrapperObject);
				long exist_tmp = (Long) queryResult.get("Count");

				process_count += InsertObjects.size();
				copy_count += InsertObjects.size() - (int) exist_tmp;
				exist_count += exist_tmp;
				System.out.println("" + process_count + "\t\t" + copy_count + "\t\t" + exist_count);

				InsertObjects.clear();
			}
		}
		if (InsertObjects.size() > 0) {
			wrapperObject = new JSONObject();
			wrapperObject.put("QueryType", "batch_insert");
			wrapperObject.put("InsertObjects", InsertObjects);
			wrapperObject.put("Database", to_database);
			wrapperObject.put("Datatable", to_table);
			queryResult = agent.Query(wrapperObject);
			long exist_tmp = (Long) queryResult.get("Count");

			process_count += InsertObjects.size();
			copy_count += InsertObjects.size() - (int) exist_tmp;
			exist_count += exist_tmp;
			System.out.println("" + process_count + "\t\t" + copy_count + "\t\t" + exist_count);

			InsertObjects.clear();
		}
		scanner.close();

		table.close();

		agent.Stop();

		System.out.println();
	}
}
