import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.zillion.util.common.HexToByte;
import org.zillion.util.format.FormatStaticUtil;
import org.zillion.util.json.FastJsonReaderUtil;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.agent.ZillionAgent;

class MyThread extends Thread {
	ZillionAgent agent;
	String table_name;
	String building;
	int cycle_number;

	public MyThread(ZillionAgent agent, String table_name, String building, int cycle_number) {
		this.agent = agent;
		this.table_name = table_name;
		this.building = building;
		this.cycle_number = cycle_number;
	}

	public void run() {
		try {
			AgentTest_originaldata.process(this.agent, this.table_name, this.building);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}

public class AgentTest_originaldata {
	static String database_name = "database_1";
	static String table_name = "originaldata6";

	static int region_count = 1;
	static int building_per_region = 1;
	static int meter_count = 10;
	static int funcid_count = 100;
	static int receivetime_count = 100;
	static int cycle_number = 1;

	// static String def_path = "etc/zz_originaldata/def.json";
	static String def_path = "etc/zz_originaldata/def_delete_UsePutV.json";

	static List<String> buildingList = new ArrayList<String>();
	static JSONArray region_keys = new JSONArray();
	static List<String> meterList = new ArrayList<String>();
	static List<Long> funcidList = new ArrayList<Long>();
	static List<String> receivetimeList = new ArrayList<String>();

	static {
		try {
			{
				long start = 1101010001;
				for (int i = 0; i < region_count; i++) {
					String building = null;
					for (int ii = 0; ii < building_per_region; ii++) {
						building = FormatStaticUtil.toString(start);
						buildingList.add(building);
						start += 100000000;
					}
					if (i < region_count - 1) {
						String hex = "01" + HexToByte.byteToHex(building.getBytes()) + "02";
						region_keys.add(hex);
					}
				}
			}
			{
				long start = 1000;
				for (int i = 0; i < meter_count; i++) {
					meterList.add(FormatStaticUtil.toString(start));
					start += 1;
				}
			}
			{
				long start = 0;
				for (int i = 0; i < funcid_count; i++) {
					funcidList.add(start);
					start += 1;
				}
			}
			{
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				Date start = sdf.parse("2000-01-01 00:00:00");
				for (int i = 0; i < receivetime_count; i++) {
					start = new Date(start.getTime() + 1000L * 60 * 5);
					receivetimeList.add(sdf.format(start));
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) throws Exception {
		Configuration configuration = HBaseConfiguration.create();
		// ZillionAgent agent = new ZillionAgent(configuration, "abc", MemoryConfig_Ignite);
		ZillionAgent agent = new ZillionAgent(configuration, "abc");

		try {
			agent.Start();
		} catch (Exception e1) {
			e1.printStackTrace();
		}

		try {
			agent.NDL_AddDatabase(database_name);
		} catch (Exception e) {
			// e.printStackTrace();
		}

		try {
			agent.DDL_DropTable(database_name, table_name);
		} catch (Exception e) {
			// e.printStackTrace();
		}

		// ʼ
		JSONObject def = FastJsonReaderUtil.Instance().ReadJSONObject(new File(def_path));
		def.put("region_keys", region_keys);
		JSONArray Indexes = (JSONArray) def.remove("Indexes");
		agent.DDL_AddTable(database_name, table_name, def, Indexes);

		List<MyThread> threadList = new ArrayList<MyThread>();
		for (int i = 0; i < buildingList.size(); i++) {
			String building = buildingList.get(i);
			MyThread thread = new MyThread(agent, table_name, building, cycle_number);
			thread.start();
			threadList.add(thread);
		}

		Date time_from = new Date();
		while (true) {
			Thread.sleep(1L);

			boolean all_finish = true;
			for (MyThread thread : threadList) {
				if (thread.isAlive()) {
					all_finish = false;
					break;
				}
			}
			if (all_finish) {
				break;
			}
		}
		Date time_to = new Date();
		LogUtil.warn("" + (time_to.getTime() - time_from.getTime()));

		agent.Stop();
		System.out.println();
	}

	static void process(ZillionAgent agent, String table_name, String building) throws Exception {
		List<List<Long>> timeListList = new ArrayList<List<Long>>();
		for (int i = 0; i < cycle_number; i++) {
			Date date1 = new Date();
			insert(agent, table_name, building);
			// Date date2 = new Date();
			// select(agent, table_name, building);
			// Date date3 = new Date();
			// delete(agent, table_name, building);
			// // delete_half_rand(agent);
			// Date date4 = new Date();
			// select(agent, table_name, building);
			// Date date5 = new Date();
			//
			// List<Long> timeList = new ArrayList<Long>();
			// timeList.add(date2.getTime() - date1.getTime());
			// timeList.add(date3.getTime() - date2.getTime());
			// timeList.add(date4.getTime() - date3.getTime());
			// timeList.add(date5.getTime() - date4.getTime());
			// timeListList.add(timeList);
			// for (List<Long> timeListInner : timeListList) {
			// StringBuffer sb = new StringBuffer();
			// for (Long time : timeListInner) {
			// sb.append("\t" + time);
			// }
			// LogUtil.info(sb.toString());
			// }

			agent.container.repositoryMap.get(database_name).GetTable(table_name).unuseTable(agent.container.ZK);
		}
	}

	static void insert(ZillionAgent agent, String table_name, String building) throws Exception {
		Random rand = new Random();
		for (int index2 = 0; index2 < meterList.size(); index2++) {
			String meter = meterList.get(index2);
			for (int index3 = 0; index3 < funcidList.size(); index3++) {
				Long funcid = funcidList.get(index3);
				JSONArray InsertObjects = new JSONArray();
				for (int index4 = 0; index4 < receivetimeList.size(); index4++) {
					String receivetime = receivetimeList.get(index4);
					JSONObject InsertObject = new JSONObject();
					InsertObject.put("building", building);
					InsertObject.put("meter", meter);
					InsertObject.put("funcid", funcid);
					InsertObject.put("receivetime", receivetime);
					InsertObject.put("data", rand.nextDouble());
					InsertObjects.add(InsertObject);

					// {
					// JSONObject wrapperObject = new JSONObject();
					// wrapperObject.put("QueryType", "put");
					// wrapperObject.put("InsertObject", InsertObject);
					// wrapperObject.put("Database", database_name);
					// wrapperObject.put("Datatable", table_name);
					// agent.Query(wrapperObject);
					// }
				}
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "batch_insert");
				wrapperObject.put("InsertObjects", InsertObjects);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				agent.Query(wrapperObject);
			}
		}
	}

	static void select(ZillionAgent agent, String table_name, String building) throws Exception {
		for (int index2 = 0; index2 < meterList.size(); index2++) {
			String meter = meterList.get(index2);
			for (int index3 = 0; index3 < funcidList.size(); index3++) {
				Long funcid = funcidList.get(index3);
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("building", building);
				InsertObject.put("meter", meter);
				InsertObject.put("funcid", funcid);
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "select");
				wrapperObject.put("Criteria", InsertObject);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				JSONObject queryResult = agent.Query(wrapperObject);
				JSONArray resultContent = (JSONArray) queryResult.get("Content");
			}
		}
	}

	static void delete(ZillionAgent agent, String table_name, String building) throws Exception {
		for (int index2 = 0; index2 < meterList.size(); index2++) {
			String meter = meterList.get(index2);
			for (int index3 = 0; index3 < funcidList.size(); index3++) {
				Long funcid = funcidList.get(index3);
				JSONObject InsertObject = new JSONObject();
				InsertObject.put("building", building);
				InsertObject.put("meter", meter);
				InsertObject.put("funcid", funcid);
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "delete");
				wrapperObject.put("Criteria", InsertObject);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				JSONObject queryResult = agent.Query(wrapperObject);
				JSONArray resultContent = (JSONArray) queryResult.get("Content");
			}
		}
	}

	static void delete_half_rand(ZillionAgent agent, String table_name, String building) throws Exception {
		Random rand = new Random();
		for (int index2 = 0; index2 < meterList.size(); index2++) {
			String meter = meterList.get(index2);
			for (int index3 = 0; index3 < funcidList.size(); index3++) {
				Long funcid = funcidList.get(index3);
				if (rand.nextInt(2) == 0) {
					JSONObject InsertObject = new JSONObject();
					InsertObject.put("building", building);
					InsertObject.put("meter", meter);
					InsertObject.put("funcid", funcid);
					JSONObject wrapperObject = new JSONObject();
					wrapperObject.put("QueryType", "delete");
					wrapperObject.put("Criteria", InsertObject);
					wrapperObject.put("Database", database_name);
					wrapperObject.put("Datatable", table_name);
					JSONObject queryResult = agent.Query(wrapperObject);
					JSONArray resultContent = (JSONArray) queryResult.get("Content");
				}
			}
		}
	}
}
