import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter.RowRange;
import org.apache.hadoop.hbase.util.Bytes;
import org.zillion.util.common.HexToByte;
import org.zillion.util.json.FastJsonCompareUtil;
import org.zillion.util.json.FastJsonReaderUtil;
import org.zillion.util.json.FastJsonUtil;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.agent.ZillionAgent;
import com.zillion.database.agent.ZillionCriteriaMatchItem;
import com.zillion.database.agent.ZillionCriteriaMatchUtil;
import com.zillion.database.util.DatabaseImpl_HBase;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.common.ZillionConstant;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.ZillionTableColumnEnum;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;

public class AgentTest_reverse {
	static String database_name = "database_1";
	static String table_name = "table_reverse";

	static List<String> stringList = new ArrayList<String>();
	static List<Boolean> booleanList = new ArrayList<Boolean>();
	static List<Long> longList = new ArrayList<Long>();

	static Random rand = new Random();
	static ZillionAgent agent;

	static {
		stringList.add(null);
		stringList.add("");
		stringList.add("a");
		stringList.add("aa");
		stringList.add("ab");

		booleanList.add(null);
		booleanList.add(false);
		booleanList.add(true);

		longList.add(null);
		longList.add(1L);
		longList.add(2L);
		longList.add(3L);
	}

	public static void main(String[] args) throws Exception {
		Configuration configuration = HBaseConfiguration.create();
		agent = new ZillionAgent(configuration, "abc");

		agent.Start();

		try {
			agent.NDL_AddDatabase(database_name);
		} catch (Exception e) {
			// e.printStackTrace();
		}

		try {
			// agent.DDL_DropTable(database_name, table_name);
			// ʼ
			JSONObject def = FastJsonReaderUtil.Instance().ReadJSONObject(new File("etc/reverse/def.json"));
			JSONArray Indexes = (JSONArray) def.remove("Indexes");
			agent.DDL_AddTable(database_name, table_name, def, Indexes);

			insert(agent);
		} catch (Exception e) {
			// e.printStackTrace();
		}

		{
			ZillionTableSchema schema = agent.container.repositoryMap.get(database_name).GetTable(table_name);
			match_equal(schema);
			match_range(schema);
		}

		agent.Stop();
		System.out.println();
	}

	static void insert(ZillionAgent agent) throws Exception {
		for (int index1 = 0; index1 < stringList.size(); index1++) {
			String building = stringList.get(index1);
			for (int index2 = 0; index2 < booleanList.size(); index2++) {
				Boolean meter = booleanList.get(index2);
				for (int index3 = 0; index3 < longList.size(); index3++) {
					Long funcid = longList.get(index3);
					JSONArray InsertObjects = new JSONArray();

					JSONObject InsertObject = new JSONObject();
					InsertObject.put("String1", building);
					InsertObject.put("Boolean1", meter);
					InsertObject.put("Long1", funcid);
					InsertObject.put("String2", building);
					InsertObject.put("Boolean2", meter);
					InsertObject.put("Long2", funcid);
					InsertObjects.add(InsertObject);

					JSONObject wrapperObject = new JSONObject();
					wrapperObject.put("QueryType", "batch_insert");
					wrapperObject.put("InsertObjects", InsertObjects);
					wrapperObject.put("Database", database_name);
					wrapperObject.put("Datatable", table_name);
					agent.Query(wrapperObject);
				}
			}
		}
	}

	static Object getValue(ZillionTableColumnEnum columnType, int index) throws Exception {
		if (columnType == ZillionTableColumnEnum.STRING) {
			return stringList.get(index - 1);
		} else if (columnType == ZillionTableColumnEnum.BOOLEAN) {
			return booleanList.get(index - 1);
		} else if (columnType == ZillionTableColumnEnum.LONG) {
			return longList.get(index - 1);
		} else {
			throw new Exception("" + columnType);
		}
	}

	static int getCount(ZillionTableColumnEnum columnType) throws Exception {
		if (columnType == ZillionTableColumnEnum.STRING) {
			return stringList.size();
		} else if (columnType == ZillionTableColumnEnum.BOOLEAN) {
			return booleanList.size();
		} else if (columnType == ZillionTableColumnEnum.LONG) {
			return longList.size();
		} else {
			throw new Exception("" + columnType);
		}
	}

	static void match_equal(ZillionTableSchema schema) throws Exception {
		Connection connection = ((DatabaseImpl_HBase) agent.container.DatabaseImpl).connection;
		{
			TableName tableName = TableName.valueOf(database_name, ZillionConstant.prefix_data + table_name);
			HTable table = (HTable) connection.getTable(tableName);
			match_equal_one(schema, null, schema.Key, table);
		}
		for (ZillionTableIndex schemaIndex : schema.Indexes) {
			TableName tableName = TableName.valueOf(database_name, ZillionConstant.prefix_index + (table_name + "." + schemaIndex.index_name));
			HTable table = (HTable) connection.getTable(tableName);
			match_equal_one(schema, schemaIndex, schemaIndex.Key, table);
		}
	}

	static void match_equal_one(ZillionTableSchema schema, ZillionTableIndex schemaIndex, String[] Key, HTable table) throws Exception {
		int[] indexes = new int[Key.length];
		while (true) {
			JSONObject Criteria = new JSONObject();
			for (int i = 0; i < indexes.length; i++) {
				int index = indexes[i];
				if (index == 0) {
					break;
				}
				String column = Key[i];
				ZillionTableColumnEnum columnType = schema.ColumnMap.get(column).Type;
				Object value = getValue(columnType, index);
				Criteria.put(column, value);
			}
			my_select(schema, schemaIndex, Key, table, Criteria);

			// 1
			indexes[0] = indexes[0] + 1;
			for (int i = 0; i < indexes.length - 1; i++) {
				int index = indexes[i];
				String column = Key[i];
				ZillionTableColumnEnum columnType = schema.ColumnMap.get(column).Type;
				int count = getCount(columnType);
				if (index == count + 1) {
					indexes[i] = 1;
					indexes[i + 1] = indexes[i + 1] + 1;
				}
			}
			{
				int index = indexes[indexes.length - 1];
				String column = Key[indexes.length - 1];
				ZillionTableColumnEnum columnType = schema.ColumnMap.get(column).Type;
				int count = getCount(columnType);
				if (index == count + 1) {
					break;
				}
			}
		}
	}

	static void match_range(ZillionTableSchema schema) throws Exception {
		Connection connection = ((DatabaseImpl_HBase) agent.container.DatabaseImpl).connection;
		{
			TableName tableName = TableName.valueOf(database_name, ZillionConstant.prefix_data + table_name);
			HTable table = (HTable) connection.getTable(tableName);
			match_range_one(schema, null, schema.Key, table);
		}
		for (ZillionTableIndex schemaIndex : schema.Indexes) {
			TableName tableName = TableName.valueOf(database_name, ZillionConstant.prefix_index + (table_name + "." + schemaIndex.index_name));
			HTable table = (HTable) connection.getTable(tableName);
			match_range_one(schema, schemaIndex, schemaIndex.Key, table);
		}
	}

	static void match_range_one(ZillionTableSchema schema, ZillionTableIndex schemaIndex, String[] Key, HTable table) throws Exception {
		int[] indexes = new int[Key.length - 1];
		while (true) {
			JSONObject CriteriaBase = new JSONObject();
			for (int i = 0; i < indexes.length; i++) {
				int index = indexes[i];
				if (index == 0) {
					break;
				}
				String column = Key[i];
				ZillionTableColumnEnum columnType = schema.ColumnMap.get(column).Type;
				Object value = getValue(columnType, index);
				CriteriaBase.put(column, value);
			}
			{
				String column = Key[CriteriaBase.size()];
				ZillionTableColumnEnum columnType = schema.ColumnMap.get(column).Type;
				List<JSONObject> rangeList = generateRange(columnType);
				for (JSONObject range : rangeList) {
					JSONObject Criteria = new JSONObject();
					Criteria.putAll(CriteriaBase);
					Criteria.put(column, range);
					my_select(schema, schemaIndex, Key, table, Criteria);
				}
			}

			// 1
			indexes[0] = indexes[0] + 1;
			for (int i = 0; i < indexes.length - 1; i++) {
				int index = indexes[i];
				String column = Key[i];
				ZillionTableColumnEnum columnType = schema.ColumnMap.get(column).Type;
				int count = getCount(columnType);
				if (index == count + 1) {
					indexes[i] = 1;
					indexes[i + 1] = indexes[i + 1] + 1;
				}
			}
			{
				int index = indexes[indexes.length - 1];
				String column = Key[indexes.length - 1];
				ZillionTableColumnEnum columnType = schema.ColumnMap.get(column).Type;
				int count = getCount(columnType);
				if (index == count + 1) {
					break;
				}
			}
		}
	}

	static List<JSONObject> generateRange(ZillionTableColumnEnum columnType) throws Exception {
		if (columnType == ZillionTableColumnEnum.STRING) {
			return generateRange(stringList);
		} else if (columnType == ZillionTableColumnEnum.BOOLEAN) {
			return generateRange(booleanList);
		} else if (columnType == ZillionTableColumnEnum.LONG) {
			return generateRange(longList);
		} else {
			throw new Exception("" + columnType);
		}
	}

	static List<JSONObject> generateRange(List<?> valueList) {
		List<JSONObject> result = new ArrayList<JSONObject>();
		for (int i = 1; i < valueList.size(); i++) {
			Object value = valueList.get(i);
			{
				JSONObject item = new JSONObject();
				item.put("$lt", value);
				result.add(item);
			}
			{
				JSONObject item = new JSONObject();
				item.put("$lte", value);
				result.add(item);
			}
			{
				JSONObject item = new JSONObject();
				item.put("$gt", value);
				result.add(item);
			}
			{
				JSONObject item = new JSONObject();
				item.put("$gte", value);
				result.add(item);
			}
			{
				JSONObject item = new JSONObject();
				item.put("$gte", value);
				item.put("$lte", value);
				result.add(item);
			}
		}
		for (int i = 1; i < valueList.size() - 1; i++) {
			Object value = valueList.get(i);
			for (int ii = i + 1; ii < valueList.size(); ii++) {
				Object valueNext = valueList.get(ii);
				{
					JSONObject item = new JSONObject();
					item.put("$gt", value);
					item.put("$lt", valueNext);
					result.add(item);
				}
				{
					JSONObject item = new JSONObject();
					item.put("$gte", value);
					item.put("$lt", valueNext);
					result.add(item);
				}
				{
					JSONObject item = new JSONObject();
					item.put("$gt", value);
					item.put("$lte", valueNext);
					result.add(item);
				}
				{
					JSONObject item = new JSONObject();
					item.put("$gte", value);
					item.put("$lte", valueNext);
					result.add(item);
				}
			}
		}
		return result;
	}

	static void my_select(ZillionTableSchema schema, ZillionTableIndex schemaIndex, String[] Key, HTable table, JSONObject Criteria)
			throws Exception {
		try {
			JSONArray queryContent_asc;
			{
				JSONObject queryJSON = new JSONObject();
				queryJSON.put("Database", database_name);
				queryJSON.put("Datatable", table_name);
				queryJSON.put("QueryType", "select");
				queryJSON.put("Criteria", Criteria);
				JSONObject queryResult = agent.Query(queryJSON);
				queryContent_asc = (JSONArray) queryResult.get("Content");
			}
			JSONArray queryContent_desc;
			{
				JSONArray OrderBy = new JSONArray();
				for (String keyItem : Key) {
					JSONObject o = new JSONObject();
					o.put("Column", keyItem);
					o.put("Asc", false);
					OrderBy.add(o);
				}
				JSONObject queryJSON = new JSONObject();
				queryJSON.put("Database", database_name);
				queryJSON.put("Datatable", table_name);
				queryJSON.put("QueryType", "select");
				queryJSON.put("Criteria", Criteria);
				queryJSON.put("OrderBy", OrderBy);
				JSONObject queryResult = agent.Query(queryJSON);
				queryContent_desc = (JSONArray) queryResult.get("Content");
			}

			JSONArray result_false = my_select(schema, schemaIndex, Key, table, Criteria, false);
			JSONArray result_true = my_select(schema, schemaIndex, Key, table, Criteria, true);
			if (!FastJsonCompareUtil.Instance().Compare(queryContent_asc, queryContent_desc)
					|| !FastJsonCompareUtil.Instance().Compare(queryContent_asc, result_false, true)
					|| !FastJsonCompareUtil.Instance().Compare(queryContent_desc, result_true, true)) {
				// if(Criteria.size()!=4){
				LogUtil.warn(FastJsonUtil.toString(Criteria) + "\t" + queryContent_asc.size() + "\t" + result_false.size() + "\t" + result_true.size()
						+ "\t" + (result_false.size() == result_true.size()));
				// }
			}
		} catch (Exception e) {
			LogUtil.warn(FastJsonUtil.toString(Criteria));
			throw e;
		}
	}

	static JSONArray my_select(ZillionTableSchema schema, ZillionTableIndex schemaIndex, String[] Key, HTable table, JSONObject Criteria,
			boolean reverse) throws Exception {
		JSONArray resultContent = new JSONArray();

		ZillionCriteriaMatchItem matchItem = ZillionCriteriaMatchUtil.CriteriaMatch(schema, Key, Criteria);
		// LogUtil.warn(HexToByte.byteToHex(matchItem.bytes_gt) + "\t" + matchItem.bytes_gt_include + "\t" + HexToByte.byteToHex(matchItem.bytes_lt)
		// + "\t" + matchItem.bytes_lt_include);

		if (Bytes.compareTo(matchItem.bytes_gt, matchItem.bytes_lt) == 0) {
			if (!matchItem.bytes_gt_include || !matchItem.bytes_lt_include) {
				return resultContent;
			}
		}

		try {
			Scan scan = new Scan();
			if (schema.delete_UsePutV) {
				scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
			}
			Iterator<String> iter = schema.ColumnMap.keySet().iterator();
			while (iter.hasNext()) {
				String col = iter.next();
				boolean is_key = false;
				for (String keyItem : Key) {
					if (keyItem.equals(col)) {
						is_key = true;
						break;
					}
				}
				if (!is_key) {
					scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(col));
				}
			}
			if (reverse) {
				scan.setReversed(true);
				scan.setStartRow(matchItem.bytes_lt);
				scan.setStopRow(matchItem.bytes_gt);
			} else {
				List<RowRange> rangeList = new ArrayList<RowRange>();
				rangeList.add(new RowRange(matchItem.bytes_gt, matchItem.bytes_gt_include, matchItem.bytes_lt, matchItem.bytes_lt_include));
				MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
				scan.setFilter(filter);
			}

			ResultScanner scanner = table.getScanner(scan);
			while (true) {
				Result res = scanner.next();
				if (res == null) {
					break;
				}
				if (schemaIndex != null) {
					if (HTableUtil.valid(schema, res.listCells())) {
						JSONObject resultObject = HTableUtil.Convert_index_data(schema, schemaIndex, res.getRow(), res.listCells());
						resultContent.add(resultObject);
						// LogUtil.info(resultObject.toJSONString());
					}
				} else {
					if (HTableUtil.valid(schema, res.listCells())) {
						JSONObject resultObject = HTableUtil.Convert(schema, res.getRow(), res.listCells());
						resultContent.add(resultObject);
						// LogUtil.info(resultObject.toJSONString());
					}
				}
			}
			scanner.close();
		} catch (Exception e) {
			LogUtil.warn("\n" + HexToByte.byteToHex(matchItem.bytes_gt, true) + "\n" + HexToByte.byteToHex(matchItem.bytes_lt, true));
			throw e;
		}

		return resultContent;
	}
}
