import java.util.Random;

import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.data.Stat;
import org.zillion.util.log.LogUtil;

import com.zillion.util.zk.ZKLockCEDU;
import com.zillion.util.zk.ZKWrapper;

public class ZKLockCEDUTest {
	public static void main(String[] args) throws Exception {
		ZKWrapper zk = new ZKWrapper("zookeeper1:2181,zookeeper2:2181,zookeeper3:2181", 10000);

		String parent_path = "/zillion";
		Stat stat = zk.exists(parent_path, false);
		if (stat == null) {
			zk.create(parent_path, "d".getBytes("UTF-8"), CreateMode.PERSISTENT);
		}

		for (int i = 0; i < 3; i++) {
			MyThread thread = new MyThread(i);
			thread.start();
		}

		while (true) {
			Thread.sleep(1L);
		}
	}

	public static class MyThread extends Thread {
		int num;
		static String[] lock_types = { "c", "e", "e", "e", "e", "d", "u", "u", "u", "u" };
		static String[] check_types = { "c", "d", "d", "d", "d", "d", "d", "d", "d", "d" };

		public MyThread(int num) {
			this.num = num;
		}

		public void run() {
			try {
				ZKWrapper zk = new ZKWrapper("node2:2181,node3:2181,node4:2181", 10000);
				String parent_path = "/zillion";

				Random rand = new Random();
				for (int i = 0; i < 10; i++) {
					int next = rand.nextInt(lock_types.length);
					String lock_type = lock_types[next];
					String check_type = check_types[next];

					boolean success = ZKLockCEDU.PreCheck(zk, parent_path, check_type);
					if (success) {
						String node_path = ZKLockCEDU.Create(zk, parent_path, lock_type, "true");
						int sequence_id = Integer.parseInt(node_path.substring(parent_path.length() + 3));
						success = ZKLockCEDU.Check(zk, parent_path, sequence_id, check_type);
						if (success) {
							LogUtil.info("node" + num + " round" + i + " " + lock_type + " ***** " + "Wait");
							ZKLockCEDU.WaitFor(zk, parent_path, node_path, 3600);
							Thread.sleep(1000L * 5);
							if (lock_type.equals("c") || lock_type.equals("d")) {
								zk.setData(parent_path, ("" + lock_type).getBytes("UTF-8"), -1);
							}
							ZKLockCEDU.Finish(zk, node_path);
							LogUtil.info("node" + num + " round" + i + " " + lock_type + " ***** " + "Finish");
						} else {
							ZKLockCEDU.Finish(zk, node_path);
							LogUtil.info("node" + num + " round" + i + " " + lock_type + " ***** " + "Check" + "fail");
							Thread.sleep(1000L * 5);
						}
					} else {
						LogUtil.info("node" + num + " round" + i + " " + lock_type + " ***** " + "PreCheck" + "fail");
						Thread.sleep(1000L * 5);
					}

					Thread.sleep(1000L);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}
