package test.compress;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.zillion.util.common.CompressUtil;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.database.agent.ZillionAgent;

public class AgentTest_zip {
	static int cycle_count = 3;
	static int[] thread_count_array = { 32, 16, 8, 4, 2, 1 };

	static String[] algo_array = { "gzip", "gzip", "deflate", "deflate" };
	static int[] deflate_level_array = { 1, 1, 1, 9 };
	static String[] Database_array = { "work_order", "work_order_gzip", "work_order_deflate_1", "work_order_deflate_9" };
	static String[] Datatable_array = { "wo_order_body", "wo_order_body", "wo_order_body", "wo_order_body" };

	static Configuration configuration;
	static ZillionAgent agent;
	static JSONArray Content = new JSONArray();

	public static void main(String[] args) throws Exception {
		configuration = HBaseConfiguration.create();
		agent = new ZillionAgent(configuration, "abc");

		try {
			agent.Start();
		} catch (Exception e) {
			e.printStackTrace();
		}

		test_insert();
		test_use();
	}

	static void test_insert() throws Exception {
		JSONArray zip_array = new JSONArray();
		String ImportZip = "D:\\Test\\wo_order_body.zip";
		ZipArchiveInputStream zis = new ZipArchiveInputStream(new FileInputStream(ImportZip));
		@SuppressWarnings("unused")
		ArchiveEntry entry;
		while ((entry = zis.getNextEntry()) != null) {
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			BufferedOutputStream Bout = new BufferedOutputStream(out);
			int b;
			while ((b = zis.read()) != -1) {
				Bout.write(b);
			}
			Bout.close();
			out.close();
			String JSONString = new String(out.toByteArray(), "UTF-8");
			JSONArray Content = (JSONArray) JSON.parse(JSONString);
			for (int index_item = 0; index_item < Content.size(); index_item++) {
				JSONObject item = (JSONObject) Content.get(index_item);
				zip_array.add(item);
			}
		}
		zis.close();

		LogUtil.error("test_insert");
		for (int index_tc = 0; index_tc < thread_count_array.length; index_tc++) {
			int thread_count = thread_count_array[index_tc];
			LogUtil.error("thread_count" + ":" + "\t" + thread_count);
			for (int index_cycle = 0; index_cycle < cycle_count; index_cycle++) {
				
				Thread.sleep(1000L * 60);
				
				long[] time_array = new long[Database_array.length];
				for (int index_array = 0; index_array < Database_array.length; index_array++) {
					CompressUtil.algo = algo_array[index_array];
					CompressUtil.deflate_level = deflate_level_array[index_array];
					String Database = Database_array[index_array];
					String Datatable = Datatable_array[index_array];
					{
						JSONObject wrapperObject = new JSONObject();
						wrapperObject.put("QueryType", "table_clear");
						wrapperObject.put("Database", Database);
						wrapperObject.put("Datatable", Datatable);
						agent.Query(wrapperObject);
					}

					order_list.clear();
					for (int index_item = 0; index_item < zip_array.size(); index_item++) {
						JSONObject item = (JSONObject) zip_array.get(index_item);
						order_list.add(item);
					}

					long t1 = (new Date()).getTime();
					List<Thread_zip_insert> threadList = new ArrayList<Thread_zip_insert>();
					for (int index = 0; index < thread_count; index++) {
						Thread_zip_insert thread = new Thread_zip_insert(agent, Database, Datatable);
						thread.start();
						threadList.add(thread);
					}
					while (true) {
						Thread.sleep(1000L);
						boolean all_finish = true;
						for (Thread_zip_insert thread : threadList) {
							if (thread.isAlive()) {
								all_finish = false;
								break;
							}
						}
						if (all_finish) {
							break;
						}
					}
					long t2 = (new Date()).getTime();
					time_array[index_array] = (t2 - t1);
				}
				StringBuffer sb = new StringBuffer();
				for (int index_array = 0; index_array < Database_array.length; index_array++) {
					if (sb.length() > 0) {
						sb.append(",");
					}
					sb.append(time_array[index_array]);
				}
				LogUtil.error((index_cycle + 1) + "," + sb.toString());
			}
		}

	}

	static void test_use() throws Exception {
		{
			CompressUtil.algo = algo_array[0];
			CompressUtil.deflate_level = deflate_level_array[0];
			String Database = Database_array[0];
			String Datatable = Datatable_array[0];
			{
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "select");
				wrapperObject.put("Database", Database);
				wrapperObject.put("Datatable", Datatable);
				JSONObject Criteria = new JSONObject();
				wrapperObject.put("Criteria", Criteria);
				JSONArray ReturnColumns = new JSONArray();
				ReturnColumns.add("order_id");
				wrapperObject.put("ReturnColumns", ReturnColumns);
				JSONObject queryResult = agent.Query(wrapperObject);
				Content = (JSONArray) queryResult.get("Content");
			}

			order_id_list.clear();
			for (int index_item = 0; index_item < Content.size(); index_item++) {
				JSONObject item = (JSONObject) Content.get(index_item);
				order_id_list.add((String) item.get("order_id"));
			}

			long t1 = (new Date()).getTime();
			List<Thread_zip_select> threadList = new ArrayList<Thread_zip_select>();
			for (int index = 0; index < thread_count_array[0]; index++) {
				Thread_zip_select thread = new Thread_zip_select(agent, Database, Datatable);
				thread.start();
				threadList.add(thread);
			}
			while (true) {
				Thread.sleep(1000L);
				boolean all_finish = true;
				for (Thread_zip_select thread : threadList) {
					if (thread.isAlive()) {
						all_finish = false;
						break;
					}
				}
				if (all_finish) {
					break;
				}
			}
			long t2 = (new Date()).getTime();
			LogUtil.error(Database + "." + Datatable + ":" + "\t" + (t2 - t1));

			int ori_length = 0;
			int[] zip_length_array = new int[Database_array.length - 1];
			for (int index_item = 0; index_item < Content.size(); index_item++) {
				JSONObject item = (JSONObject) Content.get(index_item);
				String order_id = (String) item.get("order_id");
				JSONObject ContentItem = resultMap.get(order_id);
				String wo_body = (String) ContentItem.get("wo_body");
				byte[] ori_bytes = wo_body.getBytes();
				ori_length += ori_bytes.length;
				for (int index_array = 1; index_array < Database_array.length; index_array++) {
					CompressUtil.algo = algo_array[index_array];
					CompressUtil.deflate_level = deflate_level_array[index_array];
					byte[] zip_bytes = CompressUtil.compress(ori_bytes);
					zip_length_array[index_array - 1] += zip_bytes.length;
				}
			}
			for (int index_array = 1; index_array < Database_array.length; index_array++) {
				LogUtil.error(algo_array[index_array] + "." + deflate_level_array[index_array] + ":" + "\t"
						+ (1.0 * zip_length_array[index_array - 1] / ori_length));
			}
		}

		test_select();
		test_update();

		agent.Stop();
	}

	static void test_select() throws Exception {
		LogUtil.error("test_select");
		for (int index_tc = 0; index_tc < thread_count_array.length; index_tc++) {
			int thread_count = thread_count_array[index_tc];
			LogUtil.error("thread_count" + ":" + "\t" + thread_count);
			for (int index_cycle = 0; index_cycle < cycle_count; index_cycle++) {
				long[] time_array = new long[Database_array.length];
				for (int index_array = 0; index_array < Database_array.length; index_array++) {
					CompressUtil.algo = algo_array[index_array];
					CompressUtil.deflate_level = deflate_level_array[index_array];
					String Database = Database_array[index_array];
					String Datatable = Datatable_array[index_array];

					order_id_list.clear();
					for (int index_item = 0; index_item < Content.size(); index_item++) {
						JSONObject item = (JSONObject) Content.get(index_item);
						order_id_list.add((String) item.get("order_id"));
					}

					long t1 = (new Date()).getTime();
					List<Thread_zip_select> threadList = new ArrayList<Thread_zip_select>();
					for (int index = 0; index < thread_count; index++) {
						Thread_zip_select thread = new Thread_zip_select(agent, Database, Datatable);
						thread.start();
						threadList.add(thread);
					}
					while (true) {
						Thread.sleep(1000L);
						boolean all_finish = true;
						for (Thread_zip_select thread : threadList) {
							if (thread.isAlive()) {
								all_finish = false;
								break;
							}
						}
						if (all_finish) {
							break;
						}
					}
					long t2 = (new Date()).getTime();
					time_array[index_array] = (t2 - t1);
				}
				StringBuffer sb = new StringBuffer();
				for (int index_array = 0; index_array < Database_array.length; index_array++) {
					if (sb.length() > 0) {
						sb.append(",");
					}
					sb.append(time_array[index_array]);
				}
				LogUtil.error((index_cycle + 1) + "," + sb.toString());
			}
		}
	}

	static void test_update() throws Exception {
		LogUtil.error("test_update");
		for (int index_tc = 0; index_tc < thread_count_array.length; index_tc++) {
			int thread_count = thread_count_array[index_tc];
			LogUtil.error("thread_count" + ":" + "\t" + thread_count);
			for (int index_cycle = 0; index_cycle < cycle_count; index_cycle++) {
				long[] time_array = new long[Database_array.length];
				for (int index_array = 0; index_array < Database_array.length; index_array++) {
					CompressUtil.algo = algo_array[index_array];
					CompressUtil.deflate_level = deflate_level_array[index_array];
					String Database = Database_array[index_array];
					String Datatable = Datatable_array[index_array];

					order_list.clear();
					for (int index_item = 0; index_item < Content.size(); index_item++) {
						JSONObject item = (JSONObject) Content.get(index_item);
						String order_id = (String) item.get("order_id");
						order_list.add(resultMap.get(order_id));
					}

					long t1 = (new Date()).getTime();
					List<Thread_zip_update> threadList = new ArrayList<Thread_zip_update>();
					for (int index = 0; index < thread_count; index++) {
						Thread_zip_update thread = new Thread_zip_update(agent, Database, Datatable);
						thread.start();
						threadList.add(thread);
					}
					while (true) {
						Thread.sleep(1000L);
						boolean all_finish = true;
						for (Thread_zip_update thread : threadList) {
							if (thread.isAlive()) {
								all_finish = false;
								break;
							}
						}
						if (all_finish) {
							break;
						}
					}
					long t2 = (new Date()).getTime();
					time_array[index_array] = (t2 - t1);
				}
				StringBuffer sb = new StringBuffer();
				for (int index_array = 0; index_array < Database_array.length; index_array++) {
					if (sb.length() > 0) {
						sb.append(",");
					}
					sb.append(time_array[index_array]);
				}
				LogUtil.error((index_cycle + 1) + "," + sb.toString());
			}
		}
	}

	static List<String> order_id_list = new ArrayList<String>();
	static Map<String, JSONObject> resultMap = new HashMap<String, JSONObject>();
	static Random rand = new Random();

	public static synchronized String Pop_select() {
		String result = null;
		if (order_id_list.size() > 0) {
			result = order_id_list.remove(rand.nextInt(order_id_list.size()));
			// result = order_id_list.remove(0);
		}
		return result;
	}

	public static synchronized void SetResult(String job, JSONObject result) {
		resultMap.put(job, result);
	}

	static List<JSONObject> order_list = new ArrayList<JSONObject>();

	public static synchronized JSONObject Pop_insert_update() {
		JSONObject result = null;
		if (order_list.size() > 0) {
			result = order_list.remove(rand.nextInt(order_list.size()));
			// result = order_list.remove(0);
		}
		return result;
	}
}
