package test.es;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.apache.http.HttpHost;
import org.elasticsearch.action.admin.indices.create.CreateIndexRequest;
import org.elasticsearch.action.admin.indices.create.CreateIndexResponse;
import org.elasticsearch.action.admin.indices.delete.DeleteIndexRequest;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.support.master.AcknowledgedResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.zillion.util.json.FastJsonReaderUtil;

import com.alibaba.fastjson.JSONObject;

public class ESSpeedTest1 {
	static final String index_name = "index1";
	static final String type_name = "type1";
	static final boolean bulk_insert = true;
	static final int count = 32;

	public static void main(String[] args) throws Exception {
		RestHighLevelClient client = new RestHighLevelClient(RestClient.builder(new HttpHost("localhost", 9200, "http")));
		ESSpeedTest1 ESStudy = new ESSpeedTest1();
		ESStudy.selectAll(client, "db_test_table_es", "g");
		try {
			ESStudy.delete(client, "db_test:table_es");
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			ESStudy.delete(client, index_name);
		} catch (Exception e) {
			e.printStackTrace();
		}
		ESStudy.create(client);
		ESStudy.insert(client);
		ESStudy.search(client);
		ESStudy.searchAll(client);
		client.close();
	}

	public void delete(RestHighLevelClient client, String im) throws Exception {
		DeleteIndexRequest request = new DeleteIndexRequest(im);
		AcknowledgedResponse deleteIndexResponse = client.indices().delete(request, RequestOptions.DEFAULT);
		System.out.println("isAcknowledged:" + deleteIndexResponse.isAcknowledged());
	}

	public void create(RestHighLevelClient client) throws Exception {
		CreateIndexRequest request = new CreateIndexRequest(index_name);
		request.settings(Settings.builder().put("index.number_of_shards", 1).put("index.number_of_replicas", 1));
		JSONObject wrapperObjects = FastJsonReaderUtil.Instance().ReadJSONObject(new File("etc/es/create1.json"));
		request.mapping(type_name, wrapperObjects.toJSONString(), XContentType.JSON);
		CreateIndexResponse createIndexResponse = client.indices().create(request, RequestOptions.DEFAULT);

		System.out.println("isAcknowledged:" + createIndexResponse.isAcknowledged());
		System.out.println("isShardsAcknowledged:" + createIndexResponse.isShardsAcknowledged());
	}

	public void insert(RestHighLevelClient client) throws Exception {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
		Map<String, List<Object>> valueListMap = new HashMap<String, List<Object>>();
		JSONObject wrapperObjects = FastJsonReaderUtil.Instance().ReadJSONObject(new File("etc/es/create1.json"));
		JSONObject properties = (JSONObject) wrapperObjects.get("properties");
		Iterator<String> keyIter = properties.keySet().iterator();
		while (keyIter.hasNext()) {
			String key = keyIter.next();
			JSONObject def = (JSONObject) properties.get(key);
			String type = (String) def.get("type");
			List<Object> valueList = ESSpeedUtil.generate(key, type, count);
			valueListMap.put(key, valueList);
		}

		String[] keys = properties.keySet().toArray(new String[0]);
		int[] indexes = new int[keys.length];
		int count_insert = 0;
		BulkRequest request = new BulkRequest();
		while (true) {
			Map<String, Object> jsonMap = new HashMap<>();
			for (int i = 0; i < keys.length; i++) {
				String key = keys[i];
				jsonMap.put(key, valueListMap.get(key).get(indexes[i]));
			}
			IndexRequest IndexRequest = new IndexRequest(index_name, type_name).source(jsonMap);
			if (bulk_insert) {
				request.add(IndexRequest);
				if (count_insert % 100 == 0) {
					client.bulk(request, RequestOptions.DEFAULT);
					request = new BulkRequest();
				}
			} else {
				client.index(IndexRequest, RequestOptions.DEFAULT);
			}
			count_insert++;
			if (count_insert % 1000 == 0) {
				System.out.println(sdf.format(new Date()) + "\t" + count_insert);
			}
			boolean finished = ESSpeedUtil.next(indexes, count);
			if (finished) {
				if (request.numberOfActions() > 0) {
					client.bulk(request, RequestOptions.DEFAULT);
				}
				break;
			}
		}
	}

	public void selectAll(RestHighLevelClient client, String in, String tn) throws Exception {
		SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
		searchSourceBuilder.query(QueryBuilders.matchAllQuery());

		SearchRequest searchRequest = new SearchRequest(in);
		searchRequest.types(tn);
		searchRequest.source(searchSourceBuilder);
		SearchResponse searchResponse = client.search(searchRequest, RequestOptions.DEFAULT);
		this.print(searchResponse);
	}

	public void searchAll(RestHighLevelClient client) throws Exception {
		Map<String, List<Object>> valueListMap = new HashMap<String, List<Object>>();
		JSONObject wrapperObjects = FastJsonReaderUtil.Instance().ReadJSONObject(new File("etc/es/create1.json"));
		JSONObject properties = (JSONObject) wrapperObjects.get("properties");
		String[] keys = { "user", "shenggao" };
		for (String key : keys) {
			JSONObject def = (JSONObject) properties.get(key);
			String type = (String) def.get("type");
			List<Object> valueList = ESSpeedUtil.generate(key, type, count);
			valueListMap.put(key, valueList);
		}
		int[] indexes = new int[keys.length];
		while (true) {
			BoolQueryBuilder qb = QueryBuilders.boolQuery();
			for (int i = 0; i < keys.length; i++) {
				String key = keys[i];
				qb.must(QueryBuilders.termQuery(key, valueListMap.get(key).get(indexes[i])));
			}

			SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
			sourceBuilder.query(qb);
			SearchRequest searchRequest = new SearchRequest(index_name);
			searchRequest.types(type_name);
			searchRequest.source(sourceBuilder);
			SearchResponse searchResponse = client.search(searchRequest, RequestOptions.DEFAULT);
			// this.print(searchResponse);
			boolean finished = ESSpeedUtil.next(indexes, count);
			if (finished) {
				break;
			}
		}
	}

	public void search(RestHighLevelClient client) throws Exception {
		// QueryBuilders.matchAllQuery();
		// QueryBuilders.matchQuery("name", "«402");
		// QueryBuilders.multiMatchQuery("ɽʡ̫ԭ7429ֵ", // Text you are looking for
		// "home", "now_home" // Fields you query on
		// );
		// QueryBuilders.boolQuery().must(QueryBuilders.termQuery("name", "«3033")).must(QueryBuilders.termQuery("home", "ɽʡ̫ԭ7967ֵ"))
		// .mustNot(QueryBuilders.termQuery("isRealMen", false)).should(QueryBuilders.termQuery("now_home", "ɽʡ̫ԭ"));
		// QueryBuilders.idsQuery().addIds("CHszwWRURyK08j01p0Mmug", "ojGrYKMEQCCPvh75lHJm3A");
		// QueryBuilders.fuzzyQuery("name", "«3582");
		// QueryBuilders.prefixQuery("name", "«31");
		// QueryBuilders.rangeQuery("name").from("«1000").to("«3000").includeLower(true) // ½
		// .includeUpper(true); // Ͻ

		QueryBuilder qb = QueryBuilders.boolQuery().must(QueryBuilders.rangeQuery("user").gte("name0000").lt("name0001"))
				.must(QueryBuilders.rangeQuery("shenggao").gte(0).lt(1)).must(QueryBuilders.rangeQuery("tizhong").gte(0.0).lt(1.0));

		SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
		sourceBuilder.query(qb);
		sourceBuilder.from(0);
		sourceBuilder.size(100);
		sourceBuilder.timeout(new TimeValue(60, TimeUnit.SECONDS));
		String[] includeFields = new String[] { "user", "shenggao", "tizhong", "postDate" };
		String[] excludeFields = new String[] { "_type" };
		sourceBuilder.fetchSource(includeFields, excludeFields);
		sourceBuilder.sort(SortBuilders.fieldSort("postDate").order(SortOrder.ASC));
		sourceBuilder.sort(SortBuilders.fieldSort("user").order(SortOrder.ASC));
		sourceBuilder.sort(SortBuilders.fieldSort("shenggao").order(SortOrder.ASC));

		SearchRequest searchRequest = new SearchRequest(index_name);
		searchRequest.types(type_name);
		searchRequest.source(sourceBuilder);
		SearchResponse searchResponse = client.search(searchRequest, RequestOptions.DEFAULT);
		this.print(searchResponse);
	}

	private void print(SearchResponse searchResponse) throws Exception {
		SearchHits hits = searchResponse.getHits();
		SearchHit[] searchHits = hits.getHits();
		System.out.println("SearchHit:" + searchHits.length);
		for (SearchHit hit : searchHits) {
			Map<String, Object> datas = hit.getSourceAsMap();
			System.out.println(datas.toString());
		}
	}
}
