package test.es;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.TimeUnit;

import org.apache.http.HttpHost;
import org.elasticsearch.action.admin.indices.create.CreateIndexRequest;
import org.elasticsearch.action.admin.indices.create.CreateIndexResponse;
import org.elasticsearch.action.admin.indices.delete.DeleteIndexRequest;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.delete.DeleteRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.index.IndexResponse;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.support.master.AcknowledgedResponse;
import org.elasticsearch.action.update.UpdateRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.common.unit.Fuzziness;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.MatchQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.zillion.util.json.FastJsonReaderUtil;

import com.alibaba.fastjson.JSONObject;

public class ESStudy {
	static final String index_name = "student";
	static final String type_name = "g1";

	public static void main(String[] args) throws Exception {
		RestHighLevelClient client = new RestHighLevelClient(RestClient.builder(new HttpHost("localhost", 9200, "http")));
		ESStudy ESStudy = new ESStudy();
		try {
			ESStudy.drop(client);
		} catch (Exception e) {
			// e.printStackTrace();
		}
		ESStudy.create(client);
		ESStudy.insert(client);
		ESStudy.select(client, "jack");
		Thread.sleep(1000L);
		// ESStudy.search01(client);
		// ESStudy.delete(client);
		ESStudy.select(client, "jack");
		ESStudy.update(client, "jack");
		Thread.sleep(1000L);
		ESStudy.select(client, "jack");
		client.close();
	}

	public void drop(RestHighLevelClient client) throws Exception {
		DeleteIndexRequest request = new DeleteIndexRequest(index_name);
		AcknowledgedResponse deleteIndexResponse = client.indices().delete(request, RequestOptions.DEFAULT);
		System.out.println("isAcknowledged:" + deleteIndexResponse.isAcknowledged());
	}

	public void create(RestHighLevelClient client) throws Exception {
		CreateIndexRequest request = new CreateIndexRequest(index_name);
		request.settings(Settings.builder().put("index.number_of_shards", 1).put("index.number_of_replicas", 1));
		JSONObject wrapperObjects = FastJsonReaderUtil.Instance().ReadJSONObject(new File("etc/es/create.json"));
		request.mapping(type_name, wrapperObjects.toJSONString(), XContentType.JSON);

		CreateIndexResponse createIndexResponse = client.indices().create(request, RequestOptions.DEFAULT);

		System.out.println("isAcknowledged:" + createIndexResponse.isAcknowledged());
		System.out.println("isShardsAcknowledged:" + createIndexResponse.isShardsAcknowledged());
	}

	public void insert(RestHighLevelClient client) throws Exception {
		String[] users = { "kimchy", "tim", "jack", "rose" };
		Random rand = new Random();
		boolean bulk_insert = true;
		for (String user : users) {
			BulkRequest bulkRequest = new BulkRequest();
			for (int i = 0; i < 5; i++) {
				Map<String, Object> jsonMap = new HashMap<>();
				jsonMap.put("user", user);
				jsonMap.put("shenggao", rand.nextInt(100));
				jsonMap.put("tizhong", rand.nextDouble() * 100.0);
				jsonMap.put("postDate", new Date());
				IndexRequest request = new IndexRequest(index_name, type_name).source(jsonMap);
				if (bulk_insert) {
					bulkRequest.add(request);
				} else {
					IndexResponse indexResponse = client.index(request, RequestOptions.DEFAULT);
					System.out.println("id:" + indexResponse.getId());
				}
			}
			if (bulk_insert) {
				client.bulk(bulkRequest, RequestOptions.DEFAULT);
			}
		}
	}

	public void delete(RestHighLevelClient client) throws Exception {
		List<String> ids = this.select_ids(client, "tim");
		BulkRequest bulkRequest = new BulkRequest();
		for (String id : ids) {
			System.out.println("delete\t" + id);
			DeleteRequest deleteRequest = new DeleteRequest(index_name, type_name, id);
			bulkRequest.add(deleteRequest);
		}
		client.bulk(bulkRequest, RequestOptions.DEFAULT);
	}

	public void update(RestHighLevelClient client, String user) throws Exception {
		List<String> ids = this.select_ids(client, user);
		JSONObject Set = new JSONObject();
		Set.put("tizhong", 300.0);
		BulkRequest bulkRequest = new BulkRequest();
		for (String id : ids) {
			System.out.println("update\t" + id);
			UpdateRequest deleteRequest = new UpdateRequest(index_name, type_name, id);
			deleteRequest.doc(Set.toJSONString(), XContentType.JSON);
			bulkRequest.add(deleteRequest);
		}
		client.bulk(bulkRequest, RequestOptions.DEFAULT);
	}

	public void search01(RestHighLevelClient client) throws Exception {
		SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
		searchSourceBuilder.query(QueryBuilders.matchAllQuery());

		SearchRequest searchRequest = new SearchRequest(index_name);
		searchRequest.types(type_name);
		searchRequest.source(searchSourceBuilder);
		SearchResponse searchResponse = client.search(searchRequest, RequestOptions.DEFAULT);
		this.print(searchResponse);
	}

	public List<String> select(RestHighLevelClient client, String user) throws Exception {
		List<String> result = new ArrayList<String>();
		SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
		sourceBuilder.query(QueryBuilders.termQuery("user", user));
		sourceBuilder.from(0);
		sourceBuilder.size(10);
		sourceBuilder.timeout(new TimeValue(60, TimeUnit.SECONDS));
		String[] includeFields = new String[] { "user", "shenggao", "tizhong" };
		String[] excludeFields = new String[] { "_type" };
		sourceBuilder.fetchSource(includeFields, excludeFields);
		sourceBuilder.sort(SortBuilders.fieldSort("shenggao").order(SortOrder.ASC));

		SearchRequest searchRequest = new SearchRequest(index_name);
		searchRequest.types(type_name);
		searchRequest.source(sourceBuilder);
		SearchResponse searchResponse = client.search(searchRequest, RequestOptions.DEFAULT);
		this.print(searchResponse);
		return result;
	}

	public List<String> select_ids(RestHighLevelClient client, String user) throws Exception {
		List<String> result = new ArrayList<String>();
		SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
		sourceBuilder.query(QueryBuilders.termQuery("user", user));
		sourceBuilder.from(0);
		sourceBuilder.size(10);
		sourceBuilder.timeout(new TimeValue(60, TimeUnit.SECONDS));
		String[] includeFields = new String[] { "user", "shenggao", "tizhong" };
		String[] excludeFields = new String[] { "_type" };
		sourceBuilder.fetchSource(includeFields, excludeFields);
		sourceBuilder.sort(SortBuilders.fieldSort("shenggao").order(SortOrder.ASC));

		SearchRequest searchRequest = new SearchRequest(index_name);
		searchRequest.types(type_name);
		searchRequest.source(sourceBuilder);
		SearchResponse searchResponse = client.search(searchRequest, RequestOptions.DEFAULT);
		SearchHits hits = searchResponse.getHits();
		SearchHit[] searchHits = hits.getHits();
		System.out.println("SearchHit:" + searchHits.length);
		for (SearchHit hit : searchHits) {
			result.add(hit.getId());
		}
		return result;
	}

	public void search03(RestHighLevelClient client) throws Exception {
		SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
		sourceBuilder.query(QueryBuilders.rangeQuery("user").gte("jack").lte("rose"));
		sourceBuilder.from(0);
		sourceBuilder.size(10);
		sourceBuilder.timeout(new TimeValue(60, TimeUnit.SECONDS));
		String[] includeFields = new String[] { "user", "shenggao", "tizhong" };
		String[] excludeFields = new String[] { "_type" };
		sourceBuilder.fetchSource(includeFields, excludeFields);
		sourceBuilder.sort(SortBuilders.fieldSort("user").order(SortOrder.ASC));
		sourceBuilder.sort(SortBuilders.fieldSort("shenggao").order(SortOrder.ASC));

		SearchRequest searchRequest = new SearchRequest(index_name);
		searchRequest.types(type_name);
		searchRequest.source(sourceBuilder);
		SearchResponse searchResponse = client.search(searchRequest, RequestOptions.DEFAULT);
		this.print(searchResponse);
	}

	public void search04(RestHighLevelClient client) throws Exception {
		MatchQueryBuilder matchQueryBuilder = new MatchQueryBuilder("user", "kimchy");
		matchQueryBuilder.fuzziness(Fuzziness.AUTO);
		matchQueryBuilder.prefixLength(3);
		matchQueryBuilder.maxExpansions(10);
		// MatchQueryBuilder matchQueryBuilder = QueryBuilders.matchQuery("user", "kimchy")
		// .fuzziness(Fuzziness.AUTO)
		// .prefixLength(3)
		// .maxExpansions(10);
		SearchSourceBuilder sourceQueryBuilder = new SearchSourceBuilder();
		sourceQueryBuilder.query(matchQueryBuilder);

		SearchRequest searchRequest = new SearchRequest(index_name);
		searchRequest.types(type_name);
		searchRequest.source(sourceQueryBuilder);
		SearchResponse searchResponse = client.search(searchRequest, RequestOptions.DEFAULT);
		this.print(searchResponse);
	}

	private void print(SearchResponse searchResponse) throws Exception {
		SearchHits hits = searchResponse.getHits();
		SearchHit[] searchHits = hits.getHits();
		System.out.println("SearchHit:" + searchHits.length);
		for (SearchHit hit : searchHits) {
			Map<String, Object> datas = hit.getSourceAsMap();
			System.out.println(hit.getId() + "\t" + datas.toString());
		}
	}
}
