package test.hbase;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.apache.hadoop.hbase.client.Get;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Row;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.filter.BinaryComparator;
import org.apache.hadoop.hbase.filter.CompareFilter;
import org.apache.hadoop.hbase.filter.Filter;
import org.apache.hadoop.hbase.filter.FilterList;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter.RowRange;
import org.apache.hadoop.hbase.filter.RowFilter;
import org.apache.hadoop.hbase.util.Bytes;

public class HBaseTest1 {

	static String[] name_ss = { "james", "john", "robert", "michael", "william", "david", "richard", "charles", "joseph", "thomas" };
	static String[] name2_ss = { "mary", "patricia", "linda", "barbara", "elizabeth", "jennifer", "maria", "susan", "margaret", "dorothy" };
	static int age_length = 2;
	static int age2_length = 2;
	static int score_length = 10;
	static int score2_length = 10;

	public static void main(String[] args) throws Exception {
		Configuration configuration = HBaseConfiguration.create();
		Connection connection = ConnectionFactory.createConnection(configuration);
		Admin admin = connection.getAdmin();

		int splitCount = 20;
		int totalCount = name_ss.length * name2_ss.length * age_length * age2_length * score_length * score2_length;
		int splitSize = totalCount / splitCount;
		byte[][] splitKeys = new byte[splitCount - 1][];
		for (int i = 1; i < splitCount; i++) {
			splitKeys[i - 1] = Bytes.toBytes("row" + AdminUtil.df.format(splitSize * i));
		}

		String table_name = "table_zillion";
		TableName tableName = TableName.valueOf(table_name);

		AdminUtil.deleteTable(admin, tableName);
		AdminUtil.createTable(admin, tableName, splitKeys);

		HTable table = (HTable) connection.getTable(tableName);
		{
			List<Put> batch = new ArrayList<Put>();
			{
				Put put = new Put(Bytes.toBytes("row" + "1"));
				put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name"), Bytes.toBytes("name" + "1"));
				batch.add(put);
			}
			{
//				Put put = new Put(Bytes.toBytes("row" + "2"));
//				put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name"), Bytes.toBytes("name" + "2"));
//				batch.add(put);
			}
			table.put(batch);
			table.flushCommits();
		}
		{
			List<Get> batch = new ArrayList<Get>();
			{
				Get get = new Get(Bytes.toBytes("row" + "1"));
				get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name"));
				batch.add(get);
			}
			{
				Get get = new Get(Bytes.toBytes("row" + "2"));
				get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name"));
				batch.add(get);
			}
			Result[] results = table.get(batch);
			List<byte[]> exist_row_bytesList = new ArrayList<byte[]>();
			for (Result res : results) {
				byte[] rowBytes = null;
				if (res != null && res.getRow() != null) {
					rowBytes = res.getRow();
				}
				exist_row_bytesList.add(rowBytes);
			}
			System.out.println();
		}

		// Put put = new Put(Bytes.toBytes("row1"));
		// put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("v1"),
		// Bytes.toBytes("val1"));
		// put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("v2"),
		// Bytes.toBytes("val2"));
		// table.put(put);

		Date date1 = new Date();
		Insert(table, true);

		Date date2 = new Date();
		// Get(table);
		Scanner_MultiRowRangeFilter(table);
		Date date3 = new Date();
		Update(table);
		Date date4 = new Date();

		System.out.println("Insert:\t" + (date2.getTime() - date1.getTime()));
		System.out.println("Scanner:\t" + (date3.getTime() - date2.getTime()));
		System.out.println("Update:\t" + (date4.getTime() - date3.getTime()));

		table.close();
		admin.close();
		connection.close();
	}

	public static void Insert(HTable table, boolean sequence) throws Exception {
		System.out.println(AdminUtil.sdf.format(new Date()));

		if (table.isAutoFlush()) {
			table.setAutoFlush(false, true);
		}

		if (sequence) {
			for (int name_index = 0; name_index < name_ss.length; name_index++) {
				for (int name2_index = 0; name2_index < name2_ss.length; name2_index++) {
					for (int age_index = 0; age_index < age_length; age_index++) {
						for (int age2_index = 0; age2_index < age2_length; age2_index++) {
							List<Put> batch = new ArrayList<Put>();
							for (int score_index = 0; score_index < score_length; score_index++) {
								for (int score2_index = 0; score2_index < score2_length; score2_index++) {
									int id = (((((name_index) * name2_ss.length + name2_index) * age_length + age_index) * age2_length + age2_index)
											* score_length + score_index) * score2_length + score2_index;
									Put put = new Put(Bytes.toBytes("row" + AdminUtil.df.format(id)));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name_s"), Bytes.toBytes(name_ss[name_index]));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name2_s"), Bytes.toBytes(name2_ss[name2_index]));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"), Bytes.toBytes(age_index));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"), Bytes.toBytes(age2_index));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score_d"), Bytes.toBytes(score_index));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score2_d"), Bytes.toBytes(score2_index));
									batch.add(put);
								}
							}
							table.put(batch);
							table.flushCommits();
						}
					}
					System.out.println(AdminUtil.sdf.format(new Date()) + "\t" + name_index + name2_index);
				}
			}
		} else {
			for (int score2_index = 0; score2_index < score2_length; score2_index++) {
				for (int score_index = 0; score_index < score_length; score_index++) {
					List<Put> batch = new ArrayList<Put>();
					for (int age2_index = 0; age2_index < age2_length; age2_index++) {
						for (int age_index = 0; age_index < age_length; age_index++) {
							for (int name2_index = 0; name2_index < name2_ss.length; name2_index++) {
								for (int name_index = 0; name_index < name_ss.length; name_index++) {
									int id = (((((name_index) * name2_ss.length + name2_index) * age_length + age_index) * age2_length + age2_index)
											* score_length + score_index) * score2_length + score2_index;
									Put put = new Put(Bytes.toBytes("row" + AdminUtil.df.format(id)));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name_s"), Bytes.toBytes(name_ss[name_index]));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name2_s"), Bytes.toBytes(name2_ss[name2_index]));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"), Bytes.toBytes(age_index));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"), Bytes.toBytes(age2_index));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score_d"), Bytes.toBytes(score_index));
									put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score2_d"), Bytes.toBytes(score2_index));
									batch.add(put);
								}
							}
						}
					}
					table.put(batch);
					table.flushCommits();
				}
				System.out.println(AdminUtil.sdf.format(new Date()) + "\t" + score2_index);
			}
		}

		System.out.println(AdminUtil.sdf.format(new Date()));
	}

	public static void Get(HTable table) throws Exception {
		System.out.println(AdminUtil.sdf.format(new Date()));

		for (int name_index = 0; name_index < name_ss.length; name_index++) {
			for (int name2_index = 0; name2_index < name2_ss.length; name2_index++) {
				int count = 0;
				for (int age_index = 0; age_index < age_length; age_index++) {
					for (int age2_index = 0; age2_index < age2_length; age2_index++) {
						List<Get> batch = new ArrayList<Get>();
						for (int score_index = 0; score_index < score_length; score_index++) {
							for (int score2_index = 0; score2_index < score2_length; score2_index++) {
								int id = (((((name_index) * name2_ss.length + name2_index) * age_length + age_index) * age2_length + age2_index)
										* score_length + score_index) * score2_length + score2_index;
								Get get = new Get(Bytes.toBytes("row" + AdminUtil.df.format(id)));
								get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name_s"));
								get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name2_s"));
								get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"));
								get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"));
								get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score_d"));
								get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score2_d"));
								batch.add(get);
							}
						}
						Result[] results = table.get(batch);
						for (Result res : results) {
							count++;
						}
					}
				}
				System.out.println(AdminUtil.sdf.format(new Date()) + "\t" + name_index + name2_index + "\t" + count);
			}
		}

		System.out.println(AdminUtil.sdf.format(new Date()));
	}

	public static void Scanner_FilterList(HTable table) throws Exception {
		System.out.println(AdminUtil.sdf.format(new Date()));

		for (int name_index = 0; name_index < name_ss.length; name_index++) {
			for (int name2_index = 0; name2_index < name2_ss.length; name2_index++) {
				int count = 0;
				for (int age_index = 0; age_index < age_length; age_index++) {
					for (int age2_index = 0; age2_index < age2_length; age2_index++) {
						int score_index = 0;
						int score2_index = 0;
						int start_id = (((((name_index) * name2_ss.length + name2_index) * age_length + age_index) * age2_length + age2_index)
								* score_length + score_index) * score2_length + score2_index;
						int end_id = start_id + score_length * score2_length;

						Scan scan = new Scan();
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name_s"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name2_s"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score_d"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score2_d"));

						List<Filter> filters = new ArrayList<Filter>();
						Filter filter1 = new RowFilter(CompareFilter.CompareOp.GREATER_OR_EQUAL,
								new BinaryComparator(Bytes.toBytes("row" + AdminUtil.df.format(start_id))));
						Filter filter2 = new RowFilter(CompareFilter.CompareOp.LESS,
								new BinaryComparator(Bytes.toBytes("row" + AdminUtil.df.format(end_id))));
						filters.add(filter1);
						filters.add(filter2);
						FilterList filterList = new FilterList(filters);
						scan.setFilter(filterList);
						ResultScanner scanner = table.getScanner(scan);
						while (true) {
							Result res = scanner.next();
							if (res == null) {
								break;
							}
							count++;
						}
						scanner.close();
					}
				}
				System.out.println(AdminUtil.sdf.format(new Date()) + "\t" + name_index + name2_index + "\t" + count);
			}
		}

		System.out.println(AdminUtil.sdf.format(new Date()));
	}

	public static void Scanner_MultiRowRangeFilter(HTable table) throws Exception {
		System.out.println(AdminUtil.sdf.format(new Date()));

		for (int name_index = 0; name_index < name_ss.length; name_index++) {
			for (int name2_index = 0; name2_index < name2_ss.length; name2_index++) {
				int count = 0;
				for (int age_index = 0; age_index < age_length; age_index++) {
					for (int age2_index = 0; age2_index < age2_length; age2_index++) {
						int score_index = 0;
						int score2_index = 0;
						int start_id = (((((name_index) * name2_ss.length + name2_index) * age_length + age_index) * age2_length + age2_index)
								* score_length + score_index) * score2_length + score2_index;
						int end_id = start_id + score_length * score2_length;

						Scan scan = new Scan();
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name_s"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name2_s"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score_d"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score2_d"));

						List<RowRange> rangeList = new ArrayList<RowRange>();
						rangeList.add(new RowRange(Bytes.toBytes("row" + AdminUtil.df.format(start_id)), true,
								Bytes.toBytes("row" + AdminUtil.df.format(end_id)), false));
						MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
						scan.setFilter(filter);
						ResultScanner scanner = table.getScanner(scan);
						while (true) {
							Result res = scanner.next();
							if (res == null) {
								break;
							}
							count++;
						}
						scanner.close();
					}
				}
				System.out.println(AdminUtil.sdf.format(new Date()) + "\t" + name_index + name2_index + "\t" + count);
			}
		}

		System.out.println(AdminUtil.sdf.format(new Date()));
	}

	public static void Update(HTable table) throws Exception {
		System.out.println(AdminUtil.sdf.format(new Date()));

		for (int name_index = 0; name_index < name_ss.length; name_index++) {
			for (int name2_index = 0; name2_index < name2_ss.length; name2_index++) {
				int count = 0;
				for (int age_index = 0; age_index < age_length; age_index++) {
					for (int age2_index = 0; age2_index < age2_length; age2_index++) {
						int score_index = 0;
						int score2_index = 0;
						int start_id = (((((name_index) * name2_ss.length + name2_index) * age_length + age_index) * age2_length + age2_index)
								* score_length + score_index) * score2_length + score2_index;
						int end_id = start_id + score_length * score2_length;

						List<Put> putList = new ArrayList<Put>();

						Scan scan = new Scan();
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name_s"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name2_s"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"));
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"));

						List<RowRange> rangeList = new ArrayList<RowRange>();
						rangeList.add(new RowRange(Bytes.toBytes("row" + AdminUtil.df.format(start_id)), true,
								Bytes.toBytes("row" + AdminUtil.df.format(end_id)), false));
						MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
						scan.setFilter(filter);
						ResultScanner scanner = table.getScanner(scan);
						while (true) {
							Result result = scanner.next();
							if (result == null) {
								break;
							}

							byte[] row_bytes = result.getRow();
							int age_l = Bytes.toInt(result.getValue(Bytes.toBytes("f"), Bytes.toBytes("age_l")));
							int age2_l = Bytes.toInt(result.getValue(Bytes.toBytes("f"), Bytes.toBytes("age2_l")));
							int age_l_new = age_length + age_l;
							int age2_l_new = age2_length + age2_l;
							byte[] age_bytes_new = Bytes.toBytes(age_l_new);
							byte[] age2_bytes_new = Bytes.toBytes(age2_l_new);

							Put put = new Put(row_bytes);
							put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"), age_bytes_new);
							put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"), age2_bytes_new);
							putList.add(put);

							count++;
						}
						scanner.close();

						{
							table.put(putList);
							table.flushCommits();
						}
					}
				}
				System.out.println(AdminUtil.sdf.format(new Date()) + "\t" + name_index + name2_index + "\t" + count);
			}
		}

		System.out.println(AdminUtil.sdf.format(new Date()));
	}

}
