package test.hbase;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.apache.hadoop.hbase.client.Delete;
import org.apache.hadoop.hbase.client.Get;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Row;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter.RowRange;
import org.apache.hadoop.hbase.util.Bytes;

public class HBaseTest2 {
	static String[] name_ss = { "james", "john", "robert", "michael", "william", "david", "richard", "charles", "joseph", "thomas" };
	static String[] name2_ss = { "mary", "patricia", "linda", "barbara", "elizabeth", "jennifer", "maria", "susan", "margaret", "dorothy" };
	static int age_length = 10;
	static int age2_length = 10;
	static int score_length = 10;
	static int score2_length = 10;

	public static void main(String[] args) throws Exception {
		Configuration configuration = HBaseConfiguration.create();
		Connection connection = ConnectionFactory.createConnection(configuration);
		Admin admin = connection.getAdmin();

		int splitCount = 20;
		int totalCount = name_ss.length * name2_ss.length * age_length * age2_length * score_length * score2_length;
		int splitSize = totalCount / splitCount;
		byte[][] splitKeys = new byte[splitCount - 1][];
		for (int i = 1; i < splitCount; i++) {
			splitKeys[i - 1] = Bytes.toBytes("row" + AdminUtil.df.format(splitSize * i));
		}

		int index_splitCount = 20;
		int index_totalCount = age2_length * age_length;
		int index_splitSize = index_totalCount / index_splitCount;
		byte[][] index_splitKeys = new byte[index_splitCount - 1][];
		for (int i = 1; i < index_splitCount; i++) {
			int tmp = index_splitSize * i;
			int age2_index = tmp / age_length;
			int age_index = tmp % age_length;
			byte[] age_bytes = Bytes.toBytes(age_index);
			byte[] age2_bytes = Bytes.toBytes(age2_index);
			byte[] index_bytes = new byte[age2_bytes.length + age_bytes.length];
			System.arraycopy(age2_bytes, 0, index_bytes, 0, age2_bytes.length);
			System.arraycopy(age_bytes, 0, index_bytes, age2_bytes.length, age_bytes.length);
			index_splitKeys[i - 1] = index_bytes;
		}

		String table_name = "table_zillion";
		TableName tableName = TableName.valueOf(table_name);
		String table_name_index = "table_zillion_index";
		TableName tableName_index = TableName.valueOf(table_name_index);

		AdminUtil.deleteTable(admin, tableName);
		AdminUtil.createTable(admin, tableName, splitKeys);
		AdminUtil.deleteTable(admin, tableName_index);
		AdminUtil.createTable(admin, tableName_index, index_splitKeys);

		HTable table = (HTable) connection.getTable(tableName);
		HTable table_index = (HTable) connection.getTable(tableName_index);

		Date date1 = new Date();
		Insert(table, table_index);

		Date date2 = new Date();
		Scanner(table, table_index);
		Date date3 = new Date();
		Update(table, table_index);
		Date date4 = new Date();

		System.out.println("Insert:\t" + (date2.getTime() - date1.getTime()));
		System.out.println("Scanner:\t" + (date3.getTime() - date2.getTime()));
		System.out.println("Update:\t" + (date4.getTime() - date3.getTime()));

		table.close();
		admin.close();
		connection.close();
	}

	public static void Insert(HTable table, HTable table_index) throws Exception {
		System.out.println(AdminUtil.sdf.format(new Date()));

		if (table.isAutoFlush()) {
			table.setAutoFlush(false, true);
		}

		for (int name_index = 0; name_index < name_ss.length; name_index++) {
			for (int name2_index = 0; name2_index < name2_ss.length; name2_index++) {
				for (int age_index = 0; age_index < age_length; age_index++) {
					for (int age2_index = 0; age2_index < age2_length; age2_index++) {
						byte[] name_bytes = Bytes.toBytes(name_ss[name_index]);
						byte[] name2_bytes = Bytes.toBytes(name2_ss[name2_index]);
						byte[] age_bytes = Bytes.toBytes(age_index);
						byte[] age2_bytes = Bytes.toBytes(age2_index);
						byte[] index_bytes = new byte[age2_bytes.length + age_bytes.length + name2_bytes.length + 1 + name_bytes.length];
						System.arraycopy(age2_bytes, 0, index_bytes, 0, age2_bytes.length);
						System.arraycopy(age_bytes, 0, index_bytes, age2_bytes.length, age_bytes.length);
						System.arraycopy(name2_bytes, 0, index_bytes, age2_bytes.length + age_bytes.length, name2_bytes.length);
						System.arraycopy(name_bytes, 0, index_bytes, age2_bytes.length + age_bytes.length + name2_bytes.length + 1,
								name_bytes.length);

						List<Put> batch = new ArrayList<Put>();
						List<Put> batch_index = new ArrayList<Put>();
						for (int score_index = 0; score_index < score_length; score_index++) {
							for (int score2_index = 0; score2_index < score2_length; score2_index++) {
								int id = (((((name_index) * name2_ss.length + name2_index) * age_length + age_index) * age2_length + age2_index)
										* score_length + score_index) * score2_length + score2_index;
								byte[] row_bytes = Bytes.toBytes("row" + AdminUtil.df.format(id));

								Put put = new Put(row_bytes);
								put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name_s"), name_bytes);
								put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name2_s"), name2_bytes);
								put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"), age_bytes);
								put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"), age2_bytes);
								put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score_d"), Bytes.toBytes(score_index));
								put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score2_d"), Bytes.toBytes(score2_index));
								batch.add(put);

								byte[] index_row_bytes = new byte[index_bytes.length + 1 + row_bytes.length];
								System.arraycopy(index_bytes, 0, index_row_bytes, 0, index_bytes.length);
								System.arraycopy(row_bytes, 0, index_row_bytes, index_bytes.length + 1, row_bytes.length);

								Put put_index = new Put(index_row_bytes);
								put_index.addColumn(Bytes.toBytes("f"), Bytes.toBytes("i"), Bytes.toBytes("1"));
								batch_index.add(put_index);
							}
						}
						table.put(batch);
						table.flushCommits();

						table_index.put(batch_index);
						table_index.flushCommits();
					}
				}
				System.out.println(AdminUtil.sdf.format(new Date()) + "\t" + name_index + name2_index);
			}
		}

		System.out.println(AdminUtil.sdf.format(new Date()));
	}

	public static void Scanner(HTable table, HTable table_index) throws Exception {
		System.out.println(AdminUtil.sdf.format(new Date()));

		for (int name_index = 0; name_index < name_ss.length; name_index++) {
			for (int name2_index = 0; name2_index < name2_ss.length; name2_index++) {
				int count = 0;
				for (int age_index = 0; age_index < age_length; age_index++) {
					for (int age2_index = 0; age2_index < age2_length; age2_index++) {
						byte[] name_bytes = Bytes.toBytes(name_ss[name_index]);
						byte[] name2_bytes = Bytes.toBytes(name2_ss[name2_index]);
						byte[] age_bytes = Bytes.toBytes(age_index);
						byte[] age2_bytes = Bytes.toBytes(age2_index);
						byte[] index_bytes = new byte[age2_bytes.length + age_bytes.length + name2_bytes.length + 1 + name_bytes.length];
						System.arraycopy(age2_bytes, 0, index_bytes, 0, age2_bytes.length);
						System.arraycopy(age_bytes, 0, index_bytes, age2_bytes.length, age_bytes.length);
						System.arraycopy(name2_bytes, 0, index_bytes, age2_bytes.length + age_bytes.length, name2_bytes.length);
						System.arraycopy(name_bytes, 0, index_bytes, age2_bytes.length + age_bytes.length + name2_bytes.length + 1,
								name_bytes.length);

						byte[] index_bytes_begin = new byte[index_bytes.length + 1];
						System.arraycopy(index_bytes, 0, index_bytes_begin, 0, index_bytes.length);
						byte[] index_bytes_end = new byte[index_bytes.length + 1];
						System.arraycopy(index_bytes, 0, index_bytes_end, 0, index_bytes.length);
						index_bytes_end[index_bytes_end.length - 1] = 1;

						List<byte[]> row_bytesList = new ArrayList<byte[]>();
						Scan scan = new Scan();
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("i"));

						List<RowRange> rangeList = new ArrayList<RowRange>();
						rangeList.add(new RowRange(index_bytes_begin, true, index_bytes_end, false));
						MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
						scan.setFilter(filter);
						ResultScanner scanner = table_index.getScanner(scan);
						while (true) {
							Result res = scanner.next();
							if (res == null) {
								break;
							}

							byte[] cellRowArray = res.getRow();
							byte[] row_bytes = new byte[cellRowArray.length - (index_bytes.length + 1)];
							System.arraycopy(cellRowArray, (index_bytes.length + 1), row_bytes, 0, cellRowArray.length - (index_bytes.length + 1));
							row_bytesList.add(row_bytes);
						}
						scanner.close();

						List<Get> batch = new ArrayList<Get>();
						for (byte[] row_bytes : row_bytesList) {
							Get get = new Get(row_bytes);
							get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name_s"));
							get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name2_s"));
							get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"));
							get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"));
							get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score_d"));
							get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("score2_d"));
							batch.add(get);
						}

						Result[] results = table.get(batch);
						for (Object res : results) {
							count++;
						}
					}
				}
				System.out.println(AdminUtil.sdf.format(new Date()) + "\t" + name_index + name2_index + "\t" + count);
			}
		}

		System.out.println(AdminUtil.sdf.format(new Date()));
	}

	public static void Update(HTable table, HTable table_index) throws Exception {
		System.out.println(AdminUtil.sdf.format(new Date()));

		for (int name_index = 0; name_index < name_ss.length; name_index++) {
			for (int name2_index = 0; name2_index < name2_ss.length; name2_index++) {
				int count = 0;
				for (int age_index = 0; age_index < age_length; age_index++) {
					for (int age2_index = 0; age2_index < age2_length; age2_index++) {
						byte[] name_bytes = Bytes.toBytes(name_ss[name_index]);
						byte[] name2_bytes = Bytes.toBytes(name2_ss[name2_index]);
						byte[] age_bytes = Bytes.toBytes(age_index);
						byte[] age2_bytes = Bytes.toBytes(age2_index);
						byte[] index_bytes = new byte[age2_bytes.length + age_bytes.length + name2_bytes.length + 1 + name_bytes.length];
						System.arraycopy(age2_bytes, 0, index_bytes, 0, age2_bytes.length);
						System.arraycopy(age_bytes, 0, index_bytes, age2_bytes.length, age_bytes.length);
						System.arraycopy(name2_bytes, 0, index_bytes, age2_bytes.length + age_bytes.length, name2_bytes.length);
						System.arraycopy(name_bytes, 0, index_bytes, age2_bytes.length + age_bytes.length + name2_bytes.length + 1,
								name_bytes.length);

						int age_l_new = age_length + age_index;
						int age2_l_new = age2_length + age2_index;
						byte[] age_bytes_new = Bytes.toBytes(age_l_new);
						byte[] age2_bytes_new = Bytes.toBytes(age2_l_new);
						byte[] index_bytes_new = new byte[age2_bytes_new.length + age_bytes_new.length + name2_bytes.length + 1 + name_bytes.length];
						System.arraycopy(age2_bytes_new, 0, index_bytes_new, 0, age2_bytes_new.length);
						System.arraycopy(age_bytes_new, 0, index_bytes_new, age2_bytes_new.length, age_bytes_new.length);
						System.arraycopy(name2_bytes, 0, index_bytes_new, age2_bytes_new.length + age_bytes_new.length, name2_bytes.length);
						System.arraycopy(name_bytes, 0, index_bytes_new, age2_bytes_new.length + age_bytes_new.length + name2_bytes.length + 1,
								name_bytes.length);

						byte[] index_bytes_begin = new byte[index_bytes.length + 1];
						System.arraycopy(index_bytes, 0, index_bytes_begin, 0, index_bytes.length);
						byte[] index_bytes_end = new byte[index_bytes.length + 1];
						System.arraycopy(index_bytes, 0, index_bytes_end, 0, index_bytes.length);
						index_bytes_end[index_bytes_end.length - 1] = 1;

						List<byte[]> index_row_bytesList = new ArrayList<byte[]>();
						List<byte[]> row_bytesList = new ArrayList<byte[]>();
						Scan scan = new Scan();
						scan.addColumn(Bytes.toBytes("f"), Bytes.toBytes("i"));

						List<RowRange> rangeList = new ArrayList<RowRange>();
						rangeList.add(new RowRange(index_bytes_begin, true, index_bytes_end, false));
						MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
						scan.setFilter(filter);
						ResultScanner scanner = table_index.getScanner(scan);
						while (true) {
							Result res = scanner.next();
							if (res == null) {
								break;
							}

							byte[] cellRowArray = res.getRow();
							byte[] row_bytes = new byte[cellRowArray.length - (index_bytes.length + 1)];
							System.arraycopy(cellRowArray, (index_bytes.length + 1), row_bytes, 0, cellRowArray.length - (index_bytes.length + 1));
							index_row_bytesList.add(cellRowArray);
							row_bytesList.add(row_bytes);
						}
						scanner.close();

						List<Get> batch = new ArrayList<Get>();
						for (byte[] row_bytes : row_bytesList) {
							Get get = new Get(row_bytes);
							get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name_s"));
							get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("name2_s"));
							get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"));
							get.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"));
							batch.add(get);
						}

						List<Delete> deleteList = new ArrayList<Delete>();
						List<Put> putList = new ArrayList<Put>();
						List<Delete> index_deleteList = new ArrayList<Delete>();
						List<Put> index_putList = new ArrayList<Put>();

						Result[] results = table.get(batch);
						for (Result result : results) {
							if (true) {
								byte[] row_bytes = result.getRow();
								Delete delete = new Delete(row_bytes);
								delete.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"));
								delete.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"));
								deleteList.add(delete);

								Put put = new Put(row_bytes);
								put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"), age_bytes_new);
								put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"), age2_bytes_new);
								putList.add(put);

								byte[] index_row_bytes = new byte[index_bytes.length + 1 + row_bytes.length];
								System.arraycopy(index_bytes, 0, index_row_bytes, 0, index_bytes.length);
								System.arraycopy(row_bytes, 0, index_row_bytes, index_bytes.length + 1, row_bytes.length);
								Delete index_delete = new Delete(index_row_bytes);
								index_delete.addColumn(Bytes.toBytes("f"), Bytes.toBytes("i"));
								index_deleteList.add(index_delete);

								byte[] index_row_bytes_new = new byte[index_bytes_new.length + 1 + row_bytes.length];
								System.arraycopy(index_bytes_new, 0, index_row_bytes_new, 0, index_bytes_new.length);
								System.arraycopy(row_bytes, 0, index_row_bytes_new, index_bytes_new.length + 1, row_bytes.length);
								Put put_index = new Put(index_row_bytes_new);
								put_index.addColumn(Bytes.toBytes("f"), Bytes.toBytes("i"), Bytes.toBytes("1"));
								index_putList.add(put_index);
							} else {
								UpdateResult(result, deleteList, putList, index_deleteList, index_putList);
							}

							count++;
						}

						{
							table.put(putList);
							table.flushCommits();
						}
						{
							table_index.delete(index_deleteList);
							table_index.flushCommits();
						}
						{
							table_index.put(index_putList);
							table_index.flushCommits();
						}
					}
				}
				System.out.println(AdminUtil.sdf.format(new Date()) + "\t" + name_index + name2_index + "\t" + count);
			}
		}

		System.out.println(AdminUtil.sdf.format(new Date()));
	}

	private static void UpdateResult(Result result, List<Delete> deleteList, List<Put> putList, List<Delete> index_deleteList,
			List<Put> index_putList) {
		byte[] row_bytes = result.getRow();
		String name_s = Bytes.toString(result.getValue(Bytes.toBytes("f"), Bytes.toBytes("name_s")));
		String name2_s = Bytes.toString(result.getValue(Bytes.toBytes("f"), Bytes.toBytes("name2_s")));
		int age_l = Bytes.toInt(result.getValue(Bytes.toBytes("f"), Bytes.toBytes("age_l")));
		int age2_l = Bytes.toInt(result.getValue(Bytes.toBytes("f"), Bytes.toBytes("age2_l")));
		byte[] name_bytes = Bytes.toBytes(name_s);
		byte[] name2_bytes = Bytes.toBytes(name2_s);
		byte[] age_bytes = Bytes.toBytes(age_l);
		byte[] age2_bytes = Bytes.toBytes(age2_l);
		byte[] index_bytes = new byte[age2_bytes.length + age_bytes.length + name2_bytes.length + 1 + name_bytes.length];
		System.arraycopy(age2_bytes, 0, index_bytes, 0, age2_bytes.length);
		System.arraycopy(age_bytes, 0, index_bytes, age2_bytes.length, age_bytes.length);
		System.arraycopy(name2_bytes, 0, index_bytes, age2_bytes.length + age_bytes.length, name2_bytes.length);
		System.arraycopy(name_bytes, 0, index_bytes, age2_bytes.length + age_bytes.length + name2_bytes.length + 1, name_bytes.length);

		int age_l_new = age_length + age_l;
		int age2_l_new = age2_length + age2_l;
		byte[] age_bytes_new = Bytes.toBytes(age_l_new);
		byte[] age2_bytes_new = Bytes.toBytes(age2_l_new);
		byte[] index_bytes_new = new byte[age2_bytes_new.length + age_bytes_new.length + name2_bytes.length + 1 + name_bytes.length];
		System.arraycopy(age2_bytes_new, 0, index_bytes_new, 0, age2_bytes_new.length);
		System.arraycopy(age_bytes_new, 0, index_bytes_new, age2_bytes_new.length, age_bytes_new.length);
		System.arraycopy(name2_bytes, 0, index_bytes_new, age2_bytes_new.length + age_bytes_new.length, name2_bytes.length);
		System.arraycopy(name_bytes, 0, index_bytes_new, age2_bytes_new.length + age_bytes_new.length + name2_bytes.length + 1, name_bytes.length);

		Delete delete = new Delete(row_bytes);
		delete.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"));
		delete.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"));
		deleteList.add(delete);

		Put put = new Put(row_bytes);
		put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age_l"), age_bytes_new);
		put.addColumn(Bytes.toBytes("f"), Bytes.toBytes("age2_l"), age2_bytes_new);
		putList.add(put);

		byte[] index_row_bytes = new byte[index_bytes.length + 1 + row_bytes.length];
		System.arraycopy(index_bytes, 0, index_row_bytes, 0, index_bytes.length);
		System.arraycopy(row_bytes, 0, index_row_bytes, index_bytes.length + 1, row_bytes.length);
		Delete index_delete = new Delete(index_row_bytes);
		index_delete.addColumn(Bytes.toBytes("f"), Bytes.toBytes("i"));
		index_deleteList.add(index_delete);

		byte[] index_row_bytes_new = new byte[index_bytes_new.length + 1 + row_bytes.length];
		System.arraycopy(index_bytes_new, 0, index_row_bytes_new, 0, index_bytes_new.length);
		System.arraycopy(row_bytes, 0, index_row_bytes_new, index_bytes_new.length + 1, row_bytes.length);
		Put put_index = new Put(index_row_bytes_new);
		put_index.addColumn(Bytes.toBytes("f"), Bytes.toBytes("i"), Bytes.toBytes("1"));
		index_putList.add(put_index);
	}
}
