package test.hbase;

import java.io.FileInputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Date;
import java.util.List;
import java.util.Random;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.NamespaceDescriptor;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.security.UserGroupInformation;
import org.zillion.util.common.FileUtil;
import org.zillion.util.format.FormatStaticUtil;
import org.zillion.util.log.LogUtil;

import com.zillion.util.common.ByteToObject;

public class HBaseTest_originaldata {
	static boolean UseKerberos = false;
	static String database_name = "db_public";
	static String table_name = "test_originaldata";

	static int split_count = 10;
	static int thread_per_split = 10;
	static int meter_count = 100;
	static int funcid_count = 100;
	static int receivetime_count = 100;

	static String[] split_array;

	static String[] building_array;
	static String[] meter_array;
	static Long[] funcid_array;
	static String[] receivetime_array;

	static {
		try {
			{
				building_array = new String[split_count * thread_per_split];
				split_array = new String[split_count - 1];
				long start = 1000000000;
				for (int i = 0; i < split_count * thread_per_split; i++) {
					if (i != 0 && i % thread_per_split == 0) {
						split_array[i / thread_per_split - 1] = FormatStaticUtil.toString(start);
					}
					building_array[i] = FormatStaticUtil.toString(start);
					start += 1;
				}
			}
			{
				meter_array = new String[meter_count];
				long start = 1000;
				for (int i = 0; i < meter_count; i++) {
					meter_array[i] = FormatStaticUtil.toString(start);
					start += 1;
				}
			}
			{
				funcid_array = new Long[funcid_count];
				long start = 0;
				for (int i = 0; i < funcid_count; i++) {
					funcid_array[i] = start;
					start += 1;
				}
			}
			{
				receivetime_array = new String[receivetime_count];
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				Date start = sdf.parse("2000-01-01 00:00:00");
				for (int i = 0; i < receivetime_count; i++) {
					receivetime_array[i] = sdf.format(start);
					start = new Date(start.getTime() + 1000L * 60 * 5);
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static int FinishCount = 0;

	public static synchronized void AddFinishCount(int count) {
		FinishCount += count;
	}

	public static synchronized int GetFinishCount() {
		return FinishCount;
	}

	static String get_new_principal(String principal) {
		String result = null;
		try {
			int index_ = principal.indexOf("_HOST");
			if (index_ != -1) {
				String prefix = principal.substring(0, index_);
				String suffix = principal.substring(index_ + "_HOST".length());
				if (prefix.charAt(prefix.length() - 1) == '/' && suffix.charAt(0) == '@') {
					result = prefix + InetAddress.getLocalHost().getHostName() + suffix;
				}
			}
		} catch (Exception e) {
			LogUtil.error(e.getMessage());
		}
		return result;
	}

	public static void main(String[] args) throws Exception {
		Configuration configuration = HBaseConfiguration.create();

		if (UseKerberos) {
			String zillion_kerberos_txt = configuration.get("zillion.kerberos.txt");
			String zillion_kerberos_keytab = configuration.get("zillion.kerberos.keytab");
			if (zillion_kerberos_txt != null && zillion_kerberos_keytab != null) {
				byte[] bytes = FileUtil.read_file_bytes(new FileInputStream(zillion_kerberos_txt));
				String base64String = new String(bytes, "UTF-8");
				String base64Header = "data:application/octet-stream;base64,";
				if (base64String.startsWith(base64Header)) {
					base64String = base64String.substring(base64Header.length());
				}
				byte[] keytab_bytes = Base64.getDecoder().decode(base64String);
				FileUtil.write_file(keytab_bytes, zillion_kerberos_keytab);
			}

			String principal = configuration.get("kerberos.principal");
			String file = configuration.get("keytab.file");
			if (principal != null && file != null) {
				String jaas_conf_path = configuration.get("zillion.jaas.conf");
				String krb5_conf_path = configuration.get("zillion.krb5.conf");
				System.setProperty("java.security.auth.login.config", jaas_conf_path);
				System.setProperty("java.security.krb5.conf", krb5_conf_path);
				String new_principal = get_new_principal(principal);
				if (new_principal != null) {
					principal = new_principal;
					configuration.set("kerberos.principal", principal);
				}

				UserGroupInformation.setConfiguration(configuration);
				try {
					UserGroupInformation.loginUserFromKeytab(principal, file);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

		Connection connection = ConnectionFactory.createConnection(configuration);
		Admin admin = connection.getAdmin();

		// 创建数据库
		boolean exist_namespace = false;
		NamespaceDescriptor[] namespaces = admin.listNamespaceDescriptors();
		for (NamespaceDescriptor namespace_exist : namespaces) {
			String namespace_name = namespace_exist.getName();
			if (namespace_name.equals(database_name)) {
				exist_namespace = true;
				break;
			}
		}
		if (!exist_namespace) {
			NamespaceDescriptor namespaceDesc = NamespaceDescriptor.create(database_name).build();
			admin.createNamespace(namespaceDesc);
		}

		// 删除表
		TableName tableName = TableName.valueOf(database_name, table_name);
		AdminUtil.deleteTable(admin, tableName);

		// 创建表
		byte[][] splitKeys = new byte[split_array.length][];
		for (int i = 0; i < split_array.length; i++) {
			String split = split_array[i];
			List<Object> keyValueList = new ArrayList<Object>();
			keyValueList.add(split);
			splitKeys[i] = BuildKey(keyValueList);
		}
		AdminUtil.createTable(admin, tableName, splitKeys);

		HTable table = (HTable) connection.getTable(tableName);
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Date beginTime = new Date();
		List<MyThread> threadList = new ArrayList<MyThread>();
		for (int i = 0; i < split_count * thread_per_split; i++) {
			String building = building_array[i];
			MyThread thread = new MyThread(connection, tableName, table, building, meter_array, funcid_array, receivetime_array);
			thread.start();
			threadList.add(thread);
		}

		Date lastTime = new Date();
		int lastCount = GetFinishCount();
		while (true) {
			Thread.sleep(1000L * 5);

			Date currTime = new Date();
			int currCount = GetFinishCount();
			System.out.printf(sdf.format(currTime) + "    " + "Second:%10d" + "    " + "FinishCount:%10d    Speed(num/sec):%10d",
					(currTime.getTime() - beginTime.getTime()) / 1000, currCount,
					(int) (1000.0 * (currCount - lastCount) / (currTime.getTime() - lastTime.getTime())));
			System.out.println();
			lastTime = currTime;
			lastCount = currCount;

			boolean all_finish = true;
			for (MyThread thread : threadList) {
				if (thread.isAlive()) {
					all_finish = false;
					break;
				}
			}
			if (all_finish) {
				break;
			}
		}
		Date endTime = new Date();
		int TotalCount = GetFinishCount();
		LogUtil.info("Second:" + (endTime.getTime() - beginTime.getTime()) / 1000 + "\t" + "TotalCount:" + TotalCount + "\t" + "Speed(num/sec):"
				+ (int) (1000.0 * TotalCount / (endTime.getTime() - beginTime.getTime())));

		AdminUtil.deleteTable(admin, tableName);

		admin.close();
		connection.close();
	}

	static byte[] BuildKey(List<Object> keyValueList) {
		byte[][] bytess = new byte[keyValueList.size()][];
		for (int i = 0; i < keyValueList.size(); i++) {
			Object value = keyValueList.get(i);
			bytess[i] = GenerateKeyBytes(value);
		}
		int bytes_count = 0;
		for (int i = 0; i < keyValueList.size(); i++) {
			bytes_count += bytess[i].length;
		}
		byte[] get_key = new byte[bytes_count];
		int bytes_index = 0;
		for (int i = 0; i < keyValueList.size(); i++) {
			System.arraycopy(bytess[i], 0, get_key, bytes_index, bytess[i].length);
			bytes_index += bytess[i].length;
		}
		return get_key;
	}

	static byte[] GenerateKeyBytes(Object value) {
		byte[] result = null;
		if (value == null) {
			result = new byte[1];
			result[0] = 0x00;
		} else {
			byte[] data_bytes = null;
			if (value instanceof String) {
				data_bytes = ByteToObject.toBytes((String) value);
			} else if (value instanceof Long) {
				data_bytes = ByteToObject.toBytes(((Long) value).longValue());
			} else if (value instanceof Double) {
				data_bytes = ByteToObject.toBytes(((Double) value).doubleValue());
			} else if (value instanceof Boolean) {
				data_bytes = ByteToObject.toBytes(((Boolean) value).booleanValue());
			}

			if (value instanceof String) {
				result = new byte[data_bytes.length + 2];
				result[0] = 0x01;
				System.arraycopy(data_bytes, 0, result, 1, data_bytes.length);
				result[result.length - 1] = 0x00;
			} else {
				result = new byte[data_bytes.length + 1];
				result[0] = 0x01;
				System.arraycopy(data_bytes, 0, result, 1, data_bytes.length);
			}
		}
		return result;
	}
}

class MyThread extends Thread {
	Connection connection;
	TableName tableName;
	HTable table;
	String building;
	String[] meter_array;
	Long[] funcid_array;
	String[] receivetime_array;

	public MyThread(Connection connection, TableName tableName, HTable table, String building, String[] meter_array, Long[] funcid_array,
			String[] receivetime_array) {
		this.connection = connection;
		this.tableName = tableName;
		this.table = table;
		this.building = building;
		this.meter_array = meter_array;
		this.funcid_array = funcid_array;
		this.receivetime_array = receivetime_array;
	}

	public void run() {
		try {
			// HTable tableInner = (HTable) connection.getTable(tableName);
			HTable tableInner = this.table;

			Random rand = new Random();
			for (int index2 = 0; index2 < meter_array.length; index2++) {
				String meter = meter_array[index2];
				for (int index3 = 0; index3 < funcid_array.length; index3++) {
					Long funcid = funcid_array[index3];
					List<Put> batch = new ArrayList<Put>();
					for (int index4 = 0; index4 < receivetime_array.length; index4++) {
						String receivetime = receivetime_array[index4];

						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						Put row = new Put(HBaseTest_originaldata.BuildKey(keyValueList));
						row.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("data"), ByteToObject.toBytes(rand.nextDouble()));
						batch.add(row);
					}
					tableInner.put(batch);
					HBaseTest_originaldata.AddFinishCount(batch.size());
				}
			}

			// tableInner.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}