package test.hbase;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.NamespaceDescriptor;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Put;
import org.zillion.util.format.FormatStaticUtil;

import com.zillion.util.common.ByteToObject;

public class HBaseTest_replication {
	static String database_name = "db_test";
	static String table_name = "table1";

	static int split_count = 1;
	static int building_per_split = 1;
	static int meter_count = 10;
	static int funcid_count = 10;
	static int receivetime_count = 10;

	static String[] split_array;

	static String[] building_array;
	static String[] meter_array;
	static Long[] funcid_array;
	static String[] receivetime_array;

	static {
		try {
			{
				building_array = new String[split_count * building_per_split];
				split_array = new String[split_count - 1];
				long start = 1000000000;
				for (int i = 0; i < split_count * building_per_split; i++) {
					if (i != 0 && i % building_per_split == 0) {
						split_array[i / building_per_split - 1] = FormatStaticUtil.toString(start);
					}
					building_array[i] = FormatStaticUtil.toString(start);
					start += 1;
				}
			}
			{
				meter_array = new String[meter_count];
				long start = 1000;
				for (int i = 0; i < meter_count; i++) {
					meter_array[i] = FormatStaticUtil.toString(start);
					start += 1;
				}
			}
			{
				funcid_array = new Long[funcid_count];
				long start = 0;
				for (int i = 0; i < funcid_count; i++) {
					funcid_array[i] = start;
					start += 1;
				}
			}
			{
				receivetime_array = new String[receivetime_count];
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				Date start = sdf.parse("2000-01-01 00:00:00");
				for (int i = 0; i < receivetime_count; i++) {
					receivetime_array[i] = sdf.format(start);
					start = new Date(start.getTime() + 1000L * 60 * 5);
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) throws Exception {
		Configuration configuration2 = new Configuration();
		configuration2.set("hbase.cluster.distributed", "true");
		configuration2.set("hbase.zookeeper.quorum", "zookeeper1.saga");
		Connection connection2 = ConnectionFactory.createConnection(configuration2);
		Admin admin2 = connection2.getAdmin();

		// Configuration configuration = HBaseConfiguration.create();
		Configuration configuration = new Configuration();
		configuration.set("hbase.cluster.distributed", "true");
		configuration.set("hbase.zookeeper.quorum", "zookeeper1");
		Connection connection = ConnectionFactory.createConnection(configuration);
		Admin admin = connection.getAdmin();

		// ݿ
		boolean exist_namespace = false;
		NamespaceDescriptor[] namespaces = admin.listNamespaceDescriptors();
		for (NamespaceDescriptor namespace_exist : namespaces) {
			String namespace_name = namespace_exist.getName();
			if (namespace_name.equals(database_name)) {
				exist_namespace = true;
				break;
			}
		}
		if (!exist_namespace) {
			NamespaceDescriptor namespaceDesc = NamespaceDescriptor.create(database_name).build();
			admin2.createNamespace(namespaceDesc);
			admin.createNamespace(namespaceDesc);
		}

		// ɾ
		TableName tableName = TableName.valueOf(database_name, table_name);
		AdminUtil.deleteTable(admin2, tableName);
		AdminUtil.deleteTable(admin, tableName);

		// 
		byte[][] splitKeys = new byte[split_array.length][];
		for (int i = 0; i < split_array.length; i++) {
			String split = split_array[i];
			List<Object> keyValueList = new ArrayList<Object>();
			keyValueList.add(split);
			splitKeys[i] = BuildKey(keyValueList);
		}
		AdminUtil.createTable(admin2, tableName, splitKeys);
		AdminUtil.createTable_rep(admin, tableName, splitKeys);

		HTable table = (HTable) connection.getTable(tableName);
		Random rand = new Random();
		for (int index1 = 0; index1 < building_array.length; index1++) {
			String building = building_array[index1];
			for (int index2 = 0; index2 < meter_array.length; index2++) {
				String meter = meter_array[index2];
				for (int index3 = 0; index3 < funcid_array.length; index3++) {
					Long funcid = funcid_array[index3];
					List<Put> batch = new ArrayList<Put>();
					for (int index4 = 0; index4 < receivetime_array.length; index4++) {
						String receivetime = receivetime_array[index4];

						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						Put row = new Put(HBaseTest_replication.BuildKey(keyValueList));
						row.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("data"), ByteToObject.toBytes(rand.nextDouble()));
						batch.add(row);
					}
					table.put(batch);
				}
			}
		}
		table.close();

		admin2.close();
		connection2.close();
		admin.close();
		connection.close();
	}

	static byte[] BuildKey(List<Object> keyValueList) {
		byte[][] bytess = new byte[keyValueList.size()][];
		for (int i = 0; i < keyValueList.size(); i++) {
			Object value = keyValueList.get(i);
			bytess[i] = GenerateKeyBytes(value);
		}
		int bytes_count = 0;
		for (int i = 0; i < keyValueList.size(); i++) {
			bytes_count += bytess[i].length;
		}
		byte[] get_key = new byte[bytes_count];
		int bytes_index = 0;
		for (int i = 0; i < keyValueList.size(); i++) {
			System.arraycopy(bytess[i], 0, get_key, bytes_index, bytess[i].length);
			bytes_index += bytess[i].length;
		}
		return get_key;
	}

	static byte[] GenerateKeyBytes(Object value) {
		byte[] result = null;
		if (value == null) {
			result = new byte[1];
			result[0] = 0x00;
		} else {
			byte[] data_bytes = null;
			if (value instanceof String) {
				data_bytes = ByteToObject.toBytes((String) value);
			} else if (value instanceof Long) {
				data_bytes = ByteToObject.toBytes(((Long) value).longValue());
			} else if (value instanceof Double) {
				data_bytes = ByteToObject.toBytes(((Double) value).doubleValue());
			} else if (value instanceof Boolean) {
				data_bytes = ByteToObject.toBytes(((Boolean) value).booleanValue());
			}

			if (value instanceof String) {
				result = new byte[data_bytes.length + 2];
				result[0] = 0x01;
				System.arraycopy(data_bytes, 0, result, 1, data_bytes.length);
				result[result.length - 1] = 0x00;
			} else {
				result = new byte[data_bytes.length + 1];
				result[0] = 0x01;
				System.arraycopy(data_bytes, 0, result, 1, data_bytes.length);
			}
		}
		return result;
	}
}
