package test.hbase;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.apache.hadoop.hbase.client.Delete;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter.RowRange;
import org.zillion.util.format.FormatStaticUtil;
import org.zillion.util.json.FastJsonReaderUtil;
import org.zillion.util.log.LogUtil;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.SchemaParser;
import com.zillion.util.table.ZillionTableSchema;

public class SIM_AgentTest_originaldata {
	static String[] building_array = new String[10];
	static String[] meter_array = new String[10];
	static Long[] funcid_array = new Long[10];
	static String[] receivetime_array = new String[10000];

	static String table_name = "sim_originaldata";
	static ZillionTableSchema schema;

	static {
		try {
			{
				long start = 1101070037;
				for (int i = 0; i < building_array.length; i++) {
					building_array[i] = FormatStaticUtil.toString(start);
					start += 500000000;
				}
			}
			{
				long start = 1000;
				for (int i = 0; i < meter_array.length; i++) {
					meter_array[i] = FormatStaticUtil.toString(start);
					start += 1;
				}
			}
			{
				long start = 0;
				for (int i = 0; i < funcid_array.length; i++) {
					funcid_array[i] = start;
					start += 1;
				}
			}
			{
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				Date start = sdf.parse("2000-01-01 00:00:00");
				for (int i = 0; i < receivetime_array.length; i++) {
					receivetime_array[i] = sdf.format(start);
					start = new Date(start.getTime() + 1000L * 60 * 5);
				}
			}

			JSONObject def = FastJsonReaderUtil.Instance().ReadJSONObject(new File("etc/zz_originaldata/def.json"));
			schema = new ZillionTableSchema("", table_name);
			schema.JSON = def;
			SchemaParser.ParseTableSchema(schema, def);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) throws Exception {
		Configuration configuration = HBaseConfiguration.create();
		Connection connection = ConnectionFactory.createConnection(configuration);
		Admin admin = connection.getAdmin();

		TableName tableName = TableName.valueOf(table_name);
		for (int i = 0; i < 1; i++) {
			AdminUtil.deleteTable(admin, tableName);
			AdminUtil.createTable(admin, tableName, null);

			HTable table = (HTable) connection.getTable(tableName);

			insert(table);
			select(table);
			// deleteByKeyAfterScan(table);
			// deleteByKeyAfterScan2(table);
			deleteByKeyAfterScanUsePutV(table);
			// deleteByKey(table);
			// deleteByKeyUsePut(table);
			select(table);

			table.close();
		}

		admin.close();
		connection.close();
	}

	static void insert(HTable table) throws Exception {
		Random rand = new Random();
		LogUtil.info("insert begin:" + building_array.length);
		for (int index1 = 0; index1 < building_array.length; index1++) {
			String building = building_array[index1];
			for (int index2 = 0; index2 < meter_array.length; index2++) {
				String meter = meter_array[index2];
				for (int index3 = 0; index3 < funcid_array.length; index3++) {
					Long funcid = funcid_array[index3];
					List<Put> batch = new ArrayList<Put>();
					for (int index4 = 0; index4 < receivetime_array.length; index4++) {
						String receivetime = receivetime_array[index4];

						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						Put row = new Put(BuildKey(keyValueList));
						if (schema.delete_UsePutV) {
							row.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"), ByteToObject.toBytes("1"));
						}
						row.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("data"), ByteToObject.toBytes(rand.nextDouble()));
						batch.add(row);
					}
					table.put(batch);
				}
			}
			LogUtil.info("insert finish:" + (index1 + 1) + "/" + building_array.length);
		}
	}

	static void select(HTable table) throws Exception {
		LogUtil.info("select begin:" + building_array.length);
		for (int index1 = 0; index1 < building_array.length; index1++) {
			String building = building_array[index1];
			int count = 0;
			for (int index2 = 0; index2 < meter_array.length; index2++) {
				String meter = meter_array[index2];
				for (int index3 = 0; index3 < funcid_array.length; index3++) {
					Long funcid = funcid_array[index3];

					byte[] start_key;
					byte[] end_key;
					{
						String receivetime = receivetime_array[0];
						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						start_key = BuildKey(keyValueList);
					}
					{
						String receivetime = receivetime_array[receivetime_array.length - 1];
						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						end_key = BuildKey(keyValueList);
					}

					JSONArray resultContent = new JSONArray();

					Scan scan = new Scan();
					if (schema.delete_UsePutV) {
						scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
					}
					scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("data"));
					List<RowRange> rangeList = new ArrayList<RowRange>();
					rangeList.add(new RowRange(start_key, true, end_key, true));
					MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
					scan.setFilter(filter);
					ResultScanner scanner = table.getScanner(scan);
					while (true) {
						Result res = scanner.next();
						if (res == null) {
							break;
						}

						if (HTableUtil.valid(schema, res.listCells())) {
							JSONObject resultObject = HTableUtil.Convert(schema, res.getRow(), res.listCells());
							resultContent.add(resultObject);
						}
					}
					scanner.close();

					count += resultContent.size();
				}
			}
			LogUtil.info("select finish:" + (index1 + 1) + "/" + building_array.length + "\t" + count);
		}
	}

	static void deleteByKey(HTable table) throws Exception {
		LogUtil.info("delete begin:" + building_array.length);
		for (int index1 = 0; index1 < building_array.length; index1++) {
			String building = building_array[index1];
			for (int index2 = 0; index2 < meter_array.length; index2++) {
				String meter = meter_array[index2];
				for (int index3 = 0; index3 < funcid_array.length; index3++) {
					Long funcid = funcid_array[index3];
					List<Delete> batch = new ArrayList<Delete>();
					for (int index4 = 0; index4 < receivetime_array.length; index4++) {
						String receivetime = receivetime_array[index4];

						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						Delete row = new Delete(BuildKey(keyValueList));
						row.addFamily(ByteToObject.toBytes("f"));
						batch.add(row);
					}
					table.delete(batch);
				}
			}
			LogUtil.info("delete finish:" + (index1 + 1) + "/" + building_array.length);
		}
	}

	static void deleteByKeyUsePutV(HTable table) throws Exception {
		LogUtil.info("delete begin:" + building_array.length);
		for (int index1 = 0; index1 < building_array.length; index1++) {
			String building = building_array[index1];
			for (int index2 = 0; index2 < meter_array.length; index2++) {
				String meter = meter_array[index2];
				for (int index3 = 0; index3 < funcid_array.length; index3++) {
					Long funcid = funcid_array[index3];
					List<Put> batch = new ArrayList<Put>();
					for (int index4 = 0; index4 < receivetime_array.length; index4++) {
						String receivetime = receivetime_array[index4];

						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						Put row = new Put(BuildKey(keyValueList));
						if (schema.delete_UsePutV) {
							row.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"), ByteToObject.toBytes("0"));
						}
						row.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("data"), null);
						batch.add(row);
					}
					table.put(batch);
				}
			}
			LogUtil.info("delete finish:" + (index1 + 1) + "/" + building_array.length);
		}
	}

	static void deleteByKeyAfterScan(HTable table) throws Exception {
		LogUtil.info("delete begin:" + building_array.length);
		for (int index1 = 0; index1 < building_array.length; index1++) {
			String building = building_array[index1];
			for (int index2 = 0; index2 < meter_array.length; index2++) {
				String meter = meter_array[index2];
				for (int index3 = 0; index3 < funcid_array.length; index3++) {
					Long funcid = funcid_array[index3];

					byte[] start_key;
					byte[] end_key;
					{
						String receivetime = receivetime_array[0];
						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						start_key = BuildKey(keyValueList);
					}
					{
						String receivetime = receivetime_array[receivetime_array.length - 1];
						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						end_key = BuildKey(keyValueList);
					}

					JSONArray resultContent = new JSONArray();

					Scan scan = new Scan();
					if (schema.delete_UsePutV) {
						scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
					}
					scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("data"));
					List<RowRange> rangeList = new ArrayList<RowRange>();
					rangeList.add(new RowRange(start_key, true, end_key, true));
					MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
					scan.setFilter(filter);
					ResultScanner scanner = table.getScanner(scan);
					while (true) {
						Result res = scanner.next();
						if (res == null) {
							break;
						}

						if (HTableUtil.valid(schema, res.listCells())) {
							JSONObject resultObject = HTableUtil.Convert(schema, res.getRow(), res.listCells());
							resultContent.add(resultObject);
						}
					}
					scanner.close();

					List<Delete> batch = new ArrayList<Delete>();
					for (int i = 0; i < resultContent.size(); i++) {
						JSONObject resultObject = (JSONObject) resultContent.get(i);
						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(resultObject.get("building"));
						keyValueList.add(resultObject.get("meter"));
						keyValueList.add(resultObject.get("funcid"));
						keyValueList.add(resultObject.get("receivetime"));
						Delete delete = new Delete(BuildKey(keyValueList));
						delete.addFamily(ByteToObject.toBytes("f"));
						batch.add(delete);
					}
					table.delete(batch);
				}
			}
			LogUtil.info("delete finish:" + (index1 + 1) + "/" + building_array.length);
		}
	}

	static void deleteByKeyAfterScan2(HTable table) throws Exception {
		LogUtil.info("deleteByKeyAfterScan2 begin:" + building_array.length);
		List<JSONArray> resultContentList = new ArrayList<JSONArray>();
		for (int index1 = 0; index1 < building_array.length; index1++) {
			String building = building_array[index1];
			for (int index2 = 0; index2 < meter_array.length; index2++) {
				String meter = meter_array[index2];
				for (int index3 = 0; index3 < funcid_array.length; index3++) {
					Long funcid = funcid_array[index3];

					byte[] start_key;
					byte[] end_key;
					{
						String receivetime = receivetime_array[0];
						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						start_key = BuildKey(keyValueList);
					}
					{
						String receivetime = receivetime_array[receivetime_array.length - 1];
						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						end_key = BuildKey(keyValueList);
					}

					JSONArray resultContent = new JSONArray();

					Scan scan = new Scan();
					if (schema.delete_UsePutV) {
						scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
					}
					scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("data"));
					List<RowRange> rangeList = new ArrayList<RowRange>();
					rangeList.add(new RowRange(start_key, true, end_key, true));
					MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
					scan.setFilter(filter);
					ResultScanner scanner = table.getScanner(scan);
					while (true) {
						Result res = scanner.next();
						if (res == null) {
							break;
						}

						if (HTableUtil.valid(schema, res.listCells())) {
							JSONObject resultObject = HTableUtil.Convert(schema, res.getRow(), res.listCells());
							resultContent.add(resultObject);
						}
					}
					scanner.close();

					resultContentList.add(resultContent);
				}
			}
			LogUtil.info("scan finish:" + (index1 + 1) + "/" + building_array.length);
		}

		for (int index1 = 0; index1 < building_array.length; index1++) {
			int start_index = index1 * meter_array.length * funcid_array.length;
			int end_index = (index1 + 1) * meter_array.length * funcid_array.length;
			for (int index_tmp = start_index; index_tmp < end_index; index_tmp++) {
				JSONArray resultContent = resultContentList.get(index_tmp);
				List<Delete> batch = new ArrayList<Delete>();
				for (int i = 0; i < resultContent.size(); i++) {
					JSONObject resultObject = (JSONObject) resultContent.get(i);
					List<Object> keyValueList = new ArrayList<Object>();
					keyValueList.add(resultObject.get("building"));
					keyValueList.add(resultObject.get("meter"));
					keyValueList.add(resultObject.get("funcid"));
					keyValueList.add(resultObject.get("receivetime"));
					Delete delete = new Delete(BuildKey(keyValueList));
					delete.addFamily(ByteToObject.toBytes("f"));
					batch.add(delete);
				}
				table.delete(batch);
			}
			LogUtil.info("delete finish:" + (index1 + 1) + "/" + building_array.length);
		}
	}

	static void deleteByKeyAfterScanUsePutV(HTable table) throws Exception {
		LogUtil.info("delete begin:" + building_array.length);
		for (int index1 = 0; index1 < building_array.length; index1++) {
			String building = building_array[index1];
			for (int index2 = 0; index2 < meter_array.length; index2++) {
				String meter = meter_array[index2];
				for (int index3 = 0; index3 < funcid_array.length; index3++) {
					Long funcid = funcid_array[index3];

					byte[] start_key;
					byte[] end_key;
					{
						String receivetime = receivetime_array[0];
						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						start_key = BuildKey(keyValueList);
					}
					{
						String receivetime = receivetime_array[receivetime_array.length - 1];
						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(building);
						keyValueList.add(meter);
						keyValueList.add(funcid);
						keyValueList.add(receivetime);
						end_key = BuildKey(keyValueList);
					}

					JSONArray resultContent = new JSONArray();

					Scan scan = new Scan();
					if (schema.delete_UsePutV) {
						scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
					}
					scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("data"));
					List<RowRange> rangeList = new ArrayList<RowRange>();
					rangeList.add(new RowRange(start_key, true, end_key, true));
					MultiRowRangeFilter filter = new MultiRowRangeFilter(rangeList);
					scan.setFilter(filter);
					ResultScanner scanner = table.getScanner(scan);
					while (true) {
						Result res = scanner.next();
						if (res == null) {
							break;
						}

						if (HTableUtil.valid(schema, res.listCells())) {
							JSONObject resultObject = HTableUtil.Convert(schema, res.getRow(), res.listCells());
							resultContent.add(resultObject);
						}
					}
					scanner.close();

					List<Put> batch = new ArrayList<Put>();
					for (int i = 0; i < resultContent.size(); i++) {
						JSONObject resultObject = (JSONObject) resultContent.get(i);
						List<Object> keyValueList = new ArrayList<Object>();
						keyValueList.add(resultObject.get("building"));
						keyValueList.add(resultObject.get("meter"));
						keyValueList.add(resultObject.get("funcid"));
						keyValueList.add(resultObject.get("receivetime"));
						Put row = new Put(BuildKey(keyValueList));
						if (schema.delete_UsePutV) {
							row.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"), ByteToObject.toBytes("0"));
						}
						row.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("data"), null);
						batch.add(row);
					}
					table.put(batch);
				}
			}
			LogUtil.info("delete finish:" + (index1 + 1) + "/" + building_array.length);
		}
	}

	public static byte[] BuildKey(List<Object> keyValueList) {
		byte[][] bytess = new byte[keyValueList.size()][];
		for (int i = 0; i < keyValueList.size(); i++) {
			Object value = keyValueList.get(i);
			bytess[i] = GenerateKeyBytes(value);
		}
		int bytes_count = 0;
		for (int i = 0; i < keyValueList.size(); i++) {
			bytes_count += bytess[i].length;
		}
		byte[] get_key = new byte[bytes_count];
		int bytes_index = 0;
		for (int i = 0; i < keyValueList.size(); i++) {
			System.arraycopy(bytess[i], 0, get_key, bytes_index, bytess[i].length);
			bytes_index += bytess[i].length;
		}
		return get_key;
	}

	public static byte[] GenerateKeyBytes(Object value) {
		byte[] result = null;
		if (value == null) {
			result = new byte[1];
			result[0] = 0x00;
		} else {
			byte[] data_bytes = null;
			if (value instanceof String) {
				data_bytes = ByteToObject.toBytes((String) value);
			} else if (value instanceof Long) {
				data_bytes = ByteToObject.toBytes(((Long) value).longValue());
			} else if (value instanceof Double) {
				data_bytes = ByteToObject.toBytes(((Double) value).doubleValue());
			} else if (value instanceof Boolean) {
				data_bytes = ByteToObject.toBytes(((Boolean) value).booleanValue());
			}

			if (value instanceof String) {
				result = new byte[data_bytes.length + 2];
				result[0] = 0x01;
				System.arraycopy(data_bytes, 0, result, 1, data_bytes.length);
				result[result.length - 1] = 0x00;
			} else {
				result = new byte[data_bytes.length + 1];
				result[0] = 0x01;
				System.arraycopy(data_bytes, 0, result, 1, data_bytes.length);
			}
		}
		return result;
	}
}
