package test.hdfs;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IOUtils;
import org.apache.hadoop.security.UserGroupInformation;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.zillion.util.log.LogUtil;

/**
 * HDFS
 * 
 * @author zhi
 * @since 201991018:28:24
 *
 */
public class HadoopTest {
	private Logger logger = LogManager.getLogger(this.getClass());
	private FileSystem fileSystem = null;

	public static void main(String[] args) throws Exception {
		HadoopTest instance = new HadoopTest();

		instance.before();
		instance.listFiles();
		instance.mkdir();
		instance.uploadFile();
		instance.downFile();
		instance.after();
	}

	@Before
	public void before() throws Exception {
		Configuration configuration = new Configuration();
		configuration.addResource(new FileInputStream("/core-site.xml"));
		configuration.addResource(new FileInputStream("/hdfs-site.xml"));

		// kerberos֤
		System.setProperty("java.security.krb5.conf", "C:/ProgramData/MIT/Kerberos5/krb5.ini");
		UserGroupInformation.setConfiguration(configuration);
		UserGroupInformation.loginUserFromKeytab("hadoop/menglu-PC@HADOOP.COM", "D:/Test/kerberos/kerberos.keytab");

		fileSystem = FileSystem.get(new URI("hdfs://node1:8020"), configuration);
	}

	@After
	public void after() throws Exception {
		if (fileSystem != null) {
			fileSystem.close();
		}
	}

	/**
	 * ļ
	 */
	@Test
	public void mkdir() {
		try {
			boolean result = fileSystem.mkdirs(new Path("/test"));
			logger.info("ļн" + result);
		} catch (IllegalArgumentException | IOException e) {
			logger.error("ļг", e);
		}
	}

	/**
	 * ϴļ
	 */
	@Test
	public void uploadFile() {
		String fileName = "hadoop.txt";
		InputStream input = null;
		OutputStream output = null;
		try {
			input = new FileInputStream("D:/Test/hdfs/" + fileName);
			output = fileSystem.create(new Path("/test/" + fileName));
			IOUtils.copyBytes(input, output, 4096, true);
			logger.error("ϴļɹ");
		} catch (IllegalArgumentException | IOException e) {
			logger.error("ϴļ", e);
		} finally {
			if (input != null) {
				try {
					input.close();
				} catch (IOException e) {
				}
			}
			if (output != null) {
				try {
					output.close();
				} catch (IOException e) {
				}
			}
		}
	}

	/**
	 * ļ
	 */
	@Test
	public void downFile() {
		String fileName = "hadoop.txt";
		InputStream input = null;
		OutputStream output = null;
		try {
			input = fileSystem.open(new Path("/test/" + fileName));
			output = new FileOutputStream("D:/Test/hdfs/download/" + fileName);
			IOUtils.copyBytes(input, output, 4096, true);
			logger.error("ļɹ");
		} catch (IllegalArgumentException | IOException e) {
			logger.error("ļ", e);
		} finally {
			if (input != null) {
				try {
					input.close();
				} catch (IOException e) {
				}
			}
			if (output != null) {
				try {
					output.close();
				} catch (IOException e) {
				}
			}
		}
	}

	/**
	 * ɾļ
	 */
	@Test
	public void deleteFile() {
		String fileName = "hadoop.txt";
		try {
			boolean result = fileSystem.delete(new Path("/test/" + fileName), true);
			logger.info("ɾļ" + result);
		} catch (IllegalArgumentException | IOException e) {
			logger.error("ɾļ", e);
		}
	}

	/**
	 * ļ
	 */
	@Test
	public void listFiles() {
		try {
			FileStatus[] statuses = fileSystem.listStatus(new Path("/"));
			for (FileStatus file : statuses) {
				logger.info("ɨ赽ļĿ¼ƣ" + file.getPath().getName() + "ǷΪļ" + file.isFile());
			}
		} catch (IllegalArgumentException | IOException e) {
			logger.error("ļ", e);
		}
	}
}