package test.zookeeper;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.zookeeper.WatchedEvent;
import org.apache.zookeeper.Watcher;
import org.apache.zookeeper.Watcher.Event.EventType;
import org.apache.zookeeper.Watcher.Event.KeeperState;
import org.apache.zookeeper.ZooKeeper;
import org.apache.zookeeper.data.Stat;
import org.apache.zookeeper.proto.WatcherEvent;

public class NodeWatcher implements Watcher {
	ZooKeeper zk;
	Map<String, List<String>> childrenMap_old = new HashMap<String, List<String>>();
	Map<String, List<String>> childrenMap_new = new HashMap<String, List<String>>();

	public NodeWatcher(ZooKeeper zk) {
		this.zk = zk;
	}

	public void process(WatchedEvent event) {
		String path = event.getPath();
		EventType type = event.getType();
		KeeperState state = event.getState();
		WatcherEvent wrapper = event.getWrapper();
		// System.out.println(path + "\t" + state + "\t" + type + "\t" +
		// wrapper);

		if (type == EventType.NodeChildrenChanged) {
			try {
				this.recursive(zk, this, path);
				List<String> children_old = this.childrenMap_old.get(path);
				List<String> children_new = this.childrenMap_new.get(path);

				List<String> children_add = new ArrayList<String>();
				for (String child_new : children_new) {
					boolean exist = false;
					if (children_old != null) {
						for (String child_old : children_old) {
							if (child_old.equals(child_new)) {
								exist = true;
								break;
							}
						}
					}
					if (!exist) {
						children_add.add(child_new);
					}
				}
				Collections.sort(children_add);
				StringBuffer str_add = new StringBuffer();
				str_add.append("add:");
				for (String child : children_add) {
					str_add.append(" " + child);
				}

				List<String> children_delete = new ArrayList<String>();
				if (children_old != null) {
					for (String child_old : children_old) {
						boolean exist = false;
						for (String child_new : children_new) {
							if (child_new.equals(child_old)) {
								exist = true;
								break;
							}
						}
						if (!exist) {
							children_delete.add(child_old);
						}
					}
				}
				Collections.sort(children_delete);
				StringBuffer str_delete = new StringBuffer();
				str_delete.append("delete:");
				for (String child : children_delete) {
					str_delete.append(" " + child);
				}

				System.out.println("EventType.NodeChildrenChanged:" + path + "\t" + str_add.toString() + "\t" + str_delete.toString());
			} catch (Exception e) {
				e.printStackTrace();
			}
		} else if (type == EventType.NodeCreated) {
			try {
				this.recursive(zk, this, path);

				Stat stat = new Stat();
				byte[] data = zk.getData(path, this, stat);
				String dataString = new String(data, "UTF-8");
				System.out.println("EventType.NodeCreated:" + path + "\t" + dataString + "\t" + stat.getVersion());
			} catch (Exception e) {
				e.printStackTrace();
			}
		} else if (type == EventType.NodeDataChanged) {
			try {
				Stat stat = new Stat();
				byte[] data = zk.getData(path, this, stat);
				String dataString = new String(data, "UTF-8");
				System.out.println("EventType.NodeDataChanged:" + path + "\t" + dataString + "\t" + stat.getVersion());
			} catch (Exception e) {
				e.printStackTrace();
			}
		} else if (type == EventType.NodeDeleted) {
			try {
				Stat stat = zk.exists(path, this);

				System.out.println("EventType.NodeDeleted:" + path);
			} catch (Exception e) {
				e.printStackTrace();
			}
		} else if (type == EventType.None) {
			System.out.println("EventType.None:" + path);
		}

		try {
			this.recursive(zk, this, path);
		} catch (Exception e1) {
			e1.printStackTrace();
		}
	}

	public void recursive(ZooKeeper zk, Watcher watcher, String path) throws Exception {
		Stat stat = zk.exists(path, this);
		if (stat != null) {
			zk.getData(path, this, null);
			List<String> children = zk.getChildren(path, watcher);
			if (this.childrenMap_new.containsKey(path)) {
				List<String> children_old = this.childrenMap_new.get(path);
				this.childrenMap_old.remove(path);
				this.childrenMap_old.put(path, children_old);
				this.childrenMap_new.remove(path);
			}
			this.childrenMap_new.put(path, children);
			for (String child : children) {
				String parentPath = path.length() == 1 ? "/" : path + "/";
				String childPath = parentPath + child;

				recursive(zk, watcher, childPath);
			}
		}
	}

	public void recursiveChildren(ZooKeeper zk, Watcher watcher, String path) throws Exception {
		zk.getData(path, null, null);
		List<String> children = zk.getChildren(path, null);
		for (String child : children) {
			String parentPath = path.length() == 1 ? "/" : path + "/";
			String childPath = parentPath + child;

			recursive(zk, watcher, childPath);
		}
	}
}
