package useful;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.hadoop.hbase.ClusterStatus;
import org.apache.hadoop.hbase.HRegionInfo;
import org.apache.hadoop.hbase.RegionLoad;
import org.apache.hadoop.hbase.ServerLoad;
import org.apache.hadoop.hbase.ServerName;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.protobuf.generated.AdminProtos.GetRegionInfoResponse.CompactionState;
import org.zillion.util.common.HexToByte;
import org.zillion.util.log.LogUtil;

public class HBaseAdminHelper {

	public static void Table_majorCompact(Admin admin,TableName TableName) throws Exception {
		String namespace = TableName.getNamespaceAsString();
		String tableName_tmp = TableName.getNameAsString();

		LogUtil.warn(namespace + "\t" + tableName_tmp + "\t" + "majorCompact" + "\t" + "wait");
		while (true) {
			Thread.sleep(1000L);
			CompactionState CompactionState1 = admin.getCompactionState(TableName);
			if (CompactionState1.equals(CompactionState.NONE)) {
				break;
			}
		}
		LogUtil.warn(namespace + "\t" + tableName_tmp + "\t" + "majorCompact" + "\t" + "begin");
		admin.majorCompact(TableName);
		while (true) {
			Thread.sleep(1000L);
			CompactionState CompactionState1 = admin.getCompactionState(TableName);
			if (CompactionState1.equals(CompactionState.NONE)) {
				break;
			}
		}
		LogUtil.warn(namespace + "\t" + tableName_tmp + "\t" + "majorCompact" + "\t" + "end");
	}
	
	public static void Table_merge_by_sizeMB(Admin admin,TableName TableName, int max_sizeMB) throws Exception {
		while (true) {
			Map<String, RegionLoad> regionloadMap = new HashMap<String, RegionLoad>();
			ClusterStatus clusterStatus = admin.getClusterStatus();
			Collection<ServerName> servers = clusterStatus.getServers();
			for (ServerName serverName : servers) {
				ServerLoad serverLoad = clusterStatus.getLoad(serverName);
				Map<byte[], RegionLoad> regionLoadMap = serverLoad.getRegionsLoad();
				Iterator<byte[]> iter = regionLoadMap.keySet().iterator();
				while (iter.hasNext()) {
					byte[] key = iter.next();
					byte[] start_key = get_start_key(key);
					RegionLoad regionload = regionLoadMap.get(key);
					if (regionload.getNameAsString().startsWith(TableName.getNameAsString() + ",")) {
						regionloadMap.put(HexToByte.byteToHex(start_key), regionload);
					}
				}
			}
			List<HRegionInfo> regionInfoList = admin.getTableRegions(TableName);
			for (int i = regionInfoList.size() - 1; i >= 0; i--) {
				HRegionInfo regionInfo_1 = regionInfoList.get(i);
				if (regionInfo_1.isOffline()) {
					regionInfoList.remove(i);
				}
			}
			int MB_min = Integer.MAX_VALUE;
			int merge_index = -1;
			for (int i = 0; i < regionInfoList.size() - 1; i++) {
				HRegionInfo regionInfo_1 = regionInfoList.get(i);
				HRegionInfo regionInfo_2 = regionInfoList.get(i + 1);
				RegionLoad regionload_1 = regionloadMap.get(HexToByte.byteToHex(get_start_key(regionInfo_1.getRegionName())));
				RegionLoad regionload_2 = regionloadMap.get(HexToByte.byteToHex(get_start_key(regionInfo_2.getRegionName())));
				int MB = regionload_1.getStorefileSizeMB() + regionload_2.getStorefileSizeMB();
				if (MB < MB_min) {
					merge_index = i;
					MB_min = MB;
					break;
				}
			}
			if (MB_min < max_sizeMB) {
				HRegionInfo regionInfo_1 = regionInfoList.get(merge_index);
				HRegionInfo regionInfo_2 = regionInfoList.get(merge_index + 1);
				RegionLoad regionload_1 = regionloadMap.get(HexToByte.byteToHex(get_start_key(regionInfo_1.getRegionName())));
				RegionLoad regionload_2 = regionloadMap.get(HexToByte.byteToHex(get_start_key(regionInfo_2.getRegionName())));
				LogUtil.warn("merge:");
				LogUtil.warn(regionload_1.getNameAsString() + "\t\t" + regionload_1.getDataLocality() + "\t\t" + regionload_1.getStorefileSizeMB());
				LogUtil.warn(regionload_2.getNameAsString() + "\t\t" + regionload_2.getDataLocality() + "\t\t" + regionload_2.getStorefileSizeMB());
				mergeRegions(admin,TableName, regionload_1.getName(), regionload_2.getName());
			} else {
				break;
			}
		}
	}

	static void mergeRegions(Admin admin,TableName TableName, byte[] name_1, byte[] name_2) throws Exception {
		while (true) {
			Thread.sleep(1000L);
			try {
				CompactionState CompactionState1 = admin.getCompactionState(TableName);
				if (CompactionState1.equals(CompactionState.NONE)) {
					break;
				}
			} catch (Exception e) {
				// e.printStackTrace();
			}
		}
		admin.mergeRegions(name_1, name_2, false);
		while (true) {
			Thread.sleep(1000L);
			try {
				CompactionState CompactionState1 = admin.getCompactionState(TableName);
				if (CompactionState1.equals(CompactionState.NONE)) {
					break;
				}
			} catch (Exception e) {
				// e.printStackTrace();
			}
		}
		Thread.sleep(1000L * 15);
	}
	static byte[] get_start_key(byte[] key) {
		byte douhao = ',';
		int index_first = -1;
		for (int i = 0; i < key.length; i++) {
			if (key[i] == douhao) {
				index_first = i;
				break;
			}
		}
		int index_final = -1;
		for (int i = key.length - 1; i >= 0; i--) {
			if (key[i] == douhao) {
				index_final = i;
				break;
			}
		}
		byte[] start_key = new byte[index_final - (index_first + 1)];
		System.arraycopy(key, (index_first + 1), start_key, 0, start_key.length);
		return start_key;
	}
}
