package useful;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.zillion.util.format.FormatStaticUtil;
import org.zillion.util.json.JSONReaderUtil;
import org.zillion.util.log.LogUtil;

import com.sagacloud.json.JSONArray;
import com.sagacloud.json.JSONObject;
import com.zillion.database.agent.ZillionAgent;
import com.zillion.util.table.ZillionTableColumn;
import com.zillion.util.table.ZillionTableColumnEnum;
import com.zillion.util.table.ZillionTableSchema;

public class Test_combine_select {
	static String database_name = "database_1";
	static String table_name = "table_1";
	static String def_path = "etc/zz_originaldata/def.json";

	static String[] building_array = new String[10];
	static String[] meter_array = new String[10];
	static Long[] funcid_array = new Long[10];
	static String[] receivetime_array = new String[100];
	static {
		try {
			{
				long start = 1101070037;
				for (int i = 0; i < building_array.length; i++) {
					building_array[i] = FormatStaticUtil.toString(start);
					start += 500000000;
				}
			}
			{
				long start = 1000;
				for (int i = 0; i < meter_array.length; i++) {
					meter_array[i] = FormatStaticUtil.toString(start);
					start += 1;
				}
			}
			{
				long start = 0;
				for (int i = 0; i < funcid_array.length; i++) {
					funcid_array[i] = start;
					start += 1;
				}
			}
			{
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				Date start = sdf.parse("2000-01-01 00:00:00");
				for (int i = 0; i < receivetime_array.length; i++) {
					receivetime_array[i] = sdf.format(start);
					start = new Date(start.getTime() + 1000L * 60 * 5);
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) throws Exception {
		Configuration configuration = HBaseConfiguration.create();
		ZillionAgent agent = new ZillionAgent(configuration);
		agent.Start();

		try {
			agent.NDL_AddDatabase(database_name);
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			agent.DDL_DropTable(database_name, table_name);
		} catch (Exception e1) {
			e1.printStackTrace();
		}
		// ʼ
		JSONObject def = (JSONObject) JSONReaderUtil.Instance().Read(new File(def_path));
		JSONArray Columns = (JSONArray) def.get("Columns");
		JSONArray Key = (JSONArray) def.get("Key");

		JSONObject table_schema = new JSONObject();
		table_schema.put("Columns", Columns);
		table_schema.put("Key", Key);
		if (def.containsKey("delete_UsePutV")) {
			table_schema.put("delete_UsePutV", (Boolean) def.get("delete_UsePutV"));
		}
		agent.DDL_AddTable(database_name, table_name, table_schema, (JSONArray) def.get("Indexes"));
		insert(agent);
		System.out.println("insert finish");

		ZillionTableSchema schema = agent.container.repositoryMap.get(database_name).GetTable(table_name);

		{
			JSONArray Criterias = new JSONArray();
			int count_skip = 0;
			int count_limit = meter_array.length * funcid_array.length * receivetime_array.length;
			int index_curr = 0;
			for (int index1 = 0; index1 < building_array.length; index1++) {
				String building = building_array[index1];
				for (int index2 = 0; index2 < meter_array.length; index2++) {
					String meter = meter_array[index2];
					for (int index3 = 0; index3 < funcid_array.length; index3++) {
						Long funcid = funcid_array[index3];
						for (int index4 = 0; index4 < receivetime_array.length; index4++) {
							String receivetime = receivetime_array[index4];
							if (Criterias.size() >= count_limit) {
								break;
							}
							index_curr++;
							if (index_curr > count_skip) {
								JSONObject InsertObject = new JSONObject();
								InsertObject.put("building", building);
								InsertObject.put("meter", meter);
								InsertObject.put("funcid", funcid);
								InsertObject.put("receivetime", receivetime);
								Criterias.add(InsertObject);
							}
						}
					}
				}
			}
			LogUtil.warn("" + new Date());
			{
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "combine_select");
				wrapperObject.put("Criterias", Criterias);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				JSONObject queryResult = agent.Query(wrapperObject);
				JSONArray resultContent = (JSONArray) queryResult.get("Content");
				print(schema, resultContent);
			}
			LogUtil.warn("" + new Date());
			{
				JSONObject wrapperObject = new JSONObject();
				wrapperObject.put("QueryType", "batch_select");
				wrapperObject.put("Criterias", Criterias);
				wrapperObject.put("Database", database_name);
				wrapperObject.put("Datatable", table_name);
				JSONObject queryResult = agent.Query(wrapperObject);
				JSONArray resultContent = (JSONArray) queryResult.get("Content");
				JSONArray items = new JSONArray();
				for (int i = 0; i < resultContent.size(); i++) {
					JSONObject ci = (JSONObject) resultContent.get(i);
					items.addAll((JSONArray) ci.get("Content"));
				}
				print(schema, items);
			}
			LogUtil.warn("" + new Date());

		}

		agent.Stop();

		System.out.println();
	}

	static void print(ZillionTableSchema schema, JSONArray resultContent) {
		List<String> columnList = new ArrayList<String>();
		for (String column : schema.Key) {
			columnList.add(column);
		}
		for (String column : schema.OtherColumn) {
			columnList.add(column);
		}

		for (String column : columnList) {
			System.out.printf("%20s", column);
			System.out.print("\t");
		}
		System.out.println();
		for (int i = 0; i < resultContent.size(); i++) {
			JSONObject item = (JSONObject) resultContent.get(i);
			for (String column : columnList) {
				ZillionTableColumn column_schema = schema.ColumnMap.get(column);
				if (column_schema.Type == ZillionTableColumnEnum.BOOLEAN) {
					System.out.printf("%20s", ((Boolean) item.get(column)) ? "true" : "false");
				} else if (column_schema.Type == ZillionTableColumnEnum.LONG) {
					System.out.printf("%20d", (Long) item.get(column));
				} else if (column_schema.Type == ZillionTableColumnEnum.DOUBLE) {
					System.out.printf("%20.2f", (Double) item.get(column));
				} else if (column_schema.Type == ZillionTableColumnEnum.STRING) {
					System.out.printf("%20s", (String) item.get(column));
				}
				System.out.print("\t");
			}
			System.out.println();
		}
	}

	static void insert(ZillionAgent agent) throws Exception {
		Random rand = new Random();
		LogUtil.info("insert begin:" + building_array.length);
		for (int index1 = 0; index1 < building_array.length; index1++) {
			String building = building_array[index1];
			for (int index2 = 0; index2 < meter_array.length; index2++) {
				String meter = meter_array[index2];
				for (int index3 = 0; index3 < funcid_array.length; index3++) {
					Long funcid = funcid_array[index3];
					JSONArray InsertObjects = new JSONArray();
					for (int index4 = 0; index4 < receivetime_array.length; index4++) {
						String receivetime = receivetime_array[index4];
						JSONObject InsertObject = new JSONObject();
						InsertObject.put("building", building);
						InsertObject.put("meter", meter);
						InsertObject.put("funcid", funcid);
						InsertObject.put("receivetime", receivetime);
						InsertObject.put("data", rand.nextDouble());
						InsertObjects.add(InsertObject);
					}
					JSONObject wrapperObject = new JSONObject();
					wrapperObject.put("QueryType", "batch_insert");
					wrapperObject.put("InsertObjects", InsertObjects);
					wrapperObject.put("Database", database_name);
					wrapperObject.put("Datatable", table_name);
					agent.Query(wrapperObject);
				}
			}
			LogUtil.info("insert finish:" + (index1 + 1) + "/" + building_array.length);
		}
	}
}
